from httptransport import HttpTransport
from yangbase import YangBase


class Openhltest(YangBase):
	"""This module is the top level of the test hierarchy.
	"""
	def __init__(self, ip_address, rest_port):
		self.YANG_MODULE = ''
		self.YANG_CLASS = ''
		super(Openhltest, self).__init__(HttpTransport(ip_address, rest_port), None)

	def sessions(self, name=None):
		"""Get the sessions object(s) from the server.

		A list of test tool sessions. Sessions can onl be accessed based on the api-key submitted as part of the request. To get an api-key use the authenticate rpc.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the sessions list.  

		Returns:  
			:obj:`list` of :obj:`Sessions` | :obj:`Sessions`: If arg name is None a list of Sessions objects otherwise a single Sessions object.  

		Raises:  
			NotFoundError: name is not in the list of sessions objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(Sessions(self, name))

	def create_sessions(self, name):
		"""Create a child sibling sessions instance on the server.

		A list of test tool sessions. Sessions can onl be accessed based on the api-key submitted as part of the request. To get an api-key use the authenticate rpc.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`Sessions`: An object encapsulating an instance of the sessions model.  

		Raises:  
			AlreadyExistsError: An instance of sessions with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(Sessions(self, name), locals())

	def update(self):
		"""Update this openhltest instance on the server with any changed property values.

		This module is the top level of the test hierarchy.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class Sessions(YangBase):
	"""A list of test tool sessions. Sessions can onl be accessed based on the api-key submitted as part of the request. To get an api-key use the authenticate rpc.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sessions'
		self.YANG_KEY = 'name'
		super(Sessions, self).__init__(parent, yang_key_value)

	def config(self):
		"""Get the config object from the server

		This container aggregates all other top level configuration submodules.

		Returns:  
			:obj:`SessionsConfig`: An object encapsulating an instance of the config model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfig(self))

	def statistics(self):
		"""Get the statistics object from the server

		The statistics pull model

		Returns:  
			:obj:`SessionsStatistics`: An object encapsulating an instance of the statistics model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatistics(self))

	def delete(self):
		"""Delete this sessions instance on the server.

		A list of test tool sessions. Sessions can onl be accessed based on the api-key submitted as part of the request. To get an api-key use the authenticate rpc.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sessions instance on the server with any changed property values.

		A list of test tool sessions. Sessions can onl be accessed based on the api-key submitted as part of the request. To get an api-key use the authenticate rpc.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the test tool session. Once the session has been created the name cannot be modified."""
		return self._get_value('name')


class SessionsConfig(YangBase):
	"""This container aggregates all other top level configuration submodules.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'config'
		super(SessionsConfig, self).__init__(parent, None)

	def ports(self, name=None):
		"""Get the ports object(s) from the server.

		A list of abstract test ports

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ports list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPorts` | :obj:`SessionsConfigPorts`: If arg name is None a list of SessionsConfigPorts objects otherwise a single SessionsConfigPorts object.  

		Raises:  
			NotFoundError: name is not in the list of ports objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPorts(self, name))

	def create_ports(self, name):
		"""Create a child sibling ports instance on the server.

		A list of abstract test ports

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPorts`: An object encapsulating an instance of the ports model.  

		Raises:  
			AlreadyExistsError: An instance of ports with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPorts(self, name), locals())

	def layer1_groups(self, name=None):
		"""Get the layer1-groups object(s) from the server.

		A group of layer1 settings that will be applied to each port's location. The vendor implementation should apply layer 1 settings when starting protocols. If the port's location is empty then nothing will be applied to that port.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the layer1-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigLayer1Groups` | :obj:`SessionsConfigLayer1Groups`: If arg name is None a list of SessionsConfigLayer1Groups objects otherwise a single SessionsConfigLayer1Groups object.  

		Raises:  
			NotFoundError: name is not in the list of layer1-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1Groups(self, name))

	def create_layer1_groups(self, name):
		"""Create a child sibling layer1-groups instance on the server.

		A group of layer1 settings that will be applied to each port's location. The vendor implementation should apply layer 1 settings when starting protocols. If the port's location is empty then nothing will be applied to that port.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigLayer1Groups`: An object encapsulating an instance of the layer1-groups model.  

		Raises:  
			AlreadyExistsError: An instance of layer1-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigLayer1Groups(self, name), locals())

	def global_multicast_groups(self, name=None):
		"""Get the global-multicast-groups object(s) from the server.

		A list of multi-cast groups.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the global-multicast-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalMulticastGroups` | :obj:`SessionsConfigGlobalMulticastGroups`: If arg name is None a list of SessionsConfigGlobalMulticastGroups objects otherwise a single SessionsConfigGlobalMulticastGroups object.  

		Raises:  
			NotFoundError: name is not in the list of global-multicast-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroups(self, name))

	def create_global_multicast_groups(self, name):
		"""Create a child sibling global-multicast-groups instance on the server.

		A list of multi-cast groups.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroups`: An object encapsulating an instance of the global-multicast-groups model.  

		Raises:  
			AlreadyExistsError: An instance of global-multicast-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalMulticastGroups(self, name), locals())

	def global_protocols(self):
		"""Get the global-protocols object from the server

		This list allows for configuring global protocols options.

		Returns:  
			:obj:`SessionsConfigGlobalProtocols`: An object encapsulating an instance of the global-protocols model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocols(self))

	def device_groups(self, name=None):
		"""Get the device-groups object(s) from the server.

		A list of device-groups. Each device-groups object can contain 0..n devices.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the device-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroups` | :obj:`SessionsConfigDeviceGroups`: If arg name is None a list of SessionsConfigDeviceGroups objects otherwise a single SessionsConfigDeviceGroups object.  

		Raises:  
			NotFoundError: name is not in the list of device-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroups(self, name))

	def create_device_groups(self, name):
		"""Create a child sibling device-groups instance on the server.

		A list of device-groups. Each device-groups object can contain 0..n devices.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroups`: An object encapsulating an instance of the device-groups model.  

		Raises:  
			AlreadyExistsError: An instance of device-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroups(self, name), locals())

	def port_traffic(self, name=None):
		"""Get the port-traffic object(s) from the server.

		A list of traffic streams where each traffic stream has a single transmit port as its source and a list of user defined frames.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the port-traffic list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTraffic` | :obj:`SessionsConfigPortTraffic`: If arg name is None a list of SessionsConfigPortTraffic objects otherwise a single SessionsConfigPortTraffic object.  

		Raises:  
			NotFoundError: name is not in the list of port-traffic objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTraffic(self, name))

	def create_port_traffic(self, name):
		"""Create a child sibling port-traffic instance on the server.

		A list of traffic streams where each traffic stream has a single transmit port as its source and a list of user defined frames.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTraffic`: An object encapsulating an instance of the port-traffic model.  

		Raises:  
			AlreadyExistsError: An instance of port-traffic with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTraffic(self, name), locals())

	def device_traffic(self, name=None):
		"""Get the device-traffic object(s) from the server.

		This is a collection of nodes that the server uses to generate 1..n traffic-streams. The source and destinations of the device traffic group can only come from the device-groups list or its children. 

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the device-traffic list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTraffic` | :obj:`SessionsConfigDeviceTraffic`: If arg name is None a list of SessionsConfigDeviceTraffic objects otherwise a single SessionsConfigDeviceTraffic object.  

		Raises:  
			NotFoundError: name is not in the list of device-traffic objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTraffic(self, name))

	def create_device_traffic(self, name):
		"""Create a child sibling device-traffic instance on the server.

		This is a collection of nodes that the server uses to generate 1..n traffic-streams. The source and destinations of the device traffic group can only come from the device-groups list or its children. 

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTraffic`: An object encapsulating an instance of the device-traffic model.  

		Raises:  
			AlreadyExistsError: An instance of device-traffic with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTraffic(self, name), locals())

	def delete(self):
		"""Delete this config instance on the server.

		This container aggregates all other top level configuration submodules.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this config instance on the server with any changed property values.

		This container aggregates all other top level configuration submodules.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	class PortControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class PortControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def port_control(self, port_control_input):
		"""Execute the port-control action on the server

		Control one or more physical hardware test ports and/or virtual machine test ports. An empty targets list signifies that all ports will be subject to the mode specified.

		Args:  
			port_control_input (:obj:`PortControlInput`):  An object encapsulating an instance of the port-control action input model.  

		Returns:  
			:obj:`PortControlOutput`:  An object encapsulating an instance of the port-control action output model.  

		Raises:  
			BadRequestError: The port_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/port-control', port_control_input)

	def clear(self):
		"""Execute the clear action on the server

		Clear the current configuration

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/clear')

	class LoadInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.vendor_config = ''

	def load(self, load_input):
		"""Execute the load action on the server

		Load a vendor specific configuration

		Args:  
			load_input (:obj:`LoadInput`):  An object encapsulating an instance of the load action input model.  

		Raises:  
			BadRequestError: The load_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/load', load_input)

	class SaveConfigInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.filename = ''

	def save_config(self, save_config_input):
		"""Execute the save-config action on the server

		Save configuration in vendor specific format

		Args:  
			save_config_input (:obj:`SaveConfigInput`):  An object encapsulating an instance of the save-config action input model.  

		Raises:  
			BadRequestError: The save_config_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/save-config', save_config_input)

	class ArpControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []

	def arp_control(self, arp_control_input):
		"""Execute the arp-control action on the server

		ARP control command. An empty targets list signifies that ARP will be performed globally

		Args:  
			arp_control_input (:obj:`ArpControlInput`):  An object encapsulating an instance of the arp-control action input model.  

		Raises:  
			BadRequestError: The arp_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/arp-control', arp_control_input)

	class TrafficControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class TrafficControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def traffic_control(self, traffic_control_input):
		"""Execute the traffic-control action on the server

		Control one or more traffic groups. An empty list signifies that all traffic will be subject to the mode specified.

		Args:  
			traffic_control_input (:obj:`TrafficControlInput`):  An object encapsulating an instance of the traffic-control action input model.  

		Returns:  
			:obj:`TrafficControlOutput`:  An object encapsulating an instance of the traffic-control action output model.  

		Raises:  
			BadRequestError: The traffic_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/traffic-control', traffic_control_input)

	class DeviceGroupsControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class DeviceGroupsControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def device_groups_control(self, device_groups_control_input):
		"""Execute the device-groups-control action on the server

		Control one or more device-groups. An empty list signifies that all device-groups will be subject to the mode specified.

		Args:  
			device_groups_control_input (:obj:`DeviceGroupsControlInput`):  An object encapsulating an instance of the device-groups-control action input model.  

		Returns:  
			:obj:`DeviceGroupsControlOutput`:  An object encapsulating an instance of the device-groups-control action output model.  

		Raises:  
			BadRequestError: The device_groups_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/device-groups-control', device_groups_control_input)

	class Bgpv4ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Bgpv4ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def bgpv4_control(self, bgpv4_control_input):
		"""Execute the bgpv4-control action on the server

		Start one or more emulated BGPV4 protocol groups. An empty targets list signifies that all BGPV4 protocol groups will be subject to the mode specified.

		Args:  
			bgpv4_control_input (:obj:`Bgpv4ControlInput`):  An object encapsulating an instance of the bgpv4-control action input model.  

		Returns:  
			:obj:`Bgpv4ControlOutput`:  An object encapsulating an instance of the bgpv4-control action output model.  

		Raises:  
			BadRequestError: The bgpv4_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/bgpv4-control', bgpv4_control_input)

	class Bgpv6ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Bgpv6ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def bgpv6_control(self, bgpv6_control_input):
		"""Execute the bgpv6-control action on the server

		Start one or more emulated BGPV6 protocol groups. An empty targets list signifies that all BGPV6 protocol groups will be subject to the mode specified.

		Args:  
			bgpv6_control_input (:obj:`Bgpv6ControlInput`):  An object encapsulating an instance of the bgpv6-control action input model.  

		Returns:  
			:obj:`Bgpv6ControlOutput`:  An object encapsulating an instance of the bgpv6-control action output model.  

		Raises:  
			BadRequestError: The bgpv6_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/bgpv6-control', bgpv6_control_input)

	class Ospfv2ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Ospfv2ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def ospfv2_control(self, ospfv2_control_input):
		"""Execute the ospfv2-control action on the server

		Start one or more emulated OSPFV2 protocol groups. An empty targets list signifies that all OSPFV2 protocol groups will be subject to the mode specified.

		Args:  
			ospfv2_control_input (:obj:`Ospfv2ControlInput`):  An object encapsulating an instance of the ospfv2-control action input model.  

		Returns:  
			:obj:`Ospfv2ControlOutput`:  An object encapsulating an instance of the ospfv2-control action output model.  

		Raises:  
			BadRequestError: The ospfv2_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/ospfv2-control', ospfv2_control_input)

	class Ospfv3ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Ospfv3ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def ospfv3_control(self, ospfv3_control_input):
		"""Execute the ospfv3-control action on the server

		Start one or more emulated OSPFV3 protocol groups. An empty targets list signifies that all OSPFV3 protocol groups will be subject to the mode specified.

		Args:  
			ospfv3_control_input (:obj:`Ospfv3ControlInput`):  An object encapsulating an instance of the ospfv3-control action input model.  

		Returns:  
			:obj:`Ospfv3ControlOutput`:  An object encapsulating an instance of the ospfv3-control action output model.  

		Raises:  
			BadRequestError: The ospfv3_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/ospfv3-control', ospfv3_control_input)

	class IsisControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class IsisControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def isis_control(self, isis_control_input):
		"""Execute the isis-control action on the server

		Start one or more emulated ISIS protocol groups. An empty targets list signifies that all ISIS protocol groups will be subject to the mode specified.

		Args:  
			isis_control_input (:obj:`IsisControlInput`):  An object encapsulating an instance of the isis-control action input model.  

		Returns:  
			:obj:`IsisControlOutput`:  An object encapsulating an instance of the isis-control action output model.  

		Raises:  
			BadRequestError: The isis_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/isis-control', isis_control_input)

	class Bfdv4ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Bfdv4ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def bfdv4_control(self, bfdv4_control_input):
		"""Execute the bfdv4-control action on the server

		Start one or more emulated BFD v4 protocol groups. An empty targets list signifies that all BFD v4 protocol groups will be subject to the mode specified.

		Args:  
			bfdv4_control_input (:obj:`Bfdv4ControlInput`):  An object encapsulating an instance of the bfdv4-control action input model.  

		Returns:  
			:obj:`Bfdv4ControlOutput`:  An object encapsulating an instance of the bfdv4-control action output model.  

		Raises:  
			BadRequestError: The bfdv4_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/bfdv4-control', bfdv4_control_input)

	class Bfdv6ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Bfdv6ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def bfdv6_control(self, bfdv6_control_input):
		"""Execute the bfdv6-control action on the server

		Start one or more emulated BFD v6 protocol groups. An empty targets list signifies that all BFD v6 protocol groups will be subject to the mode specified.

		Args:  
			bfdv6_control_input (:obj:`Bfdv6ControlInput`):  An object encapsulating an instance of the bfdv6-control action input model.  

		Returns:  
			:obj:`Bfdv6ControlOutput`:  An object encapsulating an instance of the bfdv6-control action output model.  

		Raises:  
			BadRequestError: The bfdv6_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/bfdv6-control', bfdv6_control_input)

	class Dhcpv4ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Dhcpv4ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def dhcpv4_control(self, dhcpv4_control_input):
		"""Execute the dhcpv4-control action on the server

		Start one or more emulated DHCPv4 Client protocol groups. An empty targets list signifies that all DHCPv4 Client protocol groups will be subject to the mode specified.

		Args:  
			dhcpv4_control_input (:obj:`Dhcpv4ControlInput`):  An object encapsulating an instance of the dhcpv4-control action input model.  

		Returns:  
			:obj:`Dhcpv4ControlOutput`:  An object encapsulating an instance of the dhcpv4-control action output model.  

		Raises:  
			BadRequestError: The dhcpv4_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/dhcpv4-control', dhcpv4_control_input)

	class Dhcpv4ServerControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Dhcpv4ServerControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def dhcpv4_server_control(self, dhcpv4_server_control_input):
		"""Execute the dhcpv4-server-control action on the server

		Start one or more emulated DHCPv4 Server protocol groups. An empty targets list signifies that all DHCPv4 Server protocol groups will be subject to the mode specified.

		Args:  
			dhcpv4_server_control_input (:obj:`Dhcpv4ServerControlInput`):  An object encapsulating an instance of the dhcpv4-server-control action input model.  

		Returns:  
			:obj:`Dhcpv4ServerControlOutput`:  An object encapsulating an instance of the dhcpv4-server-control action output model.  

		Raises:  
			BadRequestError: The dhcpv4_server_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/dhcpv4-server-control', dhcpv4_server_control_input)

	class Dhcpv6ControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''
			self.filename = ''

	class Dhcpv6ControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def dhcpv6_control(self, dhcpv6_control_input):
		"""Execute the dhcpv6-control action on the server

		Start one or more emulated DHCPv6/PD Client protocol groups. An empty targets list signifies that all DHCPv6/PD Client protocol groups will be subject to the mode specified.

		Args:  
			dhcpv6_control_input (:obj:`Dhcpv6ControlInput`):  An object encapsulating an instance of the dhcpv6-control action input model.  

		Returns:  
			:obj:`Dhcpv6ControlOutput`:  An object encapsulating an instance of the dhcpv6-control action output model.  

		Raises:  
			BadRequestError: The dhcpv6_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/dhcpv6-control', dhcpv6_control_input)

	class Dhcpv6ServerControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class Dhcpv6ServerControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def dhcpv6_server_control(self, dhcpv6_server_control_input):
		"""Execute the dhcpv6-server-control action on the server

		Start one or more emulated DHCPv6 Server protocol groups. An empty targets list signifies that all DHCPv6 Server protocol groups will be subject to the mode specified.

		Args:  
			dhcpv6_server_control_input (:obj:`Dhcpv6ServerControlInput`):  An object encapsulating an instance of the dhcpv6-server-control action input model.  

		Returns:  
			:obj:`Dhcpv6ServerControlOutput`:  An object encapsulating an instance of the dhcpv6-server-control action output model.  

		Raises:  
			BadRequestError: The dhcpv6_server_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/dhcpv6-server-control', dhcpv6_server_control_input)

	class IgmpControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''
			self.calculate_latency = ''
			self.join_failed_retry_counter = ''
			self.join_leave_delay = ''
			self.rx_data_duration = ''

	class IgmpControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def igmp_control(self, igmp_control_input):
		"""Execute the igmp-control action on the server

		Start one or more emulated IGMP Hosts protocol groups. An empty targets list signifies that all IGMP Hosts groups will be subject to the mode specified.

		Args:  
			igmp_control_input (:obj:`IgmpControlInput`):  An object encapsulating an instance of the igmp-control action input model.  

		Returns:  
			:obj:`IgmpControlOutput`:  An object encapsulating an instance of the igmp-control action output model.  

		Raises:  
			BadRequestError: The igmp_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/igmp-control', igmp_control_input)

	class IgmpQuerierControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class IgmpQuerierControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def igmp_querier_control(self, igmp_querier_control_input):
		"""Execute the igmp-querier-control action on the server

		Start or Stops one or more emulated IGMP querier protocol groups. An empty targets list signifies that all IGMP querier groups will be subject to the mode specified.

		Args:  
			igmp_querier_control_input (:obj:`IgmpQuerierControlInput`):  An object encapsulating an instance of the igmp-querier-control action input model.  

		Returns:  
			:obj:`IgmpQuerierControlOutput`:  An object encapsulating an instance of the igmp-querier-control action output model.  

		Raises:  
			BadRequestError: The igmp_querier_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/igmp-querier-control', igmp_querier_control_input)

	class MldControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''
			self.calculate_latency = ''
			self.join_failed_retry_counter = ''
			self.join_leave_delay = ''
			self.rx_data_duration = ''

	class MldControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def mld_control(self, mld_control_input):
		"""Execute the mld-control action on the server

		Start one or more emulated MLD Hosts protocol groups. An empty targets list signifies that all MLD Hosts groups will be subject to the mode specified.

		Args:  
			mld_control_input (:obj:`MldControlInput`):  An object encapsulating an instance of the mld-control action input model.  

		Returns:  
			:obj:`MldControlOutput`:  An object encapsulating an instance of the mld-control action output model.  

		Raises:  
			BadRequestError: The mld_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/mld-control', mld_control_input)

	class MldQuerierControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''

	class MldQuerierControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def mld_querier_control(self, mld_querier_control_input):
		"""Execute the mld-querier-control action on the server

		Start or Stops one or more emulated MLD querier protocol groups. An empty targets list signifies that all MLD querier groups will be subject to the mode specified.

		Args:  
			mld_querier_control_input (:obj:`MldQuerierControlInput`):  An object encapsulating an instance of the mld-querier-control action input model.  

		Returns:  
			:obj:`MldQuerierControlOutput`:  An object encapsulating an instance of the mld-querier-control action output model.  

		Raises:  
			BadRequestError: The mld_querier_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/mld-querier-control', mld_querier_control_input)

	class PimControlInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.targets = []
			self.mode = ''
			self.data_mdt_multicast_group = ''
			self.mdt_delay = ''
			self.mdt_interval = ''
			self.traffic_targets = []

	class PimControlOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.errata = []

		class Errata(object):
			"""A list of rpc errors. An empty list indicates no errors occurred
			"""
			def __init__(self):
				self.name = ''
				self.detail = ''
				self.stack_trace = ''

	def pim_control(self, pim_control_input):
		"""Execute the pim-control action on the server

		Start or Stops one or more emulated PIM protocol groups. An empty targets list signifies that all PIM groups will be subject to the mode specified.

		Args:  
			pim_control_input (:obj:`PimControlInput`):  An object encapsulating an instance of the pim-control action input model.  

		Returns:  
			:obj:`PimControlOutput`:  An object encapsulating an instance of the pim-control action output model.  

		Raises:  
			BadRequestError: The pim_control_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/pim-control', pim_control_input)

	class InvokeInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.command = ''

	class InvokeOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.retvalue = ''

	def invoke(self, invoke_input):
		"""Execute the invoke action on the server

		invoke the stc native commands

		Args:  
			invoke_input (:obj:`InvokeInput`):  An object encapsulating an instance of the invoke action input model.  

		Returns:  
			:obj:`InvokeOutput`:  An object encapsulating an instance of the invoke action output model.  

		Raises:  
			BadRequestError: The invoke_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/invoke', invoke_input)

	class VerifyTrafficInput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:input'
			self.device_traffic = ''
			self.port_traffic = ''
			self.tolerance = ''
			self.expected = ''

	class VerifyTrafficOutput(object):
		"""TBD
		"""
		def __init__(self):
			self.YANG_PATH = 'openhltest:output'
			self.status = ''
			self.loss = ''
			self.rx_frames = ''
			self.tx_frames = ''

	def verify_traffic(self, verify_traffic_input):
		"""Execute the verify-traffic action on the server

		Verify traffic results

		Args:  
			verify_traffic_input (:obj:`VerifyTrafficInput`):  An object encapsulating an instance of the verify-traffic action input model.  

		Returns:  
			:obj:`VerifyTrafficOutput`:  An object encapsulating an instance of the verify-traffic action output model.  

		Raises:  
			BadRequestError: The verify_traffic_input arg has invalid values.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/verify-traffic', verify_traffic_input)


class SessionsConfigPorts(YangBase):
	"""A list of abstract test ports
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ports'
		self.YANG_KEY = 'name'
		super(SessionsConfigPorts, self).__init__(parent, yang_key_value)

	def delete(self):
		"""Delete this ports instance on the server.

		A list of abstract test ports

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ports instance on the server with any changed property values.

		A list of abstract test ports

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of an abstract test port"""
		return self._get_value('name')

	@property
	def location(self):
		"""str: The location of the physical or virtual port that will be connected to this abstract port. The value pattern must be chassis/card/port where chassis is an ipv4/ipv6 address and card/port are unsigned byte 1-255. The separator between the chassis/card/port values must be a forward slash."""
		return self._get_value('location')

	@location.setter
	def location(self, value):
		return self._set_value('location', value)


class SessionsConfigLayer1Groups(YangBase):
	"""A group of layer1 settings that will be applied to each port's location. The vendor implementation should apply layer 1 settings when starting protocols. If the port's location is empty then nothing will be applied to that port.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'layer1-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigLayer1Groups, self).__init__(parent, yang_key_value)

	def ethernet_copper(self):
		"""Get the ethernet-copper object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsEthernetCopper`: An object encapsulating an instance of the ethernet-copper model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsEthernetCopper(self))

	def ethernet_fiber(self):
		"""Get the ethernet-fiber object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsEthernetFiber`: An object encapsulating an instance of the ethernet-fiber model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsEthernetFiber(self))

	def ten_gig_copper(self):
		"""Get the ten-gig-copper object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigCopper`: An object encapsulating an instance of the ten-gig-copper model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigCopper(self))

	def ten_gig_fiber(self):
		"""Get the ten-gig-fiber object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigFiber`: An object encapsulating an instance of the ten-gig-fiber model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigFiber(self))

	def twenty_five_gig(self):
		"""Get the twenty-five-gig object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTwentyFiveGig`: An object encapsulating an instance of the twenty-five-gig model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTwentyFiveGig(self))

	def forty_gig(self):
		"""Get the forty-gig object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsFortyGig`: An object encapsulating an instance of the forty-gig model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsFortyGig(self))

	def one_hundred_gig(self):
		"""Get the one-hundred-gig object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsOneHundredGig`: An object encapsulating an instance of the one-hundred-gig model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsOneHundredGig(self))

	def delete(self):
		"""Delete this layer1-groups instance on the server.

		A group of layer1 settings that will be applied to each port's location. The vendor implementation should apply layer 1 settings when starting protocols. If the port's location is empty then nothing will be applied to that port.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this layer1-groups instance on the server with any changed property values.

		A group of layer1 settings that will be applied to each port's location. The vendor implementation should apply layer 1 settings when starting protocols. If the port's location is empty then nothing will be applied to that port.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of a layer1 group"""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../ports/name: A list of abstract test port names. Every object in the devices and protocols lists will share the ports assigned to a device-groups object. An abstract test port name cannot be assigned to more than one device-groups object."""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)

	@property
	def layer1_type(self):
		""":str:`enum`: TBD  
		Enums:  
			ETHERNET_COPPER: TBD  
			ETHERNET_FIBER: TBD  
			TEN_GIG_COPPER: TBD  
			TEN_GIG_FIBER: TBD  
			TWENTY_FIVE_GIG: TBD  
			FORTY_GIG: TBD  
			FIFTY_GIG: TBD  
			ONE_HUNDRED_GIG: TBD  
			TWO_HUNDRED_GIG: TBD  
			FOUR_HUNDRED_GIG: TBD  
		"""
		return self._get_value('layer1-type')

	@layer1_type.setter
	def layer1_type(self, value):
		return self._set_value('layer1-type', value)


class SessionsConfigLayer1GroupsEthernetCopper(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet-copper'
		super(SessionsConfigLayer1GroupsEthernetCopper, self).__init__(parent, None)

	def auto_negotiation_capabilities(self):
		"""Get the auto-negotiation-capabilities object from the server

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Returns:  
			:obj:`SessionsConfigLayer1GroupsEthernetCopperAutoNegotiationCapabilities`: An object encapsulating an instance of the auto-negotiation-capabilities model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsEthernetCopperAutoNegotiationCapabilities(self))

	def delete(self):
		"""Delete this ethernet-copper instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet-copper instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def auto_mdix(self):
		"""bool: TBD"""
		return self._get_value('auto-mdix')

	@auto_mdix.setter
	def auto_mdix(self, value):
		return self._set_value('auto-mdix', value)

	@property
	def speed_duplex_type(self):
		""":str:`enum`: When auto negotiation is disabled a single speed and duplex type can be selected  
		Enums:  
			1000G: TBD  
			100M_FD: TBD  
			100M_HD: TBD  
			10M_FD: TBD  
			10M_HD: TBD  
		"""
		return self._get_value('speed-duplex-type')

	@speed_duplex_type.setter
	def speed_duplex_type(self, value):
		return self._set_value('speed-duplex-type', value)


class SessionsConfigLayer1GroupsEthernetCopperAutoNegotiationCapabilities(YangBase):
	"""When auto negotiation is enabled the following speeds and duplex can be advertised
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auto-negotiation-capabilities'
		super(SessionsConfigLayer1GroupsEthernetCopperAutoNegotiationCapabilities, self).__init__(parent, None)

	def delete(self):
		"""Delete this auto-negotiation-capabilities instance on the server.

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auto-negotiation-capabilities instance on the server with any changed property values.

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def advertise_1000g(self):
		"""bool: TBD"""
		return self._get_value('advertise-1000g')

	@advertise_1000g.setter
	def advertise_1000g(self, value):
		return self._set_value('advertise-1000g', value)

	@property
	def advertise_100m_fd(self):
		"""bool: TBD"""
		return self._get_value('advertise-100m-fd')

	@advertise_100m_fd.setter
	def advertise_100m_fd(self, value):
		return self._set_value('advertise-100m-fd', value)

	@property
	def advertise_100m_hd(self):
		"""bool: TBD"""
		return self._get_value('advertise-100m-hd')

	@advertise_100m_hd.setter
	def advertise_100m_hd(self, value):
		return self._set_value('advertise-100m-hd', value)

	@property
	def advertise_10m_fd(self):
		"""bool: TBD"""
		return self._get_value('advertise-10m-fd')

	@advertise_10m_fd.setter
	def advertise_10m_fd(self, value):
		return self._set_value('advertise-10m-fd', value)

	@property
	def advertise_10m_hd(self):
		"""bool: TBD"""
		return self._get_value('advertise-10m-hd')

	@advertise_10m_hd.setter
	def advertise_10m_hd(self, value):
		return self._set_value('advertise-10m-hd', value)


class SessionsConfigLayer1GroupsEthernetFiber(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet-fiber'
		super(SessionsConfigLayer1GroupsEthernetFiber, self).__init__(parent, None)

	def delete(self):
		"""Delete this ethernet-fiber instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet-fiber instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def cable_type_length(self):
		""":str:`enum`: TBD  
		Enums:  
			OPTICAL: TBD  
			COPPER_OM_2M: TBD  
			COPPER_3M: TBD  
			COPPER_5M: TBD  
			COPPER_7M: TBD  
			COPPER_OM_1M: TBD  
			COPPER_2M: TBD  
			COPPER_5M_ACTIVE: TBD  
			COPPER_7M_ACTIVE: TBD  
		"""
		return self._get_value('cable-type-length')

	@cable_type_length.setter
	def cable_type_length(self, value):
		return self._set_value('cable-type-length', value)

	@property
	def cfp_interface(self):
		""":str:`enum`: TBD  
		Enums:  
			ACC_6068A: TBD  
			ACC_6069A: TBD  
			ACC_6068A_CISCO_100G_SR10: TBD  
			ACC_6069A_CISCO_40G_SR4: TBD  
			CISCO_FINISAR_100G_LR4_v1_5: TBD  
			OTHER_CFP: TBD  
			LR4_100GBASE: TBD  
			SR10_100GBASE: TBD  
			CXP_SR10_100GBASE: TBD  
			CFP4_LR4_100GBASE: TBD  
			CFP4_SR4_100GBASE: TBD  
			CFP4_CR4_100GBASE: TBD  
			QSFP28_LR4_100GBASE: TBD  
			QSFP28_SR4_100GBASE: TBD  
			QSFP28_CR4_100GBASE: TBD  
			QSFP28_AOC_100GBASE: TBD  
		"""
		return self._get_value('cfp-interface')

	@cfp_interface.setter
	def cfp_interface(self, value):
		return self._set_value('cfp-interface', value)

	@property
	def deficit_idle_count(self):
		"""bool: Deficit Idle Count"""
		return self._get_value('deficit-idle-count')

	@deficit_idle_count.setter
	def deficit_idle_count(self, value):
		return self._set_value('deficit-idle-count', value)


class SessionsConfigLayer1GroupsTenGigCopper(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ten-gig-copper'
		super(SessionsConfigLayer1GroupsTenGigCopper, self).__init__(parent, None)

	def auto_negotiation_capabilities(self):
		"""Get the auto-negotiation-capabilities object from the server

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigCopperAutoNegotiationCapabilities`: An object encapsulating an instance of the auto-negotiation-capabilities model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigCopperAutoNegotiationCapabilities(self))

	def priority_based_flow_control(self):
		"""Get the priority-based-flow-control object from the server

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigCopperPriorityBasedFlowControl`: An object encapsulating an instance of the priority-based-flow-control model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigCopperPriorityBasedFlowControl(self))

	def data_center_bridging_exchange(self):
		"""Get the data-center-bridging-exchange object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange(self))

	def create_data_center_bridging_exchange(self):
		"""Create a child sibling data-center-bridging-exchange instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			AlreadyExistsError: An instance of data-center-bridging-exchange with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange(self), locals())

	def delete(self):
		"""Delete this ten-gig-copper instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ten-gig-copper instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def auto_mdix(self):
		"""bool: TBD"""
		return self._get_value('auto-mdix')

	@auto_mdix.setter
	def auto_mdix(self, value):
		return self._set_value('auto-mdix', value)

	@property
	def speed_duplex_type(self):
		""":str:`enum`: When auto negotiation is disabled a single speed and duplex type can be selected  
		Enums:  
			1000G: TBD  
			100M_FD: TBD  
			100M_HD: TBD  
			10M_FD: TBD  
			10M_HD: TBD  
		"""
		return self._get_value('speed-duplex-type')

	@speed_duplex_type.setter
	def speed_duplex_type(self, value):
		return self._set_value('speed-duplex-type', value)

	@property
	def forward_error_correction(self):
		"""bool: TBD"""
		return self._get_value('forward-error-correction')

	@forward_error_correction.setter
	def forward_error_correction(self, value):
		return self._set_value('forward-error-correction', value)

	@property
	def ignore_link_status(self):
		"""bool: TBD"""
		return self._get_value('ignore-link-status')

	@ignore_link_status.setter
	def ignore_link_status(self, value):
		return self._set_value('ignore-link-status', value)

	@property
	def internal_ppm_adjust(self):
		"""int8: TBD"""
		return self._get_value('internal-ppm-adjust')

	@internal_ppm_adjust.setter
	def internal_ppm_adjust(self, value):
		return self._set_value('internal-ppm-adjust', value)

	@property
	def optimized_xon(self):
		""":str:`enum`: TBD  
		Enums:  
			DISABLE: TBD  
			ENABLE: TBD  
		"""
		return self._get_value('optimized-xon')

	@optimized_xon.setter
	def optimized_xon(self, value):
		return self._set_value('optimized-xon', value)

	@property
	def transmit_clock_source(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERNAL: TBD  
			INTERNAL_PPM_ADJ: TBD  
			BITS: TBD  
			LOOP: TBD  
			EXTERNAL: TBD  
		"""
		return self._get_value('transmit-clock-source')

	@transmit_clock_source.setter
	def transmit_clock_source(self, value):
		return self._set_value('transmit-clock-source', value)


class SessionsConfigLayer1GroupsTenGigCopperAutoNegotiationCapabilities(YangBase):
	"""When auto negotiation is enabled the following speeds and duplex can be advertised
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auto-negotiation-capabilities'
		super(SessionsConfigLayer1GroupsTenGigCopperAutoNegotiationCapabilities, self).__init__(parent, None)

	def delete(self):
		"""Delete this auto-negotiation-capabilities instance on the server.

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auto-negotiation-capabilities instance on the server with any changed property values.

		When auto negotiation is enabled the following speeds and duplex can be advertised

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def advertise_1000g(self):
		"""bool: TBD"""
		return self._get_value('advertise-1000g')

	@advertise_1000g.setter
	def advertise_1000g(self, value):
		return self._set_value('advertise-1000g', value)

	@property
	def advertise_100m_fd(self):
		"""bool: TBD"""
		return self._get_value('advertise-100m-fd')

	@advertise_100m_fd.setter
	def advertise_100m_fd(self, value):
		return self._set_value('advertise-100m-fd', value)

	@property
	def advertise_100m_hd(self):
		"""bool: TBD"""
		return self._get_value('advertise-100m-hd')

	@advertise_100m_hd.setter
	def advertise_100m_hd(self, value):
		return self._set_value('advertise-100m-hd', value)

	@property
	def advertise_10m_fd(self):
		"""bool: TBD"""
		return self._get_value('advertise-10m-fd')

	@advertise_10m_fd.setter
	def advertise_10m_fd(self, value):
		return self._set_value('advertise-10m-fd', value)

	@property
	def advertise_10m_hd(self):
		"""bool: TBD"""
		return self._get_value('advertise-10m-hd')

	@advertise_10m_hd.setter
	def advertise_10m_hd(self, value):
		return self._set_value('advertise-10m-hd', value)


class SessionsConfigLayer1GroupsTenGigCopperPriorityBasedFlowControl(YangBase):
	"""Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority-based-flow-control'
		super(SessionsConfigLayer1GroupsTenGigCopperPriorityBasedFlowControl, self).__init__(parent, None)

	def delete(self):
		"""Delete this priority-based-flow-control instance on the server.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority-based-flow-control instance on the server with any changed property values.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'data-center-bridging-exchange'
		super(SessionsConfigLayer1GroupsTenGigCopperDataCenterBridgingExchange, self).__init__(parent, None)

	def delete(self):
		"""Delete this data-center-bridging-exchange instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this data-center-bridging-exchange instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsTenGigFiber(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ten-gig-fiber'
		super(SessionsConfigLayer1GroupsTenGigFiber, self).__init__(parent, None)

	def priority_based_flow_control(self):
		"""Get the priority-based-flow-control object from the server

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigFiberPriorityBasedFlowControl`: An object encapsulating an instance of the priority-based-flow-control model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigFiberPriorityBasedFlowControl(self))

	def data_center_bridging_exchange(self):
		"""Get the data-center-bridging-exchange object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange(self))

	def create_data_center_bridging_exchange(self):
		"""Create a child sibling data-center-bridging-exchange instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			AlreadyExistsError: An instance of data-center-bridging-exchange with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange(self), locals())

	def delete(self):
		"""Delete this ten-gig-fiber instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ten-gig-fiber instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def cable_type_length(self):
		""":str:`enum`: TBD  
		Enums:  
			OPTICAL: TBD  
			COPPER_OM_2M: TBD  
			COPPER_3M: TBD  
			COPPER_5M: TBD  
			COPPER_7M: TBD  
			COPPER_OM_1M: TBD  
			COPPER_2M: TBD  
			COPPER_5M_ACTIVE: TBD  
			COPPER_7M_ACTIVE: TBD  
		"""
		return self._get_value('cable-type-length')

	@cable_type_length.setter
	def cable_type_length(self, value):
		return self._set_value('cable-type-length', value)

	@property
	def cfp_interface(self):
		""":str:`enum`: TBD  
		Enums:  
			ACC_6068A: TBD  
			ACC_6069A: TBD  
			ACC_6068A_CISCO_100G_SR10: TBD  
			ACC_6069A_CISCO_40G_SR4: TBD  
			CISCO_FINISAR_100G_LR4_v1_5: TBD  
			OTHER_CFP: TBD  
			LR4_100GBASE: TBD  
			SR10_100GBASE: TBD  
			CXP_SR10_100GBASE: TBD  
			CFP4_LR4_100GBASE: TBD  
			CFP4_SR4_100GBASE: TBD  
			CFP4_CR4_100GBASE: TBD  
			QSFP28_LR4_100GBASE: TBD  
			QSFP28_SR4_100GBASE: TBD  
			QSFP28_CR4_100GBASE: TBD  
			QSFP28_AOC_100GBASE: TBD  
		"""
		return self._get_value('cfp-interface')

	@cfp_interface.setter
	def cfp_interface(self, value):
		return self._set_value('cfp-interface', value)

	@property
	def deficit_idle_count(self):
		"""bool: Deficit Idle Count"""
		return self._get_value('deficit-idle-count')

	@deficit_idle_count.setter
	def deficit_idle_count(self, value):
		return self._set_value('deficit-idle-count', value)

	@property
	def forward_error_correction(self):
		"""bool: TBD"""
		return self._get_value('forward-error-correction')

	@forward_error_correction.setter
	def forward_error_correction(self, value):
		return self._set_value('forward-error-correction', value)

	@property
	def ignore_link_status(self):
		"""bool: TBD"""
		return self._get_value('ignore-link-status')

	@ignore_link_status.setter
	def ignore_link_status(self, value):
		return self._set_value('ignore-link-status', value)

	@property
	def internal_ppm_adjust(self):
		"""int8: TBD"""
		return self._get_value('internal-ppm-adjust')

	@internal_ppm_adjust.setter
	def internal_ppm_adjust(self, value):
		return self._set_value('internal-ppm-adjust', value)

	@property
	def optimized_xon(self):
		""":str:`enum`: TBD  
		Enums:  
			DISABLE: TBD  
			ENABLE: TBD  
		"""
		return self._get_value('optimized-xon')

	@optimized_xon.setter
	def optimized_xon(self, value):
		return self._set_value('optimized-xon', value)

	@property
	def transmit_clock_source(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERNAL: TBD  
			INTERNAL_PPM_ADJ: TBD  
			BITS: TBD  
			LOOP: TBD  
			EXTERNAL: TBD  
		"""
		return self._get_value('transmit-clock-source')

	@transmit_clock_source.setter
	def transmit_clock_source(self, value):
		return self._set_value('transmit-clock-source', value)


class SessionsConfigLayer1GroupsTenGigFiberPriorityBasedFlowControl(YangBase):
	"""Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority-based-flow-control'
		super(SessionsConfigLayer1GroupsTenGigFiberPriorityBasedFlowControl, self).__init__(parent, None)

	def delete(self):
		"""Delete this priority-based-flow-control instance on the server.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority-based-flow-control instance on the server with any changed property values.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'data-center-bridging-exchange'
		super(SessionsConfigLayer1GroupsTenGigFiberDataCenterBridgingExchange, self).__init__(parent, None)

	def delete(self):
		"""Delete this data-center-bridging-exchange instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this data-center-bridging-exchange instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsTwentyFiveGig(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'twenty-five-gig'
		super(SessionsConfigLayer1GroupsTwentyFiveGig, self).__init__(parent, None)

	def delete(self):
		"""Delete this twenty-five-gig instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this twenty-five-gig instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def advertise_ieee(self):
		"""bool: TBD"""
		return self._get_value('advertise-ieee')

	@advertise_ieee.setter
	def advertise_ieee(self, value):
		return self._set_value('advertise-ieee', value)


class SessionsConfigLayer1GroupsFortyGig(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'forty-gig'
		super(SessionsConfigLayer1GroupsFortyGig, self).__init__(parent, None)

	def priority_based_flow_control(self):
		"""Get the priority-based-flow-control object from the server

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Returns:  
			:obj:`SessionsConfigLayer1GroupsFortyGigPriorityBasedFlowControl`: An object encapsulating an instance of the priority-based-flow-control model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsFortyGigPriorityBasedFlowControl(self))

	def data_center_bridging_exchange(self):
		"""Get the data-center-bridging-exchange object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange(self))

	def create_data_center_bridging_exchange(self):
		"""Create a child sibling data-center-bridging-exchange instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			AlreadyExistsError: An instance of data-center-bridging-exchange with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange(self), locals())

	def delete(self):
		"""Delete this forty-gig instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this forty-gig instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def cable_type_length(self):
		""":str:`enum`: TBD  
		Enums:  
			OPTICAL: TBD  
			COPPER_OM_2M: TBD  
			COPPER_3M: TBD  
			COPPER_5M: TBD  
			COPPER_7M: TBD  
			COPPER_OM_1M: TBD  
			COPPER_2M: TBD  
			COPPER_5M_ACTIVE: TBD  
			COPPER_7M_ACTIVE: TBD  
		"""
		return self._get_value('cable-type-length')

	@cable_type_length.setter
	def cable_type_length(self, value):
		return self._set_value('cable-type-length', value)

	@property
	def cfp_interface(self):
		""":str:`enum`: TBD  
		Enums:  
			ACC_6068A: TBD  
			ACC_6069A: TBD  
			ACC_6068A_CISCO_100G_SR10: TBD  
			ACC_6069A_CISCO_40G_SR4: TBD  
			CISCO_FINISAR_100G_LR4_v1_5: TBD  
			OTHER_CFP: TBD  
			LR4_100GBASE: TBD  
			SR10_100GBASE: TBD  
			CXP_SR10_100GBASE: TBD  
			CFP4_LR4_100GBASE: TBD  
			CFP4_SR4_100GBASE: TBD  
			CFP4_CR4_100GBASE: TBD  
			QSFP28_LR4_100GBASE: TBD  
			QSFP28_SR4_100GBASE: TBD  
			QSFP28_CR4_100GBASE: TBD  
			QSFP28_AOC_100GBASE: TBD  
		"""
		return self._get_value('cfp-interface')

	@cfp_interface.setter
	def cfp_interface(self, value):
		return self._set_value('cfp-interface', value)

	@property
	def deficit_idle_count(self):
		"""bool: Deficit Idle Count"""
		return self._get_value('deficit-idle-count')

	@deficit_idle_count.setter
	def deficit_idle_count(self, value):
		return self._set_value('deficit-idle-count', value)

	@property
	def forward_error_correction(self):
		"""bool: TBD"""
		return self._get_value('forward-error-correction')

	@forward_error_correction.setter
	def forward_error_correction(self, value):
		return self._set_value('forward-error-correction', value)

	@property
	def ignore_link_status(self):
		"""bool: TBD"""
		return self._get_value('ignore-link-status')

	@ignore_link_status.setter
	def ignore_link_status(self, value):
		return self._set_value('ignore-link-status', value)

	@property
	def internal_ppm_adjust(self):
		"""int8: TBD"""
		return self._get_value('internal-ppm-adjust')

	@internal_ppm_adjust.setter
	def internal_ppm_adjust(self, value):
		return self._set_value('internal-ppm-adjust', value)

	@property
	def optimized_xon(self):
		""":str:`enum`: TBD  
		Enums:  
			DISABLE: TBD  
			ENABLE: TBD  
		"""
		return self._get_value('optimized-xon')

	@optimized_xon.setter
	def optimized_xon(self, value):
		return self._set_value('optimized-xon', value)

	@property
	def transmit_clock_source(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERNAL: TBD  
			INTERNAL_PPM_ADJ: TBD  
			BITS: TBD  
			LOOP: TBD  
			EXTERNAL: TBD  
		"""
		return self._get_value('transmit-clock-source')

	@transmit_clock_source.setter
	def transmit_clock_source(self, value):
		return self._set_value('transmit-clock-source', value)


class SessionsConfigLayer1GroupsFortyGigPriorityBasedFlowControl(YangBase):
	"""Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority-based-flow-control'
		super(SessionsConfigLayer1GroupsFortyGigPriorityBasedFlowControl, self).__init__(parent, None)

	def delete(self):
		"""Delete this priority-based-flow-control instance on the server.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority-based-flow-control instance on the server with any changed property values.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'data-center-bridging-exchange'
		super(SessionsConfigLayer1GroupsFortyGigDataCenterBridgingExchange, self).__init__(parent, None)

	def delete(self):
		"""Delete this data-center-bridging-exchange instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this data-center-bridging-exchange instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsOneHundredGig(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'one-hundred-gig'
		super(SessionsConfigLayer1GroupsOneHundredGig, self).__init__(parent, None)

	def priority_based_flow_control(self):
		"""Get the priority-based-flow-control object from the server

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Returns:  
			:obj:`SessionsConfigLayer1GroupsOneHundredGigPriorityBasedFlowControl`: An object encapsulating an instance of the priority-based-flow-control model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsOneHundredGigPriorityBasedFlowControl(self))

	def data_center_bridging_exchange(self):
		"""Get the data-center-bridging-exchange object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange(self))

	def create_data_center_bridging_exchange(self):
		"""Create a child sibling data-center-bridging-exchange instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange`: An object encapsulating an instance of the data-center-bridging-exchange model.  

		Raises:  
			AlreadyExistsError: An instance of data-center-bridging-exchange with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange(self), locals())

	def delete(self):
		"""Delete this one-hundred-gig instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this one-hundred-gig instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def auto_instrumentation(self):
		""":str:`enum`: TBD  
		Enums:  
			END_OF_FRAME: TBD  
			FLOATING: TBD  
		"""
		return self._get_value('auto-instrumentation')

	@auto_instrumentation.setter
	def auto_instrumentation(self, value):
		return self._set_value('auto-instrumentation', value)

	@property
	def auto_negotiation_enabled(self):
		"""bool: If enabled allows for auto negotiation between ports for speed and duplex operation."""
		return self._get_value('auto-negotiation-enabled')

	@auto_negotiation_enabled.setter
	def auto_negotiation_enabled(self, value):
		return self._set_value('auto-negotiation-enabled', value)

	@property
	def master_slave_type(self):
		""":str:`enum`: When auto negotiation is enabled one of the master slave types can be specified  
		Enums:  
			MASTER: TBD  
			SLAVE: TBD  
			FAULT: TBD  
		"""
		return self._get_value('master-slave-type')

	@master_slave_type.setter
	def master_slave_type(self, value):
		return self._set_value('master-slave-type', value)

	@property
	def collision_exponent(self):
		"""uint8: TBD"""
		return self._get_value('collision-exponent')

	@collision_exponent.setter
	def collision_exponent(self, value):
		return self._set_value('collision-exponent', value)

	@property
	def custom_fec_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NONE: TBD  
			KR_FEC: TBD  
			RS_FEC: TBD  
		"""
		return self._get_value('custom-fec-mode')

	@custom_fec_mode.setter
	def custom_fec_mode(self, value):
		return self._set_value('custom-fec-mode', value)

	@property
	def data_path_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			NORMAL: TBD  
			LOOPBACK: TBD  
			LINE_MONITOR: TBD  
		"""
		return self._get_value('data-path-mode')

	@data_path_mode.setter
	def data_path_mode(self, value):
		return self._set_value('data-path-mode', value)

	@property
	def flow_control(self):
		"""bool: Enables the port's MAC flow control mechanisms to listen for a directed address pause message"""
		return self._get_value('flow-control')

	@flow_control.setter
	def flow_control(self, value):
		return self._set_value('flow-control', value)

	@property
	def flow_control_directed_address(self):
		"""str: The 48-bit MAC address that the port listens on for a directed pause."""
		return self._get_value('flow-control-directed-address')

	@flow_control_directed_address.setter
	def flow_control_directed_address(self, value):
		return self._set_value('flow-control-directed-address', value)

	@property
	def mtu(self):
		"""uint16: TBD"""
		return self._get_value('mtu')

	@mtu.setter
	def mtu(self, value):
		return self._set_value('mtu', value)

	@property
	def port_setup_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			PORTCONFIG_ONLY: TBD  
			REGISTERS_ONLY: TBD  
		"""
		return self._get_value('port-setup-mode')

	@port_setup_mode.setter
	def port_setup_mode(self, value):
		return self._set_value('port-setup-mode', value)

	@property
	def rx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			CAPTURE: TBD  
			CAPTURE_AND_MEASURE: TBD  
			MEASURE: TBD  
			PACKET_IMPAIRMENT: TBD  
		"""
		return self._get_value('rx-mode')

	@rx_mode.setter
	def rx_mode(self, value):
		return self._set_value('rx-mode', value)

	@property
	def tx_gap_control(self):
		""":str:`enum`: TBD  
		Enums:  
			AVERAGE_MODE: TBD  
			FIXED_MODE: TBD  
		"""
		return self._get_value('tx-gap-control')

	@tx_gap_control.setter
	def tx_gap_control(self, value):
		return self._set_value('tx-gap-control', value)

	@property
	def tx_mode(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERLEAVED: TBD  
			INTERLEAVED_COARSE: TBD  
			PACKET_IMPAIRMENT: TBD  
			SEQUENTIAL: TBD  
			SEQUENTIAL_COARSE: TBD  
		"""
		return self._get_value('tx-mode')

	@tx_mode.setter
	def tx_mode(self, value):
		return self._set_value('tx-mode', value)

	@property
	def cable_type_length(self):
		""":str:`enum`: TBD  
		Enums:  
			OPTICAL: TBD  
			COPPER_OM_2M: TBD  
			COPPER_3M: TBD  
			COPPER_5M: TBD  
			COPPER_7M: TBD  
			COPPER_OM_1M: TBD  
			COPPER_2M: TBD  
			COPPER_5M_ACTIVE: TBD  
			COPPER_7M_ACTIVE: TBD  
		"""
		return self._get_value('cable-type-length')

	@cable_type_length.setter
	def cable_type_length(self, value):
		return self._set_value('cable-type-length', value)

	@property
	def cfp_interface(self):
		""":str:`enum`: TBD  
		Enums:  
			ACC_6068A: TBD  
			ACC_6069A: TBD  
			ACC_6068A_CISCO_100G_SR10: TBD  
			ACC_6069A_CISCO_40G_SR4: TBD  
			CISCO_FINISAR_100G_LR4_v1_5: TBD  
			OTHER_CFP: TBD  
			LR4_100GBASE: TBD  
			SR10_100GBASE: TBD  
			CXP_SR10_100GBASE: TBD  
			CFP4_LR4_100GBASE: TBD  
			CFP4_SR4_100GBASE: TBD  
			CFP4_CR4_100GBASE: TBD  
			QSFP28_LR4_100GBASE: TBD  
			QSFP28_SR4_100GBASE: TBD  
			QSFP28_CR4_100GBASE: TBD  
			QSFP28_AOC_100GBASE: TBD  
		"""
		return self._get_value('cfp-interface')

	@cfp_interface.setter
	def cfp_interface(self, value):
		return self._set_value('cfp-interface', value)

	@property
	def deficit_idle_count(self):
		"""bool: Deficit Idle Count"""
		return self._get_value('deficit-idle-count')

	@deficit_idle_count.setter
	def deficit_idle_count(self, value):
		return self._set_value('deficit-idle-count', value)

	@property
	def forward_error_correction(self):
		"""bool: TBD"""
		return self._get_value('forward-error-correction')

	@forward_error_correction.setter
	def forward_error_correction(self, value):
		return self._set_value('forward-error-correction', value)

	@property
	def ignore_link_status(self):
		"""bool: TBD"""
		return self._get_value('ignore-link-status')

	@ignore_link_status.setter
	def ignore_link_status(self, value):
		return self._set_value('ignore-link-status', value)

	@property
	def internal_ppm_adjust(self):
		"""int8: TBD"""
		return self._get_value('internal-ppm-adjust')

	@internal_ppm_adjust.setter
	def internal_ppm_adjust(self, value):
		return self._set_value('internal-ppm-adjust', value)

	@property
	def optimized_xon(self):
		""":str:`enum`: TBD  
		Enums:  
			DISABLE: TBD  
			ENABLE: TBD  
		"""
		return self._get_value('optimized-xon')

	@optimized_xon.setter
	def optimized_xon(self, value):
		return self._set_value('optimized-xon', value)

	@property
	def transmit_clock_source(self):
		""":str:`enum`: TBD  
		Enums:  
			INTERNAL: TBD  
			INTERNAL_PPM_ADJ: TBD  
			BITS: TBD  
			LOOP: TBD  
			EXTERNAL: TBD  
		"""
		return self._get_value('transmit-clock-source')

	@transmit_clock_source.setter
	def transmit_clock_source(self, value):
		return self._set_value('transmit-clock-source', value)


class SessionsConfigLayer1GroupsOneHundredGigPriorityBasedFlowControl(YangBase):
	"""Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority-based-flow-control'
		super(SessionsConfigLayer1GroupsOneHundredGigPriorityBasedFlowControl, self).__init__(parent, None)

	def delete(self):
		"""Delete this priority-based-flow-control instance on the server.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority-based-flow-control instance on the server with any changed property values.

		Priority-based Flow Control provides a link level flow control mechanism that can be controlled independently for each frame priority. The goal of this mechanism is to ensure zero loss under congestion in DCB networks

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'data-center-bridging-exchange'
		super(SessionsConfigLayer1GroupsOneHundredGigDataCenterBridgingExchange, self).__init__(parent, None)

	def delete(self):
		"""Delete this data-center-bridging-exchange instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this data-center-bridging-exchange instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalMulticastGroups(YangBase):
	"""A list of multi-cast groups.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'global-multicast-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalMulticastGroups, self).__init__(parent, yang_key_value)

	def multicast_groups(self, name=None):
		"""Get the multicast-groups object(s) from the server.

		A list of multi-cast groups.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the multicast-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalMulticastGroupsMulticastGroups` | :obj:`SessionsConfigGlobalMulticastGroupsMulticastGroups`: If arg name is None a list of SessionsConfigGlobalMulticastGroupsMulticastGroups objects otherwise a single SessionsConfigGlobalMulticastGroupsMulticastGroups object.  

		Raises:  
			NotFoundError: name is not in the list of multicast-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroups(self, name))

	def create_multicast_groups(self, name):
		"""Create a child sibling multicast-groups instance on the server.

		A list of multi-cast groups.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroups`: An object encapsulating an instance of the multicast-groups model.  

		Raises:  
			AlreadyExistsError: An instance of multicast-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalMulticastGroupsMulticastGroups(self, name), locals())

	def delete(self):
		"""Delete this global-multicast-groups instance on the server.

		A list of multi-cast groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this global-multicast-groups instance on the server with any changed property values.

		A list of multi-cast groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the group of multicast groups object."""
		return self._get_value('name')


class SessionsConfigGlobalMulticastGroupsMulticastGroups(YangBase):
	"""A list of multi-cast groups.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroups, self).__init__(parent, yang_key_value)

	def ipv4_group(self):
		"""Get the ipv4-group object from the server

		IPV4 multicast group details

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4Group`: An object encapsulating an instance of the ipv4-group model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4Group(self))

	def ipv6_group(self):
		"""Get the ipv6-group object from the server

		IPV6 multicast group details

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6Group`: An object encapsulating an instance of the ipv6-group model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6Group(self))

	def delete(self):
		"""Delete this multicast-groups instance on the server.

		A list of multi-cast groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-groups instance on the server with any changed property values.

		A list of multi-cast groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the multicast group object."""
		return self._get_value('name')

	@property
	def number_of_groups(self):
		"""uint32:  number of multicast groups in each multicast group block."""
		return self._get_value('number-of-groups')

	@number_of_groups.setter
	def number_of_groups(self, value):
		return self._set_value('number-of-groups', value)

	@property
	def group_type(self):
		""":str:`enum`: Determines which detailed multicast group is active.  
		Enums:  
			IPV4: TBD  
			IPV6: TBD  
		"""
		return self._get_value('group-type')

	@group_type.setter
	def group_type(self, value):
		return self._set_value('group-type', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4Group(YangBase):
	"""IPV4 multicast group details
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-group'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4Group, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IP address

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLength(self))

	def delete(self):
		"""Delete this ipv4-group instance on the server.

		IPV4 multicast group details

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-group instance on the server with any changed property values.

		IPV4 multicast group details

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddress(YangBase):
	"""IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv4GroupPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6Group(YangBase):
	"""IPV6 multicast group details
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-group'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6Group, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		Start IPv6 address

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		IPv6 Prefix Length

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLength(self))

	def delete(self):
		"""Delete this ipv6-group instance on the server.

		IPV6 multicast group details

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-group instance on the server with any changed property values.

		IPV6 multicast group details

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddress(YangBase):
	"""Start IPv6 address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		Start IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		Start IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLength(YangBase):
	"""IPv6 Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		IPv6 Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		IPv6 Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalMulticastGroupsMulticastGroupsIpv6GroupPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocols(YangBase):
	"""This list allows for configuring global protocols options.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'global-protocols'
		super(SessionsConfigGlobalProtocols, self).__init__(parent, None)

	def dhcpv4(self, name=None):
		"""Get the dhcpv4 object(s) from the server.

		This list allows for configuring global DHCPv4 options.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the dhcpv4 list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalProtocolsDhcpv4` | :obj:`SessionsConfigGlobalProtocolsDhcpv4`: If arg name is None a list of SessionsConfigGlobalProtocolsDhcpv4 objects otherwise a single SessionsConfigGlobalProtocolsDhcpv4 object.  

		Raises:  
			NotFoundError: name is not in the list of dhcpv4 objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4(self, name))

	def create_dhcpv4(self, name):
		"""Create a child sibling dhcpv4 instance on the server.

		This list allows for configuring global DHCPv4 options.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4`: An object encapsulating an instance of the dhcpv4 model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv4 with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv4(self, name), locals())

	def dhcpv6(self, name=None):
		"""Get the dhcpv6 object(s) from the server.

		This list allows for configuring global DHCPv6 and PD options.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the dhcpv6 list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalProtocolsDhcpv6` | :obj:`SessionsConfigGlobalProtocolsDhcpv6`: If arg name is None a list of SessionsConfigGlobalProtocolsDhcpv6 objects otherwise a single SessionsConfigGlobalProtocolsDhcpv6 object.  

		Raises:  
			NotFoundError: name is not in the list of dhcpv6 objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6(self, name))

	def create_dhcpv6(self, name):
		"""Create a child sibling dhcpv6 instance on the server.

		This list allows for configuring global DHCPv6 and PD options.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6`: An object encapsulating an instance of the dhcpv6 model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv6 with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6(self, name), locals())

	def igmp(self, name=None):
		"""Get the igmp object(s) from the server.

		This list allows for configuring global IGMP options.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the igmp list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalProtocolsIgmp` | :obj:`SessionsConfigGlobalProtocolsIgmp`: If arg name is None a list of SessionsConfigGlobalProtocolsIgmp objects otherwise a single SessionsConfigGlobalProtocolsIgmp object.  

		Raises:  
			NotFoundError: name is not in the list of igmp objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmp(self, name))

	def create_igmp(self, name):
		"""Create a child sibling igmp instance on the server.

		This list allows for configuring global IGMP options.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmp`: An object encapsulating an instance of the igmp model.  

		Raises:  
			AlreadyExistsError: An instance of igmp with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsIgmp(self, name), locals())

	def mld(self, name=None):
		"""Get the mld object(s) from the server.

		This list allows for configuring global MLD options.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the mld list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalProtocolsMld` | :obj:`SessionsConfigGlobalProtocolsMld`: If arg name is None a list of SessionsConfigGlobalProtocolsMld objects otherwise a single SessionsConfigGlobalProtocolsMld object.  

		Raises:  
			NotFoundError: name is not in the list of mld objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMld(self, name))

	def create_mld(self, name):
		"""Create a child sibling mld instance on the server.

		This list allows for configuring global MLD options.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMld`: An object encapsulating an instance of the mld model.  

		Raises:  
			AlreadyExistsError: An instance of mld with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsMld(self, name), locals())

	def pim(self, name=None):
		"""Get the pim object(s) from the server.

		This list allows for configuring global PIM options.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the pim list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigGlobalProtocolsPim` | :obj:`SessionsConfigGlobalProtocolsPim`: If arg name is None a list of SessionsConfigGlobalProtocolsPim objects otherwise a single SessionsConfigGlobalProtocolsPim object.  

		Raises:  
			NotFoundError: name is not in the list of pim objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPim(self, name))

	def create_pim(self, name):
		"""Create a child sibling pim instance on the server.

		This list allows for configuring global PIM options.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPim`: An object encapsulating an instance of the pim model.  

		Raises:  
			AlreadyExistsError: An instance of pim with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsPim(self, name), locals())

	def delete(self):
		"""Delete this global-protocols instance on the server.

		This list allows for configuring global protocols options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this global-protocols instance on the server with any changed property values.

		This list allows for configuring global protocols options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv4(YangBase):
	"""This list allows for configuring global DHCPv4 options.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalProtocolsDhcpv4, self).__init__(parent, yang_key_value)

	def request_rate(self):
		"""Get the request-rate object from the server

		Requests per second for DHCP client leases or lease renewals.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RequestRate`: An object encapsulating an instance of the request-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RequestRate(self))

	def lease_time(self):
		"""Get the lease-time object from the server

		Suggested lease time in seconds at the DHCP port level.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4LeaseTime`: An object encapsulating an instance of the lease-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4LeaseTime(self))

	def max_msg_size(self):
		"""Get the max-msg-size object from the server

		Used to negotiate the maximum DHCP message size, in bytes. This is option 57 for the options field of the DHCP message, as described in RFC 2132.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxMsgSize`: An object encapsulating an instance of the max-msg-size model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSize(self))

	def msg_timeout(self):
		"""Get the msg-timeout object from the server

		Message timeout in seconds.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MsgTimeout`: An object encapsulating an instance of the msg-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MsgTimeout(self))

	def outstanding_session_count(self):
		"""Get the outstanding-session-count object from the server

		Number of DHCP sessions to resolve at a time.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCount`: An object encapsulating an instance of the outstanding-session-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCount(self))

	def release_rate(self):
		"""Get the release-rate object from the server

		Number of DHCP sessions that are released per second.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4ReleaseRate`: An object encapsulating an instance of the release-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4ReleaseRate(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of retries allowed.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RetryCount(self))

	def sequence_type(self):
		"""Get the sequence-type object from the server

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4SequenceType`: An object encapsulating an instance of the sequence-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4SequenceType(self))

	def start_transaction_id(self):
		"""Get the start-transaction-id object from the server

		Starting transaction ID for sessions on this port.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4StartTransactionId`: An object encapsulating an instance of the start-transaction-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4StartTransactionId(self))

	def max_dna_v4_retry_count(self):
		"""Get the max-dna-v4-retry-count object from the server

		Max DNAv4 retry count.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCount`: An object encapsulating an instance of the max-dna-v4-retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCount(self))

	def dna_v4_timeout(self):
		"""Get the dna-v4-timeout object from the server

		DNAv4 timeout.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4DnaV4Timeout`: An object encapsulating an instance of the dna-v4-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4DnaV4Timeout(self))

	def enable_custom_option_assign_for_relay_agents(self):
		"""Get the enable-custom-option-assign-for-relay-agents object from the server

		Enable custom option assignments on relay agent mode.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgents`: An object encapsulating an instance of the enable-custom-option-assign-for-relay-agents model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgents(self))

	def delete(self):
		"""Delete this dhcpv4 instance on the server.

		This list allows for configuring global DHCPv4 options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv4 instance on the server with any changed property values.

		This list allows for configuring global DHCPv4 options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name for a global protocols list item"""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../../ports/name: A list of abstract port references"""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)


class SessionsConfigGlobalProtocolsDhcpv4RequestRate(YangBase):
	"""Requests per second for DHCP client leases or lease renewals.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'request-rate'
		super(SessionsConfigGlobalProtocolsDhcpv4RequestRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RequestRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RequestRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RequestRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RequestRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RequestRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RequestRateRandom(self))

	def delete(self):
		"""Delete this request-rate instance on the server.

		Requests per second for DHCP client leases or lease renewals.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this request-rate instance on the server with any changed property values.

		Requests per second for DHCP client leases or lease renewals.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4RequestRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4RequestRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4RequestRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4RequestRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4RequestRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4RequestRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4LeaseTime(YangBase):
	"""Suggested lease time in seconds at the DHCP port level.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'lease-time'
		super(SessionsConfigGlobalProtocolsDhcpv4LeaseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4LeaseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4LeaseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4LeaseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeRandom(self))

	def delete(self):
		"""Delete this lease-time instance on the server.

		Suggested lease time in seconds at the DHCP port level.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this lease-time instance on the server with any changed property values.

		Suggested lease time in seconds at the DHCP port level.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4LeaseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4LeaseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4LeaseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4LeaseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxMsgSize(YangBase):
	"""Used to negotiate the maximum DHCP message size, in bytes. This is option 57 for the options field of the DHCP message, as described in RFC 2132.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-msg-size'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSize, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeRandom(self))

	def delete(self):
		"""Delete this max-msg-size instance on the server.

		Used to negotiate the maximum DHCP message size, in bytes. This is option 57 for the options field of the DHCP message, as described in RFC 2132.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-msg-size instance on the server with any changed property values.

		Used to negotiate the maximum DHCP message size, in bytes. This is option 57 for the options field of the DHCP message, as described in RFC 2132.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxMsgSizeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4MsgTimeout(YangBase):
	"""Message timeout in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'msg-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv4MsgTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutRandom(self))

	def delete(self):
		"""Delete this msg-timeout instance on the server.

		Message timeout in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this msg-timeout instance on the server with any changed property values.

		Message timeout in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4MsgTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCount(YangBase):
	"""Number of DHCP sessions to resolve at a time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'outstanding-session-count'
		super(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountRandom(self))

	def delete(self):
		"""Delete this outstanding-session-count instance on the server.

		Number of DHCP sessions to resolve at a time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this outstanding-session-count instance on the server with any changed property values.

		Number of DHCP sessions to resolve at a time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4OutstandingSessionCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4ReleaseRate(YangBase):
	"""Number of DHCP sessions that are released per second.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'release-rate'
		super(SessionsConfigGlobalProtocolsDhcpv4ReleaseRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4ReleaseRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4ReleaseRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4ReleaseRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateRandom(self))

	def delete(self):
		"""Delete this release-rate instance on the server.

		Number of DHCP sessions that are released per second.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this release-rate instance on the server with any changed property values.

		Number of DHCP sessions that are released per second.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4ReleaseRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4ReleaseRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4ReleaseRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4ReleaseRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4RetryCount(YangBase):
	"""Number of retries allowed.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv4RetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4RetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4RetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of retries allowed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of retries allowed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4RetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4RetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4RetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4RetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4RetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4RetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4SequenceType(YangBase):
	"""Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-type'
		super(SessionsConfigGlobalProtocolsDhcpv4SequenceType, self).__init__(parent, None)

	def delete(self):
		"""Delete this sequence-type instance on the server.

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-type instance on the server with any changed property values.

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4StartTransactionId(YangBase):
	"""Starting transaction ID for sessions on this port.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-transaction-id'
		super(SessionsConfigGlobalProtocolsDhcpv4StartTransactionId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdRandom(self))

	def delete(self):
		"""Delete this start-transaction-id instance on the server.

		Starting transaction ID for sessions on this port.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-transaction-id instance on the server with any changed property values.

		Starting transaction ID for sessions on this port.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4StartTransactionIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCount(YangBase):
	"""Max DNAv4 retry count.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-dna-v4-retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountRandom(self))

	def delete(self):
		"""Delete this max-dna-v4-retry-count instance on the server.

		Max DNAv4 retry count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-dna-v4-retry-count instance on the server with any changed property values.

		Max DNAv4 retry count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4MaxDnaV4RetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4DnaV4Timeout(YangBase):
	"""DNAv4 timeout.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dna-v4-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv4DnaV4Timeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutRandom(self))

	def delete(self):
		"""Delete this dna-v4-timeout instance on the server.

		DNAv4 timeout.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dna-v4-timeout instance on the server with any changed property values.

		DNAv4 timeout.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4DnaV4TimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgents(YangBase):
	"""Enable custom option assignments on relay agent mode.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-custom-option-assign-for-relay-agents'
		super(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgents, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsRandom(self))

	def delete(self):
		"""Delete this enable-custom-option-assign-for-relay-agents instance on the server.

		Enable custom option assignments on relay agent mode.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-custom-option-assign-for-relay-agents instance on the server with any changed property values.

		Enable custom option assignments on relay agent mode.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv4EnableCustomOptionAssignForRelayAgentsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6(YangBase):
	"""This list allows for configuring global DHCPv6 and PD options.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalProtocolsDhcpv6, self).__init__(parent, yang_key_value)

	def request_rate(self):
		"""Get the request-rate object from the server

		Requests per second for DHCPv6 client leases.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestRate`: An object encapsulating an instance of the request-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestRate(self))

	def release_rate(self):
		"""Get the release-rate object from the server

		Number of DHCPv6 sessions that are released per second. Use the Release operation to release DHCPv6 sessions.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseRate`: An object encapsulating an instance of the release-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseRate(self))

	def renew_rate(self):
		"""Get the renew-rate object from the server

		Renewals per second for DHCPv6 client renewals.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewRate`: An object encapsulating an instance of the renew-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewRate(self))

	def outstanding_session_count(self):
		"""Get the outstanding-session-count object from the server

		Number of DHCPv6 sessions to resolve at a time.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCount`: An object encapsulating an instance of the outstanding-session-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCount(self))

	def max_retry_attempt(self):
		"""Get the max-retry-attempt object from the server

		Maximum Session Level Auto Retry Count.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttempt`: An object encapsulating an instance of the max-retry-attempt model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttempt(self))

	def sequence_type(self):
		"""Get the sequence-type object from the server

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SequenceType`: An object encapsulating an instance of the sequence-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SequenceType(self))

	def enable_device_block_rate(self):
		"""Get the enable-device-block-rate object from the server

		To Control Block Rate Based On Devices. TRUE  : Control Block Rate Based On Devices. FALSE : Control Block Rate Based On Port Config.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRate`: An object encapsulating an instance of the enable-device-block-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRate(self))

	def enable_session_auto_retry(self):
		"""Get the enable-session-auto-retry object from the server

		Session Level Auto Retry. TRUE  : Enable Session Level Auto Retry. FALSE : Disable Session Level Auto Retry.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetry`: An object encapsulating an instance of the enable-session-auto-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetry(self))

	def no_wait_multi_advertise(self):
		"""Get the no-wait-multi-advertise object from the server

		DHCPv6 Client Collecting Advertise. TRUE  : DHCPv6 Client Use The First Advertise. FALSE : DHCPv6 Client Will Collect Advertise.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertise`: An object encapsulating an instance of the no-wait-multi-advertise model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertise(self))

	def solicit_message(self):
		"""Get the solicit-message object from the server

		Solicit message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessage`: An object encapsulating an instance of the solicit-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessage(self))

	def create_solicit_message(self):
		"""Create a child sibling solicit-message instance on the server.

		Solicit message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessage`: An object encapsulating an instance of the solicit-message model.  

		Raises:  
			AlreadyExistsError: An instance of solicit-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6SolicitMessage(self), locals())

	def request_message(self):
		"""Get the request-message object from the server

		Request message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessage`: An object encapsulating an instance of the request-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessage(self))

	def create_request_message(self):
		"""Create a child sibling request-message instance on the server.

		Request message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessage`: An object encapsulating an instance of the request-message model.  

		Raises:  
			AlreadyExistsError: An instance of request-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6RequestMessage(self), locals())

	def confirm_message(self):
		"""Get the confirm-message object from the server

		Confirm message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage`: An object encapsulating an instance of the confirm-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage(self))

	def create_confirm_message(self):
		"""Create a child sibling confirm-message instance on the server.

		Confirm message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage`: An object encapsulating an instance of the confirm-message model.  

		Raises:  
			AlreadyExistsError: An instance of confirm-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage(self), locals())

	def renew_message(self):
		"""Get the renew-message object from the server

		Renew message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessage`: An object encapsulating an instance of the renew-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessage(self))

	def create_renew_message(self):
		"""Create a child sibling renew-message instance on the server.

		Renew message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessage`: An object encapsulating an instance of the renew-message model.  

		Raises:  
			AlreadyExistsError: An instance of renew-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6RenewMessage(self), locals())

	def rebind_message(self):
		"""Get the rebind-message object from the server

		Rebind message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessage`: An object encapsulating an instance of the rebind-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessage(self))

	def create_rebind_message(self):
		"""Create a child sibling rebind-message instance on the server.

		Rebind message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessage`: An object encapsulating an instance of the rebind-message model.  

		Raises:  
			AlreadyExistsError: An instance of rebind-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6RebindMessage(self), locals())

	def release_message(self):
		"""Get the release-message object from the server

		Release message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage`: An object encapsulating an instance of the release-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage(self))

	def create_release_message(self):
		"""Create a child sibling release-message instance on the server.

		Release message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage`: An object encapsulating an instance of the release-message model.  

		Raises:  
			AlreadyExistsError: An instance of release-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage(self), locals())

	def decline_message(self):
		"""Get the decline-message object from the server

		Decline message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessage`: An object encapsulating an instance of the decline-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessage(self))

	def create_decline_message(self):
		"""Create a child sibling decline-message instance on the server.

		Decline message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessage`: An object encapsulating an instance of the decline-message model.  

		Raises:  
			AlreadyExistsError: An instance of decline-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6DeclineMessage(self), locals())

	def info_request_message(self):
		"""Get the info-request-message object from the server

		Information Request message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage`: An object encapsulating an instance of the info-request-message model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage(self))

	def create_info_request_message(self):
		"""Create a child sibling info-request-message instance on the server.

		Information Request message options

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage`: An object encapsulating an instance of the info-request-message model.  

		Raises:  
			AlreadyExistsError: An instance of info-request-message with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage(self), locals())

	def delete(self):
		"""Delete this dhcpv6 instance on the server.

		This list allows for configuring global DHCPv6 and PD options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv6 instance on the server with any changed property values.

		This list allows for configuring global DHCPv6 and PD options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name for a global protocols list item"""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../../ports/name: A list of abstract port references"""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestRate(YangBase):
	"""Requests per second for DHCPv6 client leases.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'request-rate'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestRateRandom(self))

	def delete(self):
		"""Delete this request-rate instance on the server.

		Requests per second for DHCPv6 client leases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this request-rate instance on the server with any changed property values.

		Requests per second for DHCPv6 client leases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseRate(YangBase):
	"""Number of DHCPv6 sessions that are released per second. Use the Release operation to release DHCPv6 sessions.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'release-rate'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateRandom(self))

	def delete(self):
		"""Delete this release-rate instance on the server.

		Number of DHCPv6 sessions that are released per second. Use the Release operation to release DHCPv6 sessions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this release-rate instance on the server with any changed property values.

		Number of DHCPv6 sessions that are released per second. Use the Release operation to release DHCPv6 sessions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewRate(YangBase):
	"""Renewals per second for DHCPv6 client renewals.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'renew-rate'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewRateRandom(self))

	def delete(self):
		"""Delete this renew-rate instance on the server.

		Renewals per second for DHCPv6 client renewals.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this renew-rate instance on the server with any changed property values.

		Renewals per second for DHCPv6 client renewals.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCount(YangBase):
	"""Number of DHCPv6 sessions to resolve at a time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'outstanding-session-count'
		super(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountRandom(self))

	def delete(self):
		"""Delete this outstanding-session-count instance on the server.

		Number of DHCPv6 sessions to resolve at a time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this outstanding-session-count instance on the server with any changed property values.

		Number of DHCPv6 sessions to resolve at a time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6OutstandingSessionCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttempt(YangBase):
	"""Maximum Session Level Auto Retry Count.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-retry-attempt'
		super(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttempt, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptRandom(self))

	def delete(self):
		"""Delete this max-retry-attempt instance on the server.

		Maximum Session Level Auto Retry Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-retry-attempt instance on the server with any changed property values.

		Maximum Session Level Auto Retry Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6MaxRetryAttemptRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SequenceType(YangBase):
	"""Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-type'
		super(SessionsConfigGlobalProtocolsDhcpv6SequenceType, self).__init__(parent, None)

	def delete(self):
		"""Delete this sequence-type instance on the server.

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-type instance on the server with any changed property values.

		Port-level sequence type that determines in what order sessions are attempted. SEQUENTIAL	: Sessions are attempted sequentially across host blocks under the port. PARALLEL	  : Sessions are attempted in parallel across host blocks under the port

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRate(YangBase):
	"""To Control Block Rate Based On Devices. TRUE  : Control Block Rate Based On Devices. FALSE : Control Block Rate Based On Port Config.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-device-block-rate'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateRandom(self))

	def delete(self):
		"""Delete this enable-device-block-rate instance on the server.

		To Control Block Rate Based On Devices. TRUE  : Control Block Rate Based On Devices. FALSE : Control Block Rate Based On Port Config.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-device-block-rate instance on the server with any changed property values.

		To Control Block Rate Based On Devices. TRUE  : Control Block Rate Based On Devices. FALSE : Control Block Rate Based On Port Config.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableDeviceBlockRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetry(YangBase):
	"""Session Level Auto Retry. TRUE  : Enable Session Level Auto Retry. FALSE : Disable Session Level Auto Retry.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-session-auto-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryRandom(self))

	def delete(self):
		"""Delete this enable-session-auto-retry instance on the server.

		Session Level Auto Retry. TRUE  : Enable Session Level Auto Retry. FALSE : Disable Session Level Auto Retry.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-session-auto-retry instance on the server with any changed property values.

		Session Level Auto Retry. TRUE  : Enable Session Level Auto Retry. FALSE : Disable Session Level Auto Retry.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6EnableSessionAutoRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertise(YangBase):
	"""DHCPv6 Client Collecting Advertise. TRUE  : DHCPv6 Client Use The First Advertise. FALSE : DHCPv6 Client Will Collect Advertise.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'no-wait-multi-advertise'
		super(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertise, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseRandom(self))

	def delete(self):
		"""Delete this no-wait-multi-advertise instance on the server.

		DHCPv6 Client Collecting Advertise. TRUE  : DHCPv6 Client Use The First Advertise. FALSE : DHCPv6 Client Will Collect Advertise.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this no-wait-multi-advertise instance on the server with any changed property values.

		DHCPv6 Client Collecting Advertise. TRUE  : DHCPv6 Client Use The First Advertise. FALSE : DHCPv6 Client Will Collect Advertise.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6NoWaitMultiAdvertiseRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessage(YangBase):
	"""Solicit message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'solicit-message'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive an ADVERTISE message from the delegating router after sending an initial SOLICIT message.If the host does not receive a valid ADVERTISE message within this time,it sends another SOLICIT message and the timeout length is doubled fromthe previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an ADVERTISE message. If an initial SOLICIT message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the SOLICIT timeout value is set to the MaximumSolicit Retry Timeout value, the client tries to resend the SOLICIT message for the number of times specified in the Solicit Retry Count field.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Specify the maximum number of retries to establish a DHCPv6/PD session with the SOLICIT message. When this number is reached, the port incrementsthe failure counter.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Enable or disable indefinite retry. TRUE  : Enable indefinite retry of solicits. FALSE : Disable indefinite retry of solicits.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating solicits will be not be retried. TRUE  : Disable retry of solicits. FALSE : Enable retry of solicits.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetries(self))

	def delete(self):
		"""Delete this solicit-message instance on the server.

		Solicit message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this solicit-message instance on the server with any changed property values.

		Solicit message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive an ADVERTISE message from the delegating router after sending an initial SOLICIT message.If the host does not receive a valid ADVERTISE message within this time,it sends another SOLICIT message and the timeout length is doubled fromthe previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive an ADVERTISE message from the delegating router after sending an initial SOLICIT message.If the host does not receive a valid ADVERTISE message within this time,it sends another SOLICIT message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive an ADVERTISE message from the delegating router after sending an initial SOLICIT message.If the host does not receive a valid ADVERTISE message within this time,it sends another SOLICIT message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an ADVERTISE message. If an initial SOLICIT message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the SOLICIT timeout value is set to the MaximumSolicit Retry Timeout value, the client tries to resend the SOLICIT message for the number of times specified in the Solicit Retry Count field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an ADVERTISE message. If an initial SOLICIT message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the SOLICIT timeout value is set to the MaximumSolicit Retry Timeout value, the client tries to resend the SOLICIT message for the number of times specified in the Solicit Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an ADVERTISE message. If an initial SOLICIT message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the SOLICIT timeout value is set to the MaximumSolicit Retry Timeout value, the client tries to resend the SOLICIT message for the number of times specified in the Solicit Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCount(YangBase):
	"""Specify the maximum number of retries to establish a DHCPv6/PD session with the SOLICIT message. When this number is reached, the port incrementsthe failure counter.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Specify the maximum number of retries to establish a DHCPv6/PD session with the SOLICIT message. When this number is reached, the port incrementsthe failure counter.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Specify the maximum number of retries to establish a DHCPv6/PD session with the SOLICIT message. When this number is reached, the port incrementsthe failure counter.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetry(YangBase):
	"""Enable or disable indefinite retry. TRUE  : Enable indefinite retry of solicits. FALSE : Disable indefinite retry of solicits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Enable or disable indefinite retry. TRUE  : Enable indefinite retry of solicits. FALSE : Disable indefinite retry of solicits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Enable or disable indefinite retry. TRUE  : Enable indefinite retry of solicits. FALSE : Disable indefinite retry of solicits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetries(YangBase):
	"""Flag indicating solicits will be not be retried. TRUE  : Disable retry of solicits. FALSE : Enable retry of solicits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating solicits will be not be retried. TRUE  : Disable retry of solicits. FALSE : Enable retry of solicits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating solicits will be not be retried. TRUE  : Disable retry of solicits. FALSE : Enable retry of solicits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6SolicitMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessage(YangBase):
	"""Request message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'request-message'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another REQUEST message and the timeout length is doubled fromthe previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REQUEST message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. Thisfield limits the retry timeout time, which can become long if it continuesto double with each retry. If the REQUEST timeout value is set to the MaximumRequest Retry Timeout value, the client tries to resend the REQUESTmessage for the number of times specified in the Request Retry Count field.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Specify the maximum number of retries to establish a DHCPv6/PD session with the REQUEST message. When this number is reached, the port increments the failure counter.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating requests will retried indefinitely. TRUE  : Enable indefinite retry of requests. FALSE : Disable indefinite retry of requests.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating requests will be not be retried. TRUE  : Disable retry of requests. FALSE : Enable retry of requests.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetries(self))

	def delete(self):
		"""Delete this request-message instance on the server.

		Request message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this request-message instance on the server with any changed property values.

		Request message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another REQUEST message and the timeout length is doubled fromthe previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another REQUEST message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another REQUEST message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REQUEST message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. Thisfield limits the retry timeout time, which can become long if it continuesto double with each retry. If the REQUEST timeout value is set to the MaximumRequest Retry Timeout value, the client tries to resend the REQUESTmessage for the number of times specified in the Request Retry Count field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REQUEST message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. Thisfield limits the retry timeout time, which can become long if it continuesto double with each retry. If the REQUEST timeout value is set to the MaximumRequest Retry Timeout value, the client tries to resend the REQUESTmessage for the number of times specified in the Request Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REQUEST message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. Thisfield limits the retry timeout time, which can become long if it continuesto double with each retry. If the REQUEST timeout value is set to the MaximumRequest Retry Timeout value, the client tries to resend the REQUESTmessage for the number of times specified in the Request Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCount(YangBase):
	"""Specify the maximum number of retries to establish a DHCPv6/PD session with the REQUEST message. When this number is reached, the port increments the failure counter.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Specify the maximum number of retries to establish a DHCPv6/PD session with the REQUEST message. When this number is reached, the port increments the failure counter.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Specify the maximum number of retries to establish a DHCPv6/PD session with the REQUEST message. When this number is reached, the port increments the failure counter.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetry(YangBase):
	"""Flag indicating requests will retried indefinitely. TRUE  : Enable indefinite retry of requests. FALSE : Disable indefinite retry of requests.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating requests will retried indefinitely. TRUE  : Enable indefinite retry of requests. FALSE : Disable indefinite retry of requests.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating requests will retried indefinitely. TRUE  : Enable indefinite retry of requests. FALSE : Disable indefinite retry of requests.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetries(YangBase):
	"""Flag indicating requests will be not be retried. TRUE  : Disable retry of requests. FALSE : Enable retry of requests.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating requests will be not be retried. TRUE  : Disable retry of requests. FALSE : Enable retry of requests.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating requests will be not be retried. TRUE  : Disable retry of requests. FALSE : Enable retry of requests.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RequestMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage(YangBase):
	"""Confirm message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'confirm-message'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial CONFIRM message.If the host does not receive a valid REPLY message within this time,it sends another CONFIRM message and the timeout length is doubled fromthe previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial CONFIRM message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the CONFIRM timeout value is set to the MaximumCONFIRM Retry Timeout value, the client tries to resend the CONFIRM message for the number of times specified in the CONFIRM Retry Count field.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeout(self))

	def max_duration(self):
		"""Get the max-duration object from the server

		Specifies an upper bound on the length of time a client may retransmit the confirm message. The message exchange fails once the duration is exceeded.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDuration`: An object encapsulating an instance of the max-duration model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDuration(self))

	def delete(self):
		"""Delete this confirm-message instance on the server.

		Confirm message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this confirm-message instance on the server with any changed property values.

		Confirm message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial CONFIRM message.If the host does not receive a valid REPLY message within this time,it sends another CONFIRM message and the timeout length is doubled fromthe previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial CONFIRM message.If the host does not receive a valid REPLY message within this time,it sends another CONFIRM message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial CONFIRM message.If the host does not receive a valid REPLY message within this time,it sends another CONFIRM message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial CONFIRM message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the CONFIRM timeout value is set to the MaximumCONFIRM Retry Timeout value, the client tries to resend the CONFIRM message for the number of times specified in the CONFIRM Retry Count field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial CONFIRM message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the CONFIRM timeout value is set to the MaximumCONFIRM Retry Timeout value, the client tries to resend the CONFIRM message for the number of times specified in the CONFIRM Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial CONFIRM message is unsuccessful and times out, the timeoutvalue for subsequent attempts is either double the previous timeout valueor the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the CONFIRM timeout value is set to the MaximumCONFIRM Retry Timeout value, the client tries to resend the CONFIRM message for the number of times specified in the CONFIRM Retry Count field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDuration(YangBase):
	"""Specifies an upper bound on the length of time a client may retransmit the confirm message. The message exchange fails once the duration is exceeded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-duration'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDuration, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationRandom(self))

	def delete(self):
		"""Delete this max-duration instance on the server.

		Specifies an upper bound on the length of time a client may retransmit the confirm message. The message exchange fails once the duration is exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-duration instance on the server with any changed property values.

		Specifies an upper bound on the length of time a client may retransmit the confirm message. The message exchange fails once the duration is exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ConfirmMessageMaxDurationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessage(YangBase):
	"""Renew message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'renew-message'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy  the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RENEW message. If the host does not receive a valid REPLY message within this time,it sends another RENEW message and the timeout length is doubled from the previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial RENEW message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the RENEW timeout value is set to the Maximum Renew Retry Timeout value, the client tries to resend the RENEW message until time T2, when the client starts sending REBIND messages.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of renew retries.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating renew will retried indefinitely. TRUE  : Enable indefinite retry of renew. FALSE : Disable indefinite retry of renew.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating renews will be not be retried. TRUE  : Disable retry of renews. FALSE : Enable retry of renews.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetries(self))

	def delete(self):
		"""Delete this renew-message instance on the server.

		Renew message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this renew-message instance on the server with any changed property values.

		Renew message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeout(YangBase):
	"""Specifiy  the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RENEW message. If the host does not receive a valid REPLY message within this time,it sends another RENEW message and the timeout length is doubled from the previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy  the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RENEW message. If the host does not receive a valid REPLY message within this time,it sends another RENEW message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy  the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RENEW message. If the host does not receive a valid REPLY message within this time,it sends another RENEW message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial RENEW message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the RENEW timeout value is set to the Maximum Renew Retry Timeout value, the client tries to resend the RENEW message until time T2, when the client starts sending REBIND messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial RENEW message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the RENEW timeout value is set to the Maximum Renew Retry Timeout value, the client tries to resend the RENEW message until time T2, when the client starts sending REBIND messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial RENEW message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the RENEW timeout value is set to the Maximum Renew Retry Timeout value, the client tries to resend the RENEW message until time T2, when the client starts sending REBIND messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCount(YangBase):
	"""Number of renew retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of renew retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of renew retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetry(YangBase):
	"""Flag indicating renew will retried indefinitely. TRUE  : Enable indefinite retry of renew. FALSE : Disable indefinite retry of renew.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating renew will retried indefinitely. TRUE  : Enable indefinite retry of renew. FALSE : Disable indefinite retry of renew.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating renew will retried indefinitely. TRUE  : Enable indefinite retry of renew. FALSE : Disable indefinite retry of renew.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetries(YangBase):
	"""Flag indicating renews will be not be retried. TRUE  : Disable retry of renews. FALSE : Enable retry of renews.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating renews will be not be retried. TRUE  : Disable retry of renews. FALSE : Enable retry of renews.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating renews will be not be retried. TRUE  : Disable retry of renews. FALSE : Enable retry of renews.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RenewMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessage(YangBase):
	"""Rebind message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rebind-message'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REBIND message.If the host does not receive a valid REPLY message within this time,it sends another REBIND message and the timeout length is doubled from the previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REBIND message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the REBIND timeout value is set to the Maximum Rebind Retry Timeout value, the client tries to resend the REBIND message until the IPv6 prefix's valid lifetime expires..

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of rebind retries.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating rebind will retried indefinitely. TRUE  : Enable indefinite retry of rebind. FALSE : Disable indefinite retry of rebind.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating rebinds will be not be retried. TRUE  : Disable retry of rebinds. FALSE : Enable retry of rebinds.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetries(self))

	def delete(self):
		"""Delete this rebind-message instance on the server.

		Rebind message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rebind-message instance on the server with any changed property values.

		Rebind message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REBIND message.If the host does not receive a valid REPLY message within this time,it sends another REBIND message and the timeout length is doubled from the previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REBIND message.If the host does not receive a valid REPLY message within this time,it sends another REBIND message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial REBIND message.If the host does not receive a valid REPLY message within this time,it sends another REBIND message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REBIND message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the REBIND timeout value is set to the Maximum Rebind Retry Timeout value, the client tries to resend the REBIND message until the IPv6 prefix's valid lifetime expires..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REBIND message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the REBIND timeout value is set to the Maximum Rebind Retry Timeout value, the client tries to resend the REBIND message until the IPv6 prefix's valid lifetime expires..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial REBIND message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the REBIND timeout value is set to the Maximum Rebind Retry Timeout value, the client tries to resend the REBIND message until the IPv6 prefix's valid lifetime expires..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCount(YangBase):
	"""Number of rebind retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of rebind retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of rebind retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetry(YangBase):
	"""Flag indicating rebind will retried indefinitely. TRUE  : Enable indefinite retry of rebind. FALSE : Disable indefinite retry of rebind.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating rebind will retried indefinitely. TRUE  : Enable indefinite retry of rebind. FALSE : Disable indefinite retry of rebind.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating rebind will retried indefinitely. TRUE  : Enable indefinite retry of rebind. FALSE : Disable indefinite retry of rebind.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetries(YangBase):
	"""Flag indicating rebinds will be not be retried. TRUE  : Disable retry of rebinds. FALSE : Enable retry of rebinds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating rebinds will be not be retried. TRUE  : Disable retry of rebinds. FALSE : Enable retry of rebinds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating rebinds will be not be retried. TRUE  : Disable retry of rebinds. FALSE : Enable retry of rebinds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6RebindMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage(YangBase):
	"""Release message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'release-message'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RELEASE message.If the host does not receive a valid REPLY message within this time,it sends another RELEASE message and the timeout length is doubled fromthe previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of release retries.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating releases will retried indefinitely. TRUE  : Enable indefinite retry of releases. FALSE : Disable indefinite retry of releases.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating releases will be not be retried. TRUE  : Disable retry of releases. FALSE : Enable retry of releases.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetries(self))

	def delete(self):
		"""Delete this release-message instance on the server.

		Release message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this release-message instance on the server with any changed property values.

		Release message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RELEASE message.If the host does not receive a valid REPLY message within this time,it sends another RELEASE message and the timeout length is doubled fromthe previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RELEASE message.If the host does not receive a valid REPLY message within this time,it sends another RELEASE message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial RELEASE message.If the host does not receive a valid REPLY message within this time,it sends another RELEASE message and the timeout length is doubled fromthe previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCount(YangBase):
	"""Number of release retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of release retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of release retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetry(YangBase):
	"""Flag indicating releases will retried indefinitely. TRUE  : Enable indefinite retry of releases. FALSE : Disable indefinite retry of releases.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating releases will retried indefinitely. TRUE  : Enable indefinite retry of releases. FALSE : Disable indefinite retry of releases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating releases will retried indefinitely. TRUE  : Enable indefinite retry of releases. FALSE : Disable indefinite retry of releases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetries(YangBase):
	"""Flag indicating releases will be not be retried. TRUE  : Disable retry of releases. FALSE : Enable retry of releases.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating releases will be not be retried. TRUE  : Disable retry of releases. FALSE : Enable retry of releases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating releases will be not be retried. TRUE  : Disable retry of releases. FALSE : Enable retry of releases.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6ReleaseMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessage(YangBase):
	"""Decline message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decline-message'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message after sending a DECLINE message.If the host does not receive a valid REPLY message within this time,it sends another DECLINE message and the timeout length is doubled from the previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of decline retries.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating declines will retried indefinitely. TRUE  : Enable indefinite retry of declines. FALSE : Disable indefinite retry of declines.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating declines will be not be retried. TRUE  : Disable retry of declines. FALSE : Enable retry of declines.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetries(self))

	def delete(self):
		"""Delete this decline-message instance on the server.

		Decline message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decline-message instance on the server with any changed property values.

		Decline message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message after sending a DECLINE message.If the host does not receive a valid REPLY message within this time,it sends another DECLINE message and the timeout length is doubled from the previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message after sending a DECLINE message.If the host does not receive a valid REPLY message within this time,it sends another DECLINE message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message after sending a DECLINE message.If the host does not receive a valid REPLY message within this time,it sends another DECLINE message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCount(YangBase):
	"""Number of decline retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of decline retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of decline retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetry(YangBase):
	"""Flag indicating declines will retried indefinitely. TRUE  : Enable indefinite retry of declines. FALSE : Disable indefinite retry of declines.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating declines will retried indefinitely. TRUE  : Enable indefinite retry of declines. FALSE : Disable indefinite retry of declines.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating declines will retried indefinitely. TRUE  : Enable indefinite retry of declines. FALSE : Disable indefinite retry of declines.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetries(YangBase):
	"""Flag indicating declines will be not be retried. TRUE  : Disable retry of declines. FALSE : Enable retry of declines.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating declines will be not be retried. TRUE  : Disable retry of declines. FALSE : Enable retry of declines.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating declines will be not be retried. TRUE  : Disable retry of declines. FALSE : Enable retry of declines.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6DeclineMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage(YangBase):
	"""Information Request message options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'info-request-message'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessage, self).__init__(parent, None)

	def initial_timeout(self):
		"""Get the initial-timeout object from the server

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial INFORMATION-REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another INFORMATION-REQUEST message and the timeout length is doubled from the previous attempt.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeout`: An object encapsulating an instance of the initial-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeout(self))

	def max_timeout(self):
		"""Get the max-timeout object from the server

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial INFORMATION-REQUEST message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the INFORMATION-REQUEST timeout value is set to the Maximum INFORMATION-REQUEST Retry Timeout value, the client tries to resend the INFORMATION-REQUEST message until the IPv6 prefix's valid lifetime expires.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeout`: An object encapsulating an instance of the max-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeout(self))

	def retry_count(self):
		"""Get the retry-count object from the server

		Number of info-request message retries.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCount`: An object encapsulating an instance of the retry-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCount(self))

	def indefinite_retry(self):
		"""Get the indefinite-retry object from the server

		Flag indicating info-request message will retried indefinitely. TRUE  : Enable indefinite retry of info-request message. FALSE : Disable indefinite retry of info-request message.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetry`: An object encapsulating an instance of the indefinite-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetry(self))

	def disable_retries(self):
		"""Get the disable-retries object from the server

		Flag indicating info-request message will be not be retried. TRUE  : Disable retry of info-request message. FALSE : Enable retry of info-request message.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetries`: An object encapsulating an instance of the disable-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetries(self))

	def delete(self):
		"""Delete this info-request-message instance on the server.

		Information Request message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this info-request-message instance on the server with any changed property values.

		Information Request message options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeout(YangBase):
	"""Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial INFORMATION-REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another INFORMATION-REQUEST message and the timeout length is doubled from the previous attempt.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'initial-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutRandom(self))

	def delete(self):
		"""Delete this initial-timeout instance on the server.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial INFORMATION-REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another INFORMATION-REQUEST message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this initial-timeout instance on the server with any changed property values.

		Specifiy the time (in seconds) that the host waits to receive a REPLY message from the delegating router after sending an initial INFORMATION-REQUEST message.If the host does not receive a valid REPLY message within this time,it sends another INFORMATION-REQUEST message and the timeout length is doubled from the previous attempt.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageInitialTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeout(YangBase):
	"""Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial INFORMATION-REQUEST message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the INFORMATION-REQUEST timeout value is set to the Maximum INFORMATION-REQUEST Retry Timeout value, the client tries to resend the INFORMATION-REQUEST message until the IPv6 prefix's valid lifetime expires.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-timeout'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutRandom(self))

	def delete(self):
		"""Delete this max-timeout instance on the server.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial INFORMATION-REQUEST message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the INFORMATION-REQUEST timeout value is set to the Maximum INFORMATION-REQUEST Retry Timeout value, the client tries to resend the INFORMATION-REQUEST message until the IPv6 prefix's valid lifetime expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-timeout instance on the server with any changed property values.

		Specify the maximum timeout value (in seconds) that the host waits for an REPLY message. If an initial INFORMATION-REQUEST message is unsuccessful and times out, the timeout value for subsequent attempts is either double the previous timeout value or the value specified for this field—whichever is lower. This field limits the retry timeout time, which can become long if it continues to double with each retry. If the INFORMATION-REQUEST timeout value is set to the Maximum INFORMATION-REQUEST Retry Timeout value, the client tries to resend the INFORMATION-REQUEST message until the IPv6 prefix's valid lifetime expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageMaxTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCount(YangBase):
	"""Number of info-request message retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retry-count'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountRandom(self))

	def delete(self):
		"""Delete this retry-count instance on the server.

		Number of info-request message retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retry-count instance on the server with any changed property values.

		Number of info-request message retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageRetryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetry(YangBase):
	"""Flag indicating info-request message will retried indefinitely. TRUE  : Enable indefinite retry of info-request message. FALSE : Disable indefinite retry of info-request message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'indefinite-retry'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetry, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryRandom(self))

	def delete(self):
		"""Delete this indefinite-retry instance on the server.

		Flag indicating info-request message will retried indefinitely. TRUE  : Enable indefinite retry of info-request message. FALSE : Disable indefinite retry of info-request message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this indefinite-retry instance on the server with any changed property values.

		Flag indicating info-request message will retried indefinitely. TRUE  : Enable indefinite retry of info-request message. FALSE : Disable indefinite retry of info-request message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageIndefiniteRetryRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetries(YangBase):
	"""Flag indicating info-request message will be not be retried. TRUE  : Disable retry of info-request message. FALSE : Enable retry of info-request message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-retries'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesRandom(self))

	def delete(self):
		"""Delete this disable-retries instance on the server.

		Flag indicating info-request message will be not be retried. TRUE  : Disable retry of info-request message. FALSE : Enable retry of info-request message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-retries instance on the server with any changed property values.

		Flag indicating info-request message will be not be retried. TRUE  : Disable retry of info-request message. FALSE : Enable retry of info-request message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsDhcpv6InfoRequestMessageDisableRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsIgmp(YangBase):
	"""This list allows for configuring global IGMP options.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalProtocolsIgmp, self).__init__(parent, yang_key_value)

	def calculate_latency_delay(self):
		"""Get the calculate-latency-delay object from the server

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelay`: An object encapsulating an instance of the calculate-latency-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelay(self))

	def max_burst(self):
		"""Get the max-burst object from the server

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpMaxBurst`: An object encapsulating an instance of the max-burst model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpMaxBurst(self))

	def rate_pps(self):
		"""Get the rate-pps object from the server

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpRatePps`: An object encapsulating an instance of the rate-pps model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpRatePps(self))

	def vlan_sub_filter_type(self):
		"""Get the vlan-sub-filter-type object from the server

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpVlanSubFilterType`: An object encapsulating an instance of the vlan-sub-filter-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpVlanSubFilterType(self))

	def delete(self):
		"""Delete this igmp instance on the server.

		This list allows for configuring global IGMP options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmp instance on the server with any changed property values.

		This list allows for configuring global IGMP options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name for a global protocols list item"""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../../ports/name: A list of abstract port references"""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)


class SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelay(YangBase):
	"""The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'calculate-latency-delay'
		super(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayRandom(self))

	def delete(self):
		"""Delete this calculate-latency-delay instance on the server.

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this calculate-latency-delay instance on the server with any changed property values.

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsIgmpCalculateLatencyDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsIgmpMaxBurst(YangBase):
	"""Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-burst'
		super(SessionsConfigGlobalProtocolsIgmpMaxBurst, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpMaxBurstIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpMaxBurstIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpMaxBurstDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpMaxBurstDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpMaxBurstRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpMaxBurstRandom(self))

	def delete(self):
		"""Delete this max-burst instance on the server.

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-burst instance on the server with any changed property values.

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsIgmpMaxBurstIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsIgmpMaxBurstIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpMaxBurstDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsIgmpMaxBurstDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpMaxBurstRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsIgmpMaxBurstRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsIgmpRatePps(YangBase):
	"""Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rate-pps'
		super(SessionsConfigGlobalProtocolsIgmpRatePps, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpRatePpsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpRatePpsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpRatePpsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpRatePpsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsIgmpRatePpsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsIgmpRatePpsRandom(self))

	def delete(self):
		"""Delete this rate-pps instance on the server.

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rate-pps instance on the server with any changed property values.

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsIgmpRatePpsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsIgmpRatePpsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpRatePpsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsIgmpRatePpsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsIgmpRatePpsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsIgmpRatePpsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsIgmpVlanSubFilterType(YangBase):
	"""To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vlan-sub-filter-type'
		super(SessionsConfigGlobalProtocolsIgmpVlanSubFilterType, self).__init__(parent, None)

	def delete(self):
		"""Delete this vlan-sub-filter-type instance on the server.

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vlan-sub-filter-type instance on the server with any changed property values.

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsMld(YangBase):
	"""This list allows for configuring global MLD options.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalProtocolsMld, self).__init__(parent, yang_key_value)

	def calculate_latency_delay(self):
		"""Get the calculate-latency-delay object from the server

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldCalculateLatencyDelay`: An object encapsulating an instance of the calculate-latency-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldCalculateLatencyDelay(self))

	def max_burst(self):
		"""Get the max-burst object from the server

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldMaxBurst`: An object encapsulating an instance of the max-burst model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldMaxBurst(self))

	def rate_pps(self):
		"""Get the rate-pps object from the server

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldRatePps`: An object encapsulating an instance of the rate-pps model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldRatePps(self))

	def vlan_sub_filter_type(self):
		"""Get the vlan-sub-filter-type object from the server

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldVlanSubFilterType`: An object encapsulating an instance of the vlan-sub-filter-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldVlanSubFilterType(self))

	def delete(self):
		"""Delete this mld instance on the server.

		This list allows for configuring global MLD options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mld instance on the server with any changed property values.

		This list allows for configuring global MLD options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name for a global protocols list item"""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../../ports/name: A list of abstract port references"""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)


class SessionsConfigGlobalProtocolsMldCalculateLatencyDelay(YangBase):
	"""The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'calculate-latency-delay'
		super(SessionsConfigGlobalProtocolsMldCalculateLatencyDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldCalculateLatencyDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldCalculateLatencyDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldCalculateLatencyDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayRandom(self))

	def delete(self):
		"""Delete this calculate-latency-delay instance on the server.

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this calculate-latency-delay instance on the server with any changed property values.

		The delay (in seconds) before latency is calculated. The timer starts after reports are sent for each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsMldCalculateLatencyDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldCalculateLatencyDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldCalculateLatencyDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsMldCalculateLatencyDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsMldMaxBurst(YangBase):
	"""Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-burst'
		super(SessionsConfigGlobalProtocolsMldMaxBurst, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldMaxBurstIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldMaxBurstIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldMaxBurstDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldMaxBurstDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldMaxBurstRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldMaxBurstRandom(self))

	def delete(self):
		"""Delete this max-burst instance on the server.

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-burst instance on the server with any changed property values.

		Short-term maximum burst size in packets. A value of 0 specifies the maximum possible burst size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsMldMaxBurstIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsMldMaxBurstIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldMaxBurstDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsMldMaxBurstDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldMaxBurstRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsMldMaxBurstRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsMldRatePps(YangBase):
	"""Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rate-pps'
		super(SessionsConfigGlobalProtocolsMldRatePps, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldRatePpsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldRatePpsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldRatePpsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldRatePpsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsMldRatePpsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsMldRatePpsRandom(self))

	def delete(self):
		"""Delete this rate-pps instance on the server.

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rate-pps instance on the server with any changed property values.

		Long-term, maximum packet rate (in packets per second). A value of 0 specifies the maximum possible output rate.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsMldRatePpsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsMldRatePpsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldRatePpsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsMldRatePpsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsMldRatePpsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsMldRatePpsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsMldVlanSubFilterType(YangBase):
	"""To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vlan-sub-filter-type'
		super(SessionsConfigGlobalProtocolsMldVlanSubFilterType, self).__init__(parent, None)

	def delete(self):
		"""Delete this vlan-sub-filter-type instance on the server.

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vlan-sub-filter-type instance on the server with any changed property values.

		To filter on the Outer or Inner VLAN tag when calculating latency for devices with a PPPoE and stacked VLAN encapsulation. OUTER	: Specifies that the analyzer filter will be programmed to use the Outer VLAN ID. INNER	: Specifies that the analyzer filter will be programmed to use the Inner VLAN ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPim(YangBase):
	"""This list allows for configuring global PIM options.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pim'
		self.YANG_KEY = 'name'
		super(SessionsConfigGlobalProtocolsPim, self).__init__(parent, yang_key_value)

	def enable_pack_group_record(self):
		"""Get the enable-pack-group-record object from the server

		Specifies that multicast groups be combined in Join/Prune messages or sent individually. TRUE  : Enables Join/Prune message packing. FALSE	: Disables Join/Prune message packing.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimEnablePackGroupRecord`: An object encapsulating an instance of the enable-pack-group-record model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimEnablePackGroupRecord(self))

	def prune_delay_options(self):
		"""Get the prune-delay-options object from the server

		Include the LAN Prune Delay option in PIM Hello messages.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptions`: An object encapsulating an instance of the prune-delay-options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptions(self))

	def create_prune_delay_options(self):
		"""Create a child sibling prune-delay-options instance on the server.

		Include the LAN Prune Delay option in PIM Hello messages.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptions`: An object encapsulating an instance of the prune-delay-options model.  

		Raises:  
			AlreadyExistsError: An instance of prune-delay-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsPimPruneDelayOptions(self), locals())

	def msg_interval(self):
		"""Get the msg-interval object from the server

		Minimum gap, in seconds, between successive PIM messages.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgInterval`: An object encapsulating an instance of the msg-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgInterval(self))

	def msg_rate(self):
		"""Get the msg-rate object from the server

		Maximum rate, per second, at which PIM messages will be transmitted.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgRate`: An object encapsulating an instance of the msg-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgRate(self))

	def trigger_hello_delay(self):
		"""Get the trigger-hello-delay object from the server

		Randomized interval, in seconds, for initial Hello message on boot up or triggered Hello message to a rebooting neighbor.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimTriggerHelloDelay`: An object encapsulating an instance of the trigger-hello-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimTriggerHelloDelay(self))

	def scalability_options(self):
		"""Get the scalability-options object from the server

		Scalability options.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptions`: An object encapsulating an instance of the scalability-options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptions(self))

	def create_scalability_options(self):
		"""Create a child sibling scalability-options instance on the server.

		Scalability options.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptions`: An object encapsulating an instance of the scalability-options model.  

		Raises:  
			AlreadyExistsError: An instance of scalability-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigGlobalProtocolsPimScalabilityOptions(self), locals())

	def delete(self):
		"""Delete this pim instance on the server.

		This list allows for configuring global PIM options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pim instance on the server with any changed property values.

		This list allows for configuring global PIM options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name for a global protocols list item"""
		return self._get_value('name')


class SessionsConfigGlobalProtocolsPimEnablePackGroupRecord(YangBase):
	"""Specifies that multicast groups be combined in Join/Prune messages or sent individually. TRUE  : Enables Join/Prune message packing. FALSE	: Disables Join/Prune message packing.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-pack-group-record'
		super(SessionsConfigGlobalProtocolsPimEnablePackGroupRecord, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimEnablePackGroupRecordIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimEnablePackGroupRecordDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimEnablePackGroupRecordRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordRandom(self))

	def delete(self):
		"""Delete this enable-pack-group-record instance on the server.

		Specifies that multicast groups be combined in Join/Prune messages or sent individually. TRUE  : Enables Join/Prune message packing. FALSE	: Disables Join/Prune message packing.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-pack-group-record instance on the server with any changed property values.

		Specifies that multicast groups be combined in Join/Prune messages or sent individually. TRUE  : Enables Join/Prune message packing. FALSE	: Disables Join/Prune message packing.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimEnablePackGroupRecordIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimEnablePackGroupRecordDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimEnablePackGroupRecordRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimEnablePackGroupRecordRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptions(YangBase):
	"""Include the LAN Prune Delay option in PIM Hello messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prune-delay-options'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptions, self).__init__(parent, None)

	def lan_prune_delay(self):
		"""Get the lan-prune-delay object from the server

		Expected message propagation delay in milliseconds.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelay`: An object encapsulating an instance of the lan-prune-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelay(self))

	def override_interval(self):
		"""Get the override-interval object from the server

		Join/Prune override interval in milliseconds.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideInterval`: An object encapsulating an instance of the override-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideInterval(self))

	def t_bit(self):
		"""Get the t-bit object from the server

		Disable or enable Joins suppression. TRUE  : Disable Joins suppression. FALSE	: Enable Joins suppression.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBit`: An object encapsulating an instance of the t-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBit(self))

	def delete(self):
		"""Delete this prune-delay-options instance on the server.

		Include the LAN Prune Delay option in PIM Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prune-delay-options instance on the server with any changed property values.

		Include the LAN Prune Delay option in PIM Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelay(YangBase):
	"""Expected message propagation delay in milliseconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'lan-prune-delay'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayRandom(self))

	def delete(self):
		"""Delete this lan-prune-delay instance on the server.

		Expected message propagation delay in milliseconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this lan-prune-delay instance on the server with any changed property values.

		Expected message propagation delay in milliseconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsLanPruneDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideInterval(YangBase):
	"""Join/Prune override interval in milliseconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'override-interval'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalRandom(self))

	def delete(self):
		"""Delete this override-interval instance on the server.

		Join/Prune override interval in milliseconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this override-interval instance on the server with any changed property values.

		Join/Prune override interval in milliseconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsOverrideIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBit(YangBase):
	"""Disable or enable Joins suppression. TRUE  : Disable Joins suppression. FALSE	: Enable Joins suppression.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 't-bit'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitRandom(self))

	def delete(self):
		"""Delete this t-bit instance on the server.

		Disable or enable Joins suppression. TRUE  : Disable Joins suppression. FALSE	: Enable Joins suppression.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this t-bit instance on the server with any changed property values.

		Disable or enable Joins suppression. TRUE  : Disable Joins suppression. FALSE	: Enable Joins suppression.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimPruneDelayOptionsTBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimMsgInterval(YangBase):
	"""Minimum gap, in seconds, between successive PIM messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'msg-interval'
		super(SessionsConfigGlobalProtocolsPimMsgInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgIntervalRandom(self))

	def delete(self):
		"""Delete this msg-interval instance on the server.

		Minimum gap, in seconds, between successive PIM messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this msg-interval instance on the server with any changed property values.

		Minimum gap, in seconds, between successive PIM messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimMsgIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimMsgIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimMsgIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimMsgIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimMsgIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimMsgIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimMsgRate(YangBase):
	"""Maximum rate, per second, at which PIM messages will be transmitted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'msg-rate'
		super(SessionsConfigGlobalProtocolsPimMsgRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimMsgRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimMsgRateRandom(self))

	def delete(self):
		"""Delete this msg-rate instance on the server.

		Maximum rate, per second, at which PIM messages will be transmitted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this msg-rate instance on the server with any changed property values.

		Maximum rate, per second, at which PIM messages will be transmitted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimMsgRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimMsgRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimMsgRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimMsgRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimMsgRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimMsgRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimTriggerHelloDelay(YangBase):
	"""Randomized interval, in seconds, for initial Hello message on boot up or triggered Hello message to a rebooting neighbor.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'trigger-hello-delay'
		super(SessionsConfigGlobalProtocolsPimTriggerHelloDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimTriggerHelloDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimTriggerHelloDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimTriggerHelloDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimTriggerHelloDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimTriggerHelloDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimTriggerHelloDelayRandom(self))

	def delete(self):
		"""Delete this trigger-hello-delay instance on the server.

		Randomized interval, in seconds, for initial Hello message on boot up or triggered Hello message to a rebooting neighbor.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this trigger-hello-delay instance on the server with any changed property values.

		Randomized interval, in seconds, for initial Hello message on boot up or triggered Hello message to a rebooting neighbor.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimTriggerHelloDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimTriggerHelloDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimTriggerHelloDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimTriggerHelloDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimTriggerHelloDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimTriggerHelloDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptions(YangBase):
	"""Scalability options.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'scalability-options'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptions, self).__init__(parent, None)

	def disable_hello_expire_timer(self):
		"""Get the disable-hello-expire-timer object from the server

		Disable hello expire timer in scalablity mode. TRUE	: Hello expire timer is disabled. FALSE	: Hello expire timer is set to user-specified value.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimer`: An object encapsulating an instance of the disable-hello-expire-timer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimer(self))

	def disable_hello_rx_in_neighbor_state(self):
		"""Get the disable-hello-rx-in-neighbor-state object from the server

		Option to not process hello messages in neighbor state to increase scalability. TRUE	: Hello messages are not processed in neighbor state. FALSE	: All hello messages are processed in neighbor state.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborState`: An object encapsulating an instance of the disable-hello-rx-in-neighbor-state model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborState(self))

	def disable_incoming_msg_processing(self):
		"""Get the disable-incoming-msg-processing object from the server

		Option to disable processing of all incoming messages in neighbor state to increase scalability. TRUE  : All incoming messages are not processed in neighbor state. FALSE	: All incoming messages are processed in neighbor state.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessing`: An object encapsulating an instance of the disable-incoming-msg-processing model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessing(self))

	def delete(self):
		"""Delete this scalability-options instance on the server.

		Scalability options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this scalability-options instance on the server with any changed property values.

		Scalability options.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimer(YangBase):
	"""Disable hello expire timer in scalablity mode. TRUE	: Hello expire timer is disabled. FALSE	: Hello expire timer is set to user-specified value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-hello-expire-timer'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerRandom(self))

	def delete(self):
		"""Delete this disable-hello-expire-timer instance on the server.

		Disable hello expire timer in scalablity mode. TRUE	: Hello expire timer is disabled. FALSE	: Hello expire timer is set to user-specified value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-hello-expire-timer instance on the server with any changed property values.

		Disable hello expire timer in scalablity mode. TRUE	: Hello expire timer is disabled. FALSE	: Hello expire timer is set to user-specified value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloExpireTimerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborState(YangBase):
	"""Option to not process hello messages in neighbor state to increase scalability. TRUE	: Hello messages are not processed in neighbor state. FALSE	: All hello messages are processed in neighbor state.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-hello-rx-in-neighbor-state'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborState, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateRandom(self))

	def delete(self):
		"""Delete this disable-hello-rx-in-neighbor-state instance on the server.

		Option to not process hello messages in neighbor state to increase scalability. TRUE	: Hello messages are not processed in neighbor state. FALSE	: All hello messages are processed in neighbor state.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-hello-rx-in-neighbor-state instance on the server with any changed property values.

		Option to not process hello messages in neighbor state to increase scalability. TRUE	: Hello messages are not processed in neighbor state. FALSE	: All hello messages are processed in neighbor state.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableHelloRxInNeighborStateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessing(YangBase):
	"""Option to disable processing of all incoming messages in neighbor state to increase scalability. TRUE  : All incoming messages are not processed in neighbor state. FALSE	: All incoming messages are processed in neighbor state.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'disable-incoming-msg-processing'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessing, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingRandom(self))

	def delete(self):
		"""Delete this disable-incoming-msg-processing instance on the server.

		Option to disable processing of all incoming messages in neighbor state to increase scalability. TRUE  : All incoming messages are not processed in neighbor state. FALSE	: All incoming messages are processed in neighbor state.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this disable-incoming-msg-processing instance on the server with any changed property values.

		Option to disable processing of all incoming messages in neighbor state to increase scalability. TRUE  : All incoming messages are not processed in neighbor state. FALSE	: All incoming messages are processed in neighbor state.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigGlobalProtocolsPimScalabilityOptionsDisableIncomingMsgProcessingRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroups(YangBase):
	"""A list of device-groups. Each device-groups object can contain 0..n devices.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'device-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroups, self).__init__(parent, yang_key_value)

	def devices(self, name=None):
		"""Get the devices object(s) from the server.

		A list of devices. Each devices object can contain 0..n protocols objects

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the devices list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevices` | :obj:`SessionsConfigDeviceGroupsDevices`: If arg name is None a list of SessionsConfigDeviceGroupsDevices objects otherwise a single SessionsConfigDeviceGroupsDevices object.  

		Raises:  
			NotFoundError: name is not in the list of devices objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevices(self, name))

	def create_devices(self, name):
		"""Create a child sibling devices instance on the server.

		A list of devices. Each devices object can contain 0..n protocols objects

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevices`: An object encapsulating an instance of the devices model.  

		Raises:  
			AlreadyExistsError: An instance of devices with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevices(self, name), locals())

	def simulated_networks(self, name=None):
		"""Get the simulated-networks object(s) from the server.

		A list of network groups. Each network group object can contain 0..n networks objects

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the simulated-networks list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworks` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworks`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworks objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworks object.  

		Raises:  
			NotFoundError: name is not in the list of simulated-networks objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworks(self, name))

	def create_simulated_networks(self, name):
		"""Create a child sibling simulated-networks instance on the server.

		A list of network groups. Each network group object can contain 0..n networks objects

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworks`: An object encapsulating an instance of the simulated-networks model.  

		Raises:  
			AlreadyExistsError: An instance of simulated-networks with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworks(self, name), locals())

	def delete(self):
		"""Delete this device-groups instance on the server.

		A list of device-groups. Each device-groups object can contain 0..n devices.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this device-groups instance on the server with any changed property values.

		A list of device-groups. Each device-groups object can contain 0..n devices.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of a device-groups object."""
		return self._get_value('name')

	@property
	def ports(self):
		"""-> ../../ports/name: A list of abstract test port names. Every object in the devices and protocols lists will share the ports assigned to a device-groups object. An abstract test port name cannot be assigned to more than one device-groups object."""
		return self._get_value('ports')

	@ports.setter
	def ports(self, value):
		return self._set_value('ports', value)


class SessionsConfigDeviceGroupsDevices(YangBase):
	"""A list of devices. Each devices object can contain 0..n protocols objects
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'devices'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevices, self).__init__(parent, yang_key_value)

	def protocols(self, name=None):
		"""Get the protocols object(s) from the server.

		A list of emulated protocols. Each emulated protocols object is a container for one and only one of the emulated protocol types. The protocol-type is used to specify what type of protocol is contained in a protocols object.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the protocols list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocols` | :obj:`SessionsConfigDeviceGroupsDevicesProtocols`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocols objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocols object.  

		Raises:  
			NotFoundError: name is not in the list of protocols objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocols(self, name))

	def create_protocols(self, name):
		"""Create a child sibling protocols instance on the server.

		A list of emulated protocols. Each emulated protocols object is a container for one and only one of the emulated protocol types. The protocol-type is used to specify what type of protocol is contained in a protocols object.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocols`: An object encapsulating an instance of the protocols model.  

		Raises:  
			AlreadyExistsError: An instance of protocols with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocols(self, name), locals())

	def delete(self):
		"""Delete this devices instance on the server.

		A list of devices. Each devices object can contain 0..n protocols objects

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this devices instance on the server with any changed property values.

		A list of devices. Each devices object can contain 0..n protocols objects

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of a devices object."""
		return self._get_value('name')

	@property
	def device_count_per_port(self):
		"""uint32: The number of devices that will be created on each abstract test port specified in the port-names list."""
		return self._get_value('device-count-per-port')

	@device_count_per_port.setter
	def device_count_per_port(self, value):
		return self._set_value('device-count-per-port', value)

	@property
	def parent_link(self):
		"""union: Identifies which devices object or simulated-networks object is connected to this object. This is used to create a devices container behind a devices or simulated networks container. An empty link indicates that the object is at the top of the stack."""
		return self._get_value('parent-link')

	@parent_link.setter
	def parent_link(self, value):
		return self._set_value('parent-link', value)


class SessionsConfigDeviceGroupsDevicesProtocols(YangBase):
	"""A list of emulated protocols. Each emulated protocols object is a container for one and only one of the emulated protocol types. The protocol-type is used to specify what type of protocol is contained in a protocols object.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'protocols'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocols, self).__init__(parent, yang_key_value)

	def ethernet(self):
		"""Get the ethernet object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsEthernet`: An object encapsulating an instance of the ethernet model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsEthernet(self))

	def vlan(self):
		"""Get the vlan object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlan`: An object encapsulating an instance of the vlan model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlan(self))

	def ipv4(self):
		"""Get the ipv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4`: An object encapsulating an instance of the ipv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4(self))

	def ipv6(self):
		"""Get the ipv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6`: An object encapsulating an instance of the ipv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6(self))

	def bgpv4(self):
		"""Get the bgpv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4`: An object encapsulating an instance of the bgpv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4(self))

	def bgpv6(self):
		"""Get the bgpv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6`: An object encapsulating an instance of the bgpv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6(self))

	def ospfv2(self):
		"""Get the ospfv2 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2`: An object encapsulating an instance of the ospfv2 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2(self))

	def ospfv3(self):
		"""Get the ospfv3 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3`: An object encapsulating an instance of the ospfv3 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3(self))

	def isis(self):
		"""Get the isis object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsis`: An object encapsulating an instance of the isis model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsis(self))

	def bfdv4(self):
		"""Get the bfdv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4`: An object encapsulating an instance of the bfdv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4(self))

	def bfdv6(self):
		"""Get the bfdv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6`: An object encapsulating an instance of the bfdv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6(self))

	def dhcpv4(self):
		"""Get the dhcpv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4`: An object encapsulating an instance of the dhcpv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4(self))

	def dhcpv4_server(self):
		"""Get the dhcpv4-server object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Server`: An object encapsulating an instance of the dhcpv4-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Server(self))

	def dhcpv6(self):
		"""Get the dhcpv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6`: An object encapsulating an instance of the dhcpv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6(self))

	def dhcpv6_server(self):
		"""Get the dhcpv6-server object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Server`: An object encapsulating an instance of the dhcpv6-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Server(self))

	def igmp(self):
		"""Get the igmp object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmp`: An object encapsulating an instance of the igmp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmp(self))

	def igmp_querier(self):
		"""Get the igmp-querier object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerier`: An object encapsulating an instance of the igmp-querier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerier(self))

	def mld(self):
		"""Get the mld object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMld`: An object encapsulating an instance of the mld model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMld(self))

	def mld_querier(self):
		"""Get the mld-querier object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerier`: An object encapsulating an instance of the mld-querier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerier(self))

	def pim(self):
		"""Get the pim object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPim`: An object encapsulating an instance of the pim model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPim(self))

	def delete(self):
		"""Delete this protocols instance on the server.

		A list of emulated protocols. Each emulated protocols object is a container for one and only one of the emulated protocol types. The protocol-type is used to specify what type of protocol is contained in a protocols object.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this protocols instance on the server with any changed property values.

		A list of emulated protocols. Each emulated protocols object is a container for one and only one of the emulated protocol types. The protocol-type is used to specify what type of protocol is contained in a protocols object.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of a protocols object"""
		return self._get_value('name')

	@property
	def parent_link(self):
		"""union: Identifies which protocols object is the parent of this protocol. This link only applies to protocol objects in one container. The link can be used to specify a vertical or horizontal relationship. To specify the first protocols object in a stack the value must be empty."""
		return self._get_value('parent-link')

	@parent_link.setter
	def parent_link(self, value):
		return self._set_value('parent-link', value)

	@property
	def flow_link(self):
		"""union: Identifies which object in another container is the next hop from this container. The name cannot be one of the names in this devices container. An empty link indicates no flow link"""
		return self._get_value('flow-link')

	@flow_link.setter
	def flow_link(self, value):
		return self._set_value('flow-link', value)

	@property
	def protocol_type(self):
		""":str:`enum`: Determines which detailed emulated protocol container is active.  
		Enums:  
			ETHERNET: TBD  
			VLAN: TBD  
			IPV4: TBD  
			IPV6: TBD  
			BGPV4: TBD  
			BGPV6: TBD  
			OSPFV2: TBD  
			OSPFV3: TBD  
			ISIS: TBD  
			BFDV4: TBD  
			BFDV6: TBD  
			DHCPV4: TBD  
			DHCPV4_SERVER: TBD  
			DHCPV6: TBD  
			DHCPV6_SERVER: TBD  
			IGMP: TBD  
			IGMP_QUERIER: TBD  
			MLD: TBD  
			MLD_QUERIER: TBD  
			PIM: TBD  
		"""
		return self._get_value('protocol-type')

	@protocol_type.setter
	def protocol_type(self, value):
		return self._set_value('protocol-type', value)


class SessionsConfigDeviceGroupsDevicesProtocolsEthernet(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet'
		super(SessionsConfigDeviceGroupsDevicesProtocolsEthernet, self).__init__(parent, None)

	def mac(self):
		"""Get the mac object from the server

		The mac address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsEthernetMac`: An object encapsulating an instance of the mac model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMac(self))

	def delete(self):
		"""Delete this ethernet instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsEthernetMac(YangBase):
	"""The mac address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mac'
		super(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMac, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacRandom(self))

	def delete(self):
		"""Delete this mac instance on the server.

		The mac address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mac instance on the server with any changed property values.

		The mac address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsEthernetMacRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlan(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vlan'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlan, self).__init__(parent, None)

	def tpid(self):
		"""Get the tpid object from the server

		Tag Protocol Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanTpid`: An object encapsulating an instance of the tpid model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanTpid(self))

	def priority(self):
		"""Get the priority object from the server

		Priority

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanPriority`: An object encapsulating an instance of the priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriority(self))

	def id(self):
		"""Get the id object from the server

		VLAN id

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanId`: An object encapsulating an instance of the id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanId(self))

	def delete(self):
		"""Delete this vlan instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vlan instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsVlanTpid(YangBase):
	"""Tag Protocol Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'tpid'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanTpid, self).__init__(parent, None)

	def delete(self):
		"""Delete this tpid instance on the server.

		Tag Protocol Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this tpid instance on the server with any changed property values.

		Tag Protocol Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanPriority(YangBase):
	"""Priority
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityRandom(self))

	def delete(self):
		"""Delete this priority instance on the server.

		Priority

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority instance on the server with any changed property values.

		Priority

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanId(YangBase):
	"""VLAN id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsVlanIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdRandom(self))

	def delete(self):
		"""Delete this id instance on the server.

		VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this id instance on the server with any changed property values.

		VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsVlanIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsVlanIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddress(self))

	def gateway_address(self):
		"""Get the gateway-address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddress`: An object encapsulating an instance of the gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddress(self))

	def prefix(self):
		"""Get the prefix object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4Prefix`: An object encapsulating an instance of the prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4Prefix(self))

	def delete(self):
		"""Delete this ipv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gateway-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this gateway-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gateway-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4Prefix(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4Prefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixRandom(self))

	def delete(self):
		"""Delete this prefix instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv4PrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddress(self))

	def gateway_address(self):
		"""Get the gateway-address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddress`: An object encapsulating an instance of the gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddress(self))

	def link_local_address(self):
		"""Get the link-local-address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddress`: An object encapsulating an instance of the link-local-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddress(self))

	def delete(self):
		"""Delete this ipv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gateway-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this gateway-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gateway-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-local-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressRandom(self))

	def delete(self):
		"""Delete this link-local-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-local-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIpv6LinkLocalAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv4'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4, self).__init__(parent, None)

	def as_number_2_byte(self):
		"""Get the as-number-2-byte object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte`: An object encapsulating an instance of the as-number-2-byte model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte(self))

	def create_as_number_2_byte(self):
		"""Create a child sibling as-number-2-byte instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte`: An object encapsulating an instance of the as-number-2-byte model.  

		Raises:  
			AlreadyExistsError: An instance of as-number-2-byte with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte(self), locals())

	def as_number_4_byte(self):
		"""Get the as-number-4-byte object from the server

		Enables the use of 4 Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte`: An object encapsulating an instance of the as-number-4-byte model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte(self))

	def create_as_number_4_byte(self):
		"""Create a child sibling as-number-4-byte instance on the server.

		Enables the use of 4 Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte`: An object encapsulating an instance of the as-number-4-byte model.  

		Raises:  
			AlreadyExistsError: An instance of as-number-4-byte with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte(self), locals())

	def as_number_set_mode(self):
		"""Get the as-number-set-mode object from the server

		Option to exclude, include or prepend the local AS number.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumberSetMode`: An object encapsulating an instance of the as-number-set-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumberSetMode(self))

	def bgp_type(self):
		"""Get the bgp-type object from the server

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4BgpType`: An object encapsulating an instance of the bgp-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4BgpType(self))

	def hold_time_interval(self):
		"""Get the hold-time-interval object from the server

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeInterval`: An object encapsulating an instance of the hold-time-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeInterval(self))

	def keep_alive_interval(self):
		"""Get the keep-alive-interval object from the server

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveInterval`: An object encapsulating an instance of the keep-alive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveInterval(self))

	def graceful_restart(self):
		"""Get the graceful-restart object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart`: An object encapsulating an instance of the graceful-restart model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart(self))

	def create_graceful_restart(self):
		"""Create a child sibling graceful-restart instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart`: An object encapsulating an instance of the graceful-restart model.  

		Raises:  
			AlreadyExistsError: An instance of graceful-restart with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart(self), locals())

	def authentication(self):
		"""Get the authentication object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication(self), locals())

	def ttl(self):
		"""Get the ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Ttl`: An object encapsulating an instance of the ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Ttl(self))

	def dut_ipv4_address(self):
		"""Get the dut-ipv4-address object from the server

		IPv4 address of the BGP peer for the session.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4Address`: An object encapsulating an instance of the dut-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4Address(self))

	def delete(self):
		"""Delete this bgpv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgpv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-2-byte'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2Byte, self).__init__(parent, None)

	def as_number(self):
		"""Get the as-number object from the server

		2-Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumber`: An object encapsulating an instance of the as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumber(self))

	def dut_as_number(self):
		"""Get the dut-as-number object from the server

		2-Byte Autonomous system number configured for the DUT.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumber`: An object encapsulating an instance of the dut-as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumber(self))

	def delete(self):
		"""Delete this as-number-2-byte instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-2-byte instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumber(YangBase):
	"""2-Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberRandom(self))

	def delete(self):
		"""Delete this as-number instance on the server.

		2-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number instance on the server with any changed property values.

		2-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumber(YangBase):
	"""2-Byte Autonomous system number configured for the DUT.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberRandom(self))

	def delete(self):
		"""Delete this dut-as-number instance on the server.

		2-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-as-number instance on the server with any changed property values.

		2-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber2ByteDutAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte(YangBase):
	"""Enables the use of 4 Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-4-byte'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4Byte, self).__init__(parent, None)

	def as_number(self):
		"""Get the as-number object from the server

		4-Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumber`: An object encapsulating an instance of the as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumber(self))

	def dut_as_number(self):
		"""Get the dut-as-number object from the server

		4-Byte Autonomous system number configured for the DUT.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumber`: An object encapsulating an instance of the dut-as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumber(self))

	def delete(self):
		"""Delete this as-number-4-byte instance on the server.

		Enables the use of 4 Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-4-byte instance on the server with any changed property values.

		Enables the use of 4 Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumber(YangBase):
	"""4-Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberRandom(self))

	def delete(self):
		"""Delete this as-number instance on the server.

		4-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number instance on the server with any changed property values.

		4-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumber(YangBase):
	"""4-Byte Autonomous system number configured for the DUT.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberRandom(self))

	def delete(self):
		"""Delete this dut-as-number instance on the server.

		4-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-as-number instance on the server with any changed property values.

		4-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumber4ByteDutAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumberSetMode(YangBase):
	"""Option to exclude, include or prepend the local AS number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-set-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AsNumberSetMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this as-number-set-mode instance on the server.

		Option to exclude, include or prepend the local AS number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-set-mode instance on the server with any changed property values.

		Option to exclude, include or prepend the local AS number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4BgpType(YangBase):
	"""The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgp-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4BgpType, self).__init__(parent, None)

	def delete(self):
		"""Delete this bgp-type instance on the server.

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgp-type instance on the server with any changed property values.

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeInterval(YangBase):
	"""BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hold-time-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalRandom(self))

	def delete(self):
		"""Delete this hold-time-interval instance on the server.

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hold-time-interval instance on the server with any changed property values.

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4HoldTimeIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveInterval(YangBase):
	"""Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalRandom(self))

	def delete(self):
		"""Delete this keep-alive-interval instance on the server.

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-interval instance on the server with any changed property values.

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4KeepAliveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'graceful-restart'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestart, self).__init__(parent, None)

	def long_lived_graceful_restart(self):
		"""Get the long-lived-graceful-restart object from the server

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestart`: An object encapsulating an instance of the long-lived-graceful-restart model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestart(self))

	def restart_time(self):
		"""Get the restart-time object from the server

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTime`: An object encapsulating an instance of the restart-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTime(self))

	def restart_delay(self):
		"""Get the restart-delay object from the server

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelay`: An object encapsulating an instance of the restart-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelay(self))

	def advertise_eor(self):
		"""Get the advertise-eor object from the server

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEor`: An object encapsulating an instance of the advertise-eor model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEor(self))

	def delete(self):
		"""Delete this graceful-restart instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this graceful-restart instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestart(YangBase):
	"""BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'long-lived-graceful-restart'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestart, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartRandom(self))

	def delete(self):
		"""Delete this long-lived-graceful-restart instance on the server.

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this long-lived-graceful-restart instance on the server with any changed property values.

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartLongLivedGracefulRestartRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTime(YangBase):
	"""BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'restart-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeRandom(self))

	def delete(self):
		"""Delete this restart-time instance on the server.

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this restart-time instance on the server with any changed property values.

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelay(YangBase):
	"""The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'restart-delay'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayRandom(self))

	def delete(self):
		"""Delete this restart-delay instance on the server.

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this restart-delay instance on the server with any changed property values.

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartRestartDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEor(YangBase):
	"""Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-eor'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEor, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorRandom(self))

	def delete(self):
		"""Delete this advertise-eor instance on the server.

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-eor instance on the server with any changed property values.

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4GracefulRestartAdvertiseEorRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Authentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationAuthenticationType(self))

	def md5_password(self):
		"""Get the md5-password object from the server

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationMd5Password`: An object encapsulating an instance of the md5-password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationMd5Password(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationMd5Password(YangBase):
	"""Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'md5-password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4AuthenticationMd5Password, self).__init__(parent, None)

	def delete(self):
		"""Delete this md5-password instance on the server.

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this md5-password instance on the server with any changed property values.

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ttl'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlRandom(self))

	def delete(self):
		"""Delete this ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4Address(YangBase):
	"""IPv4 address of the BGP peer for the session.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressRandom(self))

	def delete(self):
		"""Delete this dut-ipv4-address instance on the server.

		IPv4 address of the BGP peer for the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-ipv4-address instance on the server with any changed property values.

		IPv4 address of the BGP peer for the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv4DutIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv6'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6, self).__init__(parent, None)

	def as_number_2_byte(self):
		"""Get the as-number-2-byte object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte`: An object encapsulating an instance of the as-number-2-byte model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte(self))

	def create_as_number_2_byte(self):
		"""Create a child sibling as-number-2-byte instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte`: An object encapsulating an instance of the as-number-2-byte model.  

		Raises:  
			AlreadyExistsError: An instance of as-number-2-byte with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte(self), locals())

	def as_number_4_byte(self):
		"""Get the as-number-4-byte object from the server

		Enables the use of 4 Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte`: An object encapsulating an instance of the as-number-4-byte model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte(self))

	def create_as_number_4_byte(self):
		"""Create a child sibling as-number-4-byte instance on the server.

		Enables the use of 4 Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte`: An object encapsulating an instance of the as-number-4-byte model.  

		Raises:  
			AlreadyExistsError: An instance of as-number-4-byte with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte(self), locals())

	def as_number_set_mode(self):
		"""Get the as-number-set-mode object from the server

		Option to exclude, include or prepend the local AS number.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumberSetMode`: An object encapsulating an instance of the as-number-set-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumberSetMode(self))

	def bgp_type(self):
		"""Get the bgp-type object from the server

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6BgpType`: An object encapsulating an instance of the bgp-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6BgpType(self))

	def hold_time_interval(self):
		"""Get the hold-time-interval object from the server

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeInterval`: An object encapsulating an instance of the hold-time-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeInterval(self))

	def keep_alive_interval(self):
		"""Get the keep-alive-interval object from the server

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveInterval`: An object encapsulating an instance of the keep-alive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveInterval(self))

	def graceful_restart(self):
		"""Get the graceful-restart object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart`: An object encapsulating an instance of the graceful-restart model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart(self))

	def create_graceful_restart(self):
		"""Create a child sibling graceful-restart instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart`: An object encapsulating an instance of the graceful-restart model.  

		Raises:  
			AlreadyExistsError: An instance of graceful-restart with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart(self), locals())

	def authentication(self):
		"""Get the authentication object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication(self), locals())

	def ttl(self):
		"""Get the ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Ttl`: An object encapsulating an instance of the ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Ttl(self))

	def dut_ipv6_address(self):
		"""Get the dut-ipv6-address object from the server

		Ipv6 address of the BGP peer for the session.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6Address`: An object encapsulating an instance of the dut-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6Address(self))

	def delete(self):
		"""Delete this bgpv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgpv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-2-byte'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2Byte, self).__init__(parent, None)

	def as_number(self):
		"""Get the as-number object from the server

		2-Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumber`: An object encapsulating an instance of the as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumber(self))

	def dut_as_number(self):
		"""Get the dut-as-number object from the server

		2-Byte Autonomous system number configured for the DUT.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumber`: An object encapsulating an instance of the dut-as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumber(self))

	def delete(self):
		"""Delete this as-number-2-byte instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-2-byte instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumber(YangBase):
	"""2-Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberRandom(self))

	def delete(self):
		"""Delete this as-number instance on the server.

		2-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number instance on the server with any changed property values.

		2-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumber(YangBase):
	"""2-Byte Autonomous system number configured for the DUT.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberRandom(self))

	def delete(self):
		"""Delete this dut-as-number instance on the server.

		2-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-as-number instance on the server with any changed property values.

		2-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber2ByteDutAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte(YangBase):
	"""Enables the use of 4 Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-4-byte'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4Byte, self).__init__(parent, None)

	def as_number(self):
		"""Get the as-number object from the server

		4-Byte Autonomous system number

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumber`: An object encapsulating an instance of the as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumber(self))

	def dut_as_number(self):
		"""Get the dut-as-number object from the server

		4-Byte Autonomous system number configured for the DUT.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumber`: An object encapsulating an instance of the dut-as-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumber(self))

	def delete(self):
		"""Delete this as-number-4-byte instance on the server.

		Enables the use of 4 Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-4-byte instance on the server with any changed property values.

		Enables the use of 4 Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumber(YangBase):
	"""4-Byte Autonomous system number
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberRandom(self))

	def delete(self):
		"""Delete this as-number instance on the server.

		4-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number instance on the server with any changed property values.

		4-Byte Autonomous system number

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumber(YangBase):
	"""4-Byte Autonomous system number configured for the DUT.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-as-number'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberRandom(self))

	def delete(self):
		"""Delete this dut-as-number instance on the server.

		4-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-as-number instance on the server with any changed property values.

		4-Byte Autonomous system number configured for the DUT.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumber4ByteDutAsNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumberSetMode(YangBase):
	"""Option to exclude, include or prepend the local AS number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-number-set-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AsNumberSetMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this as-number-set-mode instance on the server.

		Option to exclude, include or prepend the local AS number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-number-set-mode instance on the server with any changed property values.

		Option to exclude, include or prepend the local AS number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6BgpType(YangBase):
	"""The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgp-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6BgpType, self).__init__(parent, None)

	def delete(self):
		"""Delete this bgp-type instance on the server.

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgp-type instance on the server with any changed property values.

		The type of BGP topology you create. Options include the following:    External BGP (EBGP)-used for BGP links between two or more Autonomous Systems (ASs)    Internal BGP (IBGP)-used within a single Autonomous System (AS)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeInterval(YangBase):
	"""BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hold-time-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalRandom(self))

	def delete(self):
		"""Delete this hold-time-interval instance on the server.

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hold-time-interval instance on the server with any changed property values.

		BGP Hold Time in seconds to use when negotiating with peers. If the router does not receive KEEPALIVE or UPDATE or NOTIFICATION messages within the Hold Time field of the OPEN message, the BGP connection to the peer will be closed.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6HoldTimeIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveInterval(YangBase):
	"""Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalRandom(self))

	def delete(self):
		"""Delete this keep-alive-interval instance on the server.

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-interval instance on the server with any changed property values.

		Number of seconds between transmissions of Keep Alive messages by the emulated router (in the absence of the sending of any other BGP packets) to the DUT. The standard keep-alive transmit time is every 30 seconds, or one-third of the Hold Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6KeepAliveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'graceful-restart'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestart, self).__init__(parent, None)

	def long_lived_graceful_restart(self):
		"""Get the long-lived-graceful-restart object from the server

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestart`: An object encapsulating an instance of the long-lived-graceful-restart model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestart(self))

	def restart_time(self):
		"""Get the restart-time object from the server

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTime`: An object encapsulating an instance of the restart-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTime(self))

	def restart_delay(self):
		"""Get the restart-delay object from the server

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelay`: An object encapsulating an instance of the restart-delay model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelay(self))

	def advertise_eor(self):
		"""Get the advertise-eor object from the server

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEor`: An object encapsulating an instance of the advertise-eor model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEor(self))

	def delete(self):
		"""Delete this graceful-restart instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this graceful-restart instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestart(YangBase):
	"""BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'long-lived-graceful-restart'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestart, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartRandom(self))

	def delete(self):
		"""Delete this long-lived-graceful-restart instance on the server.

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this long-lived-graceful-restart instance on the server with any changed property values.

		BGP long lived graceful restart allows a network operator to choose to maintain stale routing information from a failed BGP peer much longer than the existing BGP graceful restart facility.    TRUE  : Enable long lived graceful restart    FALSE : Disable long lived graceful restart

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartLongLivedGracefulRestartRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTime(YangBase):
	"""BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'restart-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeRandom(self))

	def delete(self):
		"""Delete this restart-time instance on the server.

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this restart-time instance on the server with any changed property values.

		BGP graceful restart time. The amount of time (in seconds) that the emulated router will wait for its peer to re-establish the session. If the session is not re-established within this time frame, the stale routes will be removed from the route database.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelay(YangBase):
	"""The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'restart-delay'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelay, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayRandom(self))

	def delete(self):
		"""Delete this restart-delay instance on the server.

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this restart-delay instance on the server with any changed property values.

		The amount of time to wait before initiating a new BGP session. Traffic generator will not initiate a new session until this timer expires. If the DUT initiates a new session, Spirent TestCenter will process it and establish the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartRestartDelayRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEor(YangBase):
	"""Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-eor'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEor, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorRandom(self))

	def delete(self):
		"""Delete this advertise-eor instance on the server.

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-eor instance on the server with any changed property values.

		Advertise End-of-RIB    TRUE  : Send end-of-RIB marker in the update packet    FALSE : Do not send end-of-RIB marker in the update packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6GracefulRestartAdvertiseEorRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Authentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationAuthenticationType(self))

	def md5_password(self):
		"""Get the md5-password object from the server

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationMd5Password`: An object encapsulating an instance of the md5-password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationMd5Password(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationMd5Password(YangBase):
	"""Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'md5-password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6AuthenticationMd5Password, self).__init__(parent, None)

	def delete(self):
		"""Delete this md5-password instance on the server.

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this md5-password instance on the server with any changed property values.

		Type a value to be used as a secret MD5 Key for authentication. This field is available only if you select MD5 as the type of Authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ttl'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlRandom(self))

	def delete(self):
		"""Delete this ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6Address(YangBase):
	"""Ipv6 address of the BGP peer for the session.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dut-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressRandom(self))

	def delete(self):
		"""Delete this dut-ipv6-address instance on the server.

		Ipv6 address of the BGP peer for the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dut-ipv6-address instance on the server with any changed property values.

		Ipv6 address of the BGP peer for the session.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBgpv6DutIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2, self).__init__(parent, None)

	def router_id(self):
		"""Get the router-id object from the server

		Router ID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterId`: An object encapsulating an instance of the router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterId(self))

	def area_id(self):
		"""Get the area-id object from the server

		IP address indicating the area to which the emulated router belongs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaId`: An object encapsulating an instance of the area-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaId(self))

	def network_type(self):
		"""Get the network-type object from the server

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2NetworkType`: An object encapsulating an instance of the network-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2NetworkType(self))

	def router_priority(self):
		"""Get the router-priority object from the server

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriority`: An object encapsulating an instance of the router-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriority(self))

	def interface_cost(self):
		"""Get the interface-cost object from the server

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCost`: An object encapsulating an instance of the interface-cost model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCost(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloInterval(self))

	def router_dead_interval(self):
		"""Get the router-dead-interval object from the server

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadInterval`: An object encapsulating an instance of the router-dead-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadInterval(self))

	def retransmit_interval(self):
		"""Get the retransmit-interval object from the server

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitInterval`: An object encapsulating an instance of the retransmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitInterval(self))

	def authentication(self):
		"""Get the authentication object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication(self), locals())

	def delete(self):
		"""Delete this ospfv2 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterId(YangBase):
	"""Router ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdRandom(self))

	def delete(self):
		"""Delete this router-id instance on the server.

		Router ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-id instance on the server with any changed property values.

		Router ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaId(YangBase):
	"""IP address indicating the area to which the emulated router belongs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'area-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdRandom(self))

	def delete(self):
		"""Delete this area-id instance on the server.

		IP address indicating the area to which the emulated router belongs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this area-id instance on the server with any changed property values.

		IP address indicating the area to which the emulated router belongs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AreaIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2NetworkType(YangBase):
	"""This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'network-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2NetworkType, self).__init__(parent, None)

	def delete(self):
		"""Delete this network-type instance on the server.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this network-type instance on the server with any changed property values.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriority(YangBase):
	"""Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityRandom(self))

	def delete(self):
		"""Delete this router-priority instance on the server.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-priority instance on the server with any changed property values.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCost(YangBase):
	"""Cost of the interface connecting the emulated router to the neighbor DUT router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'interface-cost'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCost, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostRandom(self))

	def delete(self):
		"""Delete this interface-cost instance on the server.

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this interface-cost instance on the server with any changed property values.

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2InterfaceCostRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloInterval(YangBase):
	"""Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2HelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadInterval(YangBase):
	"""Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-dead-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalRandom(self))

	def delete(self):
		"""Delete this router-dead-interval instance on the server.

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-dead-interval instance on the server with any changed property values.

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RouterDeadIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitInterval(YangBase):
	"""If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retransmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalRandom(self))

	def delete(self):
		"""Delete this retransmit-interval instance on the server.

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retransmit-interval instance on the server with any changed property values.

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2RetransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2Authentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationAuthenticationType(self))

	def password(self):
		"""Get the password object from the server

		Authentication password.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationPassword`: An object encapsulating an instance of the password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationPassword(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationPassword(YangBase):
	"""Authentication password.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv2AuthenticationPassword, self).__init__(parent, None)

	def delete(self):
		"""Delete this password instance on the server.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this password instance on the server with any changed property values.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3, self).__init__(parent, None)

	def router_id(self):
		"""Get the router-id object from the server

		Router ID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterId`: An object encapsulating an instance of the router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterId(self))

	def area_id(self):
		"""Get the area-id object from the server

		IP address indicating the area to which the emulated router belongs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaId`: An object encapsulating an instance of the area-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaId(self))

	def network_type(self):
		"""Get the network-type object from the server

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3NetworkType`: An object encapsulating an instance of the network-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3NetworkType(self))

	def router_priority(self):
		"""Get the router-priority object from the server

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriority`: An object encapsulating an instance of the router-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriority(self))

	def interface_cost(self):
		"""Get the interface-cost object from the server

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCost`: An object encapsulating an instance of the interface-cost model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCost(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloInterval(self))

	def router_dead_interval(self):
		"""Get the router-dead-interval object from the server

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadInterval`: An object encapsulating an instance of the router-dead-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadInterval(self))

	def retransmit_interval(self):
		"""Get the retransmit-interval object from the server

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitInterval`: An object encapsulating an instance of the retransmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitInterval(self))

	def delete(self):
		"""Delete this ospfv3 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterId(YangBase):
	"""Router ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdRandom(self))

	def delete(self):
		"""Delete this router-id instance on the server.

		Router ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-id instance on the server with any changed property values.

		Router ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaId(YangBase):
	"""IP address indicating the area to which the emulated router belongs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'area-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdRandom(self))

	def delete(self):
		"""Delete this area-id instance on the server.

		IP address indicating the area to which the emulated router belongs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this area-id instance on the server with any changed property values.

		IP address indicating the area to which the emulated router belongs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3AreaIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3NetworkType(YangBase):
	"""This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'network-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3NetworkType, self).__init__(parent, None)

	def delete(self):
		"""Delete this network-type instance on the server.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this network-type instance on the server with any changed property values.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            NATIVE    : use the adjacency implied by the port-type            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriority(YangBase):
	"""Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityRandom(self))

	def delete(self):
		"""Delete this router-priority instance on the server.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-priority instance on the server with any changed property values.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCost(YangBase):
	"""Cost of the interface connecting the emulated router to the neighbor DUT router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'interface-cost'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCost, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostRandom(self))

	def delete(self):
		"""Delete this interface-cost instance on the server.

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this interface-cost instance on the server with any changed property values.

		Cost of the interface connecting the emulated router to the neighbor DUT router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3InterfaceCostRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloInterval(YangBase):
	"""Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3HelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadInterval(YangBase):
	"""Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-dead-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalRandom(self))

	def delete(self):
		"""Delete this router-dead-interval instance on the server.

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-dead-interval instance on the server with any changed property values.

		Time interval (in seconds) that the emulated router waits to receive packets            from the neighbor DUT router before deleting that neighbor from its topology

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RouterDeadIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitInterval(YangBase):
	"""If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'retransmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalRandom(self))

	def delete(self):
		"""Delete this retransmit-interval instance on the server.

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this retransmit-interval instance on the server with any changed property values.

		If an LSA transmission fails, Spirent TestCenter waits the duration of the            retransmit interval (in seconds) before re-transmitting the packet

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsOspfv3RetransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsis(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'isis'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsis, self).__init__(parent, None)

	def system_id(self):
		"""Get the system-id object from the server

		The System ID is used to identify an emulated router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemId`: An object encapsulating an instance of the system-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemId(self))

	def ip_version(self):
		"""Get the ip-version object from the server

		IP Version            IPV4 : IPv4            IPV6 : IPv6            BOTH : IPv4 and IPv6

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisIpVersion`: An object encapsulating an instance of the ip-version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisIpVersion(self))

	def network_type(self):
		"""Get the network-type object from the server

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisNetworkType`: An object encapsulating an instance of the network-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisNetworkType(self))

	def router_priority(self):
		"""Get the router-priority object from the server

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriority`: An object encapsulating an instance of the router-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriority(self))

	def level(self):
		"""Get the level object from the server

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisLevel`: An object encapsulating an instance of the level model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisLevel(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloInterval(self))

	def authentication(self):
		"""Get the authentication object from the server

		Type of authentication to be used    NONE   : no authentication    SIMPLE : The packet is authenticated by the receiving router if the password             matches the authentication key that is included in the packet.             This method provides little security because the authentication             key can be learned by capturing packets.    MD5    : The packet contains a cryptographic checksum, but not the authentication            key itself. The receiving router performs a calculation based on the            MD5 algorithm and an authentication key ID. The packet is authenticated            if the calculated checksum matches. This method provides a stronger            assurance that routing data originated from a router with a valid            authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		Type of authentication to be used    NONE   : no authentication    SIMPLE : The packet is authenticated by the receiving router if the password             matches the authentication key that is included in the packet.             This method provides little security because the authentication             key can be learned by capturing packets.    MD5    : The packet contains a cryptographic checksum, but not the authentication            key itself. The receiving router performs a calculation based on the            MD5 algorithm and an authentication key ID. The packet is authenticated            if the calculated checksum matches. This method provides a stronger            assurance that routing data originated from a router with a valid            authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication(self), locals())

	def delete(self):
		"""Delete this isis instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this isis instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemId(YangBase):
	"""The System ID is used to identify an emulated router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'system-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdRandom(self))

	def delete(self):
		"""Delete this system-id instance on the server.

		The System ID is used to identify an emulated router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this system-id instance on the server with any changed property values.

		The System ID is used to identify an emulated router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisSystemIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisIpVersion(YangBase):
	"""IP Version            IPV4 : IPv4            IPV6 : IPv6            BOTH : IPv4 and IPv6
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ip-version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisIpVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this ip-version instance on the server.

		IP Version            IPV4 : IPv4            IPV6 : IPv6            BOTH : IPv4 and IPv6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ip-version instance on the server with any changed property values.

		IP Version            IPV4 : IPv4            IPV6 : IPv6            BOTH : IPv4 and IPv6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisNetworkType(YangBase):
	"""This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'network-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisNetworkType, self).__init__(parent, None)

	def delete(self):
		"""Delete this network-type instance on the server.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this network-type instance on the server with any changed property values.

		This setting to override the physical link type to emulate a broadcast            adjacency over POS, or a point-to-point adjacency over Ethernet            BROADCAST : Broadcast adjacency            P2P       : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriority(YangBase):
	"""Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityRandom(self))

	def delete(self):
		"""Delete this router-priority instance on the server.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-priority instance on the server with any changed property values.

		Router priority of the emulated router. Set the router priority to a higher            or lower value to influence the DR and BDR selection process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisRouterPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisLevel(YangBase):
	"""This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'level'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisLevel, self).__init__(parent, None)

	def delete(self):
		"""Delete this level instance on the server.

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this level instance on the server with any changed property values.

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloInterval(YangBase):
	"""Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Time interval (in seconds) used by the emulated routers to pace Hello packet            transmissions.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisHelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication(YangBase):
	"""Type of authentication to be used    NONE   : no authentication    SIMPLE : The packet is authenticated by the receiving router if the password             matches the authentication key that is included in the packet.             This method provides little security because the authentication             key can be learned by capturing packets.    MD5    : The packet contains a cryptographic checksum, but not the authentication            key itself. The receiving router performs a calculation based on the            MD5 algorithm and an authentication key ID. The packet is authenticated            if the calculated checksum matches. This method provides a stronger            assurance that routing data originated from a router with a valid            authentication key.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationAuthenticationType(self))

	def password(self):
		"""Get the password object from the server

		Authentication password.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationPassword`: An object encapsulating an instance of the password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationPassword(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		Type of authentication to be used    NONE   : no authentication    SIMPLE : The packet is authenticated by the receiving router if the password             matches the authentication key that is included in the packet.             This method provides little security because the authentication             key can be learned by capturing packets.    MD5    : The packet contains a cryptographic checksum, but not the authentication            key itself. The receiving router performs a calculation based on the            MD5 algorithm and an authentication key ID. The packet is authenticated            if the calculated checksum matches. This method provides a stronger            assurance that routing data originated from a router with a valid            authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		Type of authentication to be used    NONE   : no authentication    SIMPLE : The packet is authenticated by the receiving router if the password             matches the authentication key that is included in the packet.             This method provides little security because the authentication             key can be learned by capturing packets.    MD5    : The packet contains a cryptographic checksum, but not the authentication            key itself. The receiving router performs a calculation based on the            MD5 algorithm and an authentication key ID. The packet is authenticated            if the calculated checksum matches. This method provides a stronger            assurance that routing data originated from a router with a valid            authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationPassword(YangBase):
	"""Authentication password.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIsisAuthenticationPassword, self).__init__(parent, None)

	def delete(self):
		"""Delete this password instance on the server.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this password instance on the server with any changed property values.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bfdv4'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4, self).__init__(parent, None)

	def transmit_interval(self):
		"""Get the transmit-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitInterval`: An object encapsulating an instance of the transmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitInterval(self))

	def receive_interval(self):
		"""Get the receive-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveInterval`: An object encapsulating an instance of the receive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveInterval(self))

	def echo_receive_interval(self):
		"""Get the echo-receive-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveInterval`: An object encapsulating an instance of the echo-receive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveInterval(self))

	def authentication(self):
		"""Get the authentication object from the server

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication(self), locals())

	def delete(self):
		"""Delete this bfdv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bfdv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalRandom(self))

	def delete(self):
		"""Delete this transmit-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4TransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalRandom(self))

	def delete(self):
		"""Delete this receive-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4ReceiveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-receive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalRandom(self))

	def delete(self):
		"""Delete this echo-receive-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-receive-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4EchoReceiveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication(YangBase):
	"""Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4Authentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationAuthenticationType(self))

	def password(self):
		"""Get the password object from the server

		Authentication password.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationPassword`: An object encapsulating an instance of the password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationPassword(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationPassword(YangBase):
	"""Authentication password.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv4AuthenticationPassword, self).__init__(parent, None)

	def delete(self):
		"""Delete this password instance on the server.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this password instance on the server with any changed property values.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bfdv6'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6, self).__init__(parent, None)

	def transmit_interval(self):
		"""Get the transmit-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitInterval`: An object encapsulating an instance of the transmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitInterval(self))

	def receive_interval(self):
		"""Get the receive-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveInterval`: An object encapsulating an instance of the receive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveInterval(self))

	def echo_receive_interval(self):
		"""Get the echo-receive-interval object from the server

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveInterval`: An object encapsulating an instance of the echo-receive-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveInterval(self))

	def authentication(self):
		"""Get the authentication object from the server

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication(self))

	def create_authentication(self):
		"""Create a child sibling authentication instance on the server.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication`: An object encapsulating an instance of the authentication model.  

		Raises:  
			AlreadyExistsError: An instance of authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication(self), locals())

	def delete(self):
		"""Delete this bfdv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bfdv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalRandom(self))

	def delete(self):
		"""Delete this transmit-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between transmitted BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6TransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalRandom(self))

	def delete(self):
		"""Delete this receive-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Control packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6ReceiveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveInterval(YangBase):
	"""Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-receive-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalRandom(self))

	def delete(self):
		"""Delete this echo-receive-interval instance on the server.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-receive-interval instance on the server with any changed property values.

		Minimum interval, in milliseconds, that the emulated router desires between received BFD Echo packets.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6EchoReceiveIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication(YangBase):
	"""Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6Authentication, self).__init__(parent, None)

	def authentication_type(self):
		"""Get the authentication-type object from the server

		Type of authentication to be used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationAuthenticationType`: An object encapsulating an instance of the authentication-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationAuthenticationType(self))

	def password(self):
		"""Get the password object from the server

		Authentication password.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationPassword`: An object encapsulating an instance of the password model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationPassword(self))

	def delete(self):
		"""Delete this authentication instance on the server.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication instance on the server with any changed property values.

		Type of authentication to be used NONE   : no authentication SIMPLE : The packet is authenticated by the receiving router if the password matches the authentication key that is included in the packet. This method provides little security because the authentication key can be learned by capturing packets. MD5    : The packet contains a cryptographic checksum, but not the authentication key itself. The receiving router performs a calculation based on the MD5 algorithm and an authentication key ID. The packet is authenticated if the calculated checksum matches. This method provides a stronger assurance that routing data originated from a router with a valid authentication key.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationAuthenticationType(YangBase):
	"""Type of authentication to be used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationAuthenticationType, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-type instance on the server.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-type instance on the server with any changed property values.

		Type of authentication to be used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationPassword(YangBase):
	"""Authentication password.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'password'
		super(SessionsConfigDeviceGroupsDevicesProtocolsBfdv6AuthenticationPassword, self).__init__(parent, None)

	def delete(self):
		"""Delete this password instance on the server.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this password instance on the server with any changed property values.

		Authentication password.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4, self).__init__(parent, None)

	def default_host_address_prefix_length(self):
		"""Get the default-host-address-prefix-length object from the server

		Subnet mask value to use for when the server doesn't include option 1 (Subnet Mask) in the DHCPOFFER message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLength`: An object encapsulating an instance of the default-host-address-prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLength(self))

	def enable_arp_server_id(self):
		"""Get the enable-arp-server-id object from the server

		To send an ARP to the Server ID returned during binding. The MAC address obtained in this process is used as the destination address for Renew Request and Release messages. If no ARP reply is received, the configured destination MAC is used.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerId`: An object encapsulating an instance of the enable-arp-server-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerId(self))

	def enable_auto_retry(self):
		"""Get the enable-auto-retry object from the server

		Auto retry will retry sessions that fail to initially come up    NONE : Do not retry failed sessions    BLOCK_AUTO_RETRY : Retry DHCP sessions after all sessions in the same device block reach an IDLE/CONNECTED state    SESSION_AUTO_RETRY : Retry a DHCP session after it fails without waiting for the other sessions in the device block

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableAutoRetry`: An object encapsulating an instance of the enable-auto-retry model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableAutoRetry(self))

	def auto_retry_attempts(self):
		"""Get the auto-retry-attempts object from the server

		Number of retry attempts.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttempts`: An object encapsulating an instance of the auto-retry-attempts model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttempts(self))

	def enable_broadcast_flag(self):
		"""Get the enable-broadcast-flag object from the server

		Enable broadcast bit to signal the DHCP server and BOOTP relay agent to broadcast any messages to the client on the client's subnet.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlag`: An object encapsulating an instance of the enable-broadcast-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlag(self))

	def enable_client_mac_address_dataplane(self):
		"""Get the enable-client-mac-address-dataplane object from the server

		Use client MAC address for the data plane.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplane`: An object encapsulating an instance of the enable-client-mac-address-dataplane model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplane(self))

	def enable_router_option(self):
		"""Get the enable-router-option object from the server

		Enable the router option (option 3) specified in RFC 2132.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOption`: An object encapsulating an instance of the enable-router-option model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOption(self))

	def session_host_name(self):
		"""Get the session-host-name object from the server

		Unique hostname of emulated client.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4SessionHostName`: An object encapsulating an instance of the session-host-name model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4SessionHostName(self))

	def tos(self):
		"""Get the tos object from the server

		Provides an indication of the quality of service wanted.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Tos`: An object encapsulating an instance of the tos model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Tos(self))

	def option_list(self):
		"""Get the option-list object from the server

		A space-separated list of Option 55 numbers for the DHCP request messages on each session block. This attribute can have one or more of the following values:    1 - Subnet Mask Option.    3 - Router Option.    6 - Domain Name Servers Option.    15 - Domain Name Option.    33 - Static Routes Option.    44 - NetBIOS Name Servers Option.    46 - NetBIOS Node Type Option.    47 - NetBIOS Scope Option.    51 - IP Address Lease Time Option.    54 - Server Identifier Option.    58 - Renewal Time (T1) Option.    59 - Rebinding Time (T2) Option.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionList`: An object encapsulating an instance of the option-list model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionList(self))

	def dna_v4_destination_ip_address(self):
		"""Get the dna-v4-destination-ip-address object from the server

		Destination IP address for testing reachability per RFC 4436.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddress`: An object encapsulating an instance of the dna-v4-destination-ip-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddress(self))

	def dna_v4_destination_mac_address(self):
		"""Get the dna-v4-destination-mac-address object from the server

		Destination MAC address for testing reachability per RFC 4436.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddress`: An object encapsulating an instance of the dna-v4-destination-mac-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddress(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv4 custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv4 custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions(self, name), locals())

	def relay_agent(self):
		"""Get the relay-agent object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgent`: An object encapsulating an instance of the relay-agent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgent(self))

	def delete(self):
		"""Delete this dhcpv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLength(YangBase):
	"""Subnet mask value to use for when the server doesn't include option 1 (Subnet Mask) in the DHCPOFFER message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'default-host-address-prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthRandom(self))

	def delete(self):
		"""Delete this default-host-address-prefix-length instance on the server.

		Subnet mask value to use for when the server doesn't include option 1 (Subnet Mask) in the DHCPOFFER message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this default-host-address-prefix-length instance on the server with any changed property values.

		Subnet mask value to use for when the server doesn't include option 1 (Subnet Mask) in the DHCPOFFER message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DefaultHostAddressPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerId(YangBase):
	"""To send an ARP to the Server ID returned during binding. The MAC address obtained in this process is used as the destination address for Renew Request and Release messages. If no ARP reply is received, the configured destination MAC is used.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-arp-server-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdRandom(self))

	def delete(self):
		"""Delete this enable-arp-server-id instance on the server.

		To send an ARP to the Server ID returned during binding. The MAC address obtained in this process is used as the destination address for Renew Request and Release messages. If no ARP reply is received, the configured destination MAC is used.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-arp-server-id instance on the server with any changed property values.

		To send an ARP to the Server ID returned during binding. The MAC address obtained in this process is used as the destination address for Renew Request and Release messages. If no ARP reply is received, the configured destination MAC is used.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableArpServerIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableAutoRetry(YangBase):
	"""Auto retry will retry sessions that fail to initially come up    NONE : Do not retry failed sessions    BLOCK_AUTO_RETRY : Retry DHCP sessions after all sessions in the same device block reach an IDLE/CONNECTED state    SESSION_AUTO_RETRY : Retry a DHCP session after it fails without waiting for the other sessions in the device block
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-auto-retry'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableAutoRetry, self).__init__(parent, None)

	def delete(self):
		"""Delete this enable-auto-retry instance on the server.

		Auto retry will retry sessions that fail to initially come up    NONE : Do not retry failed sessions    BLOCK_AUTO_RETRY : Retry DHCP sessions after all sessions in the same device block reach an IDLE/CONNECTED state    SESSION_AUTO_RETRY : Retry a DHCP session after it fails without waiting for the other sessions in the device block

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-auto-retry instance on the server with any changed property values.

		Auto retry will retry sessions that fail to initially come up    NONE : Do not retry failed sessions    BLOCK_AUTO_RETRY : Retry DHCP sessions after all sessions in the same device block reach an IDLE/CONNECTED state    SESSION_AUTO_RETRY : Retry a DHCP session after it fails without waiting for the other sessions in the device block

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttempts(YangBase):
	"""Number of retry attempts.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auto-retry-attempts'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttempts, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsRandom(self))

	def delete(self):
		"""Delete this auto-retry-attempts instance on the server.

		Number of retry attempts.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auto-retry-attempts instance on the server with any changed property values.

		Number of retry attempts.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4AutoRetryAttemptsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlag(YangBase):
	"""Enable broadcast bit to signal the DHCP server and BOOTP relay agent to broadcast any messages to the client on the client's subnet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-broadcast-flag'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlag, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagRandom(self))

	def delete(self):
		"""Delete this enable-broadcast-flag instance on the server.

		Enable broadcast bit to signal the DHCP server and BOOTP relay agent to broadcast any messages to the client on the client's subnet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-broadcast-flag instance on the server with any changed property values.

		Enable broadcast bit to signal the DHCP server and BOOTP relay agent to broadcast any messages to the client on the client's subnet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableBroadcastFlagRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplane(YangBase):
	"""Use client MAC address for the data plane.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-client-mac-address-dataplane'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplane, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneRandom(self))

	def delete(self):
		"""Delete this enable-client-mac-address-dataplane instance on the server.

		Use client MAC address for the data plane.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-client-mac-address-dataplane instance on the server with any changed property values.

		Use client MAC address for the data plane.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableClientMacAddressDataplaneRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOption(YangBase):
	"""Enable the router option (option 3) specified in RFC 2132.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-router-option'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOption, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionRandom(self))

	def delete(self):
		"""Delete this enable-router-option instance on the server.

		Enable the router option (option 3) specified in RFC 2132.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-router-option instance on the server with any changed property values.

		Enable the router option (option 3) specified in RFC 2132.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4EnableRouterOptionRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4SessionHostName(YangBase):
	"""Unique hostname of emulated client.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'session-host-name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4SessionHostName, self).__init__(parent, None)

	def delete(self):
		"""Delete this session-host-name instance on the server.

		Unique hostname of emulated client.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this session-host-name instance on the server with any changed property values.

		Unique hostname of emulated client.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Tos(YangBase):
	"""Provides an indication of the quality of service wanted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'tos'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Tos, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosRandom(self))

	def delete(self):
		"""Delete this tos instance on the server.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this tos instance on the server with any changed property values.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4TosRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionList(YangBase):
	"""A space-separated list of Option 55 numbers for the DHCP request messages on each session block. This attribute can have one or more of the following values:    1 - Subnet Mask Option.    3 - Router Option.    6 - Domain Name Servers Option.    15 - Domain Name Option.    33 - Static Routes Option.    44 - NetBIOS Name Servers Option.    46 - NetBIOS Node Type Option.    47 - NetBIOS Scope Option.    51 - IP Address Lease Time Option.    54 - Server Identifier Option.    58 - Renewal Time (T1) Option.    59 - Rebinding Time (T2) Option.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-list'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionList, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListRandom(self))

	def delete(self):
		"""Delete this option-list instance on the server.

		A space-separated list of Option 55 numbers for the DHCP request messages on each session block. This attribute can have one or more of the following values:    1 - Subnet Mask Option.    3 - Router Option.    6 - Domain Name Servers Option.    15 - Domain Name Option.    33 - Static Routes Option.    44 - NetBIOS Name Servers Option.    46 - NetBIOS Node Type Option.    47 - NetBIOS Scope Option.    51 - IP Address Lease Time Option.    54 - Server Identifier Option.    58 - Renewal Time (T1) Option.    59 - Rebinding Time (T2) Option.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-list instance on the server with any changed property values.

		A space-separated list of Option 55 numbers for the DHCP request messages on each session block. This attribute can have one or more of the following values:    1 - Subnet Mask Option.    3 - Router Option.    6 - Domain Name Servers Option.    15 - Domain Name Option.    33 - Static Routes Option.    44 - NetBIOS Name Servers Option.    46 - NetBIOS Node Type Option.    47 - NetBIOS Scope Option.    51 - IP Address Lease Time Option.    54 - Server Identifier Option.    58 - Renewal Time (T1) Option.    59 - Rebinding Time (T2) Option.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4OptionListRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddress(YangBase):
	"""Destination IP address for testing reachability per RFC 4436.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dna-v4-destination-ip-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressRandom(self))

	def delete(self):
		"""Delete this dna-v4-destination-ip-address instance on the server.

		Destination IP address for testing reachability per RFC 4436.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dna-v4-destination-ip-address instance on the server with any changed property values.

		Destination IP address for testing reachability per RFC 4436.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationIpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddress(YangBase):
	"""Destination MAC address for testing reachability per RFC 4436.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dna-v4-destination-mac-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressRandom(self))

	def delete(self):
		"""Delete this dna-v4-destination-mac-address instance on the server.

		Destination MAC address for testing reachability per RFC 4436.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dna-v4-destination-mac-address instance on the server with any changed property values.

		Destination MAC address for testing reachability per RFC 4436.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4DnaV4DestinationMacAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions(YangBase):
	"""DHCPv4 custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in DISCOVER, REQUEST, or BOTH.    DISCOVER : Include the message option in the DISCOVER message.    REQUEST  : Include the message option in the REQUEST message.    BOTH     : Include the message option in both DISCOVER and REQUEST messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv4 custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv4 custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv4 custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsMessageType(YangBase):
	"""Includes the message option in DISCOVER, REQUEST, or BOTH.    DISCOVER : Include the message option in the DISCOVER message.    REQUEST  : Include the message option in the REQUEST message.    BOTH     : Include the message option in both DISCOVER and REQUEST messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in DISCOVER, REQUEST, or BOTH.    DISCOVER : Include the message option in the DISCOVER message.    REQUEST  : Include the message option in the REQUEST message.    BOTH     : Include the message option in both DISCOVER and REQUEST messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in DISCOVER, REQUEST, or BOTH.    DISCOVER : Include the message option in the DISCOVER message.    REQUEST  : Include the message option in the REQUEST message.    BOTH     : Include the message option in both DISCOVER and REQUEST messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4CustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgent(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgent, self).__init__(parent, None)

	def relay_agent_circuit_id(self):
		"""Get the relay-agent-circuit-id object from the server

		Relay Agent Circuit Identifier.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitId`: An object encapsulating an instance of the relay-agent-circuit-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitId(self))

	def relay_agent_link_selection(self):
		"""Get the relay-agent-link-selection object from the server

		Relay agent link selection sub-option in DHCP messages sent from emulated relay agent. The link selection sub-option is described in RFC 3527.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelection`: An object encapsulating an instance of the relay-agent-link-selection model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelection(self))

	def relay_agent_server_id_override(self):
		"""Get the relay-agent-server-id-override object from the server

		Relay agent server identifier override sub-option in DHCP messages sent from emulated relay agent. The server identifier override sub-option is described in RFC 5107.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverride`: An object encapsulating an instance of the relay-agent-server-id-override model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverride(self))

	def relay_agent_vpn_id(self):
		"""Get the relay-agent-vpn-id object from the server

		Relay agent virtual subnet selection sub-option in DHCP messages sent from emulated relay agent. The virtual subnet selection sub-option is described in RFC 6607.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnId`: An object encapsulating an instance of the relay-agent-vpn-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnId(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Remote ID sub-option in the DHCP messages that are sent from the emulated relay agent. The remote ID sub-option is described in RFC 3046.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteId(self))

	def relay_agent_ipv4_address(self):
		"""Get the relay-agent-ipv4-address object from the server

		Source IP address of the relay agent message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4Address`: An object encapsulating an instance of the relay-agent-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4Address(self))

	def relay_agent_mac_address(self):
		"""Get the relay-agent-mac-address object from the server

		MAC address of the relay agent message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddress`: An object encapsulating an instance of the relay-agent-mac-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddress(self))

	def relay_agent_pool_ipv4_address(self):
		"""Get the relay-agent-pool-ipv4-address object from the server

		IP address of the interface on the relay agent that is connectedto the DHCP hosts.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4Address`: An object encapsulating an instance of the relay-agent-pool-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4Address(self))

	def relay_agent_server_ipv4_address(self):
		"""Get the relay-agent-server-ipv4-address object from the server

		Destination IP address for the relay agent message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4Address`: An object encapsulating an instance of the relay-agent-server-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4Address(self))

	def delete(self):
		"""Delete this relay-agent instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitId(YangBase):
	"""Relay Agent Circuit Identifier.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-circuit-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-circuit-id instance on the server.

		Relay Agent Circuit Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-circuit-id instance on the server with any changed property values.

		Relay Agent Circuit Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentCircuitIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelection(YangBase):
	"""Relay agent link selection sub-option in DHCP messages sent from emulated relay agent. The link selection sub-option is described in RFC 3527.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-link-selection'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelection, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionRandom(self))

	def delete(self):
		"""Delete this relay-agent-link-selection instance on the server.

		Relay agent link selection sub-option in DHCP messages sent from emulated relay agent. The link selection sub-option is described in RFC 3527.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-link-selection instance on the server with any changed property values.

		Relay agent link selection sub-option in DHCP messages sent from emulated relay agent. The link selection sub-option is described in RFC 3527.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentLinkSelectionRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverride(YangBase):
	"""Relay agent server identifier override sub-option in DHCP messages sent from emulated relay agent. The server identifier override sub-option is described in RFC 5107.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-server-id-override'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverride, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideRandom(self))

	def delete(self):
		"""Delete this relay-agent-server-id-override instance on the server.

		Relay agent server identifier override sub-option in DHCP messages sent from emulated relay agent. The server identifier override sub-option is described in RFC 5107.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-server-id-override instance on the server with any changed property values.

		Relay agent server identifier override sub-option in DHCP messages sent from emulated relay agent. The server identifier override sub-option is described in RFC 5107.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIdOverrideRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnId(YangBase):
	"""Relay agent virtual subnet selection sub-option in DHCP messages sent from emulated relay agent. The virtual subnet selection sub-option is described in RFC 6607.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-vpn-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnId, self).__init__(parent, None)

	def vpn_id(self):
		"""Get the vpn-id object from the server

		VPN ID

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnId`: An object encapsulating an instance of the vpn-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnId(self))

	def vpn_id_type(self):
		"""Get the vpn-id-type object from the server

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnIdType`: An object encapsulating an instance of the vpn-id-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnIdType(self))

	def delete(self):
		"""Delete this relay-agent-vpn-id instance on the server.

		Relay agent virtual subnet selection sub-option in DHCP messages sent from emulated relay agent. The virtual subnet selection sub-option is described in RFC 6607.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-vpn-id instance on the server with any changed property values.

		Relay agent virtual subnet selection sub-option in DHCP messages sent from emulated relay agent. The virtual subnet selection sub-option is described in RFC 6607.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnId(YangBase):
	"""VPN ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vpn-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnId, self).__init__(parent, None)

	def delete(self):
		"""Delete this vpn-id instance on the server.

		VPN ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vpn-id instance on the server with any changed property values.

		VPN ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnIdType(YangBase):
	"""VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vpn-id-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentVpnIdVpnIdType, self).__init__(parent, None)

	def delete(self):
		"""Delete this vpn-id-type instance on the server.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vpn-id-type instance on the server with any changed property values.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteId(YangBase):
	"""Remote ID sub-option in the DHCP messages that are sent from the emulated relay agent. The remote ID sub-option is described in RFC 3046.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Remote ID sub-option in the DHCP messages that are sent from the emulated relay agent. The remote ID sub-option is described in RFC 3046.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Remote ID sub-option in the DHCP messages that are sent from the emulated relay agent. The remote ID sub-option is described in RFC 3046.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4Address(YangBase):
	"""Source IP address of the relay agent message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressRandom(self))

	def delete(self):
		"""Delete this relay-agent-ipv4-address instance on the server.

		Source IP address of the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-ipv4-address instance on the server with any changed property values.

		Source IP address of the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddress(YangBase):
	"""MAC address of the relay agent message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-mac-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressRandom(self))

	def delete(self):
		"""Delete this relay-agent-mac-address instance on the server.

		MAC address of the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-mac-address instance on the server with any changed property values.

		MAC address of the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentMacAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4Address(YangBase):
	"""IP address of the interface on the relay agent that is connectedto the DHCP hosts.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-pool-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressRandom(self))

	def delete(self):
		"""Delete this relay-agent-pool-ipv4-address instance on the server.

		IP address of the interface on the relay agent that is connectedto the DHCP hosts.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-pool-ipv4-address instance on the server with any changed property values.

		IP address of the interface on the relay agent that is connectedto the DHCP hosts.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentPoolIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4Address(YangBase):
	"""Destination IP address for the relay agent message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-server-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressRandom(self))

	def delete(self):
		"""Delete this relay-agent-server-ipv4-address instance on the server.

		Destination IP address for the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-server-ipv4-address instance on the server with any changed property values.

		Destination IP address for the relay agent message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4RelayAgentRelayAgentServerIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Server(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4Server, self).__init__(parent, None)

	def assign_strategy(self):
		"""Get the assign-strategy object from the server

		The strategy that server choose address pools which are used for assign address.    GATEWAY	       : If Client IP address is not 0 in DHCP message, assign address from those pools                    who has the same network as Client IP address.                    Else if Relay Agent IP address is not 0 in DHCP message, assign address from those                    pools who has the same network as Relay Agent IP address.                    Else if EnablePoolAddrPrefix is true, assign address from the default address pool.                    Else assign address from those pools who has the same network as DHCP server IP.    CIRCUIT_ID     : Assign address from those pools who match the relay agent circuit ID option received.    REMOTE_ID      : Assign address from those pools who match the relay agent remote ID option received.    LINK_SELECTION : Assign address from those pools who match the relay agent link selection option received.    VPN_ID	       : Assign address from those pools who match the relay agent virtual subnet                     selection option received.    POOL_BY_POOL   : Assign address from default pool, and then from the first relay agent pool if any when                    the default pool is exhausted and so on.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAssignStrategy`: An object encapsulating an instance of the assign-strategy model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAssignStrategy(self))

	def authentication_token(self):
		"""Get the authentication-token object from the server

		Token authentication for forcerenew message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAuthenticationToken`: An object encapsulating an instance of the authentication-token model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAuthenticationToken(self))

	def decline_reserve_time(self):
		"""Get the decline-reserve-time object from the server

		Time in seconds an address will be reserved after it is declined by the client.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTime`: An object encapsulating an instance of the decline-reserve-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTime(self))

	def enable_overlap_address(self):
		"""Get the enable-overlap-address object from the server

		Reuse addresses based on circuit ID. For example: say we configured 1 pool with 100 addresses and 5 different circuit ID, server can assign 100 addresses for each circuit ID. It is like we have 500 addresses..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddress`: An object encapsulating an instance of the enable-overlap-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddress(self))

	def host_name(self):
		"""Get the host-name object from the server

		Server host name.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerHostName`: An object encapsulating an instance of the host-name model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerHostName(self))

	def tos(self):
		"""Get the tos object from the server

		Provides an indication of the quality of service wanted.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTos`: An object encapsulating an instance of the tos model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTos(self))

	def lease_time(self):
		"""Get the lease-time object from the server

		Default lease time in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTime`: An object encapsulating an instance of the lease-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTime(self))

	def min_allowed_lease_time(self):
		"""Get the min-allowed-lease-time object from the server

		Minimum allowed lease time in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTime`: An object encapsulating an instance of the min-allowed-lease-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTime(self))

	def offer_reserve_time(self):
		"""Get the offer-reserve-time object from the server

		Time in seconds an address will be reserved while the server is waiting for a response for an offer..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTime`: An object encapsulating an instance of the offer-reserve-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTime(self))

	def rebinding_time_percent(self):
		"""Get the rebinding-time-percent object from the server

		(T2) Percent of the lease time at which the client should begin the rebinding process  .

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercent`: An object encapsulating an instance of the rebinding-time-percent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercent(self))

	def renewal_time_percent(self):
		"""Get the renewal-time-percent object from the server

		(T1) Percent of the lease time at which the client should begin the renewal process.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercent`: An object encapsulating an instance of the renewal-time-percent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercent(self))

	def default_server_address_pool(self):
		"""Get the default-server-address-pool object from the server

		Default pool configurations.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPool`: An object encapsulating an instance of the default-server-address-pool model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPool(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv4 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv4 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions(self, name), locals())

	def relay_agent_address_pools(self, name=None):
		"""Get the relay-agent-address-pools object(s) from the server.

		Default pool configurations.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the relay-agent-address-pools list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools object.  

		Raises:  
			NotFoundError: name is not in the list of relay-agent-address-pools objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools(self, name))

	def create_relay_agent_address_pools(self, name):
		"""Create a child sibling relay-agent-address-pools instance on the server.

		Default pool configurations.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools`: An object encapsulating an instance of the relay-agent-address-pools model.  

		Raises:  
			AlreadyExistsError: An instance of relay-agent-address-pools with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools(self, name), locals())

	def delete(self):
		"""Delete this dhcpv4-server instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv4-server instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAssignStrategy(YangBase):
	"""The strategy that server choose address pools which are used for assign address.    GATEWAY	       : If Client IP address is not 0 in DHCP message, assign address from those pools                    who has the same network as Client IP address.                    Else if Relay Agent IP address is not 0 in DHCP message, assign address from those                    pools who has the same network as Relay Agent IP address.                    Else if EnablePoolAddrPrefix is true, assign address from the default address pool.                    Else assign address from those pools who has the same network as DHCP server IP.    CIRCUIT_ID     : Assign address from those pools who match the relay agent circuit ID option received.    REMOTE_ID      : Assign address from those pools who match the relay agent remote ID option received.    LINK_SELECTION : Assign address from those pools who match the relay agent link selection option received.    VPN_ID	       : Assign address from those pools who match the relay agent virtual subnet                     selection option received.    POOL_BY_POOL   : Assign address from default pool, and then from the first relay agent pool if any when                    the default pool is exhausted and so on.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'assign-strategy'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAssignStrategy, self).__init__(parent, None)

	def delete(self):
		"""Delete this assign-strategy instance on the server.

		The strategy that server choose address pools which are used for assign address.    GATEWAY	       : If Client IP address is not 0 in DHCP message, assign address from those pools                    who has the same network as Client IP address.                    Else if Relay Agent IP address is not 0 in DHCP message, assign address from those                    pools who has the same network as Relay Agent IP address.                    Else if EnablePoolAddrPrefix is true, assign address from the default address pool.                    Else assign address from those pools who has the same network as DHCP server IP.    CIRCUIT_ID     : Assign address from those pools who match the relay agent circuit ID option received.    REMOTE_ID      : Assign address from those pools who match the relay agent remote ID option received.    LINK_SELECTION : Assign address from those pools who match the relay agent link selection option received.    VPN_ID	       : Assign address from those pools who match the relay agent virtual subnet                     selection option received.    POOL_BY_POOL   : Assign address from default pool, and then from the first relay agent pool if any when                    the default pool is exhausted and so on.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this assign-strategy instance on the server with any changed property values.

		The strategy that server choose address pools which are used for assign address.    GATEWAY	       : If Client IP address is not 0 in DHCP message, assign address from those pools                    who has the same network as Client IP address.                    Else if Relay Agent IP address is not 0 in DHCP message, assign address from those                    pools who has the same network as Relay Agent IP address.                    Else if EnablePoolAddrPrefix is true, assign address from the default address pool.                    Else assign address from those pools who has the same network as DHCP server IP.    CIRCUIT_ID     : Assign address from those pools who match the relay agent circuit ID option received.    REMOTE_ID      : Assign address from those pools who match the relay agent remote ID option received.    LINK_SELECTION : Assign address from those pools who match the relay agent link selection option received.    VPN_ID	       : Assign address from those pools who match the relay agent virtual subnet                     selection option received.    POOL_BY_POOL   : Assign address from default pool, and then from the first relay agent pool if any when                    the default pool is exhausted and so on.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAuthenticationToken(YangBase):
	"""Token authentication for forcerenew message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'authentication-token'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerAuthenticationToken, self).__init__(parent, None)

	def delete(self):
		"""Delete this authentication-token instance on the server.

		Token authentication for forcerenew message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this authentication-token instance on the server with any changed property values.

		Token authentication for forcerenew message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTime(YangBase):
	"""Time in seconds an address will be reserved after it is declined by the client.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decline-reserve-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeRandom(self))

	def delete(self):
		"""Delete this decline-reserve-time instance on the server.

		Time in seconds an address will be reserved after it is declined by the client.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decline-reserve-time instance on the server with any changed property values.

		Time in seconds an address will be reserved after it is declined by the client.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDeclineReserveTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddress(YangBase):
	"""Reuse addresses based on circuit ID. For example: say we configured 1 pool with 100 addresses and 5 different circuit ID, server can assign 100 addresses for each circuit ID. It is like we have 500 addresses..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-overlap-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressRandom(self))

	def delete(self):
		"""Delete this enable-overlap-address instance on the server.

		Reuse addresses based on circuit ID. For example: say we configured 1 pool with 100 addresses and 5 different circuit ID, server can assign 100 addresses for each circuit ID. It is like we have 500 addresses..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-overlap-address instance on the server with any changed property values.

		Reuse addresses based on circuit ID. For example: say we configured 1 pool with 100 addresses and 5 different circuit ID, server can assign 100 addresses for each circuit ID. It is like we have 500 addresses..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerEnableOverlapAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerHostName(YangBase):
	"""Server host name.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'host-name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerHostName, self).__init__(parent, None)

	def delete(self):
		"""Delete this host-name instance on the server.

		Server host name.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this host-name instance on the server with any changed property values.

		Server host name.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTos(YangBase):
	"""Provides an indication of the quality of service wanted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'tos'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTos, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosRandom(self))

	def delete(self):
		"""Delete this tos instance on the server.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this tos instance on the server with any changed property values.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerTosRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTime(YangBase):
	"""Default lease time in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'lease-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeRandom(self))

	def delete(self):
		"""Delete this lease-time instance on the server.

		Default lease time in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this lease-time instance on the server with any changed property values.

		Default lease time in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerLeaseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTime(YangBase):
	"""Minimum allowed lease time in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'min-allowed-lease-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeRandom(self))

	def delete(self):
		"""Delete this min-allowed-lease-time instance on the server.

		Minimum allowed lease time in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this min-allowed-lease-time instance on the server with any changed property values.

		Minimum allowed lease time in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerMinAllowedLeaseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTime(YangBase):
	"""Time in seconds an address will be reserved while the server is waiting for a response for an offer..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'offer-reserve-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeRandom(self))

	def delete(self):
		"""Delete this offer-reserve-time instance on the server.

		Time in seconds an address will be reserved while the server is waiting for a response for an offer..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this offer-reserve-time instance on the server with any changed property values.

		Time in seconds an address will be reserved while the server is waiting for a response for an offer..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerOfferReserveTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercent(YangBase):
	"""(T2) Percent of the lease time at which the client should begin the rebinding process  .
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rebinding-time-percent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercent, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentRandom(self))

	def delete(self):
		"""Delete this rebinding-time-percent instance on the server.

		(T2) Percent of the lease time at which the client should begin the rebinding process  .

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rebinding-time-percent instance on the server with any changed property values.

		(T2) Percent of the lease time at which the client should begin the rebinding process  .

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRebindingTimePercentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercent(YangBase):
	"""(T1) Percent of the lease time at which the client should begin the renewal process.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'renewal-time-percent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercent, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentRandom(self))

	def delete(self):
		"""Delete this renewal-time-percent instance on the server.

		(T1) Percent of the lease time at which the client should begin the renewal process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this renewal-time-percent instance on the server with any changed property values.

		(T1) Percent of the lease time at which the client should begin the renewal process.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRenewalTimePercentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPool(YangBase):
	"""Default pool configurations.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'default-server-address-pool'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPool, self).__init__(parent, None)

	def start_ipv4_address(self):
		"""Get the start-ipv4-address object from the server

		Pool starting IP address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4Address`: An object encapsulating an instance of the start-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4Address(self))

	def custom_host_address_prefix_length(self):
		"""Get the custom-host-address-prefix-length object from the server

		Customized prefix length value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLength`: An object encapsulating an instance of the custom-host-address-prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLength(self))

	def host_address_count_limit(self):
		"""Get the host-address-count-limit object from the server

		Number of addresses in a pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimit`: An object encapsulating an instance of the host-address-count-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimit(self))

	def router_list(self):
		"""Get the router-list object from the server

		Router addresses (option 3).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterList`: An object encapsulating an instance of the router-list model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterList(self))

	def domain_name(self):
		"""Get the domain-name object from the server

		Domain name (option 15).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainName`: An object encapsulating an instance of the domain-name model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainName(self))

	def domain_name_server_list(self):
		"""Get the domain-name-server-list object from the server

		Domain name servers (option 6).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerList`: An object encapsulating an instance of the domain-name-server-list model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerList(self))

	def relay_agent_circuit_id(self):
		"""Get the relay-agent-circuit-id object from the server

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitId`: An object encapsulating an instance of the relay-agent-circuit-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitId(self))

	def max_relay_agent_circuit_id_count(self):
		"""Get the max-relay-agent-circuit-id-count object from the server

		Maximum circuit ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCount`: An object encapsulating an instance of the max-relay-agent-circuit-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCount(self))

	def relay_agent_vpn_id(self):
		"""Get the relay-agent-vpn-id object from the server

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnId`: An object encapsulating an instance of the relay-agent-vpn-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnId(self))

	def relay_agent_vpn_id_type(self):
		"""Get the relay-agent-vpn-id-type object from the server

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnIdType`: An object encapsulating an instance of the relay-agent-vpn-id-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnIdType(self))

	def max_relay_agent_vpn_id_count(self):
		"""Get the max-relay-agent-vpn-id-count object from the server

		Maximum VPN ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCount`: An object encapsulating an instance of the max-relay-agent-vpn-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCount(self))

	def delete(self):
		"""Delete this default-server-address-pool instance on the server.

		Default pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this default-server-address-pool instance on the server with any changed property values.

		Default pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4Address(YangBase):
	"""Pool starting IP address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv4-address instance on the server.

		Pool starting IP address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv4-address instance on the server with any changed property values.

		Pool starting IP address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolStartIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLength(YangBase):
	"""Customized prefix length value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-host-address-prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthRandom(self))

	def delete(self):
		"""Delete this custom-host-address-prefix-length instance on the server.

		Customized prefix length value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-host-address-prefix-length instance on the server with any changed property values.

		Customized prefix length value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolCustomHostAddressPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimit(YangBase):
	"""Number of addresses in a pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'host-address-count-limit'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitRandom(self))

	def delete(self):
		"""Delete this host-address-count-limit instance on the server.

		Number of addresses in a pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this host-address-count-limit instance on the server with any changed property values.

		Number of addresses in a pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolHostAddressCountLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterList(YangBase):
	"""Router addresses (option 3).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-list'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterList, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListRandom(self))

	def delete(self):
		"""Delete this router-list instance on the server.

		Router addresses (option 3).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-list instance on the server with any changed property values.

		Router addresses (option 3).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRouterListRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainName(YangBase):
	"""Domain name (option 15).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'domain-name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainName, self).__init__(parent, None)

	def delete(self):
		"""Delete this domain-name instance on the server.

		Domain name (option 15).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this domain-name instance on the server with any changed property values.

		Domain name (option 15).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerList(YangBase):
	"""Domain name servers (option 6).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'domain-name-server-list'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerList, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListRandom(self))

	def delete(self):
		"""Delete this domain-name-server-list instance on the server.

		Domain name servers (option 6).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this domain-name-server-list instance on the server with any changed property values.

		Domain name servers (option 6).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolDomainNameServerListRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitId(YangBase):
	"""Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-circuit-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-circuit-id instance on the server.

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-circuit-id instance on the server with any changed property values.

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentCircuitIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCount(YangBase):
	"""Maximum circuit ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-circuit-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-circuit-id-count instance on the server.

		Maximum circuit ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-circuit-id-count instance on the server with any changed property values.

		Maximum circuit ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentCircuitIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnId(YangBase):
	"""Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-vpn-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnId, self).__init__(parent, None)

	def delete(self):
		"""Delete this relay-agent-vpn-id instance on the server.

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-vpn-id instance on the server with any changed property values.

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnIdType(YangBase):
	"""VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-vpn-id-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolRelayAgentVpnIdType, self).__init__(parent, None)

	def delete(self):
		"""Delete this relay-agent-vpn-id-type instance on the server.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-vpn-id-type instance on the server with any changed property values.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCount(YangBase):
	"""Maximum VPN ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-vpn-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-vpn-id-count instance on the server.

		Maximum VPN ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-vpn-id-count instance on the server with any changed property values.

		Maximum VPN ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerDefaultServerAddressPoolMaxRelayAgentVpnIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions(YangBase):
	"""DHCPv4 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK.    OFFER : Include the message option in the OFFER message.    ACK   : Include the message option in the ACK message.    NAK   : Include the message option in the NAK message

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv4 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv4 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv4 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK.    OFFER : Include the message option in the OFFER message.    ACK   : Include the message option in the ACK message.    NAK   : Include the message option in the NAK message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK.    OFFER : Include the message option in the OFFER message.    ACK   : Include the message option in the ACK message.    NAK   : Include the message option in the NAK message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK.    OFFER : Include the message option in the OFFER message.    ACK   : Include the message option in the ACK message.    NAK   : Include the message option in the NAK message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools(YangBase):
	"""Default pool configurations.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-address-pools'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPools, self).__init__(parent, yang_key_value)

	def start_ipv4_address(self):
		"""Get the start-ipv4-address object from the server

		Pool starting IP address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4Address`: An object encapsulating an instance of the start-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4Address(self))

	def custom_host_address_prefix_length(self):
		"""Get the custom-host-address-prefix-length object from the server

		Customized prefix length value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLength`: An object encapsulating an instance of the custom-host-address-prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLength(self))

	def host_address_count_limit(self):
		"""Get the host-address-count-limit object from the server

		Number of addresses in a pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimit`: An object encapsulating an instance of the host-address-count-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimit(self))

	def router_list(self):
		"""Get the router-list object from the server

		Router addresses (option 3).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterList`: An object encapsulating an instance of the router-list model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterList(self))

	def domain_name(self):
		"""Get the domain-name object from the server

		Domain name (option 15).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainName`: An object encapsulating an instance of the domain-name model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainName(self))

	def domain_name_server_list(self):
		"""Get the domain-name-server-list object from the server

		Domain name servers (option 6).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerList`: An object encapsulating an instance of the domain-name-server-list model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerList(self))

	def relay_agent_circuit_id(self):
		"""Get the relay-agent-circuit-id object from the server

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitId`: An object encapsulating an instance of the relay-agent-circuit-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitId(self))

	def max_relay_agent_circuit_id_count(self):
		"""Get the max-relay-agent-circuit-id-count object from the server

		Maximum circuit ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCount`: An object encapsulating an instance of the max-relay-agent-circuit-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCount(self))

	def relay_agent_vpn_id(self):
		"""Get the relay-agent-vpn-id object from the server

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnId`: An object encapsulating an instance of the relay-agent-vpn-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnId(self))

	def relay_agent_vpn_id_type(self):
		"""Get the relay-agent-vpn-id-type object from the server

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnIdType`: An object encapsulating an instance of the relay-agent-vpn-id-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnIdType(self))

	def max_relay_agent_vpn_id_count(self):
		"""Get the max-relay-agent-vpn-id-count object from the server

		Maximum VPN ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCount`: An object encapsulating an instance of the max-relay-agent-vpn-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCount(self))

	def delete(self):
		"""Delete this relay-agent-address-pools instance on the server.

		Default pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-address-pools instance on the server with any changed property values.

		Default pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name address pool object"""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4Address(YangBase):
	"""Pool starting IP address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv4-address instance on the server.

		Pool starting IP address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv4-address instance on the server with any changed property values.

		Pool starting IP address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsStartIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLength(YangBase):
	"""Customized prefix length value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-host-address-prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthRandom(self))

	def delete(self):
		"""Delete this custom-host-address-prefix-length instance on the server.

		Customized prefix length value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-host-address-prefix-length instance on the server with any changed property values.

		Customized prefix length value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsCustomHostAddressPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimit(YangBase):
	"""Number of addresses in a pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'host-address-count-limit'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitRandom(self))

	def delete(self):
		"""Delete this host-address-count-limit instance on the server.

		Number of addresses in a pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this host-address-count-limit instance on the server with any changed property values.

		Number of addresses in a pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsHostAddressCountLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterList(YangBase):
	"""Router addresses (option 3).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-list'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterList, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListRandom(self))

	def delete(self):
		"""Delete this router-list instance on the server.

		Router addresses (option 3).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-list instance on the server with any changed property values.

		Router addresses (option 3).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRouterListRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainName(YangBase):
	"""Domain name (option 15).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'domain-name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainName, self).__init__(parent, None)

	def delete(self):
		"""Delete this domain-name instance on the server.

		Domain name (option 15).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this domain-name instance on the server with any changed property values.

		Domain name (option 15).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerList(YangBase):
	"""Domain name servers (option 6).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'domain-name-server-list'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerList, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListRandom(self))

	def delete(self):
		"""Delete this domain-name-server-list instance on the server.

		Domain name servers (option 6).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this domain-name-server-list instance on the server with any changed property values.

		Domain name servers (option 6).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsDomainNameServerListRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitId(YangBase):
	"""Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-circuit-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-circuit-id instance on the server.

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-circuit-id instance on the server with any changed property values.

		Generate a list of circuit ID so that this pool can assign addresses for those    DHCP clients who match any of the circuit ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentCircuitIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCount(YangBase):
	"""Maximum circuit ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-circuit-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-circuit-id-count instance on the server.

		Maximum circuit ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-circuit-id-count instance on the server with any changed property values.

		Maximum circuit ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentCircuitIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those    DHCP clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnId(YangBase):
	"""Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-vpn-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnId, self).__init__(parent, None)

	def delete(self):
		"""Delete this relay-agent-vpn-id instance on the server.

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-vpn-id instance on the server with any changed property values.

		Generate a list of VPN ID so that this pool can assign addresses for those DHCP clients who match any of the VPN ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnIdType(YangBase):
	"""VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-vpn-id-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsRelayAgentVpnIdType, self).__init__(parent, None)

	def delete(self):
		"""Delete this relay-agent-vpn-id-type instance on the server.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-vpn-id-type instance on the server with any changed property values.

		VPN ID Type NVT_ASCII : Network Virtual Terminal (NVT) ASCII VPN identifier RFC_2685  : RFC 2685 VPN-ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCount(YangBase):
	"""Maximum VPN ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-vpn-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-vpn-id-count instance on the server.

		Maximum VPN ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-vpn-id-count instance on the server with any changed property values.

		Maximum VPN ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv4ServerRelayAgentAddressPoolsMaxRelayAgentVpnIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6, self).__init__(parent, None)

	def dhcpv6_client_mode(self):
		"""Get the dhcpv6-client-mode object from the server

		The type of client to emulate. DHCPV6      : The client emulates DHCPv6. DHCPPD      : The client emulates DHCP PD. DHCPV6ANDPD	: The client emulates DHCPv6 and DHCP PD.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientMode`: An object encapsulating an instance of the dhcpv6-client-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientMode(self))

	def auth_protocol(self):
		"""Get the auth-protocol object from the server

		Specifies whether to use the DHCP message authentication option used to reliably identify the source of a DHCP message and to confirm that the contents of the DHCP message have not been tampered with. Protocol used to generate the authentication information carried in the option. DELAYED_AUTH : Use the delayed authentication protocol. RECONFIG_KEY : Use the reconfigure key authentication protocol.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthProtocol`: An object encapsulating an instance of the auth-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthProtocol(self))

	def auth_keys(self, name=None):
		"""Get the auth-keys object(s) from the server.

		A list of Authentication Keys.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the auth-keys list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys object.  

		Raises:  
			NotFoundError: name is not in the list of auth-keys objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys(self, name))

	def create_auth_keys(self, name):
		"""Create a child sibling auth-keys instance on the server.

		A list of Authentication Keys.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys`: An object encapsulating an instance of the auth-keys model.  

		Raises:  
			AlreadyExistsError: An instance of auth-keys with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys(self, name), locals())

	def client_mac_addr_start(self):
		"""Get the client-mac-addr-start object from the server

		DHCPv6 client starting MAC address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStart`: An object encapsulating an instance of the client-mac-addr-start model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStart(self))

	def control_plane_prefix(self):
		"""Get the control-plane-prefix object from the server

		Control plane source for the IPv6 address prefix. LINKLOCAL           : Use the Link-Local address. ROUTERADVERTISEMENT	: Use the router advertisement.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ControlPlanePrefix`: An object encapsulating an instance of the control-plane-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ControlPlanePrefix(self))

	def duplicate_addr_detection(self):
		"""Get the duplicate-addr-detection object from the server

		Duplicate address detection. Note that the session will not go bound until the duplicate address detection is complete. If duplicate address detection fails, the address will be declined and the session will go idle.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection`: An object encapsulating an instance of the duplicate-addr-detection model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection(self))

	def create_duplicate_addr_detection(self):
		"""Create a child sibling duplicate-addr-detection instance on the server.

		Duplicate address detection. Note that the session will not go bound until the duplicate address detection is complete. If duplicate address detection fails, the address will be declined and the session will go idle.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection`: An object encapsulating an instance of the duplicate-addr-detection model.  

		Raises:  
			AlreadyExistsError: An instance of duplicate-addr-detection with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection(self), locals())

	def dhcp_realm(self):
		"""Get the dhcp-realm object from the server

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DhcpRealm`: An object encapsulating an instance of the dhcp-realm model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DhcpRealm(self))

	def dst_addr_type(self):
		"""Get the dst-addr-type object from the server

		Determines the DHCP control plane multicast address used as destination ip address. ALL_DHCP_RELAY_AGENTS_AND_SERVERS : (FF02::1:2) A link-scoped multicast address used by a                                    client to communicate with neighboring relay agents and servers. ALL_DHCP_SERVERS : (FF05::1:3) A site-scoped multicast address used by a relay agent to communicate                    with servers.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DstAddrType`: An object encapsulating an instance of the dst-addr-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DstAddrType(self))

	def duid_type(self):
		"""Get the duid-type object from the server

		DHCPv6 unique identifier type. LLT    : Link-layer address plus time. EN     : Vendor-assigned unique ID based on enterprise number. LL     : Link-layer address. CUSTOM : Custom type.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidType`: An object encapsulating an instance of the duid-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidType(self))

	def duid_enterprise(self):
		"""Get the duid-enterprise object from the server

		Enterprise number to be used in the DUID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterprise`: An object encapsulating an instance of the duid-enterprise model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterprise(self))

	def duid_value(self):
		"""Get the duid-value object from the server

		DHCPv6/PD Unique Identifier Value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValue`: An object encapsulating an instance of the duid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValue(self))

	def duid_custom_value(self):
		"""Get the duid-custom-value object from the server

		Custom DHCPv6/PD Unique Identifier Value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValue`: An object encapsulating an instance of the duid-custom-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValue(self))

	def enable_relay_agent(self):
		"""Get the enable-relay-agent object from the server

		Relay Agent emulation            TRUE  : Enable relay agent emulation.            FALSE : Disable relay agent emulation.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgent`: An object encapsulating an instance of the enable-relay-agent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgent(self))

	def enable_ldra(self):
		"""Get the enable-ldra object from the server

		Enable or disable a Lightweight DHCPv6 Relay Agent(LDRA).            TRUE  : Enable a Lightweight DHCPv6 Relay Agent(LDRA).            FALSE : Disable a Lightweight DHCPv6 Relay Agent(LDRA).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdra`: An object encapsulating an instance of the enable-ldra model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdra(self))

	def relay_server_ipv6_addr(self):
		"""Get the relay-server-ipv6-addr object from the server

		Relay server IPv6 address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6Addr`: An object encapsulating an instance of the relay-server-ipv6-addr model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6Addr(self))

	def use_relay_agent_mac_for_dataplane(self):
		"""Get the use-relay-agent-mac-for-dataplane object from the server

		Use the relay agent MAC address (forward-facing device) for traffic instead of the emulated session blocks MAC address. TRUE  : Use the relay agent's MAC address for the data plane. FALSE : Use the linked session block MAC address for the data plane.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplane`: An object encapsulating an instance of the use-relay-agent-mac-for-dataplane model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplane(self))

	def requested_prefix_length(self):
		"""Get the requested-prefix-length object from the server

		Specify the requested length (in bits) of the IPv6 prefix. A value of 0 allows the server to specify this value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLength`: An object encapsulating an instance of the requested-prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLength(self))

	def requested_prefix_start_address(self):
		"""Get the requested-prefix-start-address object from the server

		Specify the requested IPv6 prefix. A value of 0 allows the server to specify this value.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddress`: An object encapsulating an instance of the requested-prefix-start-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddress(self))

	def iaid_start_value(self):
		"""Get the iaid-start-value object from the server

		Start value of IAID of each device.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValue`: An object encapsulating an instance of the iaid-start-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValue(self))

	def home_gateway_start_addr(self):
		"""Get the home-gateway-start-addr object from the server

		Home Gateway starting MAC address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddr`: An object encapsulating an instance of the home-gateway-start-addr model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddr(self))

	def interface_id(self):
		"""Get the interface-id object from the server

		Identifies the interface on which the client message was received.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6InterfaceId`: An object encapsulating an instance of the interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6InterfaceId(self))

	def remote_id(self):
		"""Get the remote-id object from the server

		This option may be added by DHCPv6 relay agents that terminate switched or permanent circuits and have mechanisms to identify the remote host end of the circuit.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteId`: An object encapsulating an instance of the remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteId(self))

	def remote_id_enterprise(self):
		"""Get the remote-id-enterprise object from the server

		For DHCPv6 LDRA or DHCPv6 Relay Agent to identify the vendor’s registered Enterprise Number as registered with IANA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterprise`: An object encapsulating an instance of the remote-id-enterprise model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterprise(self))

	def dhcpv6_client_requested_addr(self):
		"""Get the dhcpv6-client-requested-addr object from the server

		DHCPv6 client requested starting address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddr`: An object encapsulating an instance of the dhcpv6-client-requested-addr model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddr(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetime(self))

	def t1_timer(self):
		"""Get the t1-timer object from the server

		Time in seconds at which the client contacts the server from which the addresses were obtained to extend the lifetimes of the assigned addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1Timer`: An object encapsulating an instance of the t1-timer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1Timer(self))

	def t2_timer(self):
		"""Get the t2-timer object from the server

		Time in seconds at which the client contacts any available server to extend the lifetimes of the assigned addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2Timer`: An object encapsulating an instance of the t2-timer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2Timer(self))

	def enable_renew(self):
		"""Get the enable-renew object from the server

		Specifies whether the host sends a RENEW message to the delegating            router when the DHCPv6/PD session's T1 time has expired. The host            enters the Renewing state when the elapsed lease lifetime is between            T1 and T2, but the State field does not change accordingly.            TRUE  : Specifies that host will send a RENEW message when T1 expires.            FALSE : Specifies that host will not send a RENEW message when T1 expires.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenew`: An object encapsulating an instance of the enable-renew model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenew(self))

	def enable_rebind(self):
		"""Get the enable-rebind object from the server

		Specifies whether the host sends a REBIND message to the delegating router when            the DHCPv6/PD session's T2 time has expired. The lifetime of the prefixes (DHCPv6-PD)            or addresses (DHCPv6 client) continues until the Valid Lifetime value is reached.            TRUE  : Specifies that the host will send a REBIND message when T2 expires.            FALSE : Specifies that the host will not send a REBIND message when T2 expires.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebind`: An object encapsulating an instance of the enable-rebind model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebind(self))

	def enable_reconfig_accept(self):
		"""Get the enable-reconfig-accept object from the server

		Specifies whether the client is willing to accept Reconfigure messages            from the server. The default behavior, in the absence of this option,            means unwillingness to accept Reconfigure messages.            TRUE  : Specifies that the client is willing to accept Reconfigure messages from the server.            FALSE : Specifies that the client is unwilling to accept Reconfigure messages from the server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAccept`: An object encapsulating an instance of the enable-reconfig-accept model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAccept(self))

	def request_rate(self):
		"""Get the request-rate object from the server

		The the rate (in sessions per second) for a host block at whichSOLICIT or REBIND messages are sent or at which REQUEST messages aresent after an ADVERTISE message is received. For example, if this rateis 100 sessions per second and there are three host blocks, the ratefor the entire port is 300 sessions per second. The messages apply toDHCPv6/PD prefixes.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRate`: An object encapsulating an instance of the request-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRate(self))

	def release_rate(self):
		"""Get the release-rate object from the server

		The rate (in sessions per second) at which RELEASE messages aresent for a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRate`: An object encapsulating an instance of the release-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRate(self))

	def renew_rate(self):
		"""Get the renew-rate object from the server

		The rate (in sessions per second) at which RENEW messages are sentfor a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRate`: An object encapsulating an instance of the renew-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRate(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Traffic Class Value in IP Header.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClass(self))

	def rapid_commit_mode(self):
		"""Get the rapid-commit-mode object from the server

		Rapid commit operation mode. DISABLE : Disable rapid commit mode. ENABLE  : Enable rapid commit mode and only honor replies with the Rapid Commit option. SERVER  : Enable rapid commit mode and continue binding process whether or not the server        responds with a Rapid Commit option. The Rapid Commit option isincluded in the        ADVERTISE message. If the Rapid Commit option isenabled on the DUT, the two-message        exchanged is used. If the RapidCommit option is disabled on the DUT, the four-message        exchange is used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RapidCommitMode`: An object encapsulating an instance of the rapid-commit-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RapidCommitMode(self))

	def delete(self):
		"""Delete this dhcpv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientMode(YangBase):
	"""The type of client to emulate. DHCPV6      : The client emulates DHCPv6. DHCPPD      : The client emulates DHCP PD. DHCPV6ANDPD	: The client emulates DHCPv6 and DHCP PD.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-client-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this dhcpv6-client-mode instance on the server.

		The type of client to emulate. DHCPV6      : The client emulates DHCPv6. DHCPPD      : The client emulates DHCP PD. DHCPV6ANDPD	: The client emulates DHCPv6 and DHCP PD.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv6-client-mode instance on the server with any changed property values.

		The type of client to emulate. DHCPV6      : The client emulates DHCPv6. DHCPPD      : The client emulates DHCP PD. DHCPV6ANDPD	: The client emulates DHCPv6 and DHCP PD.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthProtocol(YangBase):
	"""Specifies whether to use the DHCP message authentication option used to reliably identify the source of a DHCP message and to confirm that the contents of the DHCP message have not been tampered with. Protocol used to generate the authentication information carried in the option. DELAYED_AUTH : Use the delayed authentication protocol. RECONFIG_KEY : Use the reconfigure key authentication protocol.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-protocol'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthProtocol, self).__init__(parent, None)

	def delete(self):
		"""Delete this auth-protocol instance on the server.

		Specifies whether to use the DHCP message authentication option used to reliably identify the source of a DHCP message and to confirm that the contents of the DHCP message have not been tampered with. Protocol used to generate the authentication information carried in the option. DELAYED_AUTH : Use the delayed authentication protocol. RECONFIG_KEY : Use the reconfigure key authentication protocol.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-protocol instance on the server with any changed property values.

		Specifies whether to use the DHCP message authentication option used to reliably identify the source of a DHCP message and to confirm that the contents of the DHCP message have not been tampered with. Protocol used to generate the authentication information carried in the option. DELAYED_AUTH : Use the delayed authentication protocol. RECONFIG_KEY : Use the reconfigure key authentication protocol.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys(YangBase):
	"""A list of Authentication Keys.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-keys'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6AuthKeys, self).__init__(parent, yang_key_value)

	def delete(self):
		"""Delete this auth-keys instance on the server.

		A list of Authentication Keys.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-keys instance on the server with any changed property values.

		A list of Authentication Keys.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of authentication key."""
		return self._get_value('name')

	@property
	def key_id(self):
		"""int32: Key identifier that identified the key used to generate the HMAC-MD5 value."""
		return self._get_value('key-id')

	@key_id.setter
	def key_id(self, value):
		return self._set_value('key-id', value)

	@property
	def key_value_type(self):
		""":str:`enum`: Key used to generate the HMAC-MD5 value.  
		Enums:  
			ASCII: Specifies that the key is specified in ASCII format.  
			HEX: Specifies that the key is specified in HEX format.  
		"""
		return self._get_value('key-value-type')

	@key_value_type.setter
	def key_value_type(self, value):
		return self._set_value('key-value-type', value)

	@property
	def key_value(self):
		"""str: Key used to generate the HMAC-MD5 value."""
		return self._get_value('key-value')

	@key_value.setter
	def key_value(self, value):
		return self._set_value('key-value', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStart(YangBase):
	"""DHCPv6 client starting MAC address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'client-mac-addr-start'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStart, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartRandom(self))

	def delete(self):
		"""Delete this client-mac-addr-start instance on the server.

		DHCPv6 client starting MAC address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this client-mac-addr-start instance on the server with any changed property values.

		DHCPv6 client starting MAC address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ClientMacAddrStartRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ControlPlanePrefix(YangBase):
	"""Control plane source for the IPv6 address prefix. LINKLOCAL           : Use the Link-Local address. ROUTERADVERTISEMENT	: Use the router advertisement.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'control-plane-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ControlPlanePrefix, self).__init__(parent, None)

	def delete(self):
		"""Delete this control-plane-prefix instance on the server.

		Control plane source for the IPv6 address prefix. LINKLOCAL           : Use the Link-Local address. ROUTERADVERTISEMENT	: Use the router advertisement.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this control-plane-prefix instance on the server with any changed property values.

		Control plane source for the IPv6 address prefix. LINKLOCAL           : Use the Link-Local address. ROUTERADVERTISEMENT	: Use the router advertisement.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection(YangBase):
	"""Duplicate address detection. Note that the session will not go bound until the duplicate address detection is complete. If duplicate address detection fails, the address will be declined and the session will go idle.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'duplicate-addr-detection'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetection, self).__init__(parent, None)

	def dad_timeout(self):
		"""Get the dad-timeout object from the server

		Length of time in seconds that must elapse before the Neighbor Solicit times out.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeout`: An object encapsulating an instance of the dad-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeout(self))

	def dad_transmits(self):
		"""Get the dad-transmits object from the server

		Number of Neighbor Solicit messages to send out per session during duplicate address detection.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmits`: An object encapsulating an instance of the dad-transmits model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmits(self))

	def delete(self):
		"""Delete this duplicate-addr-detection instance on the server.

		Duplicate address detection. Note that the session will not go bound until the duplicate address detection is complete. If duplicate address detection fails, the address will be declined and the session will go idle.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this duplicate-addr-detection instance on the server with any changed property values.

		Duplicate address detection. Note that the session will not go bound until the duplicate address detection is complete. If duplicate address detection fails, the address will be declined and the session will go idle.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeout(YangBase):
	"""Length of time in seconds that must elapse before the Neighbor Solicit times out.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dad-timeout'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutRandom(self))

	def delete(self):
		"""Delete this dad-timeout instance on the server.

		Length of time in seconds that must elapse before the Neighbor Solicit times out.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dad-timeout instance on the server with any changed property values.

		Length of time in seconds that must elapse before the Neighbor Solicit times out.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmits(YangBase):
	"""Number of Neighbor Solicit messages to send out per session during duplicate address detection.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dad-transmits'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmits, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsRandom(self))

	def delete(self):
		"""Delete this dad-transmits instance on the server.

		Number of Neighbor Solicit messages to send out per session during duplicate address detection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dad-transmits instance on the server with any changed property values.

		Number of Neighbor Solicit messages to send out per session during duplicate address detection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuplicateAddrDetectionDadTransmitsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DhcpRealm(YangBase):
	"""DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcp-realm'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DhcpRealm, self).__init__(parent, None)

	def delete(self):
		"""Delete this dhcp-realm instance on the server.

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcp-realm instance on the server with any changed property values.

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DstAddrType(YangBase):
	"""Determines the DHCP control plane multicast address used as destination ip address. ALL_DHCP_RELAY_AGENTS_AND_SERVERS : (FF02::1:2) A link-scoped multicast address used by a                                    client to communicate with neighboring relay agents and servers. ALL_DHCP_SERVERS : (FF05::1:3) A site-scoped multicast address used by a relay agent to communicate                    with servers.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dst-addr-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DstAddrType, self).__init__(parent, None)

	def delete(self):
		"""Delete this dst-addr-type instance on the server.

		Determines the DHCP control plane multicast address used as destination ip address. ALL_DHCP_RELAY_AGENTS_AND_SERVERS : (FF02::1:2) A link-scoped multicast address used by a                                    client to communicate with neighboring relay agents and servers. ALL_DHCP_SERVERS : (FF05::1:3) A site-scoped multicast address used by a relay agent to communicate                    with servers.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dst-addr-type instance on the server with any changed property values.

		Determines the DHCP control plane multicast address used as destination ip address. ALL_DHCP_RELAY_AGENTS_AND_SERVERS : (FF02::1:2) A link-scoped multicast address used by a                                    client to communicate with neighboring relay agents and servers. ALL_DHCP_SERVERS : (FF05::1:3) A site-scoped multicast address used by a relay agent to communicate                    with servers.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidType(YangBase):
	"""DHCPv6 unique identifier type. LLT    : Link-layer address plus time. EN     : Vendor-assigned unique ID based on enterprise number. LL     : Link-layer address. CUSTOM : Custom type.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'duid-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidType, self).__init__(parent, None)

	def delete(self):
		"""Delete this duid-type instance on the server.

		DHCPv6 unique identifier type. LLT    : Link-layer address plus time. EN     : Vendor-assigned unique ID based on enterprise number. LL     : Link-layer address. CUSTOM : Custom type.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this duid-type instance on the server with any changed property values.

		DHCPv6 unique identifier type. LLT    : Link-layer address plus time. EN     : Vendor-assigned unique ID based on enterprise number. LL     : Link-layer address. CUSTOM : Custom type.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterprise(YangBase):
	"""Enterprise number to be used in the DUID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'duid-enterprise'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterprise, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseRandom(self))

	def delete(self):
		"""Delete this duid-enterprise instance on the server.

		Enterprise number to be used in the DUID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this duid-enterprise instance on the server with any changed property values.

		Enterprise number to be used in the DUID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidEnterpriseRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValue(YangBase):
	"""DHCPv6/PD Unique Identifier Value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'duid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueRandom(self))

	def delete(self):
		"""Delete this duid-value instance on the server.

		DHCPv6/PD Unique Identifier Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this duid-value instance on the server with any changed property values.

		DHCPv6/PD Unique Identifier Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValue(YangBase):
	"""Custom DHCPv6/PD Unique Identifier Value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'duid-custom-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueRandom(self))

	def delete(self):
		"""Delete this duid-custom-value instance on the server.

		Custom DHCPv6/PD Unique Identifier Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this duid-custom-value instance on the server with any changed property values.

		Custom DHCPv6/PD Unique Identifier Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6DuidCustomValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgent(YangBase):
	"""Relay Agent emulation            TRUE  : Enable relay agent emulation.            FALSE : Disable relay agent emulation.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-relay-agent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgent, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentRandom(self))

	def delete(self):
		"""Delete this enable-relay-agent instance on the server.

		Relay Agent emulation            TRUE  : Enable relay agent emulation.            FALSE : Disable relay agent emulation.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-relay-agent instance on the server with any changed property values.

		Relay Agent emulation            TRUE  : Enable relay agent emulation.            FALSE : Disable relay agent emulation.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRelayAgentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdra(YangBase):
	"""Enable or disable a Lightweight DHCPv6 Relay Agent(LDRA).            TRUE  : Enable a Lightweight DHCPv6 Relay Agent(LDRA).            FALSE : Disable a Lightweight DHCPv6 Relay Agent(LDRA).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-ldra'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdra, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraRandom(self))

	def delete(self):
		"""Delete this enable-ldra instance on the server.

		Enable or disable a Lightweight DHCPv6 Relay Agent(LDRA).            TRUE  : Enable a Lightweight DHCPv6 Relay Agent(LDRA).            FALSE : Disable a Lightweight DHCPv6 Relay Agent(LDRA).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-ldra instance on the server with any changed property values.

		Enable or disable a Lightweight DHCPv6 Relay Agent(LDRA).            TRUE  : Enable a Lightweight DHCPv6 Relay Agent(LDRA).            FALSE : Disable a Lightweight DHCPv6 Relay Agent(LDRA).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableLdraRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6Addr(YangBase):
	"""Relay server IPv6 address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-server-ipv6-addr'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6Addr, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrRandom(self))

	def delete(self):
		"""Delete this relay-server-ipv6-addr instance on the server.

		Relay server IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-server-ipv6-addr instance on the server with any changed property values.

		Relay server IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RelayServerIpv6AddrRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplane(YangBase):
	"""Use the relay agent MAC address (forward-facing device) for traffic instead of the emulated session blocks MAC address. TRUE  : Use the relay agent's MAC address for the data plane. FALSE : Use the linked session block MAC address for the data plane.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'use-relay-agent-mac-for-dataplane'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplane, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneRandom(self))

	def delete(self):
		"""Delete this use-relay-agent-mac-for-dataplane instance on the server.

		Use the relay agent MAC address (forward-facing device) for traffic instead of the emulated session blocks MAC address. TRUE  : Use the relay agent's MAC address for the data plane. FALSE : Use the linked session block MAC address for the data plane.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this use-relay-agent-mac-for-dataplane instance on the server with any changed property values.

		Use the relay agent MAC address (forward-facing device) for traffic instead of the emulated session blocks MAC address. TRUE  : Use the relay agent's MAC address for the data plane. FALSE : Use the linked session block MAC address for the data plane.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6UseRelayAgentMacForDataplaneRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLength(YangBase):
	"""Specify the requested length (in bits) of the IPv6 prefix. A value of 0 allows the server to specify this value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'requested-prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthRandom(self))

	def delete(self):
		"""Delete this requested-prefix-length instance on the server.

		Specify the requested length (in bits) of the IPv6 prefix. A value of 0 allows the server to specify this value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this requested-prefix-length instance on the server with any changed property values.

		Specify the requested length (in bits) of the IPv6 prefix. A value of 0 allows the server to specify this value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddress(YangBase):
	"""Specify the requested IPv6 prefix. A value of 0 allows the server to specify this value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'requested-prefix-start-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressRandom(self))

	def delete(self):
		"""Delete this requested-prefix-start-address instance on the server.

		Specify the requested IPv6 prefix. A value of 0 allows the server to specify this value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this requested-prefix-start-address instance on the server with any changed property values.

		Specify the requested IPv6 prefix. A value of 0 allows the server to specify this value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestedPrefixStartAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValue(YangBase):
	"""Start value of IAID of each device.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'iaid-start-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueRandom(self))

	def delete(self):
		"""Delete this iaid-start-value instance on the server.

		Start value of IAID of each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this iaid-start-value instance on the server with any changed property values.

		Start value of IAID of each device.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6IaidStartValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddr(YangBase):
	"""Home Gateway starting MAC address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'home-gateway-start-addr'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddr, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrRandom(self))

	def delete(self):
		"""Delete this home-gateway-start-addr instance on the server.

		Home Gateway starting MAC address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this home-gateway-start-addr instance on the server with any changed property values.

		Home Gateway starting MAC address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6HomeGatewayStartAddrRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6InterfaceId(YangBase):
	"""Identifies the interface on which the client message was received.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'interface-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6InterfaceId, self).__init__(parent, None)

	def delete(self):
		"""Delete this interface-id instance on the server.

		Identifies the interface on which the client message was received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this interface-id instance on the server with any changed property values.

		Identifies the interface on which the client message was received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteId(YangBase):
	"""This option may be added by DHCPv6 relay agents that terminate switched or permanent circuits and have mechanisms to identify the remote host end of the circuit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteId, self).__init__(parent, None)

	def delete(self):
		"""Delete this remote-id instance on the server.

		This option may be added by DHCPv6 relay agents that terminate switched or permanent circuits and have mechanisms to identify the remote host end of the circuit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this remote-id instance on the server with any changed property values.

		This option may be added by DHCPv6 relay agents that terminate switched or permanent circuits and have mechanisms to identify the remote host end of the circuit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterprise(YangBase):
	"""For DHCPv6 LDRA or DHCPv6 Relay Agent to identify the vendor’s registered Enterprise Number as registered with IANA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'remote-id-enterprise'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterprise, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseRandom(self))

	def delete(self):
		"""Delete this remote-id-enterprise instance on the server.

		For DHCPv6 LDRA or DHCPv6 Relay Agent to identify the vendor’s registered Enterprise Number as registered with IANA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this remote-id-enterprise instance on the server with any changed property values.

		For DHCPv6 LDRA or DHCPv6 Relay Agent to identify the vendor’s registered Enterprise Number as registered with IANA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RemoteIdEnterpriseRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddr(YangBase):
	"""DHCPv6 client requested starting address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-client-requested-addr'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddr, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrRandom(self))

	def delete(self):
		"""Delete this dhcpv6-client-requested-addr instance on the server.

		DHCPv6 client requested starting address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv6-client-requested-addr instance on the server with any changed property values.

		DHCPv6 client requested starting address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Dhcpv6ClientRequestedAddrRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6PreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1Timer(YangBase):
	"""Time in seconds at which the client contacts the server from which the addresses were obtained to extend the lifetimes of the assigned addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 't1-timer'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1Timer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerRandom(self))

	def delete(self):
		"""Delete this t1-timer instance on the server.

		Time in seconds at which the client contacts the server from which the addresses were obtained to extend the lifetimes of the assigned addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this t1-timer instance on the server with any changed property values.

		Time in seconds at which the client contacts the server from which the addresses were obtained to extend the lifetimes of the assigned addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T1TimerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2Timer(YangBase):
	"""Time in seconds at which the client contacts any available server to extend the lifetimes of the assigned addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 't2-timer'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2Timer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerRandom(self))

	def delete(self):
		"""Delete this t2-timer instance on the server.

		Time in seconds at which the client contacts any available server to extend the lifetimes of the assigned addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this t2-timer instance on the server with any changed property values.

		Time in seconds at which the client contacts any available server to extend the lifetimes of the assigned addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6T2TimerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenew(YangBase):
	"""Specifies whether the host sends a RENEW message to the delegating            router when the DHCPv6/PD session's T1 time has expired. The host            enters the Renewing state when the elapsed lease lifetime is between            T1 and T2, but the State field does not change accordingly.            TRUE  : Specifies that host will send a RENEW message when T1 expires.            FALSE : Specifies that host will not send a RENEW message when T1 expires.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-renew'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenew, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewRandom(self))

	def delete(self):
		"""Delete this enable-renew instance on the server.

		Specifies whether the host sends a RENEW message to the delegating            router when the DHCPv6/PD session's T1 time has expired. The host            enters the Renewing state when the elapsed lease lifetime is between            T1 and T2, but the State field does not change accordingly.            TRUE  : Specifies that host will send a RENEW message when T1 expires.            FALSE : Specifies that host will not send a RENEW message when T1 expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-renew instance on the server with any changed property values.

		Specifies whether the host sends a RENEW message to the delegating            router when the DHCPv6/PD session's T1 time has expired. The host            enters the Renewing state when the elapsed lease lifetime is between            T1 and T2, but the State field does not change accordingly.            TRUE  : Specifies that host will send a RENEW message when T1 expires.            FALSE : Specifies that host will not send a RENEW message when T1 expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRenewRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebind(YangBase):
	"""Specifies whether the host sends a REBIND message to the delegating router when            the DHCPv6/PD session's T2 time has expired. The lifetime of the prefixes (DHCPv6-PD)            or addresses (DHCPv6 client) continues until the Valid Lifetime value is reached.            TRUE  : Specifies that the host will send a REBIND message when T2 expires.            FALSE : Specifies that the host will not send a REBIND message when T2 expires.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-rebind'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebind, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindRandom(self))

	def delete(self):
		"""Delete this enable-rebind instance on the server.

		Specifies whether the host sends a REBIND message to the delegating router when            the DHCPv6/PD session's T2 time has expired. The lifetime of the prefixes (DHCPv6-PD)            or addresses (DHCPv6 client) continues until the Valid Lifetime value is reached.            TRUE  : Specifies that the host will send a REBIND message when T2 expires.            FALSE : Specifies that the host will not send a REBIND message when T2 expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-rebind instance on the server with any changed property values.

		Specifies whether the host sends a REBIND message to the delegating router when            the DHCPv6/PD session's T2 time has expired. The lifetime of the prefixes (DHCPv6-PD)            or addresses (DHCPv6 client) continues until the Valid Lifetime value is reached.            TRUE  : Specifies that the host will send a REBIND message when T2 expires.            FALSE : Specifies that the host will not send a REBIND message when T2 expires.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableRebindRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAccept(YangBase):
	"""Specifies whether the client is willing to accept Reconfigure messages            from the server. The default behavior, in the absence of this option,            means unwillingness to accept Reconfigure messages.            TRUE  : Specifies that the client is willing to accept Reconfigure messages from the server.            FALSE : Specifies that the client is unwilling to accept Reconfigure messages from the server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-reconfig-accept'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAccept, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptRandom(self))

	def delete(self):
		"""Delete this enable-reconfig-accept instance on the server.

		Specifies whether the client is willing to accept Reconfigure messages            from the server. The default behavior, in the absence of this option,            means unwillingness to accept Reconfigure messages.            TRUE  : Specifies that the client is willing to accept Reconfigure messages from the server.            FALSE : Specifies that the client is unwilling to accept Reconfigure messages from the server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-reconfig-accept instance on the server with any changed property values.

		Specifies whether the client is willing to accept Reconfigure messages            from the server. The default behavior, in the absence of this option,            means unwillingness to accept Reconfigure messages.            TRUE  : Specifies that the client is willing to accept Reconfigure messages from the server.            FALSE : Specifies that the client is unwilling to accept Reconfigure messages from the server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6EnableReconfigAcceptRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRate(YangBase):
	"""The the rate (in sessions per second) for a host block at whichSOLICIT or REBIND messages are sent or at which REQUEST messages aresent after an ADVERTISE message is received. For example, if this rateis 100 sessions per second and there are three host blocks, the ratefor the entire port is 300 sessions per second. The messages apply toDHCPv6/PD prefixes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'request-rate'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateRandom(self))

	def delete(self):
		"""Delete this request-rate instance on the server.

		The the rate (in sessions per second) for a host block at whichSOLICIT or REBIND messages are sent or at which REQUEST messages aresent after an ADVERTISE message is received. For example, if this rateis 100 sessions per second and there are three host blocks, the ratefor the entire port is 300 sessions per second. The messages apply toDHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this request-rate instance on the server with any changed property values.

		The the rate (in sessions per second) for a host block at whichSOLICIT or REBIND messages are sent or at which REQUEST messages aresent after an ADVERTISE message is received. For example, if this rateis 100 sessions per second and there are three host blocks, the ratefor the entire port is 300 sessions per second. The messages apply toDHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RequestRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRate(YangBase):
	"""The rate (in sessions per second) at which RELEASE messages aresent for a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'release-rate'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateRandom(self))

	def delete(self):
		"""Delete this release-rate instance on the server.

		The rate (in sessions per second) at which RELEASE messages aresent for a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this release-rate instance on the server with any changed property values.

		The rate (in sessions per second) at which RELEASE messages aresent for a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ReleaseRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRate(YangBase):
	"""The rate (in sessions per second) at which RENEW messages are sentfor a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'renew-rate'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRate, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateRandom(self))

	def delete(self):
		"""Delete this renew-rate instance on the server.

		The rate (in sessions per second) at which RENEW messages are sentfor a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this renew-rate instance on the server with any changed property values.

		The rate (in sessions per second) at which RENEW messages are sentfor a host block. For example, if this rate is 100 sessions per secondand there are three host blocks, the rate for the entire port is 300 sessionsper second. The messages apply to DHCPv6/PD prefixes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RenewRateRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClass(YangBase):
	"""Traffic Class Value in IP Header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Traffic Class Value in IP Header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Traffic Class Value in IP Header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Ipv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RapidCommitMode(YangBase):
	"""Rapid commit operation mode. DISABLE : Disable rapid commit mode. ENABLE  : Enable rapid commit mode and only honor replies with the Rapid Commit option. SERVER  : Enable rapid commit mode and continue binding process whether or not the server        responds with a Rapid Commit option. The Rapid Commit option isincluded in the        ADVERTISE message. If the Rapid Commit option isenabled on the DUT, the two-message        exchanged is used. If the RapidCommit option is disabled on the DUT, the four-message        exchange is used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rapid-commit-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6RapidCommitMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this rapid-commit-mode instance on the server.

		Rapid commit operation mode. DISABLE : Disable rapid commit mode. ENABLE  : Enable rapid commit mode and only honor replies with the Rapid Commit option. SERVER  : Enable rapid commit mode and continue binding process whether or not the server        responds with a Rapid Commit option. The Rapid Commit option isincluded in the        ADVERTISE message. If the Rapid Commit option isenabled on the DUT, the two-message        exchanged is used. If the RapidCommit option is disabled on the DUT, the four-message        exchange is used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rapid-commit-mode instance on the server with any changed property values.

		Rapid commit operation mode. DISABLE : Disable rapid commit mode. ENABLE  : Enable rapid commit mode and only honor replies with the Rapid Commit option. SERVER  : Enable rapid commit mode and continue binding process whether or not the server        responds with a Rapid Commit option. The Rapid Commit option isincluded in the        ADVERTISE message. If the Rapid Commit option isenabled on the DUT, the two-message        exchanged is used. If the RapidCommit option is disabled on the DUT, the four-message        exchange is used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Server(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6Server, self).__init__(parent, None)

	def assign_strategy(self):
		"""Get the assign-strategy object from the server

		The strategy that server choose address pools which are used for assign address. INTERFACE_ID : Assign address from those pools who match the relay agent interface ID option received. REMOTE_ID	 : Assign address from those pools who match the relay agent remote ID option received.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAssignStrategy`: An object encapsulating an instance of the assign-strategy model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAssignStrategy(self))

	def emulation_mode(self):
		"""Get the emulation-mode object from the server

		Server emulation mode. DHCPV6    : Server will emulate a full DHCPv6 server. DHCPV6_PD : Server will emulate only a DHCP-PD server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerEmulationMode`: An object encapsulating an instance of the emulation-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerEmulationMode(self))

	def delayed_authentication(self):
		"""Get the delayed-authentication object from the server

		Delayed authentication

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication`: An object encapsulating an instance of the delayed-authentication model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication(self))

	def create_delayed_authentication(self):
		"""Create a child sibling delayed-authentication instance on the server.

		Delayed authentication

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication`: An object encapsulating an instance of the delayed-authentication model.  

		Raises:  
			AlreadyExistsError: An instance of delayed-authentication with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication(self), locals())

	def reconfiguration_key(self):
		"""Get the reconfiguration-key object from the server

		Specify reconfiguration keys

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey`: An object encapsulating an instance of the reconfiguration-key model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey(self))

	def create_reconfiguration_key(self):
		"""Create a child sibling reconfiguration-key instance on the server.

		Specify reconfiguration keys

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey`: An object encapsulating an instance of the reconfiguration-key model.  

		Raises:  
			AlreadyExistsError: An instance of reconfiguration-key with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey(self), locals())

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Traffic Class Value in IP Header.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClass(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetime(self))

	def rebinding_time_percent(self):
		"""Get the rebinding-time-percent object from the server

		Rebinding lease time (T2) as a percentage of Lease Time.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercent`: An object encapsulating an instance of the rebinding-time-percent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercent(self))

	def renewal_time_percent(self):
		"""Get the renewal-time-percent object from the server

		Renewal lease time (T2) as a percentage of Lease Time.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercent`: An object encapsulating an instance of the renewal-time-percent model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercent(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv6 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv6 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions(self, name), locals())

	def server_address_pool(self):
		"""Get the server-address-pool object from the server

		Default Address pool configurations.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPool`: An object encapsulating an instance of the server-address-pool model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPool(self))

	def server_prefix_pool(self):
		"""Get the server-prefix-pool object from the server

		Default Prefix pool configurations.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPool`: An object encapsulating an instance of the server-prefix-pool model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPool(self))

	def additional_server_address_pool(self, name=None):
		"""Get the additional-server-address-pool object(s) from the server.

		Additional address pools configurations.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the additional-server-address-pool list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool object.  

		Raises:  
			NotFoundError: name is not in the list of additional-server-address-pool objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool(self, name))

	def create_additional_server_address_pool(self, name):
		"""Create a child sibling additional-server-address-pool instance on the server.

		Additional address pools configurations.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool`: An object encapsulating an instance of the additional-server-address-pool model.  

		Raises:  
			AlreadyExistsError: An instance of additional-server-address-pool with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool(self, name), locals())

	def additional_server_prefix_pool(self, name=None):
		"""Get the additional-server-prefix-pool object(s) from the server.

		Additional prefix delegation pools configurations.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the additional-server-prefix-pool list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool object.  

		Raises:  
			NotFoundError: name is not in the list of additional-server-prefix-pool objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool(self, name))

	def create_additional_server_prefix_pool(self, name):
		"""Create a child sibling additional-server-prefix-pool instance on the server.

		Additional prefix delegation pools configurations.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool`: An object encapsulating an instance of the additional-server-prefix-pool model.  

		Raises:  
			AlreadyExistsError: An instance of additional-server-prefix-pool with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool(self, name), locals())

	def delete(self):
		"""Delete this dhcpv6-server instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcpv6-server instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAssignStrategy(YangBase):
	"""The strategy that server choose address pools which are used for assign address. INTERFACE_ID : Assign address from those pools who match the relay agent interface ID option received. REMOTE_ID	 : Assign address from those pools who match the relay agent remote ID option received.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'assign-strategy'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAssignStrategy, self).__init__(parent, None)

	def delete(self):
		"""Delete this assign-strategy instance on the server.

		The strategy that server choose address pools which are used for assign address. INTERFACE_ID : Assign address from those pools who match the relay agent interface ID option received. REMOTE_ID	 : Assign address from those pools who match the relay agent remote ID option received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this assign-strategy instance on the server with any changed property values.

		The strategy that server choose address pools which are used for assign address. INTERFACE_ID : Assign address from those pools who match the relay agent interface ID option received. REMOTE_ID	 : Assign address from those pools who match the relay agent remote ID option received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerEmulationMode(YangBase):
	"""Server emulation mode. DHCPV6    : Server will emulate a full DHCPv6 server. DHCPV6_PD : Server will emulate only a DHCP-PD server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'emulation-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerEmulationMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this emulation-mode instance on the server.

		Server emulation mode. DHCPV6    : Server will emulate a full DHCPv6 server. DHCPV6_PD : Server will emulate only a DHCP-PD server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this emulation-mode instance on the server with any changed property values.

		Server emulation mode. DHCPV6    : Server will emulate a full DHCPv6 server. DHCPV6_PD : Server will emulate only a DHCP-PD server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication(YangBase):
	"""Delayed authentication
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'delayed-authentication'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthentication, self).__init__(parent, None)

	def dhcp_realm(self):
		"""Get the dhcp-realm object from the server

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationDhcpRealm`: An object encapsulating an instance of the dhcp-realm model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationDhcpRealm(self))

	def auth_keys(self, name=None):
		"""Get the auth-keys object(s) from the server.

		A list of Authentication Keys.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the auth-keys list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys object.  

		Raises:  
			NotFoundError: name is not in the list of auth-keys objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys(self, name))

	def create_auth_keys(self, name):
		"""Create a child sibling auth-keys instance on the server.

		A list of Authentication Keys.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys`: An object encapsulating an instance of the auth-keys model.  

		Raises:  
			AlreadyExistsError: An instance of auth-keys with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys(self, name), locals())

	def delete(self):
		"""Delete this delayed-authentication instance on the server.

		Delayed authentication

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this delayed-authentication instance on the server with any changed property values.

		Delayed authentication

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationDhcpRealm(YangBase):
	"""DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcp-realm'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationDhcpRealm, self).__init__(parent, None)

	def delete(self):
		"""Delete this dhcp-realm instance on the server.

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dhcp-realm instance on the server with any changed property values.

		DHCP realm that identifies the key used to generate the HMAC-MD5 value when using delayed authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys(YangBase):
	"""A list of Authentication Keys.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-keys'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerDelayedAuthenticationAuthKeys, self).__init__(parent, yang_key_value)

	def delete(self):
		"""Delete this auth-keys instance on the server.

		A list of Authentication Keys.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-keys instance on the server with any changed property values.

		A list of Authentication Keys.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of authentication key."""
		return self._get_value('name')

	@property
	def key_id(self):
		"""int32: Key identifier that identified the key used to generate the HMAC-MD5 value."""
		return self._get_value('key-id')

	@key_id.setter
	def key_id(self, value):
		return self._set_value('key-id', value)

	@property
	def key_value_type(self):
		""":str:`enum`: Key used to generate the HMAC-MD5 value.  
		Enums:  
			ASCII: Specifies that the key is specified in ASCII format.  
			HEX: Specifies that the key is specified in HEX format.  
		"""
		return self._get_value('key-value-type')

	@key_value_type.setter
	def key_value_type(self, value):
		return self._set_value('key-value-type', value)

	@property
	def key_value(self):
		"""str: Key used to generate the HMAC-MD5 value."""
		return self._get_value('key-value')

	@key_value.setter
	def key_value(self, value):
		return self._set_value('key-value', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey(YangBase):
	"""Specify reconfiguration keys
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reconfiguration-key'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKey, self).__init__(parent, None)

	def key_value(self):
		"""Get the key-value object from the server

		Key used for reconfigure key authentication.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValue`: An object encapsulating an instance of the key-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValue(self))

	def key_value_type(self):
		"""Get the key-value-type object from the server

		Reconfigure key value type. ASCII : Reconfigure key value is specified in ASCII. HEX	  : Reconfigure key value is specified in hexadecimal.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValueType`: An object encapsulating an instance of the key-value-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValueType(self))

	def delete(self):
		"""Delete this reconfiguration-key instance on the server.

		Specify reconfiguration keys

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reconfiguration-key instance on the server with any changed property values.

		Specify reconfiguration keys

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValue(YangBase):
	"""Key used for reconfigure key authentication.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValue, self).__init__(parent, None)

	def delete(self):
		"""Delete this key-value instance on the server.

		Key used for reconfigure key authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key-value instance on the server with any changed property values.

		Key used for reconfigure key authentication.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValueType(YangBase):
	"""Reconfigure key value type. ASCII : Reconfigure key value is specified in ASCII. HEX	  : Reconfigure key value is specified in hexadecimal.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key-value-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerReconfigurationKeyKeyValueType, self).__init__(parent, None)

	def delete(self):
		"""Delete this key-value-type instance on the server.

		Reconfigure key value type. ASCII : Reconfigure key value is specified in ASCII. HEX	  : Reconfigure key value is specified in hexadecimal.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key-value-type instance on the server with any changed property values.

		Reconfigure key value type. ASCII : Reconfigure key value is specified in ASCII. HEX	  : Reconfigure key value is specified in hexadecimal.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClass(YangBase):
	"""Traffic Class Value in IP Header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Traffic Class Value in IP Header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Traffic Class Value in IP Header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerPreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercent(YangBase):
	"""Rebinding lease time (T2) as a percentage of Lease Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rebinding-time-percent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercent, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentRandom(self))

	def delete(self):
		"""Delete this rebinding-time-percent instance on the server.

		Rebinding lease time (T2) as a percentage of Lease Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rebinding-time-percent instance on the server with any changed property values.

		Rebinding lease time (T2) as a percentage of Lease Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRebindingTimePercentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercent(YangBase):
	"""Renewal lease time (T2) as a percentage of Lease Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'renewal-time-percent'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercent, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentRandom(self))

	def delete(self):
		"""Delete this renewal-time-percent instance on the server.

		Renewal lease time (T2) as a percentage of Lease Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this renewal-time-percent instance on the server with any changed property values.

		Renewal lease time (T2) as a percentage of Lease Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerRenewalTimePercentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions(YangBase):
	"""DHCPv6 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv6 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPool(YangBase):
	"""Default Address pool configurations.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'server-address-pool'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPool, self).__init__(parent, None)

	def start_ipv6_address(self):
		"""Get the start-ipv6-address object from the server

		Pool starting IPv6 address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6Address`: An object encapsulating an instance of the start-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6Address(self))

	def number_of_pools_per_server(self):
		"""Get the number-of-pools-per-server object from the server

		Number of pools per server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServer`: An object encapsulating an instance of the number-of-pools-per-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServer(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Client prefix length.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLength(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetime(self))

	def min_iaid_value(self):
		"""Get the min-iaid-value object from the server

		The minimum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValue`: An object encapsulating an instance of the min-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValue(self))

	def max_iaid_value(self):
		"""Get the max-iaid-value object from the server

		The maximum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValue`: An object encapsulating an instance of the max-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValue(self))

	def relay_agent_interface_id(self):
		"""Get the relay-agent-interface-id object from the server

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceId`: An object encapsulating an instance of the relay-agent-interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceId(self))

	def max_relay_agent_interface_id_count(self):
		"""Get the max-relay-agent-interface-id-count object from the server

		Maximum interface ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCount`: An object encapsulating an instance of the max-relay-agent-interface-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCount(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv6 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv6 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions(self, name), locals())

	def delete(self):
		"""Delete this server-address-pool instance on the server.

		Default Address pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this server-address-pool instance on the server with any changed property values.

		Default Address pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6Address(YangBase):
	"""Pool starting IPv6 address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv6-address instance on the server.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv6-address instance on the server with any changed property values.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolStartIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServer(YangBase):
	"""Number of pools per server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-pools-per-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerRandom(self))

	def delete(self):
		"""Delete this number-of-pools-per-server instance on the server.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-pools-per-server instance on the server with any changed property values.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolNumberOfPoolsPerServerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLength(YangBase):
	"""Client prefix length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolPreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValue(YangBase):
	"""The minimum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'min-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueRandom(self))

	def delete(self):
		"""Delete this min-iaid-value instance on the server.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this min-iaid-value instance on the server with any changed property values.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMinIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValue(YangBase):
	"""The maximum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueRandom(self))

	def delete(self):
		"""Delete this max-iaid-value instance on the server.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-iaid-value instance on the server with any changed property values.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceId(YangBase):
	"""Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-interface-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-interface-id instance on the server.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-interface-id instance on the server with any changed property values.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentInterfaceIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCount(YangBase):
	"""Maximum interface ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-interface-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-interface-id-count instance on the server.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-interface-id-count instance on the server with any changed property values.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentInterfaceIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions(YangBase):
	"""DHCPv6 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv6 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerAddressPoolCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPool(YangBase):
	"""Default Prefix pool configurations.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'server-prefix-pool'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPool, self).__init__(parent, None)

	def start_ipv6_address(self):
		"""Get the start-ipv6-address object from the server

		Pool starting IPv6 address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6Address`: An object encapsulating an instance of the start-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6Address(self))

	def number_of_pools_per_server(self):
		"""Get the number-of-pools-per-server object from the server

		Number of pools per server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServer`: An object encapsulating an instance of the number-of-pools-per-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServer(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Client prefix length.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLength(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetime(self))

	def min_iaid_value(self):
		"""Get the min-iaid-value object from the server

		The minimum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValue`: An object encapsulating an instance of the min-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValue(self))

	def max_iaid_value(self):
		"""Get the max-iaid-value object from the server

		The maximum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValue`: An object encapsulating an instance of the max-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValue(self))

	def relay_agent_interface_id(self):
		"""Get the relay-agent-interface-id object from the server

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceId`: An object encapsulating an instance of the relay-agent-interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceId(self))

	def max_relay_agent_interface_id_count(self):
		"""Get the max-relay-agent-interface-id-count object from the server

		Maximum interface ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCount`: An object encapsulating an instance of the max-relay-agent-interface-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCount(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv6 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv6 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions(self, name), locals())

	def delete(self):
		"""Delete this server-prefix-pool instance on the server.

		Default Prefix pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this server-prefix-pool instance on the server with any changed property values.

		Default Prefix pool configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6Address(YangBase):
	"""Pool starting IPv6 address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv6-address instance on the server.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv6-address instance on the server with any changed property values.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolStartIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServer(YangBase):
	"""Number of pools per server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-pools-per-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerRandom(self))

	def delete(self):
		"""Delete this number-of-pools-per-server instance on the server.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-pools-per-server instance on the server with any changed property values.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolNumberOfPoolsPerServerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLength(YangBase):
	"""Client prefix length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolPreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValue(YangBase):
	"""The minimum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'min-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueRandom(self))

	def delete(self):
		"""Delete this min-iaid-value instance on the server.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this min-iaid-value instance on the server with any changed property values.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMinIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValue(YangBase):
	"""The maximum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueRandom(self))

	def delete(self):
		"""Delete this max-iaid-value instance on the server.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-iaid-value instance on the server with any changed property values.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceId(YangBase):
	"""Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-interface-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-interface-id instance on the server.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-interface-id instance on the server with any changed property values.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentInterfaceIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCount(YangBase):
	"""Maximum interface ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-interface-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-interface-id-count instance on the server.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-interface-id-count instance on the server with any changed property values.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions(YangBase):
	"""DHCPv6 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv6 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerServerPrefixPoolCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool(YangBase):
	"""Additional address pools configurations.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'additional-server-address-pool'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPool, self).__init__(parent, yang_key_value)

	def start_ipv6_address(self):
		"""Get the start-ipv6-address object from the server

		Pool starting IPv6 address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6Address`: An object encapsulating an instance of the start-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6Address(self))

	def number_of_pools_per_server(self):
		"""Get the number-of-pools-per-server object from the server

		Number of pools per server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServer`: An object encapsulating an instance of the number-of-pools-per-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServer(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Client prefix length.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLength(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetime(self))

	def min_iaid_value(self):
		"""Get the min-iaid-value object from the server

		The minimum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValue`: An object encapsulating an instance of the min-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValue(self))

	def max_iaid_value(self):
		"""Get the max-iaid-value object from the server

		The maximum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValue`: An object encapsulating an instance of the max-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValue(self))

	def relay_agent_interface_id(self):
		"""Get the relay-agent-interface-id object from the server

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceId`: An object encapsulating an instance of the relay-agent-interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceId(self))

	def max_relay_agent_interface_id_count(self):
		"""Get the max-relay-agent-interface-id-count object from the server

		Maximum interface ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCount`: An object encapsulating an instance of the max-relay-agent-interface-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCount(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv6 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv6 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions(self, name), locals())

	def delete(self):
		"""Delete this additional-server-address-pool instance on the server.

		Additional address pools configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this additional-server-address-pool instance on the server with any changed property values.

		Additional address pools configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name address pool object"""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6Address(YangBase):
	"""Pool starting IPv6 address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv6-address instance on the server.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv6-address instance on the server with any changed property values.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolStartIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServer(YangBase):
	"""Number of pools per server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-pools-per-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerRandom(self))

	def delete(self):
		"""Delete this number-of-pools-per-server instance on the server.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-pools-per-server instance on the server with any changed property values.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolNumberOfPoolsPerServerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLength(YangBase):
	"""Client prefix length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolPreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValue(YangBase):
	"""The minimum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'min-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueRandom(self))

	def delete(self):
		"""Delete this min-iaid-value instance on the server.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this min-iaid-value instance on the server with any changed property values.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMinIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValue(YangBase):
	"""The maximum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueRandom(self))

	def delete(self):
		"""Delete this max-iaid-value instance on the server.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-iaid-value instance on the server with any changed property values.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceId(YangBase):
	"""Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-interface-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-interface-id instance on the server.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-interface-id instance on the server with any changed property values.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentInterfaceIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCount(YangBase):
	"""Maximum interface ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-interface-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-interface-id-count instance on the server.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-interface-id-count instance on the server with any changed property values.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentInterfaceIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions(YangBase):
	"""DHCPv6 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv6 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerAddressPoolCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool(YangBase):
	"""Additional prefix delegation pools configurations.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'additional-server-prefix-pool'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPool, self).__init__(parent, yang_key_value)

	def start_ipv6_address(self):
		"""Get the start-ipv6-address object from the server

		Pool starting IPv6 address.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6Address`: An object encapsulating an instance of the start-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6Address(self))

	def number_of_pools_per_server(self):
		"""Get the number-of-pools-per-server object from the server

		Number of pools per server.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServer`: An object encapsulating an instance of the number-of-pools-per-server model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServer(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Client prefix length.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLength(self))

	def preferred_lifetime(self):
		"""Get the preferred-lifetime object from the server

		Preferred lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetime`: An object encapsulating an instance of the preferred-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetime(self))

	def valid_lifetime(self):
		"""Get the valid-lifetime object from the server

		Valid lifetime in seconds for the addresses.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetime`: An object encapsulating an instance of the valid-lifetime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetime(self))

	def min_iaid_value(self):
		"""Get the min-iaid-value object from the server

		The minimum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValue`: An object encapsulating an instance of the min-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValue(self))

	def max_iaid_value(self):
		"""Get the max-iaid-value object from the server

		The maximum identifier for an IA to accept in the pool.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValue`: An object encapsulating an instance of the max-iaid-value model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValue(self))

	def relay_agent_interface_id(self):
		"""Get the relay-agent-interface-id object from the server

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceId`: An object encapsulating an instance of the relay-agent-interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceId(self))

	def max_relay_agent_interface_id_count(self):
		"""Get the max-relay-agent-interface-id-count object from the server

		Maximum interface ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCount`: An object encapsulating an instance of the max-relay-agent-interface-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCount(self))

	def relay_agent_remote_id(self):
		"""Get the relay-agent-remote-id object from the server

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteId`: An object encapsulating an instance of the relay-agent-remote-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteId(self))

	def max_relay_agent_remote_id_count(self):
		"""Get the max-relay-agent-remote-id-count object from the server

		Maximum remote ID count that can be generated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCount`: An object encapsulating an instance of the max-relay-agent-remote-id-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCount(self))

	def custom_options(self, name=None):
		"""Get the custom-options object(s) from the server.

		DHCPv6 Server custom options

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-options list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions object.  

		Raises:  
			NotFoundError: name is not in the list of custom-options objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions(self, name))

	def create_custom_options(self, name):
		"""Create a child sibling custom-options instance on the server.

		DHCPv6 Server custom options

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions`: An object encapsulating an instance of the custom-options model.  

		Raises:  
			AlreadyExistsError: An instance of custom-options with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions(self, name), locals())

	def delete(self):
		"""Delete this additional-server-prefix-pool instance on the server.

		Additional prefix delegation pools configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this additional-server-prefix-pool instance on the server with any changed property values.

		Additional prefix delegation pools configurations.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name prefix delegation pool object"""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6Address(YangBase):
	"""Pool starting IPv6 address.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressRandom(self))

	def delete(self):
		"""Delete this start-ipv6-address instance on the server.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-ipv6-address instance on the server with any changed property values.

		Pool starting IPv6 address.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolStartIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServer(YangBase):
	"""Number of pools per server.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-pools-per-server'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerRandom(self))

	def delete(self):
		"""Delete this number-of-pools-per-server instance on the server.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-pools-per-server instance on the server with any changed property values.

		Number of pools per server.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolNumberOfPoolsPerServerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLength(YangBase):
	"""Client prefix length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Client prefix length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetime(YangBase):
	"""Preferred lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preferred-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeRandom(self))

	def delete(self):
		"""Delete this preferred-lifetime instance on the server.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preferred-lifetime instance on the server with any changed property values.

		Preferred lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolPreferredLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetime(YangBase):
	"""Valid lifetime in seconds for the addresses.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'valid-lifetime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeRandom(self))

	def delete(self):
		"""Delete this valid-lifetime instance on the server.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this valid-lifetime instance on the server with any changed property values.

		Valid lifetime in seconds for the addresses.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolValidLifetimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValue(YangBase):
	"""The minimum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'min-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueRandom(self))

	def delete(self):
		"""Delete this min-iaid-value instance on the server.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this min-iaid-value instance on the server with any changed property values.

		The minimum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMinIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValue(YangBase):
	"""The maximum identifier for an IA to accept in the pool.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-iaid-value'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValue, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueRandom(self))

	def delete(self):
		"""Delete this max-iaid-value instance on the server.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-iaid-value instance on the server with any changed property values.

		The maximum identifier for an IA to accept in the pool.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxIaidValueRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceId(YangBase):
	"""Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-interface-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-interface-id instance on the server.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-interface-id instance on the server with any changed property values.

		Generate a list of interface ID so that this pool can assign addresses for those DHCPv6 clients who match any of the interface ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentInterfaceIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCount(YangBase):
	"""Maximum interface ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-interface-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-interface-id-count instance on the server.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-interface-id-count instance on the server with any changed property values.

		Maximum interface ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentInterfaceIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteId(YangBase):
	"""Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'relay-agent-remote-id'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdRandom(self))

	def delete(self):
		"""Delete this relay-agent-remote-id instance on the server.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this relay-agent-remote-id instance on the server with any changed property values.

		Generate a list of remote ID so that this pool can assign addresses for those DHCPv6 clients who match any of the remote ID in the list.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolRelayAgentRemoteIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCount(YangBase):
	"""Maximum remote ID count that can be generated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-relay-agent-remote-id-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountRandom(self))

	def delete(self):
		"""Delete this max-relay-agent-remote-id-count instance on the server.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-relay-agent-remote-id-count instance on the server with any changed property values.

		Maximum remote ID count that can be generated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolMaxRelayAgentRemoteIdCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions(YangBase):
	"""DHCPv6 Server custom options
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-options'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptions, self).__init__(parent, yang_key_value)

	def option_type(self):
		"""Get the option-type object from the server

		Option Identifier

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionType`: An object encapsulating an instance of the option-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionType(self))

	def message_type(self):
		"""Get the message-type object from the server

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsMessageType`: An object encapsulating an instance of the message-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsMessageType(self))

	def option_payload(self):
		"""Get the option-payload object from the server

		Option Payload

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionPayload`: An object encapsulating an instance of the option-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionPayload(self))

	def delete(self):
		"""Delete this custom-options instance on the server.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-options instance on the server with any changed property values.

		DHCPv6 Server custom options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the DHCPv6 Server custom object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionType(YangBase):
	"""Option Identifier
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeRandom(self))

	def delete(self):
		"""Delete this option-type instance on the server.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-type instance on the server with any changed property values.

		Option Identifier

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsMessageType(YangBase):
	"""Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'message-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsMessageType, self).__init__(parent, None)

	def delete(self):
		"""Delete this message-type instance on the server.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this message-type instance on the server with any changed property values.

		Includes the message option in OFFER, ACK and NAK. ADVERTISE   : Include option in the ADVERTISE message. REPLY       : Include option in the REPLY message. RECONFIGURE	: Include option in the RECONFIGURE message. RELAY_REPLY  : Include option in the RELAY_REPLY message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionPayload(YangBase):
	"""Option Payload
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'option-payload'
		super(SessionsConfigDeviceGroupsDevicesProtocolsDhcpv6ServerAdditionalServerPrefixPoolCustomOptionsOptionPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this option-payload instance on the server.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this option-payload instance on the server with any changed property values.

		Option Payload

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			STRING: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def string(self):
		"""str: Vendor specific string patterns"""
		return self._get_value('string')

	@string.setter
	def string(self, value):
		return self._set_value('string', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmp(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmp, self).__init__(parent, None)

	def version(self):
		"""Get the version object from the server

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpVersion(self))

	def force_leave(self):
		"""Get the force-leave object from the server

		Affects all hosts except the last one, which is always required to send an IGMPv2 leave report.            IGMPv2 hosts leaving a multicast group may optionally send a leave report to the all-routers multicast group.            The force-leave flag controls whether or not all hosts are required to send leave reports when leaving the multicast group.            TRUE    : Force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeave`: An object encapsulating an instance of the force-leave model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeave(self))

	def force_simple_join(self):
		"""Get the force-simple-join object from the server

		Forces the IGMP host(s) to send a single join report per group.            Enabling this option effectively bypasses the RFC defined behavior of            sending (Robustness Variable - 1) reports. Otherwise, IGMP host(s) will            behave according to RFC specification.            TRUE    :Force IGMP host(s) to send a single join report per group.            FALSE	:Default behavior that is compliant to RFC specification. IGMP host(s) will send (Robustness Variable - 1) reports.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoin`: An object encapsulating an instance of the force-simple-join model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoin(self))

	def insert_checksum_errors(self):
		"""Get the insert-checksum-errors object from the server

		Controls the insertion of checksum errors into IGMP messages by the hardware.            TRUE    :IGMP checksum of the transmitted packet is flipped by the protocol stack.            FALSE	:IGMP checksum of the transmitted packet is not modified.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrors`: An object encapsulating an instance of the insert-checksum-errors model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrors(self))

	def insert_length_errors(self):
		"""Get the insert-length-errors object from the server

		Controls the insertion of message length errors into the IGMP messages by the IGMP stack.            TRUE    :Every IGMP packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	:IGMP packet lengths will not be modified.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrors`: An object encapsulating an instance of the insert-length-errors model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrors(self))

	def ipv4_dont_fragment(self):
		"""Get the ipv4-dont-fragment object from the server

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragment`: An object encapsulating an instance of the ipv4-dont-fragment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragment(self))

	def pack_reports(self):
		"""Get the pack-reports object from the server

		Allows IGMPv3 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets..            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReports`: An object encapsulating an instance of the pack-reports model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReports(self))

	def router_alert(self):
		"""Get the router-alert object from the server

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlert`: An object encapsulating an instance of the router-alert model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlert(self))

	def ipv4_tos(self):
		"""Get the ipv4-tos object from the server

		Provides an indication of the quality of service wanted.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4Tos`: An object encapsulating an instance of the ipv4-tos model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4Tos(self))

	def force_robust_join(self):
		"""Get the force-robust-join object from the server

		When an IGMPv1/IGMPv2 host joins a multicast group, it immediately transmits an initial unsolicited            membership report for that group, in case it is the first member of that group on the network.            In case the initial report gets damaged or lost, a second unsolicited report is recommended be sent out.            The force-robust-join flag controls whether or not a second report is transmitted.            TRUE    :Forces the host to send a second IGMPv1/IGMPv2 join report.            FALSE	:Does not force the host to send a second IGMPv1/IGMPv2 join report.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoin`: An object encapsulating an instance of the force-robust-join model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoin(self))

	def robustness_variable(self):
		"""Get the robustness-variable object from the server

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariable`: An object encapsulating an instance of the robustness-variable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariable(self))

	def unsolicited_report_interval(self):
		"""Get the unsolicited-report-interval object from the server

		Time between repetitions of a host's initial report of membership in a group.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportInterval`: An object encapsulating an instance of the unsolicited-report-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportInterval(self))

	def v1_router_present_timeout(self):
		"""Get the v1-router-present-timeout object from the server

		Amount of time a host must wait after hearing a Version 1 Query before it may send any IGMPv2 messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeout`: An object encapsulating an instance of the v1-router-present-timeout model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeout(self))

	def delete(self):
		"""Delete this igmp instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmp instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def multicast_group(self):
		"""-> ../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpVersion(YangBase):
	"""Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this version instance on the server.

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeave(YangBase):
	"""Affects all hosts except the last one, which is always required to send an IGMPv2 leave report.            IGMPv2 hosts leaving a multicast group may optionally send a leave report to the all-routers multicast group.            The force-leave flag controls whether or not all hosts are required to send leave reports when leaving the multicast group.            TRUE    : Force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-leave'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeave, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveRandom(self))

	def delete(self):
		"""Delete this force-leave instance on the server.

		Affects all hosts except the last one, which is always required to send an IGMPv2 leave report.            IGMPv2 hosts leaving a multicast group may optionally send a leave report to the all-routers multicast group.            The force-leave flag controls whether or not all hosts are required to send leave reports when leaving the multicast group.            TRUE    : Force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-leave instance on the server with any changed property values.

		Affects all hosts except the last one, which is always required to send an IGMPv2 leave report.            IGMPv2 hosts leaving a multicast group may optionally send a leave report to the all-routers multicast group.            The force-leave flag controls whether or not all hosts are required to send leave reports when leaving the multicast group.            TRUE    : Force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the IGMPv2 host(s) to send a Leave Group message when leaving a multicast group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceLeaveRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoin(YangBase):
	"""Forces the IGMP host(s) to send a single join report per group.            Enabling this option effectively bypasses the RFC defined behavior of            sending (Robustness Variable - 1) reports. Otherwise, IGMP host(s) will            behave according to RFC specification.            TRUE    :Force IGMP host(s) to send a single join report per group.            FALSE	:Default behavior that is compliant to RFC specification. IGMP host(s) will send (Robustness Variable - 1) reports.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-simple-join'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoin, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinRandom(self))

	def delete(self):
		"""Delete this force-simple-join instance on the server.

		Forces the IGMP host(s) to send a single join report per group.            Enabling this option effectively bypasses the RFC defined behavior of            sending (Robustness Variable - 1) reports. Otherwise, IGMP host(s) will            behave according to RFC specification.            TRUE    :Force IGMP host(s) to send a single join report per group.            FALSE	:Default behavior that is compliant to RFC specification. IGMP host(s) will send (Robustness Variable - 1) reports.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-simple-join instance on the server with any changed property values.

		Forces the IGMP host(s) to send a single join report per group.            Enabling this option effectively bypasses the RFC defined behavior of            sending (Robustness Variable - 1) reports. Otherwise, IGMP host(s) will            behave according to RFC specification.            TRUE    :Force IGMP host(s) to send a single join report per group.            FALSE	:Default behavior that is compliant to RFC specification. IGMP host(s) will send (Robustness Variable - 1) reports.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceSimpleJoinRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrors(YangBase):
	"""Controls the insertion of checksum errors into IGMP messages by the hardware.            TRUE    :IGMP checksum of the transmitted packet is flipped by the protocol stack.            FALSE	:IGMP checksum of the transmitted packet is not modified.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'insert-checksum-errors'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrors, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsRandom(self))

	def delete(self):
		"""Delete this insert-checksum-errors instance on the server.

		Controls the insertion of checksum errors into IGMP messages by the hardware.            TRUE    :IGMP checksum of the transmitted packet is flipped by the protocol stack.            FALSE	:IGMP checksum of the transmitted packet is not modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this insert-checksum-errors instance on the server with any changed property values.

		Controls the insertion of checksum errors into IGMP messages by the hardware.            TRUE    :IGMP checksum of the transmitted packet is flipped by the protocol stack.            FALSE	:IGMP checksum of the transmitted packet is not modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertChecksumErrorsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrors(YangBase):
	"""Controls the insertion of message length errors into the IGMP messages by the IGMP stack.            TRUE    :Every IGMP packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	:IGMP packet lengths will not be modified.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'insert-length-errors'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrors, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsRandom(self))

	def delete(self):
		"""Delete this insert-length-errors instance on the server.

		Controls the insertion of message length errors into the IGMP messages by the IGMP stack.            TRUE    :Every IGMP packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	:IGMP packet lengths will not be modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this insert-length-errors instance on the server with any changed property values.

		Controls the insertion of message length errors into the IGMP messages by the IGMP stack.            TRUE    :Every IGMP packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	:IGMP packet lengths will not be modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpInsertLengthErrorsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragment(YangBase):
	"""Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-dont-fragment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragment, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentRandom(self))

	def delete(self):
		"""Delete this ipv4-dont-fragment instance on the server.

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-dont-fragment instance on the server with any changed property values.

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4DontFragmentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReports(YangBase):
	"""Allows IGMPv3 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets..            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pack-reports'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReports, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsRandom(self))

	def delete(self):
		"""Delete this pack-reports instance on the server.

		Allows IGMPv3 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets..            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pack-reports instance on the server with any changed property values.

		Allows IGMPv3 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets..            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpPackReportsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlert(YangBase):
	"""Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-alert'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlert, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertRandom(self))

	def delete(self):
		"""Delete this router-alert instance on the server.

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-alert instance on the server with any changed property values.

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :Reports will contain multiple group records.            FALSE	:An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRouterAlertRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4Tos(YangBase):
	"""Provides an indication of the quality of service wanted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-tos'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4Tos, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosRandom(self))

	def delete(self):
		"""Delete this ipv4-tos instance on the server.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-tos instance on the server with any changed property values.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpIpv4TosRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoin(YangBase):
	"""When an IGMPv1/IGMPv2 host joins a multicast group, it immediately transmits an initial unsolicited            membership report for that group, in case it is the first member of that group on the network.            In case the initial report gets damaged or lost, a second unsolicited report is recommended be sent out.            The force-robust-join flag controls whether or not a second report is transmitted.            TRUE    :Forces the host to send a second IGMPv1/IGMPv2 join report.            FALSE	:Does not force the host to send a second IGMPv1/IGMPv2 join report.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-robust-join'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoin, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinRandom(self))

	def delete(self):
		"""Delete this force-robust-join instance on the server.

		When an IGMPv1/IGMPv2 host joins a multicast group, it immediately transmits an initial unsolicited            membership report for that group, in case it is the first member of that group on the network.            In case the initial report gets damaged or lost, a second unsolicited report is recommended be sent out.            The force-robust-join flag controls whether or not a second report is transmitted.            TRUE    :Forces the host to send a second IGMPv1/IGMPv2 join report.            FALSE	:Does not force the host to send a second IGMPv1/IGMPv2 join report.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-robust-join instance on the server with any changed property values.

		When an IGMPv1/IGMPv2 host joins a multicast group, it immediately transmits an initial unsolicited            membership report for that group, in case it is the first member of that group on the network.            In case the initial report gets damaged or lost, a second unsolicited report is recommended be sent out.            The force-robust-join flag controls whether or not a second report is transmitted.            TRUE    :Forces the host to send a second IGMPv1/IGMPv2 join report.            FALSE	:Does not force the host to send a second IGMPv1/IGMPv2 join report.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpForceRobustJoinRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariable(YangBase):
	"""Robustness Variable, which is used in the calculationof default values for various timers and counters.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'robustness-variable'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariable, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableRandom(self))

	def delete(self):
		"""Delete this robustness-variable instance on the server.

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this robustness-variable instance on the server with any changed property values.

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpRobustnessVariableRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportInterval(YangBase):
	"""Time between repetitions of a host's initial report of membership in a group.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unsolicited-report-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalRandom(self))

	def delete(self):
		"""Delete this unsolicited-report-interval instance on the server.

		Time between repetitions of a host's initial report of membership in a group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unsolicited-report-interval instance on the server with any changed property values.

		Time between repetitions of a host's initial report of membership in a group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpUnsolicitedReportIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeout(YangBase):
	"""Amount of time a host must wait after hearing a Version 1 Query before it may send any IGMPv2 messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'v1-router-present-timeout'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeout, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutRandom(self))

	def delete(self):
		"""Delete this v1-router-present-timeout instance on the server.

		Amount of time a host must wait after hearing a Version 1 Query before it may send any IGMPv2 messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this v1-router-present-timeout instance on the server with any changed property values.

		Amount of time a host must wait after hearing a Version 1 Query before it may send any IGMPv2 messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpV1RouterPresentTimeoutRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerier(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-querier'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerier, self).__init__(parent, None)

	def version(self):
		"""Get the version object from the server

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierVersion(self))

	def ipv4_dont_fragment(self):
		"""Get the ipv4-dont-fragment object from the server

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragment`: An object encapsulating an instance of the ipv4-dont-fragment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragment(self))

	def router_alert(self):
		"""Get the router-alert object from the server

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :force the IPv4 header to add Router Alert Option information in IGMP Query packet.            FALSE	:the IPv4 header don't add Router Alert Option information in IGMP Query packet.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlert`: An object encapsulating an instance of the router-alert model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlert(self))

	def ignore_v1_reports(self):
		"""Get the ignore-v1-reports object from the server

		Ignore IGMPv1 Report messages. Only IGMPv2 Reportmessages are accepted by the router.            TRUE    : Ignores IGMPv1 messages.            FALSE	: Processes IGMPv1 messages

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1Reports`: An object encapsulating an instance of the ignore-v1-reports model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1Reports(self))

	def ipv4_tos(self):
		"""Get the ipv4-tos object from the server

		Provides an indication of the quality of service wanted.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4Tos`: An object encapsulating an instance of the ipv4-tos model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4Tos(self))

	def robustness_variable(self):
		"""Get the robustness-variable object from the server

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariable`: An object encapsulating an instance of the robustness-variable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariable(self))

	def last_member_query_count(self):
		"""Get the last-member-query-count object from the server

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCount`: An object encapsulating an instance of the last-member-query-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCount(self))

	def last_member_query_interval(self):
		"""Get the last-member-query-interval object from the server

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryInterval`: An object encapsulating an instance of the last-member-query-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryInterval(self))

	def query_interval(self):
		"""Get the query-interval object from the server

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryInterval`: An object encapsulating an instance of the query-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryInterval(self))

	def query_response_interval(self):
		"""Get the query-response-interval object from the server

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseInterval`: An object encapsulating an instance of the query-response-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseInterval(self))

	def start_up_query_count(self):
		"""Get the start-up-query-count object from the server

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCount`: An object encapsulating an instance of the start-up-query-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCount(self))

	def delete(self):
		"""Delete this igmp-querier instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmp-querier instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierVersion(YangBase):
	"""Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this version instance on the server.

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		Multicast protocol used to manage multicast group memberships                IGMP_V1 :   Second version (obsoleted IGMPv0) of IGMP, specified in RFC 1112.                IGMP_V2	:   IGMP version specified in RFC 2236. Improved IGMP version that adds                            leave messages, shortening the amount of time required for a router to                            determine that no stations are in a particular group.                IGMP_V3	:   Specified in RFC 3376, this major revision of the IGMP protocol adds the                            ability to specify the source(s) that a receiver is willing to listen to.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragment(YangBase):
	"""Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-dont-fragment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragment, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentRandom(self))

	def delete(self):
		"""Delete this ipv4-dont-fragment instance on the server.

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-dont-fragment instance on the server with any changed property values.

		Controls the fragmentation of packets larger than the MTU (Maximum Transmission Unit) size.            TRUE    :Packets larger than the allowable MTU are not fragmented.            FALSE	:Packets larger than the allowable MTU will be divided into fragments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4DontFragmentRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlert(YangBase):
	"""Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :force the IPv4 header to add Router Alert Option information in IGMP Query packet.            FALSE	:the IPv4 header don't add Router Alert Option information in IGMP Query packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-alert'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlert, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertRandom(self))

	def delete(self):
		"""Delete this router-alert instance on the server.

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :force the IPv4 header to add Router Alert Option information in IGMP Query packet.            FALSE	:the IPv4 header don't add Router Alert Option information in IGMP Query packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-alert instance on the server with any changed property values.

		Enable/Disable router alert option in the IPV4 header of IGMP packet.            TRUE    :force the IPv4 header to add Router Alert Option information in IGMP Query packet.            FALSE	:the IPv4 header don't add Router Alert Option information in IGMP Query packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRouterAlertRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1Reports(YangBase):
	"""Ignore IGMPv1 Report messages. Only IGMPv2 Reportmessages are accepted by the router.            TRUE    : Ignores IGMPv1 messages.            FALSE	: Processes IGMPv1 messages
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ignore-v1-reports'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1Reports, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsRandom(self))

	def delete(self):
		"""Delete this ignore-v1-reports instance on the server.

		Ignore IGMPv1 Report messages. Only IGMPv2 Reportmessages are accepted by the router.            TRUE    : Ignores IGMPv1 messages.            FALSE	: Processes IGMPv1 messages

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ignore-v1-reports instance on the server with any changed property values.

		Ignore IGMPv1 Report messages. Only IGMPv2 Reportmessages are accepted by the router.            TRUE    : Ignores IGMPv1 messages.            FALSE	: Processes IGMPv1 messages

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIgnoreV1ReportsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4Tos(YangBase):
	"""Provides an indication of the quality of service wanted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-tos'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4Tos, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosRandom(self))

	def delete(self):
		"""Delete this ipv4-tos instance on the server.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-tos instance on the server with any changed property values.

		Provides an indication of the quality of service wanted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierIpv4TosRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariable(YangBase):
	"""Robustness Variable, which is used in the calculation of default values for various timers and counters.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'robustness-variable'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariable, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableRandom(self))

	def delete(self):
		"""Delete this robustness-variable instance on the server.

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this robustness-variable instance on the server with any changed property values.

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierRobustnessVariableRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCount(YangBase):
	"""Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'last-member-query-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountRandom(self))

	def delete(self):
		"""Delete this last-member-query-count instance on the server.

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this last-member-query-count instance on the server with any changed property values.

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryInterval(YangBase):
	"""Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'last-member-query-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalRandom(self))

	def delete(self):
		"""Delete this last-member-query-interval instance on the server.

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this last-member-query-interval instance on the server with any changed property values.

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierLastMemberQueryIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryInterval(YangBase):
	"""Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'query-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalRandom(self))

	def delete(self):
		"""Delete this query-interval instance on the server.

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this query-interval instance on the server with any changed property values.

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseInterval(YangBase):
	"""Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'query-response-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalRandom(self))

	def delete(self):
		"""Delete this query-response-interval instance on the server.

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this query-response-interval instance on the server with any changed property values.

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierQueryResponseIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCount(YangBase):
	"""Number of queries sent out on startup, separated by the Startup Query Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-up-query-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountRandom(self))

	def delete(self):
		"""Delete this start-up-query-count instance on the server.

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-up-query-count instance on the server with any changed property values.

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsIgmpQuerierStartUpQueryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMld(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMld, self).__init__(parent, None)

	def version(self):
		"""Get the version object from the server

		Multicast protocol used to manage multicast group memberships                MLD_V1  :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2  :   Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldVersion(self))

	def force_leave(self):
		"""Get the force-leave object from the server

		Affects all hosts except the last one, which is always required to send an MLDv1 leave report.            MLDv1 hosts leaving a multicast group may optionally send a leave report to the all-routers            multicast group. This option controls whether or not all hosts are required to send leave reports            when leaving the multicast group..            TRUE    : Force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeave`: An object encapsulating an instance of the force-leave model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeave(self))

	def force_robust_join(self):
		"""Get the force-robust-join object from the server

		When an MLDv1 host joins a multicast group, it immediately transmits an initial unsolicited membership             report for that group, in case it is the first member of that group on the network. In case the initial             report gets damaged or lost, a second unsolicited report is recommended be sent out. This option controls             whether or not a second report is transmitted.            TRUE    : Forces the host to send a second MLDv1 join report.            FALSE	: Does not force the host to send a second MLDv1 join report.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoin`: An object encapsulating an instance of the force-robust-join model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoin(self))

	def force_simple_join(self):
		"""Get the force-simple-join object from the server

		Forces the MLD host(s) to send a single join report per group. Enabling this option effectively bypasses             the RFC defined behavior of sending (Robustness Variable - 1) reports. Otherwise, MLD host(s) will behave             according to RFC specification.            TRUE    : Force MLD host(s) to send a single join report per group.            FALSE	: Default behavior that is compliant to RFC specification. MLD host(s) will send (Robustness Variable - 1) reports.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoin`: An object encapsulating an instance of the force-simple-join model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoin(self))

	def insert_checksum_errors(self):
		"""Get the insert-checksum-errors object from the server

		Controls the insertion of checksum errors into MLD messages by the hardware.            TRUE    : MLD checksum of the transmitted packet is flipped by the protocol stack.            FALSE	: MLD checksum of the transmitted packet is not modified.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrors`: An object encapsulating an instance of the insert-checksum-errors model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrors(self))

	def insert_length_errors(self):
		"""Get the insert-length-errors object from the server

		Controls the insertion of message length errors into the MLD messages by the MLD stack.            TRUE    : Every MLD packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	: MLD packet lengths will not be modified.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrors`: An object encapsulating an instance of the insert-length-errors model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrors(self))

	def pack_reports(self):
		"""Get the pack-reports object from the server

		Allows MLDv2 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets.            TRUE    : Reports will contain multiple group records.            FALSE	: An individual report will be sent for each group record.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldPackReports`: An object encapsulating an instance of the pack-reports model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReports(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the value of the Traffic Class field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClass(self))

	def robustness_variable(self):
		"""Get the robustness-variable object from the server

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariable`: An object encapsulating an instance of the robustness-variable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariable(self))

	def unsolicited_report_interval(self):
		"""Get the unsolicited-report-interval object from the server

		Time between repetitions of a host's initial report of membership in a group.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportInterval`: An object encapsulating an instance of the unsolicited-report-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportInterval(self))

	def delete(self):
		"""Delete this mld instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mld instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def multicast_group(self):
		"""-> ../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldVersion(YangBase):
	"""Multicast protocol used to manage multicast group memberships                MLD_V1  :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2  :   Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this version instance on the server.

		Multicast protocol used to manage multicast group memberships                MLD_V1  :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2  :   Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		Multicast protocol used to manage multicast group memberships                MLD_V1  :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2  :   Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeave(YangBase):
	"""Affects all hosts except the last one, which is always required to send an MLDv1 leave report.            MLDv1 hosts leaving a multicast group may optionally send a leave report to the all-routers            multicast group. This option controls whether or not all hosts are required to send leave reports            when leaving the multicast group..            TRUE    : Force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-leave'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeave, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveRandom(self))

	def delete(self):
		"""Delete this force-leave instance on the server.

		Affects all hosts except the last one, which is always required to send an MLDv1 leave report.            MLDv1 hosts leaving a multicast group may optionally send a leave report to the all-routers            multicast group. This option controls whether or not all hosts are required to send leave reports            when leaving the multicast group..            TRUE    : Force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-leave instance on the server with any changed property values.

		Affects all hosts except the last one, which is always required to send an MLDv1 leave report.            MLDv1 hosts leaving a multicast group may optionally send a leave report to the all-routers            multicast group. This option controls whether or not all hosts are required to send leave reports            when leaving the multicast group..            TRUE    : Force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.            FALSE	: Do not force the MLDv1 host(s) to send a Leave Group message when leaving a multicast group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceLeaveRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoin(YangBase):
	"""When an MLDv1 host joins a multicast group, it immediately transmits an initial unsolicited membership             report for that group, in case it is the first member of that group on the network. In case the initial             report gets damaged or lost, a second unsolicited report is recommended be sent out. This option controls             whether or not a second report is transmitted.            TRUE    : Forces the host to send a second MLDv1 join report.            FALSE	: Does not force the host to send a second MLDv1 join report.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-robust-join'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoin, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinRandom(self))

	def delete(self):
		"""Delete this force-robust-join instance on the server.

		When an MLDv1 host joins a multicast group, it immediately transmits an initial unsolicited membership             report for that group, in case it is the first member of that group on the network. In case the initial             report gets damaged or lost, a second unsolicited report is recommended be sent out. This option controls             whether or not a second report is transmitted.            TRUE    : Forces the host to send a second MLDv1 join report.            FALSE	: Does not force the host to send a second MLDv1 join report.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-robust-join instance on the server with any changed property values.

		When an MLDv1 host joins a multicast group, it immediately transmits an initial unsolicited membership             report for that group, in case it is the first member of that group on the network. In case the initial             report gets damaged or lost, a second unsolicited report is recommended be sent out. This option controls             whether or not a second report is transmitted.            TRUE    : Forces the host to send a second MLDv1 join report.            FALSE	: Does not force the host to send a second MLDv1 join report.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceRobustJoinRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoin(YangBase):
	"""Forces the MLD host(s) to send a single join report per group. Enabling this option effectively bypasses             the RFC defined behavior of sending (Robustness Variable - 1) reports. Otherwise, MLD host(s) will behave             according to RFC specification.            TRUE    : Force MLD host(s) to send a single join report per group.            FALSE	: Default behavior that is compliant to RFC specification. MLD host(s) will send (Robustness Variable - 1) reports.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'force-simple-join'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoin, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinRandom(self))

	def delete(self):
		"""Delete this force-simple-join instance on the server.

		Forces the MLD host(s) to send a single join report per group. Enabling this option effectively bypasses             the RFC defined behavior of sending (Robustness Variable - 1) reports. Otherwise, MLD host(s) will behave             according to RFC specification.            TRUE    : Force MLD host(s) to send a single join report per group.            FALSE	: Default behavior that is compliant to RFC specification. MLD host(s) will send (Robustness Variable - 1) reports.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this force-simple-join instance on the server with any changed property values.

		Forces the MLD host(s) to send a single join report per group. Enabling this option effectively bypasses             the RFC defined behavior of sending (Robustness Variable - 1) reports. Otherwise, MLD host(s) will behave             according to RFC specification.            TRUE    : Force MLD host(s) to send a single join report per group.            FALSE	: Default behavior that is compliant to RFC specification. MLD host(s) will send (Robustness Variable - 1) reports.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldForceSimpleJoinRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrors(YangBase):
	"""Controls the insertion of checksum errors into MLD messages by the hardware.            TRUE    : MLD checksum of the transmitted packet is flipped by the protocol stack.            FALSE	: MLD checksum of the transmitted packet is not modified.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'insert-checksum-errors'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrors, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsRandom(self))

	def delete(self):
		"""Delete this insert-checksum-errors instance on the server.

		Controls the insertion of checksum errors into MLD messages by the hardware.            TRUE    : MLD checksum of the transmitted packet is flipped by the protocol stack.            FALSE	: MLD checksum of the transmitted packet is not modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this insert-checksum-errors instance on the server with any changed property values.

		Controls the insertion of checksum errors into MLD messages by the hardware.            TRUE    : MLD checksum of the transmitted packet is flipped by the protocol stack.            FALSE	: MLD checksum of the transmitted packet is not modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertChecksumErrorsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrors(YangBase):
	"""Controls the insertion of message length errors into the MLD messages by the MLD stack.            TRUE    : Every MLD packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	: MLD packet lengths will not be modified.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'insert-length-errors'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrors, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsRandom(self))

	def delete(self):
		"""Delete this insert-length-errors instance on the server.

		Controls the insertion of message length errors into the MLD messages by the MLD stack.            TRUE    : Every MLD packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	: MLD packet lengths will not be modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this insert-length-errors instance on the server with any changed property values.

		Controls the insertion of message length errors into the MLD messages by the MLD stack.            TRUE    : Every MLD packet transmitted by the host will be 2 bytes shorter than normal.            FALSE	: MLD packet lengths will not be modified.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldInsertLengthErrorsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldPackReports(YangBase):
	"""Allows MLDv2 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets.            TRUE    : Reports will contain multiple group records.            FALSE	: An individual report will be sent for each group record.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pack-reports'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReports, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsRandom(self))

	def delete(self):
		"""Delete this pack-reports instance on the server.

		Allows MLDv2 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets.            TRUE    : Reports will contain multiple group records.            FALSE	: An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pack-reports instance on the server with any changed property values.

		Allows MLDv2 host(s) to send reports that contain multiple group records,            to allow reporting of full current state using fewer packets.            TRUE    : Reports will contain multiple group records.            FALSE	: An individual report will be sent for each group record.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldPackReportsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClass(YangBase):
	"""Specifies the value of the Traffic Class field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the value of the Traffic Class field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the value of the Traffic Class field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariable(YangBase):
	"""Robustness Variable, which is used in the calculationof default values for various timers and counters.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'robustness-variable'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariable, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableRandom(self))

	def delete(self):
		"""Delete this robustness-variable instance on the server.

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this robustness-variable instance on the server with any changed property values.

		Robustness Variable, which is used in the calculationof default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldRobustnessVariableRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportInterval(YangBase):
	"""Time between repetitions of a host's initial report of membership in a group.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unsolicited-report-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalRandom(self))

	def delete(self):
		"""Delete this unsolicited-report-interval instance on the server.

		Time between repetitions of a host's initial report of membership in a group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unsolicited-report-interval instance on the server with any changed property values.

		Time between repetitions of a host's initial report of membership in a group.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldUnsolicitedReportIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerier(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-querier'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerier, self).__init__(parent, None)

	def version(self):
		"""Get the version object from the server

		Multicast protocol used to manage multicast group memberships                MLD_V1 :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2	:  Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierVersion(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the value of the Traffic Class field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClass(self))

	def robustness_variable(self):
		"""Get the robustness-variable object from the server

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariable`: An object encapsulating an instance of the robustness-variable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariable(self))

	def last_member_query_count(self):
		"""Get the last-member-query-count object from the server

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCount`: An object encapsulating an instance of the last-member-query-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCount(self))

	def last_member_query_interval(self):
		"""Get the last-member-query-interval object from the server

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryInterval`: An object encapsulating an instance of the last-member-query-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryInterval(self))

	def query_interval(self):
		"""Get the query-interval object from the server

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryInterval`: An object encapsulating an instance of the query-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryInterval(self))

	def query_response_interval(self):
		"""Get the query-response-interval object from the server

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseInterval`: An object encapsulating an instance of the query-response-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseInterval(self))

	def start_up_query_count(self):
		"""Get the start-up-query-count object from the server

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCount`: An object encapsulating an instance of the start-up-query-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCount(self))

	def delete(self):
		"""Delete this mld-querier instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mld-querier instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierVersion(YangBase):
	"""Multicast protocol used to manage multicast group memberships                MLD_V1 :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2	:  Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this version instance on the server.

		Multicast protocol used to manage multicast group memberships                MLD_V1 :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2	:  Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		Multicast protocol used to manage multicast group memberships                MLD_V1 :   Initial multicast protocol version for IPv6, similar to IGMPv2. It is specified in RFC 2710.                MLD_V2	:  Version of MLD, specified in RFC 3810, that adds the include and exclude filter functionality as in IGMPv3.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClass(YangBase):
	"""Specifies the value of the Traffic Class field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the value of the Traffic Class field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the value of the Traffic Class field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariable(YangBase):
	"""Robustness Variable, which is used in the calculation of default values for various timers and counters.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'robustness-variable'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariable, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableRandom(self))

	def delete(self):
		"""Delete this robustness-variable instance on the server.

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this robustness-variable instance on the server with any changed property values.

		Robustness Variable, which is used in the calculation of default values for various timers and counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierRobustnessVariableRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCount(YangBase):
	"""Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'last-member-query-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountRandom(self))

	def delete(self):
		"""Delete this last-member-query-count instance on the server.

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this last-member-query-count instance on the server with any changed property values.

		Value for the Max Response Time field (in milliseconds) that is inserted into Group-Specific and Group-and-Source-Specific Query messages in response to Leave Group messages. This value also specifies the interval between transmissions of Group-Specific and Group-and-Source-Specific Query messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryInterval(YangBase):
	"""Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'last-member-query-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalRandom(self))

	def delete(self):
		"""Delete this last-member-query-interval instance on the server.

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this last-member-query-interval instance on the server with any changed property values.

		Maximum amount of time between group-specific query messages, including those sent in response to leave-group messages..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierLastMemberQueryIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryInterval(YangBase):
	"""Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'query-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalRandom(self))

	def delete(self):
		"""Delete this query-interval instance on the server.

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this query-interval instance on the server with any changed property values.

		Duration (in seconds) between transmissions of General Query messages. General Query messages are used to learn which multicast groups have members on a connected network.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseInterval(YangBase):
	"""Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'query-response-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalRandom(self))

	def delete(self):
		"""Delete this query-response-interval instance on the server.

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this query-response-interval instance on the server with any changed property values.

		Value for the Max Response Time field (in milliseconds) that is inserted into the General Query messages. This time is the maximum amount allowed for a host to send a responding report to the General Query message. This interval must be less than the Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierQueryResponseIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCount(YangBase):
	"""Number of queries sent out on startup, separated by the Startup Query Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'start-up-query-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountRandom(self))

	def delete(self):
		"""Delete this start-up-query-count instance on the server.

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this start-up-query-count instance on the server with any changed property values.

		Number of queries sent out on startup, separated by the Startup Query Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsMldQuerierStartUpQueryCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPim(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pim'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPim, self).__init__(parent, None)

	def pim_mode(self):
		"""Get the pim-mode object from the server

		Multicast protocol used to manage multicast group memberships            SM	: Router supports all three group types: (S,G), (*,*,RP), and (*,G).            SSM	: Router will not send Join/Prune messages for groups that are not (S,G).                You cannot add (*,*,RP) or (*,G) groups.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimPimMode`: An object encapsulating an instance of the pim-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimPimMode(self))

	def ip_version(self):
		"""Get the ip-version object from the server

		IP version to be used for communication with the neighbor.                IPV4	: Uses IP version 4.                IPV6	: Uses IP version 6.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpVersion`: An object encapsulating an instance of the ip-version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpVersion(self))

	def gen_id_mode(self):
		"""Get the gen-id-mode object from the server

		In PIM, routers send the Generation ID as an option in the Hello messages.             Every time a router starts up, it selects a random number. The router uses             that number as long as it remains operational. If the Generation ID changes,             it is an indication to the neighboring routers that this router has gone through             a shutdown-restart cycle. This causes the neighboring routers to reset their             databases and start fresh. If the generation ID mode is Fixed, it emulates normal             PIM router behavior. Incremental or Random means Spirent TestCenter will send             different generation IDs in successive Hello messages, causing the DUTs to reset             and rebuild their databases often.             The interactive menu option, Increment Generation ID, sends one Hello message with             incremented Generation ID                FIXED	    : Emulates normal PIM router behavior.                INCREMENT   : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            incrementing by one each time.                RANDOM	    : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            selecting a random number each time.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimGenIdMode`: An object encapsulating an instance of the gen-id-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimGenIdMode(self))

	def bi_dir_option(self):
		"""Get the bi-dir-option object from the server

		Include the BI-DIR option in Hellomessages.            TRUE    : Enables sending BI-DIR option in Hello messages.            FALSE	: Disables sending BI-DIR option in Hello messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOption`: An object encapsulating an instance of the bi-dir-option model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOption(self))

	def bootstrap_message_interval(self):
		"""Get the bootstrap-message-interval object from the server

		Frequency in seconds with which bootstrap messages are transmitted.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageInterval`: An object encapsulating an instance of the bootstrap-message-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageInterval(self))

	def bsr_priority(self):
		"""Get the bsr-priority object from the server

		8-bit priority of the emulated bootstrap router (BSR).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriority`: An object encapsulating an instance of the bsr-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriority(self))

	def dr_priority(self):
		"""Get the dr-priority object from the server

		Designated Router (DR) priority of this router. This value is sent as an option in the Hello message.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriority`: An object encapsulating an instance of the dr-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriority(self))

	def enable_bsr(self):
		"""Get the enable-bsr object from the server

		PIM routers enabled for bootstrap routing (BSR) functionality generate Bootstrap messages periodically.            TRUE    : Enables sending Bootstrap messages.            FALSE	: Disables sending Bootstrap messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsr`: An object encapsulating an instance of the enable-bsr model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsr(self))

	def hello_holdtime(self):
		"""Get the hello-holdtime object from the server

		Hold time in seconds to keep neighbor state alive.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtime`: An object encapsulating an instance of the hello-holdtime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtime(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Periodic interval in seconds for Hello messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloInterval(self))

	def join_prune_holdtime(self):
		"""Get the join-prune-holdtime object from the server

		Hold time in seconds to advertise in Join/Prune messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtime`: An object encapsulating an instance of the join-prune-holdtime model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtime(self))

	def join_prune_interval(self):
		"""Get the join-prune-interval object from the server

		Frequency in seconds at which Join/Prune messages are sent.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneInterval`: An object encapsulating an instance of the join-prune-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneInterval(self))

	def null_register_only_mode(self):
		"""Get the null-register-only-mode object from the server

		Determines whether this PIM router supports full emulation or only NULL register messages.    TRUE    : This PIM router only supports NULL register messages.    FALSE	: This PIM router supports the full emulation.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyMode`: An object encapsulating an instance of the null-register-only-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyMode(self))

	def neighbor_ipv4_address(self):
		"""Get the neighbor-ipv4-address object from the server

		IPv4 upstream neighbor address to be used in Join/Prune messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4Address`: An object encapsulating an instance of the neighbor-ipv4-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4Address(self))

	def neighbor_ipv6_address(self):
		"""Get the neighbor-ipv6-address object from the server

		IPv6 upstream neighbor address to be used in Join/Prune messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6Address`: An object encapsulating an instance of the neighbor-ipv6-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6Address(self))

	def ipv4_groups(self, name=None):
		"""Get the ipv4-groups object(s) from the server.

		A list of IPv4 Multicast Group block.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv4-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups object.  

		Raises:  
			NotFoundError: name is not in the list of ipv4-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups(self, name))

	def create_ipv4_groups(self, name):
		"""Create a child sibling ipv4-groups instance on the server.

		A list of IPv4 Multicast Group block.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups`: An object encapsulating an instance of the ipv4-groups model.  

		Raises:  
			AlreadyExistsError: An instance of ipv4-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups(self, name), locals())

	def ipv6_groups(self, name=None):
		"""Get the ipv6-groups object(s) from the server.

		A list of IPv6 Multicast Group block.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv6-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups object.  

		Raises:  
			NotFoundError: name is not in the list of ipv6-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups(self, name))

	def create_ipv6_groups(self, name):
		"""Create a child sibling ipv6-groups instance on the server.

		A list of IPv6 Multicast Group block.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups`: An object encapsulating an instance of the ipv6-groups model.  

		Raises:  
			AlreadyExistsError: An instance of ipv6-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups(self, name), locals())

	def ipv4_rp_maps(self, name=None):
		"""Get the ipv4-rp-maps object(s) from the server.

		A list of IPv4 Group-RP (Rendezvous Point) mapping.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv4-rp-maps list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps object.  

		Raises:  
			NotFoundError: name is not in the list of ipv4-rp-maps objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps(self, name))

	def create_ipv4_rp_maps(self, name):
		"""Create a child sibling ipv4-rp-maps instance on the server.

		A list of IPv4 Group-RP (Rendezvous Point) mapping.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps`: An object encapsulating an instance of the ipv4-rp-maps model.  

		Raises:  
			AlreadyExistsError: An instance of ipv4-rp-maps with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps(self, name), locals())

	def ipv6_rp_maps(self, name=None):
		"""Get the ipv6-rp-maps object(s) from the server.

		A list of IPv6 Group-RP (Rendezvous Point) mapping.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv6-rp-maps list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps object.  

		Raises:  
			NotFoundError: name is not in the list of ipv6-rp-maps objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps(self, name))

	def create_ipv6_rp_maps(self, name):
		"""Create a child sibling ipv6-rp-maps instance on the server.

		A list of IPv6 Group-RP (Rendezvous Point) mapping.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps`: An object encapsulating an instance of the ipv6-rp-maps model.  

		Raises:  
			AlreadyExistsError: An instance of ipv6-rp-maps with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps(self, name), locals())

	def ipv4_register_groups(self, name=None):
		"""Get the ipv4-register-groups object(s) from the server.

		A list of IPv4 PIM Registers block.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv4-register-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups object.  

		Raises:  
			NotFoundError: name is not in the list of ipv4-register-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups(self, name))

	def create_ipv4_register_groups(self, name):
		"""Create a child sibling ipv4-register-groups instance on the server.

		A list of IPv4 PIM Registers block.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups`: An object encapsulating an instance of the ipv4-register-groups model.  

		Raises:  
			AlreadyExistsError: An instance of ipv4-register-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups(self, name), locals())

	def ipv6_register_groups(self, name=None):
		"""Get the ipv6-register-groups object(s) from the server.

		A list of IPv6 PIM Registers block.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv6-register-groups list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups` | :obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups`: If arg name is None a list of SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups objects otherwise a single SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups object.  

		Raises:  
			NotFoundError: name is not in the list of ipv6-register-groups objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups(self, name))

	def create_ipv6_register_groups(self, name):
		"""Create a child sibling ipv6-register-groups instance on the server.

		A list of IPv6 PIM Registers block.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups`: An object encapsulating an instance of the ipv6-register-groups model.  

		Raises:  
			AlreadyExistsError: An instance of ipv6-register-groups with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups(self, name), locals())

	def delete(self):
		"""Delete this pim instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pim instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsDevicesProtocolsPimPimMode(YangBase):
	"""Multicast protocol used to manage multicast group memberships            SM	: Router supports all three group types: (S,G), (*,*,RP), and (*,G).            SSM	: Router will not send Join/Prune messages for groups that are not (S,G).                You cannot add (*,*,RP) or (*,G) groups.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pim-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimPimMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this pim-mode instance on the server.

		Multicast protocol used to manage multicast group memberships            SM	: Router supports all three group types: (S,G), (*,*,RP), and (*,G).            SSM	: Router will not send Join/Prune messages for groups that are not (S,G).                You cannot add (*,*,RP) or (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pim-mode instance on the server with any changed property values.

		Multicast protocol used to manage multicast group memberships            SM	: Router supports all three group types: (S,G), (*,*,RP), and (*,G).            SSM	: Router will not send Join/Prune messages for groups that are not (S,G).                You cannot add (*,*,RP) or (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpVersion(YangBase):
	"""IP version to be used for communication with the neighbor.                IPV4	: Uses IP version 4.                IPV6	: Uses IP version 6.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ip-version'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpVersion, self).__init__(parent, None)

	def delete(self):
		"""Delete this ip-version instance on the server.

		IP version to be used for communication with the neighbor.                IPV4	: Uses IP version 4.                IPV6	: Uses IP version 6.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ip-version instance on the server with any changed property values.

		IP version to be used for communication with the neighbor.                IPV4	: Uses IP version 4.                IPV6	: Uses IP version 6.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimGenIdMode(YangBase):
	"""In PIM, routers send the Generation ID as an option in the Hello messages.             Every time a router starts up, it selects a random number. The router uses             that number as long as it remains operational. If the Generation ID changes,             it is an indication to the neighboring routers that this router has gone through             a shutdown-restart cycle. This causes the neighboring routers to reset their             databases and start fresh. If the generation ID mode is Fixed, it emulates normal             PIM router behavior. Incremental or Random means Spirent TestCenter will send             different generation IDs in successive Hello messages, causing the DUTs to reset             and rebuild their databases often.             The interactive menu option, Increment Generation ID, sends one Hello message with             incremented Generation ID                FIXED	    : Emulates normal PIM router behavior.                INCREMENT   : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            incrementing by one each time.                RANDOM	    : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            selecting a random number each time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gen-id-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimGenIdMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this gen-id-mode instance on the server.

		In PIM, routers send the Generation ID as an option in the Hello messages.             Every time a router starts up, it selects a random number. The router uses             that number as long as it remains operational. If the Generation ID changes,             it is an indication to the neighboring routers that this router has gone through             a shutdown-restart cycle. This causes the neighboring routers to reset their             databases and start fresh. If the generation ID mode is Fixed, it emulates normal             PIM router behavior. Incremental or Random means Spirent TestCenter will send             different generation IDs in successive Hello messages, causing the DUTs to reset             and rebuild their databases often.             The interactive menu option, Increment Generation ID, sends one Hello message with             incremented Generation ID                FIXED	    : Emulates normal PIM router behavior.                INCREMENT   : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            incrementing by one each time.                RANDOM	    : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            selecting a random number each time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gen-id-mode instance on the server with any changed property values.

		In PIM, routers send the Generation ID as an option in the Hello messages.             Every time a router starts up, it selects a random number. The router uses             that number as long as it remains operational. If the Generation ID changes,             it is an indication to the neighboring routers that this router has gone through             a shutdown-restart cycle. This causes the neighboring routers to reset their             databases and start fresh. If the generation ID mode is Fixed, it emulates normal             PIM router behavior. Incremental or Random means Spirent TestCenter will send             different generation IDs in successive Hello messages, causing the DUTs to reset             and rebuild their databases often.             The interactive menu option, Increment Generation ID, sends one Hello message with             incremented Generation ID                FIXED	    : Emulates normal PIM router behavior.                INCREMENT   : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            incrementing by one each time.                RANDOM	    : Spirent TestCenter sends different Generation IDs in successive Hello messages,                            selecting a random number each time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOption(YangBase):
	"""Include the BI-DIR option in Hellomessages.            TRUE    : Enables sending BI-DIR option in Hello messages.            FALSE	: Disables sending BI-DIR option in Hello messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bi-dir-option'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOption, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionRandom(self))

	def delete(self):
		"""Delete this bi-dir-option instance on the server.

		Include the BI-DIR option in Hellomessages.            TRUE    : Enables sending BI-DIR option in Hello messages.            FALSE	: Disables sending BI-DIR option in Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bi-dir-option instance on the server with any changed property values.

		Include the BI-DIR option in Hellomessages.            TRUE    : Enables sending BI-DIR option in Hello messages.            FALSE	: Disables sending BI-DIR option in Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBiDirOptionRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageInterval(YangBase):
	"""Frequency in seconds with which bootstrap messages are transmitted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bootstrap-message-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalRandom(self))

	def delete(self):
		"""Delete this bootstrap-message-interval instance on the server.

		Frequency in seconds with which bootstrap messages are transmitted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bootstrap-message-interval instance on the server with any changed property values.

		Frequency in seconds with which bootstrap messages are transmitted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBootstrapMessageIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriority(YangBase):
	"""8-bit priority of the emulated bootstrap router (BSR).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bsr-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityRandom(self))

	def delete(self):
		"""Delete this bsr-priority instance on the server.

		8-bit priority of the emulated bootstrap router (BSR).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bsr-priority instance on the server with any changed property values.

		8-bit priority of the emulated bootstrap router (BSR).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimBsrPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriority(YangBase):
	"""Designated Router (DR) priority of this router. This value is sent as an option in the Hello message.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dr-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityRandom(self))

	def delete(self):
		"""Delete this dr-priority instance on the server.

		Designated Router (DR) priority of this router. This value is sent as an option in the Hello message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this dr-priority instance on the server with any changed property values.

		Designated Router (DR) priority of this router. This value is sent as an option in the Hello message.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimDrPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsr(YangBase):
	"""PIM routers enabled for bootstrap routing (BSR) functionality generate Bootstrap messages periodically.            TRUE    : Enables sending Bootstrap messages.            FALSE	: Disables sending Bootstrap messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enable-bsr'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsr, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrRandom(self))

	def delete(self):
		"""Delete this enable-bsr instance on the server.

		PIM routers enabled for bootstrap routing (BSR) functionality generate Bootstrap messages periodically.            TRUE    : Enables sending Bootstrap messages.            FALSE	: Disables sending Bootstrap messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enable-bsr instance on the server with any changed property values.

		PIM routers enabled for bootstrap routing (BSR) functionality generate Bootstrap messages periodically.            TRUE    : Enables sending Bootstrap messages.            FALSE	: Disables sending Bootstrap messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimEnableBsrRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtime(YangBase):
	"""Hold time in seconds to keep neighbor state alive.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-holdtime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeRandom(self))

	def delete(self):
		"""Delete this hello-holdtime instance on the server.

		Hold time in seconds to keep neighbor state alive.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-holdtime instance on the server with any changed property values.

		Hold time in seconds to keep neighbor state alive.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloHoldtimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloInterval(YangBase):
	"""Periodic interval in seconds for Hello messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Periodic interval in seconds for Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Periodic interval in seconds for Hello messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimHelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtime(YangBase):
	"""Hold time in seconds to advertise in Join/Prune messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-prune-holdtime'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeRandom(self))

	def delete(self):
		"""Delete this join-prune-holdtime instance on the server.

		Hold time in seconds to advertise in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-prune-holdtime instance on the server with any changed property values.

		Hold time in seconds to advertise in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneHoldtimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneInterval(YangBase):
	"""Frequency in seconds at which Join/Prune messages are sent.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-prune-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalRandom(self))

	def delete(self):
		"""Delete this join-prune-interval instance on the server.

		Frequency in seconds at which Join/Prune messages are sent.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-prune-interval instance on the server with any changed property values.

		Frequency in seconds at which Join/Prune messages are sent.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimJoinPruneIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyMode(YangBase):
	"""Determines whether this PIM router supports full emulation or only NULL register messages.    TRUE    : This PIM router only supports NULL register messages.    FALSE	: This PIM router supports the full emulation.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'null-register-only-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyMode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeRandom(self))

	def delete(self):
		"""Delete this null-register-only-mode instance on the server.

		Determines whether this PIM router supports full emulation or only NULL register messages.    TRUE    : This PIM router only supports NULL register messages.    FALSE	: This PIM router supports the full emulation.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this null-register-only-mode instance on the server with any changed property values.

		Determines whether this PIM router supports full emulation or only NULL register messages.    TRUE    : This PIM router only supports NULL register messages.    FALSE	: This PIM router supports the full emulation.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNullRegisterOnlyModeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4Address(YangBase):
	"""IPv4 upstream neighbor address to be used in Join/Prune messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbor-ipv4-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressRandom(self))

	def delete(self):
		"""Delete this neighbor-ipv4-address instance on the server.

		IPv4 upstream neighbor address to be used in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbor-ipv4-address instance on the server with any changed property values.

		IPv4 upstream neighbor address to be used in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv4AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6Address(YangBase):
	"""IPv6 upstream neighbor address to be used in Join/Prune messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbor-ipv6-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6Address, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressRandom(self))

	def delete(self):
		"""Delete this neighbor-ipv6-address instance on the server.

		IPv6 upstream neighbor address to be used in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbor-ipv6-address instance on the server with any changed property values.

		IPv6 upstream neighbor address to be used in Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimNeighborIpv6AddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups(YangBase):
	"""A list of IPv4 Multicast Group block.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4Groups, self).__init__(parent, yang_key_value)

	def group_type(self):
		"""Get the group-type object from the server

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsGroupType`: An object encapsulating an instance of the group-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsGroupType(self))

	def rp_address(self):
		"""Get the rp-address object from the server

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddress(self))

	def join_source_address(self):
		"""Get the join-source-address object from the server

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddress`: An object encapsulating an instance of the join-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddress(self))

	def join_source_prefix(self):
		"""Get the join-source-prefix object from the server

		Prefix length of the joined sourceIP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefix`: An object encapsulating an instance of the join-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefix(self))

	def enabling_pruning(self):
		"""Get the enabling-pruning object from the server

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruning`: An object encapsulating an instance of the enabling-pruning model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruning(self))

	def prune_source_address(self):
		"""Get the prune-source-address object from the server

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request thatNO traffic be sent (i.e., do not send traffic to this group from this source).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddress`: An object encapsulating an instance of the prune-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddress(self))

	def prune_source_prefix(self):
		"""Get the prune-source-prefix object from the server

		Prefix length of the pruned source IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefix`: An object encapsulating an instance of the prune-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefix(self))

	def delete(self):
		"""Delete this ipv4-groups instance on the server.

		A list of IPv4 Multicast Group block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-groups instance on the server with any changed property values.

		A list of IPv4 Multicast Group block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the PIM IPv4 group block object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsGroupType(YangBase):
	"""Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsGroupType, self).__init__(parent, None)

	def delete(self):
		"""Delete this group-type instance on the server.

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-type instance on the server with any changed property values.

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddress(YangBase):
	"""Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddress(YangBase):
	"""The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressRandom(self))

	def delete(self):
		"""Delete this join-source-address instance on the server.

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-source-address instance on the server with any changed property values.

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefix(YangBase):
	"""Prefix length of the joined sourceIP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixRandom(self))

	def delete(self):
		"""Delete this join-source-prefix instance on the server.

		Prefix length of the joined sourceIP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-source-prefix instance on the server with any changed property values.

		Prefix length of the joined sourceIP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsJoinSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruning(YangBase):
	"""Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enabling-pruning'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruning, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningRandom(self))

	def delete(self):
		"""Delete this enabling-pruning instance on the server.

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enabling-pruning instance on the server with any changed property values.

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsEnablingPruningRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddress(YangBase):
	"""The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request thatNO traffic be sent (i.e., do not send traffic to this group from this source).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prune-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressRandom(self))

	def delete(self):
		"""Delete this prune-source-address instance on the server.

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request thatNO traffic be sent (i.e., do not send traffic to this group from this source).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prune-source-address instance on the server with any changed property values.

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request thatNO traffic be sent (i.e., do not send traffic to this group from this source).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefix(YangBase):
	"""Prefix length of the pruned source IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prune-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixRandom(self))

	def delete(self):
		"""Delete this prune-source-prefix instance on the server.

		Prefix length of the pruned source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prune-source-prefix instance on the server with any changed property values.

		Prefix length of the pruned source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4GroupsPruneSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups(YangBase):
	"""A list of IPv6 Multicast Group block.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6Groups, self).__init__(parent, yang_key_value)

	def group_type(self):
		"""Get the group-type object from the server

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsGroupType`: An object encapsulating an instance of the group-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsGroupType(self))

	def rp_address(self):
		"""Get the rp-address object from the server

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddress(self))

	def join_source_address(self):
		"""Get the join-source-address object from the server

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddress`: An object encapsulating an instance of the join-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddress(self))

	def join_source_prefix(self):
		"""Get the join-source-prefix object from the server

		Prefix length of the joined source IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefix`: An object encapsulating an instance of the join-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefix(self))

	def enabling_pruning(self):
		"""Get the enabling-pruning object from the server

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruning`: An object encapsulating an instance of the enabling-pruning model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruning(self))

	def prune_source_address(self):
		"""Get the prune-source-address object from the server

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request that NO traffic be sent (i.e., do not send traffic to this group from this source).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddress`: An object encapsulating an instance of the prune-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddress(self))

	def prune_source_prefix(self):
		"""Get the prune-source-prefix object from the server

		Prefix length of the pruned source IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefix`: An object encapsulating an instance of the prune-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefix(self))

	def delete(self):
		"""Delete this ipv6-groups instance on the server.

		A list of IPv6 Multicast Group block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-groups instance on the server with any changed property values.

		A list of IPv6 Multicast Group block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the PIM IPv6 group block object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsGroupType(YangBase):
	"""Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-type'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsGroupType, self).__init__(parent, None)

	def delete(self):
		"""Delete this group-type instance on the server.

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-type instance on the server with any changed property values.

		Group designation. Notation represents a routing entry. STARG	: Default group-specific multicast group set type that is used in        Join/Prune messages with (*,G) source list entries. SG	    : Group-specific multicast group set type that is used in Join/Prune         messages with (S,G) source-specific source list entries. STARSTARRP	: Wildcard multicast group set type. This is used in (*,*,RP)            Join/Prune messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddress(YangBase):
	"""Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		Rendezvous Point Router (RPR) address = a PIM router configured as the root of a multicast distribution tree. Required for (*,*,RP) and (*,G) groups.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddress(YangBase):
	"""The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressRandom(self))

	def delete(self):
		"""Delete this join-source-address instance on the server.

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-source-address instance on the server with any changed property values.

		The source block (Source Start IP Address and Source Prefix Length)represents the source address range from which the (S,G) grouprequests traffic. Each Join/Prune message corresponding to this (S,G) groupincludes the specified source address in its Join source list (or inthe Prune source list, if the group is being deleted or made inactive).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefix(YangBase):
	"""Prefix length of the joined source IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'join-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixRandom(self))

	def delete(self):
		"""Delete this join-source-prefix instance on the server.

		Prefix length of the joined source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this join-source-prefix instance on the server with any changed property values.

		Prefix length of the joined source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsJoinSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruning(YangBase):
	"""Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'enabling-pruning'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruning, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningRandom(self))

	def delete(self):
		"""Delete this enabling-pruning instance on the server.

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this enabling-pruning instance on the server with any changed property values.

		Enables including the optional Prune source for (*,G) groups. TRUE    : Enables Prune source. FALSE	: Disables Prune Source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsEnablingPruningRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddress(YangBase):
	"""The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request that NO traffic be sent (i.e., do not send traffic to this group from this source).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prune-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressRandom(self))

	def delete(self):
		"""Delete this prune-source-address instance on the server.

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request that NO traffic be sent (i.e., do not send traffic to this group from this source).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prune-source-address instance on the server with any changed property values.

		The Prune source address block (Prune IP Address and Prune Prefix Length)represents the source range from which (*,G) groups request that NO traffic be sent (i.e., do not send traffic to this group from this source).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefix(YangBase):
	"""Prefix length of the pruned source IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prune-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixRandom(self))

	def delete(self):
		"""Delete this prune-source-prefix instance on the server.

		Prefix length of the pruned source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prune-source-prefix instance on the server with any changed property values.

		Prefix length of the pruned source IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6GroupsPruneSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps(YangBase):
	"""A list of IPv4 Group-RP (Rendezvous Point) mapping.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-rp-maps'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMaps, self).__init__(parent, yang_key_value)

	def rp_address(self):
		"""Get the rp-address object from the server

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddress(self))

	def rp_hold_time(self):
		"""Get the rp-hold-time object from the server

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTime`: An object encapsulating an instance of the rp-hold-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTime(self))

	def rp_priority(self):
		"""Get the rp-priority object from the server

		Priority specified by the candidate Rendezvous Point (RP) router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriority`: An object encapsulating an instance of the rp-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriority(self))

	def delete(self):
		"""Delete this ipv4-rp-maps instance on the server.

		A list of IPv4 Group-RP (Rendezvous Point) mapping.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-rp-maps instance on the server with any changed property values.

		A list of IPv4 Group-RP (Rendezvous Point) mapping.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv4 Group-RP (Rendezvous Point) object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddress(YangBase):
	"""IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTime(YangBase):
	"""Hold time specified by the candidate Rendezvous Point (RP) router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-hold-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeRandom(self))

	def delete(self):
		"""Delete this rp-hold-time instance on the server.

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-hold-time instance on the server with any changed property values.

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpHoldTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriority(YangBase):
	"""Priority specified by the candidate Rendezvous Point (RP) router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityRandom(self))

	def delete(self):
		"""Delete this rp-priority instance on the server.

		Priority specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-priority instance on the server with any changed property values.

		Priority specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RpMapsRpPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps(YangBase):
	"""A list of IPv6 Group-RP (Rendezvous Point) mapping.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-rp-maps'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMaps, self).__init__(parent, yang_key_value)

	def rp_address(self):
		"""Get the rp-address object from the server

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddress(self))

	def rp_hold_time(self):
		"""Get the rp-hold-time object from the server

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTime`: An object encapsulating an instance of the rp-hold-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTime(self))

	def rp_priority(self):
		"""Get the rp-priority object from the server

		Priority specified by the candidate Rendezvous Point (RP) router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriority`: An object encapsulating an instance of the rp-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriority(self))

	def delete(self):
		"""Delete this ipv6-rp-maps instance on the server.

		A list of IPv6 Group-RP (Rendezvous Point) mapping.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-rp-maps instance on the server with any changed property values.

		A list of IPv6 Group-RP (Rendezvous Point) mapping.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv6 Group-RP (Rendezvous Point) object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddress(YangBase):
	"""IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTime(YangBase):
	"""Hold time specified by the candidate Rendezvous Point (RP) router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-hold-time'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeRandom(self))

	def delete(self):
		"""Delete this rp-hold-time instance on the server.

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-hold-time instance on the server with any changed property values.

		Hold time specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpHoldTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriority(YangBase):
	"""Priority specified by the candidate Rendezvous Point (RP) router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-priority'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityRandom(self))

	def delete(self):
		"""Delete this rp-priority instance on the server.

		Priority specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-priority instance on the server with any changed property values.

		Priority specified by the candidate Rendezvous Point (RP) router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RpMapsRpPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups(YangBase):
	"""A list of IPv4 PIM Registers block.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-register-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroups, self).__init__(parent, yang_key_value)

	def rp_address(self):
		"""Get the rp-address object from the server

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddress(self))

	def fixed_mode_count(self):
		"""Get the fixed-mode-count object from the server

		Number of PIM null register messages to send.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCount`: An object encapsulating an instance of the fixed-mode-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCount(self))

	def register_transmit_interval(self):
		"""Get the register-transmit-interval object from the server

		Number of seconds to wait between sending Register messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitInterval`: An object encapsulating an instance of the register-transmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitInterval(self))

	def multicast_group_to_source_distribution(self):
		"""Get the multicast-group-to-source-distribution object from the server

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastGroupToSourceDistribution`: An object encapsulating an instance of the multicast-group-to-source-distribution model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastGroupToSourceDistribution(self))

	def register_transmit_mode(self):
		"""Get the register-transmit-mode object from the server

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitMode`: An object encapsulating an instance of the register-transmit-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitMode(self))

	def multicast_source_address(self):
		"""Get the multicast-source-address object from the server

		Address of the first multicast source.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddress`: An object encapsulating an instance of the multicast-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddress(self))

	def multicast_source_prefix(self):
		"""Get the multicast-source-prefix object from the server

		Length of the prefix of the address of the multicast source

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefix`: An object encapsulating an instance of the multicast-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefix(self))

	def delete(self):
		"""Delete this ipv4-register-groups instance on the server.

		A list of IPv4 PIM Registers block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-register-groups instance on the server with any changed property values.

		A list of IPv4 PIM Registers block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv4 PIM Registers block object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddress(YangBase):
	"""IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCount(YangBase):
	"""Number of PIM null register messages to send.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fixed-mode-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountRandom(self))

	def delete(self):
		"""Delete this fixed-mode-count instance on the server.

		Number of PIM null register messages to send.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fixed-mode-count instance on the server with any changed property values.

		Number of PIM null register messages to send.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsFixedModeCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitInterval(YangBase):
	"""Number of seconds to wait between sending Register messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'register-transmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalRandom(self))

	def delete(self):
		"""Delete this register-transmit-interval instance on the server.

		Number of seconds to wait between sending Register messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this register-transmit-interval instance on the server with any changed property values.

		Number of seconds to wait between sending Register messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastGroupToSourceDistribution(YangBase):
	"""PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-group-to-source-distribution'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastGroupToSourceDistribution, self).__init__(parent, None)

	def delete(self):
		"""Delete this multicast-group-to-source-distribution instance on the server.

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-group-to-source-distribution instance on the server with any changed property values.

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitMode(YangBase):
	"""PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'register-transmit-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsRegisterTransmitMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this register-transmit-mode instance on the server.

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this register-transmit-mode instance on the server with any changed property values.

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddress(YangBase):
	"""Address of the first multicast source.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressRandom(self))

	def delete(self):
		"""Delete this multicast-source-address instance on the server.

		Address of the first multicast source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-source-address instance on the server with any changed property values.

		Address of the first multicast source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefix(YangBase):
	"""Length of the prefix of the address of the multicast source
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixRandom(self))

	def delete(self):
		"""Delete this multicast-source-prefix instance on the server.

		Length of the prefix of the address of the multicast source

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-source-prefix instance on the server with any changed property values.

		Length of the prefix of the address of the multicast source

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv4RegisterGroupsMulticastSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups(YangBase):
	"""A list of IPv6 PIM Registers block.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-register-groups'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroups, self).__init__(parent, yang_key_value)

	def rp_address(self):
		"""Get the rp-address object from the server

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddress`: An object encapsulating an instance of the rp-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddress(self))

	def fixed_mode_count(self):
		"""Get the fixed-mode-count object from the server

		Number of PIM null register messages to send.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCount`: An object encapsulating an instance of the fixed-mode-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCount(self))

	def register_transmit_interval(self):
		"""Get the register-transmit-interval object from the server

		Number of seconds to wait between sending Register messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitInterval`: An object encapsulating an instance of the register-transmit-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitInterval(self))

	def multicast_group_to_source_distribution(self):
		"""Get the multicast-group-to-source-distribution object from the server

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastGroupToSourceDistribution`: An object encapsulating an instance of the multicast-group-to-source-distribution model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastGroupToSourceDistribution(self))

	def register_transmit_mode(self):
		"""Get the register-transmit-mode object from the server

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitMode`: An object encapsulating an instance of the register-transmit-mode model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitMode(self))

	def multicast_source_address(self):
		"""Get the multicast-source-address object from the server

		Address of the first multicast source.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddress`: An object encapsulating an instance of the multicast-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddress(self))

	def multicast_source_prefix(self):
		"""Get the multicast-source-prefix object from the server

		Length of the prefix of the address of the multicast source

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefix`: An object encapsulating an instance of the multicast-source-prefix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefix(self))

	def delete(self):
		"""Delete this ipv6-register-groups instance on the server.

		A list of IPv6 PIM Registers block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-register-groups instance on the server with any changed property values.

		A list of IPv6 PIM Registers block.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv6 PIM Registers block object."""
		return self._get_value('name')

	@property
	def multicast_group(self):
		"""-> ../../../../../../global-multicast-groups/name: Reference to list of global multicast groups."""
		return self._get_value('multicast-group')

	@multicast_group.setter
	def multicast_group(self, value):
		return self._set_value('multicast-group', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddress(YangBase):
	"""IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'rp-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressRandom(self))

	def delete(self):
		"""Delete this rp-address instance on the server.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this rp-address instance on the server with any changed property values.

		IP address of the RP router, a PIM-SM router configured as the root of a multicast distribution tree.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRpAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCount(YangBase):
	"""Number of PIM null register messages to send.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fixed-mode-count'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountRandom(self))

	def delete(self):
		"""Delete this fixed-mode-count instance on the server.

		Number of PIM null register messages to send.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fixed-mode-count instance on the server with any changed property values.

		Number of PIM null register messages to send.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsFixedModeCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitInterval(YangBase):
	"""Number of seconds to wait between sending Register messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'register-transmit-interval'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalRandom(self))

	def delete(self):
		"""Delete this register-transmit-interval instance on the server.

		Number of seconds to wait between sending Register messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this register-transmit-interval instance on the server with any changed property values.

		Number of seconds to wait between sending Register messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastGroupToSourceDistribution(YangBase):
	"""PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-group-to-source-distribution'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastGroupToSourceDistribution, self).__init__(parent, None)

	def delete(self):
		"""Delete this multicast-group-to-source-distribution instance on the server.

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-group-to-source-distribution instance on the server with any changed property values.

		PIM Multicast Group to Source Distribution. PAIR    : One source is paired with one multicast group BACKBONE : Each source is meshed with all the multicast group addresses

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitMode(YangBase):
	"""PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'register-transmit-mode'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsRegisterTransmitMode, self).__init__(parent, None)

	def delete(self):
		"""Delete this register-transmit-mode instance on the server.

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this register-transmit-mode instance on the server with any changed property values.

		PIM will send a fixed number of register messages or send messages continuously.. FIXED	    : PIM will send a fixed number of register messages CONTINUOUS	: PIM will send a register messages continuously

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddress(YangBase):
	"""Address of the first multicast source.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-source-address'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressRandom(self))

	def delete(self):
		"""Delete this multicast-source-address instance on the server.

		Address of the first multicast source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-source-address instance on the server with any changed property values.

		Address of the first multicast source.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefix(YangBase):
	"""Length of the prefix of the address of the multicast source
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'multicast-source-prefix'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefix, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixRandom(self))

	def delete(self):
		"""Delete this multicast-source-prefix instance on the server.

		Length of the prefix of the address of the multicast source

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this multicast-source-prefix instance on the server with any changed property values.

		Length of the prefix of the address of the multicast source

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsDevicesProtocolsPimIpv6RegisterGroupsMulticastSourcePrefixRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworks(YangBase):
	"""A list of network groups. Each network group object can contain 0..n networks objects
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'simulated-networks'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworks, self).__init__(parent, yang_key_value)

	def networks(self, name=None):
		"""Get the networks object(s) from the server.

		A list of networks. Each networks object is a container for one and only one of the network types.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the networks list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworks` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworks`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworksNetworks objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworksNetworks object.  

		Raises:  
			NotFoundError: name is not in the list of networks objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworks(self, name))

	def create_networks(self, name):
		"""Create a child sibling networks instance on the server.

		A list of networks. Each networks object is a container for one and only one of the network types.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworks`: An object encapsulating an instance of the networks model.  

		Raises:  
			AlreadyExistsError: An instance of networks with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworks(self, name), locals())

	def delete(self):
		"""Delete this simulated-networks instance on the server.

		A list of network groups. Each network group object can contain 0..n networks objects

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this simulated-networks instance on the server with any changed property values.

		A list of network groups. Each network group object can contain 0..n networks objects

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the network groups object."""
		return self._get_value('name')

	@property
	def parent_link(self):
		"""union: Identifies which devices object or networks object is connected to this object. This is used to create a netwwork groups container behind a devices or networks container."""
		return self._get_value('parent-link')

	@parent_link.setter
	def parent_link(self, value):
		return self._set_value('parent-link', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworks(YangBase):
	"""A list of networks. Each networks object is a container for one and only one of the network types.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'networks'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworks, self).__init__(parent, yang_key_value)

	def bgpv4_route_range(self):
		"""Get the bgpv4-route-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRange`: An object encapsulating an instance of the bgpv4-route-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRange(self))

	def bgpv6_route_range(self):
		"""Get the bgpv6-route-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRange`: An object encapsulating an instance of the bgpv6-route-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRange(self))

	def ospfv2_route_range(self):
		"""Get the ospfv2-route-range object from the server

		OSPFV2 Router LSA

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRange`: An object encapsulating an instance of the ospfv2-route-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRange(self))

	def ospfv2_summary_range(self):
		"""Get the ospfv2-summary-range object from the server

		OSPFV2 Summary LSA (Type-3), generated by area border routers (ABR) and sent to neighboring areas, advertise area routing information.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRange`: An object encapsulating an instance of the ospfv2-summary-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRange(self))

	def ospfv2_external_range(self):
		"""Get the ospfv2-external-range object from the server

		OSPFV2 external LSA (Type-5), External LSAs (OSPF Type 5 LSAs), originated by Autonomous System Boundary Routers (ASBRs), advertise routes to destinations external to the AS, including default routes for an Autonomous System. Type 5 LSAs are flooded into all non-stub areas of an Autonomous System.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRange`: An object encapsulating an instance of the ospfv2-external-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRange(self))

	def ospfv2_nssa_range(self):
		"""Get the ospfv2-nssa-range object from the server

		OSPFV2 NSSA LSA (Type-7), NNot So Stubby Area (NSSA) LSAs (OSPF Type 7), are originated by ASBRs within NSSAs, but are flooded only within the NSSAs, by default..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRange`: An object encapsulating an instance of the ospfv2-nssa-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRange(self))

	def ospfv3_route_range(self):
		"""Get the ospfv3-route-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRange`: An object encapsulating an instance of the ospfv3-route-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRange(self))

	def ospfv3_inter_area_prefix_range(self):
		"""Get the ospfv3-inter-area-prefix-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRange`: An object encapsulating an instance of the ospfv3-inter-area-prefix-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRange(self))

	def ospfv3_intra_area_prefix_range(self):
		"""Get the ospfv3-intra-area-prefix-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRange`: An object encapsulating an instance of the ospfv3-intra-area-prefix-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRange(self))

	def ospfv3_as_external_prefix_range(self):
		"""Get the ospfv3-as-external-prefix-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRange`: An object encapsulating an instance of the ospfv3-as-external-prefix-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRange(self))

	def isis_route_range(self):
		"""Get the isis-route-range object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRange`: An object encapsulating an instance of the isis-route-range model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRange(self))

	def delete(self):
		"""Delete this networks instance on the server.

		A list of networks. Each networks object is a container for one and only one of the network types.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this networks instance on the server with any changed property values.

		A list of networks. Each networks object is a container for one and only one of the network types.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the networks object."""
		return self._get_value('name')

	@property
	def network_count_per_port(self):
		"""uint32: The number of network devices that will be created on each abstract test port specified in the port-names list."""
		return self._get_value('network-count-per-port')

	@network_count_per_port.setter
	def network_count_per_port(self, value):
		return self._set_value('network-count-per-port', value)

	@property
	def flow_link(self):
		"""union: Identifies which object in another container is the next hop from this networks object. The name cannot be one of the names in this networks container. An empty link indicates no flow link"""
		return self._get_value('flow-link')

	@flow_link.setter
	def flow_link(self, value):
		return self._set_value('flow-link', value)

	@property
	def network_type(self):
		""":str:`enum`: Determines which detailed network container is active.  
		Enums:  
			BGPV4_ROUTE_RANGE: TBD  
			BGPV6_ROUTE_RANGE: TBD  
			OSPFV2_ROUTE_RANGE: TBD  
			OSPFV2_SUMMARY_RANGE: TBD  
			OSPFV2_EXTERNAL_RANGE: TBD  
			OSPFV2_NSSA_RANGE: TBD  
			OSPFV3_ROUTE_RANGE: TBD  
			OSPFV3_INTER_AREA_PREFIX_RANGE: TBD  
			OSPFV3_INTRA_AREA_PREFIX_RANGE: TBD  
			OSPFV3_AS_EXTERNAL_PREFIX_RANGE: TBD  
			ISIS_ROUTE_RANGE: TBD  
		"""
		return self._get_value('network-type')

	@network_type.setter
	def network_type(self, value):
		return self._set_value('network-type', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv4-route-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLength(self))

	def as_path(self):
		"""Get the as-path object from the server

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPath`: An object encapsulating an instance of the as-path model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPath(self))

	def next_hop_address(self):
		"""Get the next-hop-address object from the server

		The next hop is the node to which packets should be sent to get them closer            to the destination. Specify the IP address of the border router that should be            used as the next hop to the destinations listed in the UPDATE message.            This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddress`: An object encapsulating an instance of the next-hop-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddress(self))

	def aigp_metric(self):
		"""Get the aigp-metric object from the server

		Value of the first accumulated interior gateway protocol(AIGP) metric

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetric`: An object encapsulating an instance of the aigp-metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetric(self))

	def atomic_aggregate(self):
		"""Get the atomic-aggregate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate`: An object encapsulating an instance of the atomic-aggregate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate(self))

	def create_atomic_aggregate(self):
		"""Create a child sibling atomic-aggregate instance on the server.

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate`: An object encapsulating an instance of the atomic-aggregate model.  

		Raises:  
			AlreadyExistsError: An instance of atomic-aggregate with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate(self), locals())

	def delete(self):
		"""Delete this bgpv4-route-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgpv4-route-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def active(self):
		"""bool: TBD"""
		return self._get_value('active')

	@active.setter
	def active(self, value):
		return self._set_value('active', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPath(YangBase):
	"""Used in the AS_PATH attribute in BGP UPDATE messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-path'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPath, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathRandom(self))

	def delete(self):
		"""Delete this as-path instance on the server.

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-path instance on the server with any changed property values.

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAsPathRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddress(YangBase):
	"""The next hop is the node to which packets should be sent to get them closer            to the destination. Specify the IP address of the border router that should be            used as the next hop to the destinations listed in the UPDATE message.            This is the mandatory NEXT_HOP path attribute in UPDATE messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'next-hop-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressRandom(self))

	def delete(self):
		"""Delete this next-hop-address instance on the server.

		The next hop is the node to which packets should be sent to get them closer            to the destination. Specify the IP address of the border router that should be            used as the next hop to the destinations listed in the UPDATE message.            This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this next-hop-address instance on the server with any changed property values.

		The next hop is the node to which packets should be sent to get them closer            to the destination. Specify the IP address of the border router that should be            used as the next hop to the destinations listed in the UPDATE message.            This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeNextHopAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetric(YangBase):
	"""Value of the first accumulated interior gateway protocol(AIGP) metric
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'aigp-metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricRandom(self))

	def delete(self):
		"""Delete this aigp-metric instance on the server.

		Value of the first accumulated interior gateway protocol(AIGP) metric

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this aigp-metric instance on the server with any changed property values.

		Value of the first accumulated interior gateway protocol(AIGP) metric

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAigpMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'atomic-aggregate'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregate, self).__init__(parent, None)

	def aggregate_as(self):
		"""Get the aggregate-as object from the server

		The AS number to use for the AGGREGATOR attribute

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAs`: An object encapsulating an instance of the aggregate-as model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAs(self))

	def aggregate_ip(self):
		"""Get the aggregate-ip object from the server

		The IP address to use for the AGGREGATOR attribute.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIp`: An object encapsulating an instance of the aggregate-ip model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIp(self))

	def delete(self):
		"""Delete this atomic-aggregate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this atomic-aggregate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAs(YangBase):
	"""The AS number to use for the AGGREGATOR attribute
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'aggregate-as'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAs, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsRandom(self))

	def delete(self):
		"""Delete this aggregate-as instance on the server.

		The AS number to use for the AGGREGATOR attribute

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this aggregate-as instance on the server with any changed property values.

		The AS number to use for the AGGREGATOR attribute

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateAsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIp(YangBase):
	"""The IP address to use for the AGGREGATOR attribute.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'aggregate-ip'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpRandom(self))

	def delete(self):
		"""Delete this aggregate-ip instance on the server.

		The IP address to use for the AGGREGATOR attribute.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this aggregate-ip instance on the server with any changed property values.

		The IP address to use for the AGGREGATOR attribute.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv4RouteRangeAtomicAggregateAggregateIpRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv6-route-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLength(self))

	def as_path(self):
		"""Get the as-path object from the server

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPath`: An object encapsulating an instance of the as-path model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPath(self))

	def next_hop_address(self):
		"""Get the next-hop-address object from the server

		The next hop is the node to which packets should be sent to get them closer to the destination. Specify the IP address of the border router that should be used as the next hop to the destinations listed in the UPDATE message. This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddress`: An object encapsulating an instance of the next-hop-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddress(self))

	def delete(self):
		"""Delete this bgpv6-route-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this bgpv6-route-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def active(self):
		"""bool: TBD"""
		return self._get_value('active')

	@active.setter
	def active(self, value):
		return self._set_value('active', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPath(YangBase):
	"""Used in the AS_PATH attribute in BGP UPDATE messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'as-path'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPath, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathRandom(self))

	def delete(self):
		"""Delete this as-path instance on the server.

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this as-path instance on the server with any changed property values.

		Used in the AS_PATH attribute in BGP UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeAsPathRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddress(YangBase):
	"""The next hop is the node to which packets should be sent to get them closer to the destination. Specify the IP address of the border router that should be used as the next hop to the destinations listed in the UPDATE message. This is the mandatory NEXT_HOP path attribute in UPDATE messages.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'next-hop-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressRandom(self))

	def delete(self):
		"""Delete this next-hop-address instance on the server.

		The next hop is the node to which packets should be sent to get them closer to the destination. Specify the IP address of the border router that should be used as the next hop to the destinations listed in the UPDATE message. This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this next-hop-address instance on the server with any changed property values.

		The next hop is the node to which packets should be sent to get them closer to the destination. Specify the IP address of the border router that should be used as the next hop to the destinations listed in the UPDATE message. This is the mandatory NEXT_HOP path attribute in UPDATE messages.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksBgpv6RouteRangeNextHopAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRange(YangBase):
	"""OSPFV2 Router LSA
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-route-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRange, self).__init__(parent, None)

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterId(self))

	def router_type(self):
		"""Get the router-type object from the server

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeRouterType`: An object encapsulating an instance of the router-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeRouterType(self))

	def options(self):
		"""Get the options object from the server

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptions`: An object encapsulating an instance of the options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptions(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAge(self))

	def ospfv2_router_link(self, name=None):
		"""Get the ospfv2-router-link object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ospfv2-router-link list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink object.  

		Raises:  
			NotFoundError: name is not in the list of ospfv2-router-link objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink(self, name))

	def create_ospfv2_router_link(self, name):
		"""Create a child sibling ospfv2-router-link instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink`: An object encapsulating an instance of the ospfv2-router-link model.  

		Raises:  
			AlreadyExistsError: An instance of ospfv2-router-link with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink(self, name), locals())

	def delete(self):
		"""Delete this ospfv2-route-range instance on the server.

		OSPFV2 Router LSA

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-route-range instance on the server with any changed property values.

		OSPFV2 Router LSA

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def active(self):
		"""bool: TBD"""
		return self._get_value('active')

	@active.setter
	def active(self, value):
		return self._set_value('active', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterId(YangBase):
	"""Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeRouterType(YangBase):
	"""Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeRouterType, self).__init__(parent, None)

	def delete(self):
		"""Delete this router-type instance on the server.

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-type instance on the server with any changed property values.

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptions(YangBase):
	"""Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsRandom(self))

	def delete(self):
		"""Delete this options instance on the server.

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options instance on the server with any changed property values.

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-router-link'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLink, self).__init__(parent, yang_key_value)

	def router_link_type(self):
		"""Get the router-link-type object from the server

		Type of link VL             : Virtual Link POINT_TO_POINT : P2P (point-to-point) network STUB_NETWORK   : STUB Network

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkType`: An object encapsulating an instance of the router-link-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkType(self))

	def router_link_id(self):
		"""Get the router-link-id object from the server

		Link ID (IP address) to be used for the simulated link.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkId`: An object encapsulating an instance of the router-link-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkId(self))

	def delete(self):
		"""Delete this ospfv2-router-link instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-router-link instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the networks object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkType(YangBase):
	"""Type of link VL             : Virtual Link POINT_TO_POINT : P2P (point-to-point) network STUB_NETWORK   : STUB Network
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-link-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkType, self).__init__(parent, None)

	def delete(self):
		"""Delete this router-link-type instance on the server.

		Type of link VL             : Virtual Link POINT_TO_POINT : P2P (point-to-point) network STUB_NETWORK   : STUB Network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-link-type instance on the server with any changed property values.

		Type of link VL             : Virtual Link POINT_TO_POINT : P2P (point-to-point) network STUB_NETWORK   : STUB Network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkId(YangBase):
	"""Link ID (IP address) to be used for the simulated link.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-link-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdRandom(self))

	def delete(self):
		"""Delete this router-link-id instance on the server.

		Link ID (IP address) to be used for the simulated link.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-link-id instance on the server with any changed property values.

		Link ID (IP address) to be used for the simulated link.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2RouteRangeOspfv2RouterLinkRouterLinkIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRange(YangBase):
	"""OSPFV2 Summary LSA (Type-3), generated by area border routers (ABR) and sent to neighboring areas, advertise area routing information.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-summary-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Router ID of the simulated router that will advertise Summary LSAs

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterId(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeRouteCategory(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Initial value of the LS sequence number field in the LSA header.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeChecksum(self))

	def metric(self):
		"""Get the metric object from the server

		Route-cost metric to be used when advertising the specified summary LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetric(self))

	def options(self):
		"""Get the options object from the server

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptions`: An object encapsulating an instance of the options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptions(self))

	def segment_routing(self):
		"""Get the segment-routing object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRouting`: An object encapsulating an instance of the segment-routing model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRouting(self))

	def delete(self):
		"""Delete this ospfv2-summary-range instance on the server.

		OSPFV2 Summary LSA (Type-3), generated by area border routers (ABR) and sent to neighboring areas, advertise area routing information.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-summary-range instance on the server with any changed property values.

		OSPFV2 Summary LSA (Type-3), generated by area border routers (ABR) and sent to neighboring areas, advertise area routing information.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddress(YangBase):
	"""IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterId(YangBase):
	"""Router ID of the simulated router that will advertise Summary LSAs
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Router ID of the simulated router that will advertise Summary LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Router ID of the simulated router that will advertise Summary LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumber(YangBase):
	"""Initial value of the LS sequence number field in the LSA header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Initial value of the LS sequence number field in the LSA header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Initial value of the LS sequence number field in the LSA header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetric(YangBase):
	"""Route-cost metric to be used when advertising the specified summary LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Route-cost metric to be used when advertising the specified summary LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Route-cost metric to be used when advertising the specified summary LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptions(YangBase):
	"""Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsRandom(self))

	def delete(self):
		"""Delete this options instance on the server.

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options instance on the server with any changed property values.

		Summary LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRouting(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-routing'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRouting, self).__init__(parent, None)

	def segment_id(self):
		"""Get the segment-id object from the server

		Value of the Segment identifier(SID).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentId`: An object encapsulating an instance of the segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentId(self))

	def prefix_segment_id(self):
		"""Get the prefix-segment-id object from the server

		Value of the Prefix Segment identifier(SID). NBIT    Node SID Flag NPBIT   No-PHP Flag MBIT    Mapping Server Flag EBIT    Explicit-Null Flag VBIT    Value/Index Flag LBIT    Local/Global Flag

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentId`: An object encapsulating an instance of the prefix-segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentId(self))

	def delete(self):
		"""Delete this segment-routing instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-routing instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentId(YangBase):
	"""Value of the Segment identifier(SID).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdRandom(self))

	def delete(self):
		"""Delete this segment-id instance on the server.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-id instance on the server with any changed property values.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentId(YangBase):
	"""Value of the Prefix Segment identifier(SID). NBIT    Node SID Flag NPBIT   No-PHP Flag MBIT    Mapping Server Flag EBIT    Explicit-Null Flag VBIT    Value/Index Flag LBIT    Local/Global Flag
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdRandom(self))

	def delete(self):
		"""Delete this prefix-segment-id instance on the server.

		Value of the Prefix Segment identifier(SID). NBIT    Node SID Flag NPBIT   No-PHP Flag MBIT    Mapping Server Flag EBIT    Explicit-Null Flag VBIT    Value/Index Flag LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-segment-id instance on the server with any changed property values.

		Value of the Prefix Segment identifier(SID). NBIT    Node SID Flag NPBIT   No-PHP Flag MBIT    Mapping Server Flag EBIT    Explicit-Null Flag VBIT    Value/Index Flag LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2SummaryRangeSegmentRoutingPrefixSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRange(YangBase):
	"""OSPFV2 external LSA (Type-5), External LSAs (OSPF Type 5 LSAs), originated by Autonomous System Boundary Routers (ASBRs), advertise routes to destinations external to the AS, including default routes for an Autonomous System. Type 5 LSAs are flooded into all non-stub areas of an Autonomous System.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-external-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Router ID of the simulated router that will advertise external LSAs

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterId(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeRouteCategory(self))

	def forwarding_address(self):
		"""Get the forwarding-address object from the server

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddress`: An object encapsulating an instance of the forwarding-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddress(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeChecksum(self))

	def metric_type(self):
		"""Get the metric-type object from the server

		Metric type.    TYPE1  When calculating the cost of the path to an external route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricType`: An object encapsulating an instance of the metric-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricType(self))

	def metric(self):
		"""Get the metric object from the server

		Route-cost metric to be used when advertising the specified external LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetric(self))

	def options(self):
		"""Get the options object from the server

		external LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptions`: An object encapsulating an instance of the options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptions(self))

	def segment_routing(self):
		"""Get the segment-routing object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRouting`: An object encapsulating an instance of the segment-routing model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRouting(self))

	def delete(self):
		"""Delete this ospfv2-external-range instance on the server.

		OSPFV2 external LSA (Type-5), External LSAs (OSPF Type 5 LSAs), originated by Autonomous System Boundary Routers (ASBRs), advertise routes to destinations external to the AS, including default routes for an Autonomous System. Type 5 LSAs are flooded into all non-stub areas of an Autonomous System.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-external-range instance on the server with any changed property values.

		OSPFV2 external LSA (Type-5), External LSAs (OSPF Type 5 LSAs), originated by Autonomous System Boundary Routers (ASBRs), advertise routes to destinations external to the AS, including default routes for an Autonomous System. Type 5 LSAs are flooded into all non-stub areas of an Autonomous System.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddress(YangBase):
	"""IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterId(YangBase):
	"""Router ID of the simulated router that will advertise external LSAs
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Router ID of the simulated router that will advertise external LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Router ID of the simulated router that will advertise external LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddress(YangBase):
	"""IPv4 forwarding address to use in the forwarding address field of the LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'forwarding-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressRandom(self))

	def delete(self):
		"""Delete this forwarding-address instance on the server.

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this forwarding-address instance on the server with any changed property values.

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeForwardingAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumber(YangBase):
	"""Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricType(YangBase):
	"""Metric type.    TYPE1  When calculating the cost of the path to an external route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricType, self).__init__(parent, None)

	def delete(self):
		"""Delete this metric-type instance on the server.

		Metric type.    TYPE1  When calculating the cost of the path to an external route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric-type instance on the server with any changed property values.

		Metric type.    TYPE1  When calculating the cost of the path to an external route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetric(YangBase):
	"""Route-cost metric to be used when advertising the specified external LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Route-cost metric to be used when advertising the specified external LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Route-cost metric to be used when advertising the specified external LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptions(YangBase):
	"""external LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsRandom(self))

	def delete(self):
		"""Delete this options instance on the server.

		external LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options instance on the server with any changed property values.

		external LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRouting(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-routing'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRouting, self).__init__(parent, None)

	def segment_id(self):
		"""Get the segment-id object from the server

		Value of the Segment identifier(SID).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentId`: An object encapsulating an instance of the segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentId(self))

	def prefix_segment_id(self):
		"""Get the prefix-segment-id object from the server

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentId`: An object encapsulating an instance of the prefix-segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentId(self))

	def delete(self):
		"""Delete this segment-routing instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-routing instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentId(YangBase):
	"""Value of the Segment identifier(SID).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdRandom(self))

	def delete(self):
		"""Delete this segment-id instance on the server.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-id instance on the server with any changed property values.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentId(YangBase):
	"""Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdRandom(self))

	def delete(self):
		"""Delete this prefix-segment-id instance on the server.

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-segment-id instance on the server with any changed property values.

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2ExternalRangeSegmentRoutingPrefixSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRange(YangBase):
	"""OSPFV2 NSSA LSA (Type-7), NNot So Stubby Area (NSSA) LSAs (OSPF Type 7), are originated by ASBRs within NSSAs, but are flooded only within the NSSAs, by default..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-nssa-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IP address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Router ID of the simulated router that will advertise NSSA LSAs

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterId(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeRouteCategory(self))

	def forwarding_address(self):
		"""Get the forwarding-address object from the server

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddress`: An object encapsulating an instance of the forwarding-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddress(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeChecksum(self))

	def metric_type(self):
		"""Get the metric-type object from the server

		Metric type.    TYPE1  When calculating the cost of the path to an NSSA route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricType`: An object encapsulating an instance of the metric-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricType(self))

	def metric(self):
		"""Get the metric object from the server

		Route-cost metric to be used when advertising the specified NSSA LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetric(self))

	def options(self):
		"""Get the options object from the server

		NSSA LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptions`: An object encapsulating an instance of the options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptions(self))

	def segment_routing(self):
		"""Get the segment-routing object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRouting`: An object encapsulating an instance of the segment-routing model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRouting(self))

	def delete(self):
		"""Delete this ospfv2-nssa-range instance on the server.

		OSPFV2 NSSA LSA (Type-7), NNot So Stubby Area (NSSA) LSAs (OSPF Type 7), are originated by ASBRs within NSSAs, but are flooded only within the NSSAs, by default..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-nssa-range instance on the server with any changed property values.

		OSPFV2 NSSA LSA (Type-7), NNot So Stubby Area (NSSA) LSAs (OSPF Type 7), are originated by ASBRs within NSSAs, but are flooded only within the NSSAs, by default..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddress(YangBase):
	"""IP address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IP address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterId(YangBase):
	"""Router ID of the simulated router that will advertise NSSA LSAs
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Router ID of the simulated router that will advertise NSSA LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Router ID of the simulated router that will advertise NSSA LSAs

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddress(YangBase):
	"""IPv4 forwarding address to use in the forwarding address field of the LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'forwarding-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressRandom(self))

	def delete(self):
		"""Delete this forwarding-address instance on the server.

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this forwarding-address instance on the server with any changed property values.

		IPv4 forwarding address to use in the forwarding address field of the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeForwardingAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumber(YangBase):
	"""Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Initial value of the LS sequence number field in the LSA header. Used to detect old and duplicate LSAs. The larger the sequence number, the more recent the LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricType(YangBase):
	"""Metric type.    TYPE1  When calculating the cost of the path to an NSSA route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricType, self).__init__(parent, None)

	def delete(self):
		"""Delete this metric-type instance on the server.

		Metric type.    TYPE1  When calculating the cost of the path to an NSSA route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric-type instance on the server with any changed property values.

		Metric type.    TYPE1  When calculating the cost of the path to an NSSA route, the costs of internal links are included.    TYPE2  Costs of internal links are not included in total cost calculation. Type 2 routes have a lower priority and are considered in path selection after Type 1 costs have been evaluated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetric(YangBase):
	"""Route-cost metric to be used when advertising the specified NSSA LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Route-cost metric to be used when advertising the specified NSSA LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Route-cost metric to be used when advertising the specified NSSA LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptions(YangBase):
	"""NSSA LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsRandom(self))

	def delete(self):
		"""Delete this options instance on the server.

		NSSA LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options instance on the server with any changed property values.

		NSSA LSA Options.    TBIT     TOS: Type of Service (T,0).    EBIT     External Routing: Specifies the way AS-external-LSAs are flooded (E,1).    MCBIT    Multicast: Specifies whether IP multicast datagrams are forwarded (MC,2).    NPBIT    NSSA: Specifies the handling of Type-7 LSAs (N/P,3).    EABIT    External Attribute: Specifies the router's willingness to receive and forward External-Attributes-LSAs (EA,4).    DCBIT    Demand Circuit: Specifies the router's handling of demand circuits (DC,5).    OBIT     Opaque: Specifies the router's willingness to receive and forward Opaque LSAs as specified in RFC 2370 (O,6).    UNUSED7  Unused: This bit is not used

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRouting(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-routing'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRouting, self).__init__(parent, None)

	def segment_id(self):
		"""Get the segment-id object from the server

		Value of the Segment identifier(SID).

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentId`: An object encapsulating an instance of the segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentId(self))

	def prefix_segment_id(self):
		"""Get the prefix-segment-id object from the server

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentId`: An object encapsulating an instance of the prefix-segment-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentId(self))

	def delete(self):
		"""Delete this segment-routing instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-routing instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentId(YangBase):
	"""Value of the Segment identifier(SID).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdRandom(self))

	def delete(self):
		"""Delete this segment-id instance on the server.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this segment-id instance on the server with any changed property values.

		Value of the Segment identifier(SID).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentId(YangBase):
	"""Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-segment-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdRandom(self))

	def delete(self):
		"""Delete this prefix-segment-id instance on the server.

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-segment-id instance on the server with any changed property values.

		Value of the Prefix Segment identifier(SID).    NBIT    Node SID Flag    NPBIT   No-PHP Flag    MBIT    Mapping Server Flag    EBIT    Explicit-Null Flag    VBIT    Value/Index Flag    LBIT    Local/Global Flag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv2NssaRangeSegmentRoutingPrefixSegmentIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-route-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRange, self).__init__(parent, None)

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterId(self))

	def router_type(self):
		"""Get the router-type object from the server

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeRouterType`: An object encapsulating an instance of the router-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeRouterType(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAge(self))

	def ospfv3_router_link(self, name=None):
		"""Get the ospfv3-router-link object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ospfv3-router-link list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink object.  

		Raises:  
			NotFoundError: name is not in the list of ospfv3-router-link objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink(self, name))

	def create_ospfv3_router_link(self, name):
		"""Create a child sibling ospfv3-router-link instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink`: An object encapsulating an instance of the ospfv3-router-link model.  

		Raises:  
			AlreadyExistsError: An instance of ospfv3-router-link with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink(self, name), locals())

	def delete(self):
		"""Delete this ospfv3-route-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3-route-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def active(self):
		"""bool: TBD"""
		return self._get_value('active')

	@active.setter
	def active(self, value):
		return self._set_value('active', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterId(YangBase):
	"""Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeRouterType(YangBase):
	"""Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeRouterType, self).__init__(parent, None)

	def delete(self):
		"""Delete this router-type instance on the server.

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-type instance on the server with any changed property values.

		Router Type ASBR    : AS Boundary Router (E-bit set) ABR     : Border Router (B-bit set)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-router-link'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLink, self).__init__(parent, yang_key_value)

	def router_link_type(self):
		"""Get the router-link-type object from the server

		Type of link VIRTUAL_LINK   : Virtual Link POINT_TO_POINT : P2P (point-to-point) adjacency

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkRouterLinkType`: An object encapsulating an instance of the router-link-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkRouterLinkType(self))

	def interface_id(self):
		"""Get the interface-id object from the server

		The Interface ID is an interface index number that a router uses to uniquely identify an interface. The Interface ID must be unique within the router.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceId`: An object encapsulating an instance of the interface-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceId(self))

	def delete(self):
		"""Delete this ospfv3-router-link instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3-router-link instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the networks object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkRouterLinkType(YangBase):
	"""Type of link VIRTUAL_LINK   : Virtual Link POINT_TO_POINT : P2P (point-to-point) adjacency
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-link-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkRouterLinkType, self).__init__(parent, None)

	def delete(self):
		"""Delete this router-link-type instance on the server.

		Type of link VIRTUAL_LINK   : Virtual Link POINT_TO_POINT : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-link-type instance on the server with any changed property values.

		Type of link VIRTUAL_LINK   : Virtual Link POINT_TO_POINT : P2P (point-to-point) adjacency

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceId(YangBase):
	"""The Interface ID is an interface index number that a router uses to uniquely identify an interface. The Interface ID must be unique within the router.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'interface-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdRandom(self))

	def delete(self):
		"""Delete this interface-id instance on the server.

		The Interface ID is an interface index number that a router uses to uniquely identify an interface. The Interface ID must be unique within the router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this interface-id instance on the server with any changed property values.

		The Interface ID is an interface index number that a router uses to uniquely identify an interface. The Interface ID must be unique within the router.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3RouteRangeOspfv3RouterLinkInterfaceIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-inter-area-prefix-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IPv6 address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterId(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeChecksum(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeRouteCategory(self))

	def metric(self):
		"""Get the metric object from the server

		Cost of the advertised network prefix.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetric(self))

	def prefix_options(self):
		"""Get the prefix-options object from the server

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptions`: An object encapsulating an instance of the prefix-options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptions(self))

	def link_state_id(self):
		"""Get the link-state-id object from the server

		Link State ID. ID of the interface connected to the transit network

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateId`: An object encapsulating an instance of the link-state-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateId(self))

	def delete(self):
		"""Delete this ospfv3-inter-area-prefix-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3-inter-area-prefix-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddress(YangBase):
	"""IPv6 address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterId(YangBase):
	"""Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetric(YangBase):
	"""Cost of the advertised network prefix.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptions(YangBase):
	"""8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsRandom(self))

	def delete(self):
		"""Delete this prefix-options instance on the server.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-options instance on the server with any changed property values.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangePrefixOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateId(YangBase):
	"""Link State ID. ID of the interface connected to the transit network
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-state-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdRandom(self))

	def delete(self):
		"""Delete this link-state-id instance on the server.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-state-id instance on the server with any changed property values.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3InterAreaPrefixRangeLinkStateIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-intra-area-prefix-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IPv6 address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterId(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeChecksum(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeRouteCategory(self))

	def metric(self):
		"""Get the metric object from the server

		Cost of the advertised network prefix.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetric(self))

	def prefix_options(self):
		"""Get the prefix-options object from the server

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptions`: An object encapsulating an instance of the prefix-options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptions(self))

	def link_state_id(self):
		"""Get the link-state-id object from the server

		Link State ID. ID of the interface connected to the transit network

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateId`: An object encapsulating an instance of the link-state-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateId(self))

	def referenced_lsa(self):
		"""Get the referenced-lsa object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsa`: An object encapsulating an instance of the referenced-lsa model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsa(self))

	def delete(self):
		"""Delete this ospfv3-intra-area-prefix-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3-intra-area-prefix-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddress(YangBase):
	"""IPv6 address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterId(YangBase):
	"""Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetric(YangBase):
	"""Cost of the advertised network prefix.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptions(YangBase):
	"""8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsRandom(self))

	def delete(self):
		"""Delete this prefix-options instance on the server.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-options instance on the server with any changed property values.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangePrefixOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateId(YangBase):
	"""Link State ID. ID of the interface connected to the transit network
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-state-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdRandom(self))

	def delete(self):
		"""Delete this link-state-id instance on the server.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-state-id instance on the server with any changed property values.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeLinkStateIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsa(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'referenced-lsa'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsa, self).__init__(parent, None)

	def type(self):
		"""Get the type object from the server

		Type of the referenced LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaType`: An object encapsulating an instance of the type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaType(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Router ID of the router that originated the LSA or the Designated Router 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterId(self))

	def link_state_id(self):
		"""Get the link-state-id object from the server

		Referenced Link State ID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateId`: An object encapsulating an instance of the link-state-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateId(self))

	def delete(self):
		"""Delete this referenced-lsa instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this referenced-lsa instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaType(YangBase):
	"""Type of the referenced LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeRandom(self))

	def delete(self):
		"""Delete this type instance on the server.

		Type of the referenced LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this type instance on the server with any changed property values.

		Type of the referenced LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterId(YangBase):
	"""Router ID of the router that originated the LSA or the Designated Router 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Router ID of the router that originated the LSA or the Designated Router 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Router ID of the router that originated the LSA or the Designated Router 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateId(YangBase):
	"""Referenced Link State ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-state-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdRandom(self))

	def delete(self):
		"""Delete this link-state-id instance on the server.

		Referenced Link State ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-state-id instance on the server with any changed property values.

		Referenced Link State ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3IntraAreaPrefixRangeReferencedLsaLinkStateIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-as-external-prefix-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRange, self).__init__(parent, None)

	def address(self):
		"""Get the address object from the server

		IPv6 address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		Prefix Length

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLength(self))

	def advertise_router_id(self):
		"""Get the advertise-router-id object from the server

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterId`: An object encapsulating an instance of the advertise-router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterId(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumber(self))

	def age(self):
		"""Get the age object from the server

		Age of the LSA, in seconds.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAge`: An object encapsulating an instance of the age model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAge(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the LSA will be advertised with a good or a bad checksum.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeChecksum(self))

	def route_category(self):
		"""Get the route-category object from the server

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeRouteCategory`: An object encapsulating an instance of the route-category model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeRouteCategory(self))

	def metric_type(self):
		"""Get the metric-type object from the server

		Metric type.    TRUE  Metric is considered larger than any intra-AS path.(External Metric)    FALSE Metric is expressed in the same units as interface cost.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricType`: An object encapsulating an instance of the metric-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricType(self))

	def metric(self):
		"""Get the metric object from the server

		Cost of the advertised network prefix.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetric`: An object encapsulating an instance of the metric model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetric(self))

	def prefix_options(self):
		"""Get the prefix-options object from the server

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptions`: An object encapsulating an instance of the prefix-options model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptions(self))

	def link_state_id(self):
		"""Get the link-state-id object from the server

		Link State ID. ID of the interface connected to the transit network

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateId`: An object encapsulating an instance of the link-state-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateId(self))

	def extended_lsa(self):
		"""Get the extended-lsa object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsa`: An object encapsulating an instance of the extended-lsa model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsa(self))

	def referenced_lsa(self):
		"""Get the referenced-lsa object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsa`: An object encapsulating an instance of the referenced-lsa model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsa(self))

	def delete(self):
		"""Delete this ospfv3-as-external-prefix-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv3-as-external-prefix-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddress(YangBase):
	"""IPv6 address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		IPv6 address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLength(YangBase):
	"""Prefix Length
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		Prefix Length

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterId(YangBase):
	"""Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-router-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdRandom(self))

	def delete(self):
		"""Delete this advertise-router-id instance on the server.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-router-id instance on the server with any changed property values.

		Advertising Router ID.  Specifies the 32-bit router ID of the router that advertises a given LSA 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAdvertiseRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAge(YangBase):
	"""Age of the LSA, in seconds.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'age'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAge, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeRandom(self))

	def delete(self):
		"""Delete this age instance on the server.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this age instance on the server with any changed property values.

		Age of the LSA, in seconds.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeAgeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeChecksum(YangBase):
	"""Specifies the LSA will be advertised with a good or a bad checksum.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeChecksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the LSA will be advertised with a good or a bad checksum.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeRouteCategory(YangBase):
	"""Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-category'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeRouteCategory, self).__init__(parent, None)

	def delete(self):
		"""Delete this route-category instance on the server.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-category instance on the server with any changed property values.

		Route Category can be used to filter routes by type.    UNDEFINED The default category assigned to a manually created block.    PRIMARY   A preferred route that has duplicates (secondary routes) advertised by other ports.    SECONDARY Secondary route.    UNIQUE    A less preferred backup route.    ANY       Indicates that a single port in the test configuration advertises this route.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricType(YangBase):
	"""Metric type.    TRUE  Metric is considered larger than any intra-AS path.(External Metric)    FALSE Metric is expressed in the same units as interface cost.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric-type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeRandom(self))

	def delete(self):
		"""Delete this metric-type instance on the server.

		Metric type.    TRUE  Metric is considered larger than any intra-AS path.(External Metric)    FALSE Metric is expressed in the same units as interface cost.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric-type instance on the server with any changed property values.

		Metric type.    TRUE  Metric is considered larger than any intra-AS path.(External Metric)    FALSE Metric is expressed in the same units as interface cost.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetric(YangBase):
	"""Cost of the advertised network prefix.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'metric'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetric, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricRandom(self))

	def delete(self):
		"""Delete this metric instance on the server.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this metric instance on the server with any changed property values.

		Cost of the advertised network prefix.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeMetricRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptions(YangBase):
	"""8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-options'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptions, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsRandom(self))

	def delete(self):
		"""Delete this prefix-options instance on the server.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-options instance on the server with any changed property values.

		8- bit field of capabilities advertised with each prefix.    NUBIT    No Unicast capability bit. If set, the prefix is excluded from IPv6 unicast calculations.    LABIT    Local Address capability bit. If set, the prefix becomes an IPv6 interface address of the advertising router.    MCBIT    Multicast capability bit. If set, the prefix is included in the IPv6 multicast calculations.    PBIT     Propagate bit. Set this on the NSSA area prefixes that should be readvertised at the NSSA border.    DNBIT    Downward bit. Controls an inter-area-prefix-LSAs or AS-external-LSAs re-advertisement in a VPN environment    NBIT     Set in PrefixOptions for a host address (PrefixLength=128) that identifies the advertising router    UNUSED6  Bit not defined.    UNUSED7  Bit not defined. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangePrefixOptionsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateId(YangBase):
	"""Link State ID. ID of the interface connected to the transit network
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-state-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdRandom(self))

	def delete(self):
		"""Delete this link-state-id instance on the server.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-state-id instance on the server with any changed property values.

		Link State ID. ID of the interface connected to the transit network

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeLinkStateIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsa(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'extended-lsa'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsa, self).__init__(parent, None)

	def extended_lsa_tlvs(self):
		"""Get the extended-lsa-tlvs object from the server

		Include extended LSA tlvs.    IPV6_FORWARDING_ADDR IPv6  IPv6 Forwarding Address sub-TLV.    IPV4_FORWARDING_ADDR IPv4  IPv4 Forwarding Address sub-TLV.    ROUTE_TAG                  Route-Tag sub-TLV.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvs`: An object encapsulating an instance of the extended-lsa-tlvs model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvs(self))

	def ipv6_forwarding_address(self):
		"""Get the ipv6-forwarding-address object from the server

		IPv6 Forwarding address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddress`: An object encapsulating an instance of the ipv6-forwarding-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddress(self))

	def ipv4_forwarding_address(self):
		"""Get the ipv4-forwarding-address object from the server

		IPv4 Forwarding address

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddress`: An object encapsulating an instance of the ipv4-forwarding-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddress(self))

	def route_tag(self):
		"""Get the route-tag object from the server

		Route tag

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTag`: An object encapsulating an instance of the route-tag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTag(self))

	def delete(self):
		"""Delete this extended-lsa instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this extended-lsa instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvs(YangBase):
	"""Include extended LSA tlvs.    IPV6_FORWARDING_ADDR IPv6  IPv6 Forwarding Address sub-TLV.    IPV4_FORWARDING_ADDR IPv4  IPv4 Forwarding Address sub-TLV.    ROUTE_TAG                  Route-Tag sub-TLV.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'extended-lsa-tlvs'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvs, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsRandom(self))

	def delete(self):
		"""Delete this extended-lsa-tlvs instance on the server.

		Include extended LSA tlvs.    IPV6_FORWARDING_ADDR IPv6  IPv6 Forwarding Address sub-TLV.    IPV4_FORWARDING_ADDR IPv4  IPv4 Forwarding Address sub-TLV.    ROUTE_TAG                  Route-Tag sub-TLV.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this extended-lsa-tlvs instance on the server with any changed property values.

		Include extended LSA tlvs.    IPV6_FORWARDING_ADDR IPv6  IPv6 Forwarding Address sub-TLV.    IPV4_FORWARDING_ADDR IPv4  IPv4 Forwarding Address sub-TLV.    ROUTE_TAG                  Route-Tag sub-TLV.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaExtendedLsaTlvsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddress(YangBase):
	"""IPv6 Forwarding address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-forwarding-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-forwarding-address instance on the server.

		IPv6 Forwarding address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-forwarding-address instance on the server with any changed property values.

		IPv6 Forwarding address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv6ForwardingAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddress(YangBase):
	"""IPv4 Forwarding address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-forwarding-address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-forwarding-address instance on the server.

		IPv4 Forwarding address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-forwarding-address instance on the server with any changed property values.

		IPv4 Forwarding address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaIpv4ForwardingAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTag(YangBase):
	"""Route tag
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'route-tag'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTag, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagRandom(self))

	def delete(self):
		"""Delete this route-tag instance on the server.

		Route tag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this route-tag instance on the server with any changed property values.

		Route tag

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeExtendedLsaRouteTagRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsa(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'referenced-lsa'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsa, self).__init__(parent, None)

	def type(self):
		"""Get the type object from the server

		Type of the referenced LSA.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaType`: An object encapsulating an instance of the type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaType(self))

	def link_state_id(self):
		"""Get the link-state-id object from the server

		Referenced Link State ID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateId`: An object encapsulating an instance of the link-state-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateId(self))

	def delete(self):
		"""Delete this referenced-lsa instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this referenced-lsa instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaType(YangBase):
	"""Type of the referenced LSA.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'type'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeRandom(self))

	def delete(self):
		"""Delete this type instance on the server.

		Type of the referenced LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this type instance on the server with any changed property values.

		Type of the referenced LSA.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateId(YangBase):
	"""Referenced Link State ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'link-state-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdRandom(self))

	def delete(self):
		"""Delete this link-state-id instance on the server.

		Referenced Link State ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this link-state-id instance on the server with any changed property values.

		Referenced Link State ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksOspfv3AsExternalPrefixRangeReferencedLsaLinkStateIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRange(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'isis-route-range'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRange, self).__init__(parent, None)

	def system_id(self):
		"""Get the system-id object from the server

		System ID.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemId`: An object encapsulating an instance of the system-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemId(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number. Used to detect old and duplicate LSAs.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumber(self))

	def level(self):
		"""Get the level object from the server

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeLevel`: An object encapsulating an instance of the level model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeLevel(self))

	def ipv4_routes(self, name=None):
		"""Get the ipv4-routes object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv4-routes list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes object.  

		Raises:  
			NotFoundError: name is not in the list of ipv4-routes objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes(self, name))

	def create_ipv4_routes(self, name):
		"""Create a child sibling ipv4-routes instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes`: An object encapsulating an instance of the ipv4-routes model.  

		Raises:  
			AlreadyExistsError: An instance of ipv4-routes with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes(self, name), locals())

	def ipv6_routes(self, name=None):
		"""Get the ipv6-routes object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ipv6-routes list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes` | :obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes`: If arg name is None a list of SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes objects otherwise a single SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes object.  

		Raises:  
			NotFoundError: name is not in the list of ipv6-routes objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes(self, name))

	def create_ipv6_routes(self, name):
		"""Create a child sibling ipv6-routes instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes`: An object encapsulating an instance of the ipv6-routes model.  

		Raises:  
			AlreadyExistsError: An instance of ipv6-routes with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes(self, name), locals())

	def delete(self):
		"""Delete this isis-route-range instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this isis-route-range instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemId(YangBase):
	"""System ID.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'system-id'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdRandom(self))

	def delete(self):
		"""Delete this system-id instance on the server.

		System ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this system-id instance on the server with any changed property values.

		System ID.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSystemIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumber(YangBase):
	"""Sequence Number. Used to detect old and duplicate LSAs.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number. Used to detect old and duplicate LSAs.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeLevel(YangBase):
	"""This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'level'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeLevel, self).__init__(parent, None)

	def delete(self):
		"""Delete this level instance on the server.

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this level instance on the server with any changed property values.

		This is the circuit type of the emulated router. It defines the type of adjacency            Traffic Generator establishes with the DUT..

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-routes'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4Routes, self).__init__(parent, yang_key_value)

	def address(self):
		"""Get the address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLength(self))

	def delete(self):
		"""Delete this ipv4-routes instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-routes instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv4 route object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv4RoutesPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-routes'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6Routes, self).__init__(parent, yang_key_value)

	def address(self):
		"""Get the address object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddress(self))

	def prefix_length(self):
		"""Get the prefix-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLength`: An object encapsulating an instance of the prefix-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLength(self))

	def delete(self):
		"""Delete this ipv6-routes instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-routes instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of the IPv4 route object."""
		return self._get_value('name')


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'prefix-length'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthRandom(self))

	def delete(self):
		"""Delete this prefix-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this prefix-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceGroupsSimulatedNetworksNetworksIsisRouteRangeIpv6RoutesPrefixLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTraffic(YangBase):
	"""A list of traffic streams where each traffic stream has a single transmit port as its source and a list of user defined frames.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'port-traffic'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTraffic, self).__init__(parent, yang_key_value)

	def frames(self, name=None):
		"""Get the frames object(s) from the server.

		List of user defined frames. The order of frames in the list will be the order of frames on the wire

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the frames list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTrafficFrames` | :obj:`SessionsConfigPortTrafficFrames`: If arg name is None a list of SessionsConfigPortTrafficFrames objects otherwise a single SessionsConfigPortTrafficFrames object.  

		Raises:  
			NotFoundError: name is not in the list of frames objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrames(self, name))

	def create_frames(self, name):
		"""Create a child sibling frames instance on the server.

		List of user defined frames. The order of frames in the list will be the order of frames on the wire

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTrafficFrames`: An object encapsulating an instance of the frames model.  

		Raises:  
			AlreadyExistsError: An instance of frames with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTrafficFrames(self, name), locals())

	def frame_length(self):
		"""Get the frame-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLength`: An object encapsulating an instance of the frame-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLength(self))

	def frame_rate(self):
		"""Get the frame-rate object from the server

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameRate`: An object encapsulating an instance of the frame-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameRate(self))

	def packet_payload(self):
		"""Get the packet-payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficPacketPayload`: An object encapsulating an instance of the packet-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficPacketPayload(self))

	def delete(self):
		"""Delete this port-traffic instance on the server.

		A list of traffic streams where each traffic stream has a single transmit port as its source and a list of user defined frames.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this port-traffic instance on the server with any changed property values.

		A list of traffic streams where each traffic stream has a single transmit port as its source and a list of user defined frames.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of this traffic stream"""
		return self._get_value('name')

	@property
	def source(self):
		"""-> ../../ports/name: The transmit port of this traffic stream"""
		return self._get_value('source')

	@source.setter
	def source(self, value):
		return self._set_value('source', value)

	@property
	def destinations(self):
		"""-> ../../ports/name: The intended receive ports for this traffic stream. Any empty list means that any port can receive these frames. For a non-empty list, any frame received by a port that is not in the list will be counted as stray."""
		return self._get_value('destinations')

	@destinations.setter
	def destinations(self, value):
		return self._set_value('destinations', value)


class SessionsConfigPortTrafficFrames(YangBase):
	"""List of user defined frames. The order of frames in the list will be the order of frames on the wire
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frames'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTrafficFrames, self).__init__(parent, yang_key_value)

	def ethernet(self):
		"""Get the ethernet object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernet`: An object encapsulating an instance of the ethernet model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernet(self))

	def vlan(self):
		"""Get the vlan object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlan`: An object encapsulating an instance of the vlan model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlan(self))

	def ipv4(self):
		"""Get the ipv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4`: An object encapsulating an instance of the ipv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4(self))

	def ipv6(self):
		"""Get the ipv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6`: An object encapsulating an instance of the ipv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6(self))

	def tcp(self):
		"""Get the tcp object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcp`: An object encapsulating an instance of the tcp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcp(self))

	def udp(self):
		"""Get the udp object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdp`: An object encapsulating an instance of the udp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdp(self))

	def gre(self):
		"""Get the gre object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGre`: An object encapsulating an instance of the gre model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGre(self))

	def icmp_destination_unreachable(self):
		"""Get the icmp-destination-unreachable object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachable`: An object encapsulating an instance of the icmp-destination-unreachable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachable(self))

	def icmp_echo_reply(self):
		"""Get the icmp-echo-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReply`: An object encapsulating an instance of the icmp-echo-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReply(self))

	def icmp_echo_request(self):
		"""Get the icmp-echo-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequest`: An object encapsulating an instance of the icmp-echo-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequest(self))

	def icmp_information_reply(self):
		"""Get the icmp-information-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReply`: An object encapsulating an instance of the icmp-information-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReply(self))

	def icmp_information_request(self):
		"""Get the icmp-information-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequest`: An object encapsulating an instance of the icmp-information-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequest(self))

	def icmp_address_mask_reply(self):
		"""Get the icmp-address-mask-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReply`: An object encapsulating an instance of the icmp-address-mask-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReply(self))

	def icmp_address_mask_request(self):
		"""Get the icmp-address-mask-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequest`: An object encapsulating an instance of the icmp-address-mask-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequest(self))

	def icmp_parameter_problem(self):
		"""Get the icmp-parameter-problem object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblem`: An object encapsulating an instance of the icmp-parameter-problem model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblem(self))

	def icmp_redirect(self):
		"""Get the icmp-redirect object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirect`: An object encapsulating an instance of the icmp-redirect model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirect(self))

	def icmp_router_advertisement(self):
		"""Get the icmp-router-advertisement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisement`: An object encapsulating an instance of the icmp-router-advertisement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisement(self))

	def icmp_router_solicitation(self):
		"""Get the icmp-router-solicitation object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitation`: An object encapsulating an instance of the icmp-router-solicitation model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitation(self))

	def icmp_source_quench(self):
		"""Get the icmp-source-quench object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuench`: An object encapsulating an instance of the icmp-source-quench model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuench(self))

	def icmp_time_exceeded(self):
		"""Get the icmp-time-exceeded object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceeded`: An object encapsulating an instance of the icmp-time-exceeded model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceeded(self))

	def icmp_timestamp_reply(self):
		"""Get the icmp-timestamp-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReply`: An object encapsulating an instance of the icmp-timestamp-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReply(self))

	def icmp_timestamp_request(self):
		"""Get the icmp-timestamp-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequest`: An object encapsulating an instance of the icmp-timestamp-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequest(self))

	def icmpv6_destination_unreachable(self):
		"""Get the icmpv6-destination-unreachable object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachable`: An object encapsulating an instance of the icmpv6-destination-unreachable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachable(self))

	def icmpv6_echo_reply(self):
		"""Get the icmpv6-echo-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReply`: An object encapsulating an instance of the icmpv6-echo-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReply(self))

	def icmpv6_echo_request(self):
		"""Get the icmpv6-echo-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequest`: An object encapsulating an instance of the icmpv6-echo-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequest(self))

	def icmpv6_parameter_problem(self):
		"""Get the icmpv6-parameter-problem object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblem`: An object encapsulating an instance of the icmpv6-parameter-problem model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblem(self))

	def icmpv6_time_exceeded(self):
		"""Get the icmpv6-time-exceeded object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceeded`: An object encapsulating an instance of the icmpv6-time-exceeded model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceeded(self))

	def icmpv6_packet_too_big(self):
		"""Get the icmpv6-packet-too-big object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBig`: An object encapsulating an instance of the icmpv6-packet-too-big model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBig(self))

	def igmpv1_query(self):
		"""Get the igmpv1-query object from the server

		IGMPv1 Query message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1Query`: An object encapsulating an instance of the igmpv1-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1Query(self))

	def igmpv1_report(self):
		"""Get the igmpv1-report object from the server

		IGMPv1 Report message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1Report`: An object encapsulating an instance of the igmpv1-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1Report(self))

	def igmpv2_query(self):
		"""Get the igmpv2-query object from the server

		IGMPv2 Query message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2Query`: An object encapsulating an instance of the igmpv2-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2Query(self))

	def igmpv2_report(self):
		"""Get the igmpv2-report object from the server

		IGMPv2 Report message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2Report`: An object encapsulating an instance of the igmpv2-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2Report(self))

	def igmpv3_query(self):
		"""Get the igmpv3-query object from the server

		IGMPv3 Query message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3Query`: An object encapsulating an instance of the igmpv3-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3Query(self))

	def igmpv3_report(self):
		"""Get the igmpv3-report object from the server

		IGMPv3 Report message

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3Report`: An object encapsulating an instance of the igmpv3-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3Report(self))

	def ospfv2_hello(self):
		"""Get the ospfv2-hello object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2Hello`: An object encapsulating an instance of the ospfv2-hello model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2Hello(self))

	def custom(self):
		"""Get the custom object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesCustom`: An object encapsulating an instance of the custom model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesCustom(self))

	def delete(self):
		"""Delete this frames instance on the server.

		List of user defined frames. The order of frames in the list will be the order of frames on the wire

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frames instance on the server with any changed property values.

		List of user defined frames. The order of frames in the list will be the order of frames on the wire

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')

	@property
	def frame_type(self):
		""":str:`enum`: TBD  
		Enums:  
			CUSTOM: TBD  
			ETHERNET: TBD  
			VLAN: TBD  
			IPV4: TBD  
			IPV6: TBD  
			TCP: TBD  
			UDP: TBD  
			GRE: TBD  
			ICMP_DESTINATION_UNREACHABLE: TBD  
			ICMP_ECHO_REPLY: TBD  
			ICMP_ECHO_REQUEST: TBD  
			ICMP_INFORMATION_REPLY: TBD  
			ICMP_INFORMATION_REQUEST: TBD  
			ICMP_ADDRESS_MASK_REPLY: TBD  
			ICMP_ADDRESS_MASK_REQUEST: TBD  
			ICMP_PARAMETER_PROBLEM: TBD  
			ICMP_REDIRECT: TBD  
			ICMP_ROUTER_ADVERTISEMENT: TBD  
			ICMP_ROUTER_SOLICITATION: TBD  
			ICMP_SOURCE_QUENCH: TBD  
			ICMP_TIME_EXCEEDED: TBD  
			ICMP_TIMESTAMP_REPLY: TBD  
			ICMP_TIMESTAMP_REQUEST: TBD  
			ICMPV6_DESTINATION_UNREACHABLE: TBD  
			ICMPV6_ECHO_REPLY: TBD  
			ICMPV6_ECHO_REQUEST: TBD  
			ICMPV6_PARAMETER_PROBLEM: TBD  
			ICMPV6_TIME_EXCEEDED: TBD  
			ICMPV6_PACKET_TOO_BIG: TBD  
			IGMPV1_QUERY: TBD  
			IGMPV1_REPORT: TBD  
			IGMPV2_QUERY: TBD  
			IGMPV2_REPORT: TBD  
			IGMPV3_QUERY: TBD  
			IGMPV3_REPORT: TBD  
			OSPFV2_HELLO: TBD  
		"""
		return self._get_value('frame-type')

	@frame_type.setter
	def frame_type(self, value):
		return self._set_value('frame-type', value)


class SessionsConfigPortTrafficFramesEthernet(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet'
		super(SessionsConfigPortTrafficFramesEthernet, self).__init__(parent, None)

	def destination(self):
		"""Get the destination object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetDestination`: An object encapsulating an instance of the destination model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetDestination(self))

	def source(self):
		"""Get the source object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetSource`: An object encapsulating an instance of the source model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetSource(self))

	def ethernet_type(self):
		"""Get the ethernet-type object from the server

		Ethernet type

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetEthernetType`: An object encapsulating an instance of the ethernet-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetEthernetType(self))

	def delete(self):
		"""Delete this ethernet instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesEthernetDestination(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination'
		super(SessionsConfigPortTrafficFramesEthernetDestination, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetDestinationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetDestinationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetDestinationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetDestinationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetDestinationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetDestinationRandom(self))

	def delete(self):
		"""Delete this destination instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)

	@property
	def count(self):
		"""int32: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetDestinationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesEthernetDestinationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetDestinationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesEthernetDestinationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetDestinationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesEthernetDestinationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesEthernetSource(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source'
		super(SessionsConfigPortTrafficFramesEthernetSource, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetSourceIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetSourceIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetSourceDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetSourceDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesEthernetSourceRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesEthernetSourceRandom(self))

	def delete(self):
		"""Delete this source instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)

	@property
	def count(self):
		"""int32: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetSourceIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesEthernetSourceIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetSourceDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesEthernetSourceDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesEthernetSourceRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesEthernetSourceRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesEthernetEthernetType(YangBase):
	"""Ethernet type
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet-type'
		super(SessionsConfigPortTrafficFramesEthernetEthernetType, self).__init__(parent, None)

	def delete(self):
		"""Delete this ethernet-type instance on the server.

		Ethernet type

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet-type instance on the server with any changed property values.

		Ethernet type

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesVlan(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vlan'
		super(SessionsConfigPortTrafficFramesVlan, self).__init__(parent, None)

	def priority(self):
		"""Get the priority object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanPriority`: An object encapsulating an instance of the priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanPriority(self))

	def id(self):
		"""Get the id object from the server

		A single-tagged VLAN id

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanId`: An object encapsulating an instance of the id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanId(self))

	def delete(self):
		"""Delete this vlan instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vlan instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesVlanPriority(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority'
		super(SessionsConfigPortTrafficFramesVlanPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanPriorityRandom(self))

	def delete(self):
		"""Delete this priority instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesVlanPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesVlanPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesVlanPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesVlanPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesVlanPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesVlanPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesVlanId(YangBase):
	"""A single-tagged VLAN id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'id'
		super(SessionsConfigPortTrafficFramesVlanId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesVlanIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesVlanIdRandom(self))

	def delete(self):
		"""Delete this id instance on the server.

		A single-tagged VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this id instance on the server with any changed property values.

		A single-tagged VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesVlanIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesVlanIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesVlanIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesVlanIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesVlanIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesVlanIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4'
		super(SessionsConfigPortTrafficFramesIpv4, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4SourceAddress(self))

	def destination_address(self):
		"""Get the destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DestinationAddress`: An object encapsulating an instance of the destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DestinationAddress(self))

	def ttl(self):
		"""Get the ttl object from the server

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4Ttl`: An object encapsulating an instance of the ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4Ttl(self))

	def header_length(self):
		"""Get the header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4HeaderLength`: An object encapsulating an instance of the header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4HeaderLength(self))

	def identification(self):
		"""Get the identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4Identification`: An object encapsulating an instance of the identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4Identification(self))

	def fragment_offset(self):
		"""Get the fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4FragmentOffset`: An object encapsulating an instance of the fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4FragmentOffset(self))

	def protocol(self):
		"""Get the protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4Protocol`: An object encapsulating an instance of the protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4Protocol(self))

	def checksum(self):
		"""Get the checksum object from the server

		Verifies that packets are not corrupted.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4Checksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4Checksum(self))

	def reserved_bit(self):
		"""Get the reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ReservedBit`: An object encapsulating an instance of the reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ReservedBit(self))

	def mf_bit(self):
		"""Get the mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4MfBit`: An object encapsulating an instance of the mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4MfBit(self))

	def df_bit(self):
		"""Get the df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DfBit`: An object encapsulating an instance of the df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DfBit(self))

	def delete(self):
		"""Delete this ipv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigPortTrafficFramesIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-address'
		super(SessionsConfigPortTrafficFramesIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before            the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ttl'
		super(SessionsConfigPortTrafficFramesIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ttl instance on the server.

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'header-length'
		super(SessionsConfigPortTrafficFramesIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identification'
		super(SessionsConfigPortTrafficFramesIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fragment-offset'
		super(SessionsConfigPortTrafficFramesIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'protocol'
		super(SessionsConfigPortTrafficFramesIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4Checksum(YangBase):
	"""Verifies that packets are not corrupted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIpv4Checksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Verifies that packets are not corrupted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Verifies that packets are not corrupted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-bit'
		super(SessionsConfigPortTrafficFramesIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mf-bit'
		super(SessionsConfigPortTrafficFramesIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'df-bit'
		super(SessionsConfigPortTrafficFramesIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6'
		super(SessionsConfigPortTrafficFramesIpv6, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		Specifies the source IPv6 address of the first generated packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6SourceAddress(self))

	def destination_address(self):
		"""Get the destination-address object from the server

		Specifies the destination IPv6 address of the first generated packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6DestinationAddress`: An object encapsulating an instance of the destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6DestinationAddress(self))

	def traffic_class(self):
		"""Get the traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6TrafficClass`: An object encapsulating an instance of the traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6TrafficClass(self))

	def flow_label(self):
		"""Get the flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6FlowLabel`: An object encapsulating an instance of the flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6FlowLabel(self))

	def payload_length(self):
		"""Get the payload-length object from the server

		The two-byte payload length field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6PayloadLength`: An object encapsulating an instance of the payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6PayloadLength(self))

	def next_header(self):
		"""Get the next-header object from the server

		The next header field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6NextHeader`: An object encapsulating an instance of the next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6NextHeader(self))

	def hop_limit(self):
		"""Get the hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6HopLimit`: An object encapsulating an instance of the hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6HopLimit(self))

	def delete(self):
		"""Delete this ipv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIpv6SourceAddress(YangBase):
	"""Specifies the source IPv6 address of the first generated packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigPortTrafficFramesIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		Specifies the source IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		Specifies the source IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6DestinationAddress(YangBase):
	"""Specifies the destination IPv6 address of the first generated packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-address'
		super(SessionsConfigPortTrafficFramesIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this destination-address instance on the server.

		Specifies the destination IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-address instance on the server with any changed property values.

		Specifies the destination IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'traffic-class'
		super(SessionsConfigPortTrafficFramesIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'flow-label'
		super(SessionsConfigPortTrafficFramesIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6PayloadLength(YangBase):
	"""The two-byte payload length field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload-length'
		super(SessionsConfigPortTrafficFramesIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this payload-length instance on the server.

		The two-byte payload length field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload-length instance on the server with any changed property values.

		The two-byte payload length field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6NextHeader(YangBase):
	"""The next header field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'next-header'
		super(SessionsConfigPortTrafficFramesIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this next-header instance on the server.

		The next header field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this next-header instance on the server with any changed property values.

		The next header field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hop-limit'
		super(SessionsConfigPortTrafficFramesIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcp(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'tcp'
		super(SessionsConfigPortTrafficFramesTcp, self).__init__(parent, None)

	def source_port(self):
		"""Get the source-port object from the server

		Specifies the port on the sending TCP module.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSourcePort`: An object encapsulating an instance of the source-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSourcePort(self))

	def destination_port(self):
		"""Get the destination-port object from the server

		Specifies the port on the receiving TCP module.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpDestinationPort`: An object encapsulating an instance of the destination-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpDestinationPort(self))

	def acknowledgement_number(self):
		"""Get the acknowledgement-number object from the server

		Identifies the next expected TCP octet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpAcknowledgementNumber`: An object encapsulating an instance of the acknowledgement-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpAcknowledgementNumber(self))

	def header_length(self):
		"""Get the header-length object from the server

		Specifies the data offset field in the TCP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpHeaderLength`: An object encapsulating an instance of the header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpHeaderLength(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserves TCP bits.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpReserved(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Identifies the position of the data within the data stream.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSequenceNumber(self))

	def urgent_pointer(self):
		"""Get the urgent-pointer object from the server

		Specifies the position in the segment where urgent data ends.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpUrgentPointer`: An object encapsulating an instance of the urgent-pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpUrgentPointer(self))

	def window_size(self):
		"""Get the window-size object from the server

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpWindowSize`: An object encapsulating an instance of the window-size model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpWindowSize(self))

	def acknowledgement_flag(self):
		"""Get the acknowledgement-flag object from the server

		Indicates whether the data identified by the sequence number has been received.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpAcknowledgementFlag`: An object encapsulating an instance of the acknowledgement-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpAcknowledgementFlag(self))

	def fin_flag(self):
		"""Get the fin-flag object from the server

		Indicates whether a connection is terminated.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpFinFlag`: An object encapsulating an instance of the fin-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpFinFlag(self))

	def push_flag(self):
		"""Get the push-flag object from the server

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpPushFlag`: An object encapsulating an instance of the push-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpPushFlag(self))

	def reset_flag(self):
		"""Get the reset-flag object from the server

		Resets the connection when a segment arrives that is not intended for the current connection.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpResetFlag`: An object encapsulating an instance of the reset-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpResetFlag(self))

	def sync_flag(self):
		"""Get the sync-flag object from the server

		Indicates whether the port is open for connection.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSyncFlag`: An object encapsulating an instance of the sync-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSyncFlag(self))

	def urgent_flag(self):
		"""Get the urgent-flag object from the server

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpUrgentFlag`: An object encapsulating an instance of the urgent-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpUrgentFlag(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the TCP checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpChecksum(self))

	def delete(self):
		"""Delete this tcp instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this tcp instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesTcpSourcePort(YangBase):
	"""Specifies the port on the sending TCP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-port'
		super(SessionsConfigPortTrafficFramesTcpSourcePort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSourcePortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSourcePortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSourcePortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSourcePortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSourcePortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSourcePortRandom(self))

	def delete(self):
		"""Delete this source-port instance on the server.

		Specifies the port on the sending TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-port instance on the server with any changed property values.

		Specifies the port on the sending TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpSourcePortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpSourcePortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpSourcePortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpSourcePortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpSourcePortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpSourcePortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpDestinationPort(YangBase):
	"""Specifies the port on the receiving TCP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-port'
		super(SessionsConfigPortTrafficFramesTcpDestinationPort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpDestinationPortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpDestinationPortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpDestinationPortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpDestinationPortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpDestinationPortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpDestinationPortRandom(self))

	def delete(self):
		"""Delete this destination-port instance on the server.

		Specifies the port on the receiving TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-port instance on the server with any changed property values.

		Specifies the port on the receiving TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpDestinationPortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpDestinationPortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpDestinationPortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpDestinationPortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpDestinationPortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpDestinationPortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpAcknowledgementNumber(YangBase):
	"""Identifies the next expected TCP octet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'acknowledgement-number'
		super(SessionsConfigPortTrafficFramesTcpAcknowledgementNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpAcknowledgementNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpAcknowledgementNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpAcknowledgementNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberRandom(self))

	def delete(self):
		"""Delete this acknowledgement-number instance on the server.

		Identifies the next expected TCP octet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this acknowledgement-number instance on the server with any changed property values.

		Identifies the next expected TCP octet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpAcknowledgementNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpAcknowledgementNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpAcknowledgementNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpAcknowledgementNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpHeaderLength(YangBase):
	"""Specifies the data offset field in the TCP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'header-length'
		super(SessionsConfigPortTrafficFramesTcpHeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpHeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpHeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpHeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpHeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpHeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpHeaderLengthRandom(self))

	def delete(self):
		"""Delete this header-length instance on the server.

		Specifies the data offset field in the TCP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this header-length instance on the server with any changed property values.

		Specifies the data offset field in the TCP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpHeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpHeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpHeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpHeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpHeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpHeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpReserved(YangBase):
	"""Reserves TCP bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigPortTrafficFramesTcpReserved, self).__init__(parent, None)

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserves TCP bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserves TCP bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpSequenceNumber(YangBase):
	"""Identifies the position of the data within the data stream.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesTcpSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Identifies the position of the data within the data stream.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Identifies the position of the data within the data stream.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpUrgentPointer(YangBase):
	"""Specifies the position in the segment where urgent data ends.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'urgent-pointer'
		super(SessionsConfigPortTrafficFramesTcpUrgentPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpUrgentPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpUrgentPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpUrgentPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpUrgentPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpUrgentPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpUrgentPointerRandom(self))

	def delete(self):
		"""Delete this urgent-pointer instance on the server.

		Specifies the position in the segment where urgent data ends.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this urgent-pointer instance on the server with any changed property values.

		Specifies the position in the segment where urgent data ends.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpUrgentPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpUrgentPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpUrgentPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpUrgentPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpUrgentPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpUrgentPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpWindowSize(YangBase):
	"""This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'window-size'
		super(SessionsConfigPortTrafficFramesTcpWindowSize, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpWindowSizeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpWindowSizeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpWindowSizeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpWindowSizeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpWindowSizeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpWindowSizeRandom(self))

	def delete(self):
		"""Delete this window-size instance on the server.

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this window-size instance on the server with any changed property values.

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpWindowSizeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpWindowSizeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpWindowSizeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpWindowSizeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpWindowSizeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpWindowSizeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesTcpAcknowledgementFlag(YangBase):
	"""Indicates whether the data identified by the sequence number has been received.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'acknowledgement-flag'
		super(SessionsConfigPortTrafficFramesTcpAcknowledgementFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this acknowledgement-flag instance on the server.

		Indicates whether the data identified by the sequence number has been received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this acknowledgement-flag instance on the server with any changed property values.

		Indicates whether the data identified by the sequence number has been received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpFinFlag(YangBase):
	"""Indicates whether a connection is terminated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fin-flag'
		super(SessionsConfigPortTrafficFramesTcpFinFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this fin-flag instance on the server.

		Indicates whether a connection is terminated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fin-flag instance on the server with any changed property values.

		Indicates whether a connection is terminated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpPushFlag(YangBase):
	"""Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'push-flag'
		super(SessionsConfigPortTrafficFramesTcpPushFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this push-flag instance on the server.

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this push-flag instance on the server with any changed property values.

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpResetFlag(YangBase):
	"""Resets the connection when a segment arrives that is not intended for the current connection.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reset-flag'
		super(SessionsConfigPortTrafficFramesTcpResetFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this reset-flag instance on the server.

		Resets the connection when a segment arrives that is not intended for the current connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reset-flag instance on the server with any changed property values.

		Resets the connection when a segment arrives that is not intended for the current connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpSyncFlag(YangBase):
	"""Indicates whether the port is open for connection.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sync-flag'
		super(SessionsConfigPortTrafficFramesTcpSyncFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this sync-flag instance on the server.

		Indicates whether the port is open for connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sync-flag instance on the server with any changed property values.

		Indicates whether the port is open for connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpUrgentFlag(YangBase):
	"""Identifies the incoming data as urgent, giving it priority over the other segments.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'urgent-flag'
		super(SessionsConfigPortTrafficFramesTcpUrgentFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this urgent-flag instance on the server.

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this urgent-flag instance on the server with any changed property values.

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpChecksum(YangBase):
	"""Specifies the TCP checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesTcpChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesTcpChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesTcpChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the TCP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the TCP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesTcpChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesTcpChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesTcpChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesTcpChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesTcpChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesUdp(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'udp'
		super(SessionsConfigPortTrafficFramesUdp, self).__init__(parent, None)

	def source_port(self):
		"""Get the source-port object from the server

		Specifies the port on the sending UDP module.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpSourcePort`: An object encapsulating an instance of the source-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpSourcePort(self))

	def destination_port(self):
		"""Get the destination-port object from the server

		Specifies the port on the receiving UDP module.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpDestinationPort`: An object encapsulating an instance of the destination-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpDestinationPort(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the UDP checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpChecksum(self))

	def delete(self):
		"""Delete this udp instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this udp instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesUdpSourcePort(YangBase):
	"""Specifies the port on the sending UDP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-port'
		super(SessionsConfigPortTrafficFramesUdpSourcePort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpSourcePortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpSourcePortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpSourcePortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpSourcePortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpSourcePortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpSourcePortRandom(self))

	def delete(self):
		"""Delete this source-port instance on the server.

		Specifies the port on the sending UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-port instance on the server with any changed property values.

		Specifies the port on the sending UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesUdpSourcePortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesUdpSourcePortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpSourcePortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesUdpSourcePortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpSourcePortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesUdpSourcePortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesUdpDestinationPort(YangBase):
	"""Specifies the port on the receiving UDP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-port'
		super(SessionsConfigPortTrafficFramesUdpDestinationPort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpDestinationPortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpDestinationPortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpDestinationPortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpDestinationPortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpDestinationPortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpDestinationPortRandom(self))

	def delete(self):
		"""Delete this destination-port instance on the server.

		Specifies the port on the receiving UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-port instance on the server with any changed property values.

		Specifies the port on the receiving UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesUdpDestinationPortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesUdpDestinationPortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpDestinationPortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesUdpDestinationPortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpDestinationPortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesUdpDestinationPortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesUdpChecksum(YangBase):
	"""Specifies the UDP checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesUdpChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesUdpChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesUdpChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the UDP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the UDP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesUdpChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesUdpChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesUdpChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesUdpChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesUdpChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGre(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gre'
		super(SessionsConfigPortTrafficFramesGre, self).__init__(parent, None)

	def checksum_bit(self):
		"""Get the checksum-bit object from the server

		Checksum bit.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreChecksumBit`: An object encapsulating an instance of the checksum-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreChecksumBit(self))

	def routing_bit(self):
		"""Get the routing-bit object from the server

		Routing bit.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreRoutingBit`: An object encapsulating an instance of the routing-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreRoutingBit(self))

	def key_bit(self):
		"""Get the key-bit object from the server

		Key bit.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyBit`: An object encapsulating an instance of the key-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyBit(self))

	def sequence_number_bit(self):
		"""Get the sequence-number-bit object from the server

		Sequence Number bit.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberBit`: An object encapsulating an instance of the sequence-number-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberBit(self))

	def reserved_0(self):
		"""Get the reserved-0 object from the server

		Reserved-0 bits.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved0`: An object encapsulating an instance of the reserved-0 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved0(self))

	def version(self):
		"""Get the version object from the server

		GRE Version.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreVersion(self))

	def protocol_type(self):
		"""Get the protocol-type object from the server

		Indicates the ether protocol type of the encapsulated payload.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreProtocolType`: An object encapsulating an instance of the protocol-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreProtocolType(self))

	def gre_checksum(self):
		"""Get the gre-checksum object from the server

		GRE Version.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreGreChecksum`: An object encapsulating an instance of the gre-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreGreChecksum(self))

	def reserved_1(self):
		"""Get the reserved-1 object from the server

		Reserved-1 bits.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved1`: An object encapsulating an instance of the reserved-1 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved1(self))

	def key(self):
		"""Get the key object from the server

		Key Value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKey`: An object encapsulating an instance of the key model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKey(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumber(self))

	def keep_alive_bit(self):
		"""Get the keep-alive-bit object from the server

		Keep Alive bit.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveBit`: An object encapsulating an instance of the keep-alive-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveBit(self))

	def keep_alive_period(self):
		"""Get the keep-alive-period object from the server

		Keep alive period.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAlivePeriod`: An object encapsulating an instance of the keep-alive-period model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAlivePeriod(self))

	def keep_alive_retries(self):
		"""Get the keep-alive-retries object from the server

		Keep alive retries.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveRetries`: An object encapsulating an instance of the keep-alive-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveRetries(self))

	def delete(self):
		"""Delete this gre instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gre instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesGreChecksumBit(YangBase):
	"""Checksum bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum-bit'
		super(SessionsConfigPortTrafficFramesGreChecksumBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreChecksumBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreChecksumBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreChecksumBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreChecksumBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreChecksumBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreChecksumBitRandom(self))

	def delete(self):
		"""Delete this checksum-bit instance on the server.

		Checksum bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum-bit instance on the server with any changed property values.

		Checksum bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreChecksumBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreChecksumBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreChecksumBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreChecksumBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreChecksumBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreChecksumBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreRoutingBit(YangBase):
	"""Routing bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'routing-bit'
		super(SessionsConfigPortTrafficFramesGreRoutingBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreRoutingBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreRoutingBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreRoutingBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreRoutingBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreRoutingBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreRoutingBitRandom(self))

	def delete(self):
		"""Delete this routing-bit instance on the server.

		Routing bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this routing-bit instance on the server with any changed property values.

		Routing bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreRoutingBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreRoutingBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreRoutingBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreRoutingBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreRoutingBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreRoutingBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreKeyBit(YangBase):
	"""Key bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key-bit'
		super(SessionsConfigPortTrafficFramesGreKeyBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyBitRandom(self))

	def delete(self):
		"""Delete this key-bit instance on the server.

		Key bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key-bit instance on the server with any changed property values.

		Key bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreKeyBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreKeyBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeyBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreKeyBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeyBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreKeyBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberBit(YangBase):
	"""Sequence Number bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number-bit'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberBitRandom(self))

	def delete(self):
		"""Delete this sequence-number-bit instance on the server.

		Sequence Number bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number-bit instance on the server with any changed property values.

		Sequence Number bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreReserved0(YangBase):
	"""Reserved-0 bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-0'
		super(SessionsConfigPortTrafficFramesGreReserved0, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved0Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved0Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved0Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved0Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved0Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved0Random(self))

	def delete(self):
		"""Delete this reserved-0 instance on the server.

		Reserved-0 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-0 instance on the server with any changed property values.

		Reserved-0 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreReserved0Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreReserved0Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreReserved0Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreReserved0Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreReserved0Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreReserved0Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreVersion(YangBase):
	"""GRE Version.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigPortTrafficFramesGreVersion, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreVersionIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreVersionIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreVersionDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreVersionDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreVersionRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreVersionRandom(self))

	def delete(self):
		"""Delete this version instance on the server.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreVersionIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreVersionIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreVersionDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreVersionDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreVersionRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreVersionRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreProtocolType(YangBase):
	"""Indicates the ether protocol type of the encapsulated payload.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'protocol-type'
		super(SessionsConfigPortTrafficFramesGreProtocolType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreProtocolTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreProtocolTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreProtocolTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreProtocolTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreProtocolTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreProtocolTypeRandom(self))

	def delete(self):
		"""Delete this protocol-type instance on the server.

		Indicates the ether protocol type of the encapsulated payload.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this protocol-type instance on the server with any changed property values.

		Indicates the ether protocol type of the encapsulated payload.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreProtocolTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreProtocolTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreProtocolTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreProtocolTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreProtocolTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreProtocolTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreGreChecksum(YangBase):
	"""GRE Version.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gre-checksum'
		super(SessionsConfigPortTrafficFramesGreGreChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreGreChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreGreChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreGreChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreGreChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreGreChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreGreChecksumRandom(self))

	def delete(self):
		"""Delete this gre-checksum instance on the server.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gre-checksum instance on the server with any changed property values.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreGreChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreGreChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreGreChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreGreChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreGreChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreGreChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreReserved1(YangBase):
	"""Reserved-1 bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-1'
		super(SessionsConfigPortTrafficFramesGreReserved1, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved1Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved1Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved1Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved1Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreReserved1Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreReserved1Random(self))

	def delete(self):
		"""Delete this reserved-1 instance on the server.

		Reserved-1 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-1 instance on the server with any changed property values.

		Reserved-1 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreReserved1Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreReserved1Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreReserved1Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreReserved1Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreReserved1Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreReserved1Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreKey(YangBase):
	"""Key Value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key'
		super(SessionsConfigPortTrafficFramesGreKey, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeyRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeyRandom(self))

	def delete(self):
		"""Delete this key instance on the server.

		Key Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key instance on the server with any changed property values.

		Key Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreKeyIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreKeyIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeyDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreKeyDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeyRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreKeyRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesGreSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreKeepAliveBit(YangBase):
	"""Keep Alive bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-bit'
		super(SessionsConfigPortTrafficFramesGreKeepAliveBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveBitRandom(self))

	def delete(self):
		"""Delete this keep-alive-bit instance on the server.

		Keep Alive bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-bit instance on the server with any changed property values.

		Keep Alive bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreKeepAliveBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreKeepAliveBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAliveBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreKeepAliveBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAliveBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreKeepAliveBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreKeepAlivePeriod(YangBase):
	"""Keep alive period.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-period'
		super(SessionsConfigPortTrafficFramesGreKeepAlivePeriod, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAlivePeriodIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAlivePeriodIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAlivePeriodDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAlivePeriodDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAlivePeriodRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAlivePeriodRandom(self))

	def delete(self):
		"""Delete this keep-alive-period instance on the server.

		Keep alive period.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-period instance on the server with any changed property values.

		Keep alive period.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreKeepAlivePeriodIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreKeepAlivePeriodIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAlivePeriodDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreKeepAlivePeriodDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAlivePeriodRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreKeepAlivePeriodRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesGreKeepAliveRetries(YangBase):
	"""Keep alive retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-retries'
		super(SessionsConfigPortTrafficFramesGreKeepAliveRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesGreKeepAliveRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesGreKeepAliveRetriesRandom(self))

	def delete(self):
		"""Delete this keep-alive-retries instance on the server.

		Keep alive retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-retries instance on the server with any changed property values.

		Keep alive retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesGreKeepAliveRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesGreKeepAliveRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAliveRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesGreKeepAliveRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesGreKeepAliveRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesGreKeepAliveRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachable(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-destination-unreachable'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachable, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Destination Unreachable code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableCode(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnused(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Data(self))

	def delete(self):
		"""Delete this icmp-destination-unreachable instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-destination-unreachable instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableCode(YangBase):
	"""ICMP Destination Unreachable code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Destination Unreachable code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Destination Unreachable code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpDestinationUnreachableIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-echo-reply'
		super(SessionsConfigPortTrafficFramesIcmpEchoReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Echo reply code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksum(self))

	def delete(self):
		"""Delete this icmp-echo-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-echo-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpEchoReplyCode(YangBase):
	"""ICMP Echo reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-echo-request'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Echo request code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksum(self))

	def delete(self):
		"""Delete this icmp-echo-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-echo-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpEchoRequestCode(YangBase):
	"""ICMP Echo request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpEchoRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-information-reply'
		super(SessionsConfigPortTrafficFramesIcmpInformationReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP information reply code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksum(self))

	def delete(self):
		"""Delete this icmp-information-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-information-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpInformationReplyCode(YangBase):
	"""ICMP information reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP information reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP information reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-information-request'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP information request code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksum(self))

	def delete(self):
		"""Delete this icmp-information-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-information-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpInformationRequestCode(YangBase):
	"""ICMP information request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP information request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP information request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpInformationRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-address-mask-reply'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP address mask reply code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksum(self))

	def address_mask(self):
		"""Get the address-mask object from the server

		Address Mask.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMask`: An object encapsulating an instance of the address-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMask(self))

	def delete(self):
		"""Delete this icmp-address-mask-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-address-mask-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCode(YangBase):
	"""ICMP address mask reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP address mask reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP address mask reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMask(YangBase):
	"""Address Mask.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-mask'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskRandom(self))

	def delete(self):
		"""Delete this address-mask instance on the server.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-mask instance on the server with any changed property values.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskReplyAddressMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-address-mask-request'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP address mask request code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksum(self))

	def address_mask(self):
		"""Get the address-mask object from the server

		Address Mask.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMask`: An object encapsulating an instance of the address-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMask(self))

	def delete(self):
		"""Delete this icmp-address-mask-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-address-mask-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCode(YangBase):
	"""ICMP address mask request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP address mask request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP address mask request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMask(YangBase):
	"""Address Mask.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-mask'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskRandom(self))

	def delete(self):
		"""Delete this address-mask instance on the server.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-mask instance on the server with any changed property values.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpAddressMaskRequestAddressMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblem(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-parameter-problem'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblem, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Destination Parameter problem code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemCode(self))

	def pointer(self):
		"""Get the pointer object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemPointer`: An object encapsulating an instance of the pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemPointer(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemUnused(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Data(self))

	def delete(self):
		"""Delete this icmp-parameter-problem instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-parameter-problem instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpParameterProblemCode(YangBase):
	"""ICMP Destination Parameter problem code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Destination Parameter problem code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Destination Parameter problem code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemPointer(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pointer'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerRandom(self))

	def delete(self):
		"""Delete this pointer instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pointer instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpParameterProblemIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirect(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-redirect'
		super(SessionsConfigPortTrafficFramesIcmpRedirect, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Redirect code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectCode(self))

	def gateway_address(self):
		"""Get the gateway-address object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddress`: An object encapsulating an instance of the gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Data(self))

	def delete(self):
		"""Delete this icmp-redirect instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-redirect instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpRedirectCode(YangBase):
	"""ICMP Redirect code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpRedirectCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Redirect code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Redirect code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddress(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressRandom(self))

	def delete(self):
		"""Delete this gateway-address instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gateway-address instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectGatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpRedirectChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRedirectIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-advertisement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisement, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Router Advertisement code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCode(self))

	def advertise_count(self):
		"""Get the advertise-count object from the server

		Advertise Count.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCount`: An object encapsulating an instance of the advertise-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCount(self))

	def life_time(self):
		"""Get the life-time object from the server

		Life Time.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTime`: An object encapsulating an instance of the life-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTime(self))

	def address_entry_size(self):
		"""Get the address-entry-size object from the server

		Address Entry Size.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySize`: An object encapsulating an instance of the address-entry-size model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySize(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksum(self))

	def icmp_router_address(self, name=None):
		"""Get the icmp-router-address object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the icmp-router-address list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress` | :obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress`: If arg name is None a list of SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress objects otherwise a single SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress object.  

		Raises:  
			NotFoundError: name is not in the list of icmp-router-address objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(self, name))

	def create_icmp_router_address(self, name):
		"""Create a child sibling icmp-router-address instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress`: An object encapsulating an instance of the icmp-router-address model.  

		Raises:  
			AlreadyExistsError: An instance of icmp-router-address with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(self, name), locals())

	def delete(self):
		"""Delete this icmp-router-advertisement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-advertisement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCode(YangBase):
	"""ICMP Router Advertisement code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Router Advertisement code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Router Advertisement code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCount(YangBase):
	"""Advertise Count.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-count'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom(self))

	def delete(self):
		"""Delete this advertise-count instance on the server.

		Advertise Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-count instance on the server with any changed property values.

		Advertise Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTime(YangBase):
	"""Life Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'life-time'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeRandom(self))

	def delete(self):
		"""Delete this life-time instance on the server.

		Life Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this life-time instance on the server with any changed property values.

		Life Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementLifeTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySize(YangBase):
	"""Address Entry Size.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-entry-size'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySize, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom(self))

	def delete(self):
		"""Delete this address-entry-size instance on the server.

		Address Entry Size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-entry-size instance on the server with any changed property values.

		Address Entry Size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-address'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress, self).__init__(parent, yang_key_value)

	def router_address(self):
		"""Get the router-address object from the server

		Router Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress`: An object encapsulating an instance of the router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress(self))

	def preference_level(self):
		"""Get the preference-level object from the server

		Preference Level

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel`: An object encapsulating an instance of the preference-level model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel(self))

	def delete(self):
		"""Delete this icmp-router-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""str: Name"""
		return self._get_value('name')


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress(YangBase):
	"""Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-address'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom(self))

	def delete(self):
		"""Delete this router-address instance on the server.

		Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-address instance on the server with any changed property values.

		Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel(YangBase):
	"""Preference Level
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preference-level'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom(self))

	def delete(self):
		"""Delete this preference-level instance on the server.

		Preference Level

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preference-level instance on the server with any changed property values.

		Preference Level

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitation(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-solicitation'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitation, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Router Solicitation code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCode(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReserved(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksum(self))

	def delete(self):
		"""Delete this icmp-router-solicitation instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-solicitation instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationCode(YangBase):
	"""ICMP Router Solicitation code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Router Solicitation code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Router Solicitation code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationReserved(YangBase):
	"""Reserved.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpRouterSolicitationChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuench(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-source-quench'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuench, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Source Quench code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Data(self))

	def delete(self):
		"""Delete this icmp-source-quench instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-source-quench instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpSourceQuenchCode(YangBase):
	"""ICMP Source Quench code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Source Quench code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Source Quench code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpSourceQuenchIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceeded(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-time-exceeded'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceeded, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Time exceeded code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Data(self))

	def delete(self):
		"""Delete this icmp-time-exceeded instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-time-exceeded instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpTimeExceededCode(YangBase):
	"""ICMP Time exceeded code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Time exceeded code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Time exceeded code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimeExceededIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-timestamp-reply'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Timestamp reply code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifier(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksum(self))

	def sequence_number(self):
		"""Get the sequence_number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_number`: An object encapsulating an instance of the sequence_number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_number(self))

	def originate_timestamp(self):
		"""Get the originate-timestamp object from the server

		Originate timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestamp`: An object encapsulating an instance of the originate-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestamp(self))

	def receive_timestamp(self):
		"""Get the receive-timestamp object from the server

		Receive timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestamp`: An object encapsulating an instance of the receive-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestamp(self))

	def transmit_timestamp(self):
		"""Get the transmit-timestamp object from the server

		Transmit timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestamp`: An object encapsulating an instance of the transmit-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestamp(self))

	def delete(self):
		"""Delete this icmp-timestamp-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-timestamp-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpTimestampReplyCode(YangBase):
	"""ICMP Timestamp reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Timestamp reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Timestamp reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifier(YangBase):
	"""Identifier.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_number(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence_number'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_number, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberRandom(self))

	def delete(self):
		"""Delete this sequence_number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence_number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplySequence_numberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestamp(YangBase):
	"""Originate timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'originate-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampRandom(self))

	def delete(self):
		"""Delete this originate-timestamp instance on the server.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this originate-timestamp instance on the server with any changed property values.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyOriginateTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestamp(YangBase):
	"""Receive timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampRandom(self))

	def delete(self):
		"""Delete this receive-timestamp instance on the server.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-timestamp instance on the server with any changed property values.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyReceiveTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestamp(YangBase):
	"""Transmit timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampRandom(self))

	def delete(self):
		"""Delete this transmit-timestamp instance on the server.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-timestamp instance on the server with any changed property values.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampReplyTransmitTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-timestamp-request'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Timestamp request code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifier(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksum(self))

	def sequence_number(self):
		"""Get the sequence_number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_number`: An object encapsulating an instance of the sequence_number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_number(self))

	def originate_timestamp(self):
		"""Get the originate-timestamp object from the server

		Originate timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestamp`: An object encapsulating an instance of the originate-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestamp(self))

	def receive_timestamp(self):
		"""Get the receive-timestamp object from the server

		Receive timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestamp`: An object encapsulating an instance of the receive-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestamp(self))

	def transmit_timestamp(self):
		"""Get the transmit-timestamp object from the server

		Transmit timestamp.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestamp`: An object encapsulating an instance of the transmit-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestamp(self))

	def delete(self):
		"""Delete this icmp-timestamp-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-timestamp-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpTimestampRequestCode(YangBase):
	"""ICMP Timestamp request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Timestamp request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Timestamp request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifier(YangBase):
	"""Identifier.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_number(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence_number'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_number, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberRandom(self))

	def delete(self):
		"""Delete this sequence_number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence_number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestSequence_numberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestamp(YangBase):
	"""Originate timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'originate-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampRandom(self))

	def delete(self):
		"""Delete this originate-timestamp instance on the server.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this originate-timestamp instance on the server with any changed property values.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestOriginateTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestamp(YangBase):
	"""Receive timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampRandom(self))

	def delete(self):
		"""Delete this receive-timestamp instance on the server.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-timestamp instance on the server with any changed property values.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestReceiveTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestamp(YangBase):
	"""Transmit timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-timestamp'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampRandom(self))

	def delete(self):
		"""Delete this transmit-timestamp instance on the server.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-timestamp instance on the server with any changed property values.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpTimestampRequestTransmitTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachable(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-destination-unreachable'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachable, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableCode(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnused(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-destination-unreachable instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-destination-unreachable instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableCode(YangBase):
	"""ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-echo-reply'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Echo reply code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksum(self))

	def delete(self):
		"""Delete this icmpv6-echo-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-echo-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyCode(YangBase):
	"""ICMPv6 Echo reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-echo-request'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Echo request code.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksum(self))

	def delete(self):
		"""Delete this icmpv6-echo-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-echo-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestCode(YangBase):
	"""ICMPv6 Echo request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6EchoRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblem(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-parameter-problem'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblem, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemCode(self))

	def pointer(self):
		"""Get the pointer object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointer`: An object encapsulating an instance of the pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointer(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-parameter-problem instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-parameter-problem instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemCode(YangBase):
	"""ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointer(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pointer'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerRandom(self))

	def delete(self):
		"""Delete this pointer instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pointer instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6ParameterProblemIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceeded(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-time-exceeded'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceeded, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-time-exceeded instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-time-exceeded instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededCode(YangBase):
	"""ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6TimeExceededIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBig(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-packet-too-big'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBig, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Packet too big.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCode(self))

	def mtu(self):
		"""Get the mtu object from the server

		MTU.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtu`: An object encapsulating an instance of the mtu model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtu(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-packet-too-big instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-packet-too-big instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCode(YangBase):
	"""ICMPv6 Packet too big.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Packet too big.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Packet too big.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtu(YangBase):
	"""MTU.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mtu'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtu, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuRandom(self))

	def delete(self):
		"""Delete this mtu instance on the server.

		MTU.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mtu instance on the server with any changed property values.

		MTU.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigMtuRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIcmpv6PacketTooBigIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1Query(YangBase):
	"""IGMPv1 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv1-query'
		super(SessionsConfigPortTrafficFramesIgmpv1Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryUnused(self))

	def delete(self):
		"""Delete this igmpv1-query instance on the server.

		IGMPv1 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv1-query instance on the server with any changed property values.

		IGMPv1 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1QueryUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1QueryUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1QueryUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1Report(YangBase):
	"""IGMPv1 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv1-report'
		super(SessionsConfigPortTrafficFramesIgmpv1Report, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportUnused(self))

	def delete(self):
		"""Delete this igmpv1-report instance on the server.

		IGMPv1 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv1-report instance on the server with any changed property values.

		IGMPv1 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv1ReportUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv1ReportUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv1ReportUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2Query(YangBase):
	"""IGMPv2 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv2-query'
		super(SessionsConfigPortTrafficFramesIgmpv2Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTime(self))

	def delete(self):
		"""Delete this igmpv2-query instance on the server.

		IGMPv2 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv2-query instance on the server with any changed property values.

		IGMPv2 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2QueryMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2Report(YangBase):
	"""IGMPv2 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv2-report'
		super(SessionsConfigPortTrafficFramesIgmpv2Report, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTime(self))

	def delete(self):
		"""Delete this igmpv2-report instance on the server.

		IGMPv2 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv2-report instance on the server with any changed property values.

		IGMPv2 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv2ReportMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3Query(YangBase):
	"""IGMPv3 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv3-query'
		super(SessionsConfigPortTrafficFramesIgmpv3Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTime(self))

	def number_of_sources(self):
		"""Get the number-of-sources object from the server

		Number of Sources.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSources`: An object encapsulating an instance of the number-of-sources model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSources(self))

	def suppress_flag(self):
		"""Get the suppress-flag object from the server

		Suppress Flag.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlag`: An object encapsulating an instance of the suppress-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlag(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryReserved(self))

	def qqic(self):
		"""Get the qqic object from the server

		QQIC(Querier's Query Interval Code).

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQqic`: An object encapsulating an instance of the qqic model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQqic(self))

	def qrv(self):
		"""Get the qrv object from the server

		QRV bits(Querier's Robustness Variable).

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQrv`: An object encapsulating an instance of the qrv model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQrv(self))

	def source_address_list(self, name=None):
		"""Get the source-address-list object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the source-address-list list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList` | :obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList`: If arg name is None a list of SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList objects otherwise a single SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList object.  

		Raises:  
			NotFoundError: name is not in the list of source-address-list objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList(self, name))

	def create_source_address_list(self, name):
		"""Create a child sibling source-address-list instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList`: An object encapsulating an instance of the source-address-list model.  

		Raises:  
			AlreadyExistsError: An instance of source-address-list with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList(self, name), locals())

	def delete(self):
		"""Delete this igmpv3-query instance on the server.

		IGMPv3 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv3-query instance on the server with any changed property values.

		IGMPv3 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSources(YangBase):
	"""Number of Sources.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-sources'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSources, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesRandom(self))

	def delete(self):
		"""Delete this number-of-sources instance on the server.

		Number of Sources.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-sources instance on the server with any changed property values.

		Number of Sources.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryNumberOfSourcesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlag(YangBase):
	"""Suppress Flag.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'suppress-flag'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlag, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagRandom(self))

	def delete(self):
		"""Delete this suppress-flag instance on the server.

		Suppress Flag.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this suppress-flag instance on the server with any changed property values.

		Suppress Flag.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySuppressFlagRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryReserved(YangBase):
	"""Reserved.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQqic(YangBase):
	"""QQIC(Querier's Query Interval Code).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'qqic'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQqic, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQqicIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQqicIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQqicDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQqicDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQqicRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQqicRandom(self))

	def delete(self):
		"""Delete this qqic instance on the server.

		QQIC(Querier's Query Interval Code).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this qqic instance on the server with any changed property values.

		QQIC(Querier's Query Interval Code).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQqicIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQqicIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQqicDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQqicDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQqicRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQqicRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQrv(YangBase):
	"""QRV bits(Querier's Robustness Variable).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'qrv'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQrv, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQrvIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQrvIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQrvDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQrvDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QueryQrvRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QueryQrvRandom(self))

	def delete(self):
		"""Delete this qrv instance on the server.

		QRV bits(Querier's Robustness Variable).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this qrv instance on the server with any changed property values.

		QRV bits(Querier's Robustness Variable).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQrvIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQrvIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQrvDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQrvDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QueryQrvRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QueryQrvRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address-list'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressList, self).__init__(parent, yang_key_value)

	def address(self):
		"""Get the address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddress(self))

	def delete(self):
		"""Delete this source-address-list instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address-list instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')


class SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3QuerySourceAddressListAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3Report(YangBase):
	"""IGMPv3 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv3-report'
		super(SessionsConfigPortTrafficFramesIgmpv3Report, self).__init__(parent, None)

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportChecksum(self))

	def number_of_group_records(self):
		"""Get the number-of-group-records object from the server

		Number of group records.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecords`: An object encapsulating an instance of the number-of-group-records model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecords(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReserved(self))

	def reserved2(self):
		"""Get the reserved2 object from the server

		Reserved2

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReserved2`: An object encapsulating an instance of the reserved2 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2(self))

	def delete(self):
		"""Delete this igmpv3-report instance on the server.

		IGMPv3 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv3-report instance on the server with any changed property values.

		IGMPv3 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesIgmpv3ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecords(YangBase):
	"""Number of group records.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-group-records'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecords, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom(self))

	def delete(self):
		"""Delete this number-of-group-records instance on the server.

		Number of group records.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-group-records instance on the server with any changed property values.

		Number of group records.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReserved(YangBase):
	"""Reserved
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReserved2(YangBase):
	"""Reserved2
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved2'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Random(self))

	def delete(self):
		"""Delete this reserved2 instance on the server.

		Reserved2

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved2 instance on the server with any changed property values.

		Reserved2

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesIgmpv3ReportReserved2Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2Hello(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-hello'
		super(SessionsConfigPortTrafficFramesOspfv2Hello, self).__init__(parent, None)

	def designated_router_address(self):
		"""Get the designated-router-address object from the server

		Designated Router Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddress`: An object encapsulating an instance of the designated-router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddress(self))

	def backup_designated_router_address(self):
		"""Get the backup-designated-router-address object from the server

		Backup Designated Router Address

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddress`: An object encapsulating an instance of the backup-designated-router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddress(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Hello Interval.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloHelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloHelloInterval(self))

	def router_dead_interval(self):
		"""Get the router-dead-interval object from the server

		Router Dead Interval.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadInterval`: An object encapsulating an instance of the router-dead-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadInterval(self))

	def router_priority(self):
		"""Get the router-priority object from the server

		Router Priority.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterPriority`: An object encapsulating an instance of the router-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriority(self))

	def network_mask(self):
		"""Get the network-mask object from the server

		Network mask

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNetworkMask`: An object encapsulating an instance of the network-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMask(self))

	def area_id(self):
		"""Get the area-id object from the server

		Area Id

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAreaId`: An object encapsulating an instance of the area-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAreaId(self))

	def router_id(self):
		"""Get the router-id object from the server

		Router Id

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterId`: An object encapsulating an instance of the router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterId(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloChecksum(self))

	def packet_length(self):
		"""Get the packet-length object from the server

		Packet Length.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloPacketLength`: An object encapsulating an instance of the packet-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloPacketLength(self))

	def auth_type(self):
		"""Get the auth-type object from the server

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthType`: An object encapsulating an instance of the auth-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthType(self))

	def auth_value1(self):
		"""Get the auth-value1 object from the server

		Authentication Value 1.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1`: An object encapsulating an instance of the auth-value1 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1(self))

	def auth_value2(self):
		"""Get the auth-value2 object from the server

		Authentication Value 2.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2`: An object encapsulating an instance of the auth-value2 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2(self))

	def options_reserved_bit_7(self):
		"""Get the options-reserved-bit-7 object from the server

		Reserved Bit 7

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7`: An object encapsulating an instance of the options-reserved-bit-7 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7(self))

	def options_reserved_bit_6(self):
		"""Get the options-reserved-bit-6 object from the server

		Reserved Bit 6

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6`: An object encapsulating an instance of the options-reserved-bit-6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6(self))

	def options_dc_bit(self):
		"""Get the options-dc-bit object from the server

		Demand Circuits Bit

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBit`: An object encapsulating an instance of the options-dc-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBit(self))

	def options_ea_bit(self):
		"""Get the options-ea-bit object from the server

		EA Bit

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBit`: An object encapsulating an instance of the options-ea-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBit(self))

	def options_np_bit(self):
		"""Get the options-np-bit object from the server

		NSSA Support

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBit`: An object encapsulating an instance of the options-np-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBit(self))

	def options_mc_bit(self):
		"""Get the options-mc-bit object from the server

		Multicast Capable

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBit`: An object encapsulating an instance of the options-mc-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBit(self))

	def options_e_bit(self):
		"""Get the options-e-bit object from the server

		E Bit, External Routing Capability.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBit`: An object encapsulating an instance of the options-e-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBit(self))

	def options_reserved_bit_0(self):
		"""Get the options-reserved-bit-0 object from the server

		Reserved Bit 0

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0`: An object encapsulating an instance of the options-reserved-bit-0 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0(self))

	def neighbors(self, name=None):
		"""Get the neighbors object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the neighbors list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighbors` | :obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighbors`: If arg name is None a list of SessionsConfigPortTrafficFramesOspfv2HelloNeighbors objects otherwise a single SessionsConfigPortTrafficFramesOspfv2HelloNeighbors object.  

		Raises:  
			NotFoundError: name is not in the list of neighbors objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNeighbors(self, name))

	def create_neighbors(self, name):
		"""Create a child sibling neighbors instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighbors`: An object encapsulating an instance of the neighbors model.  

		Raises:  
			AlreadyExistsError: An instance of neighbors with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTrafficFramesOspfv2HelloNeighbors(self, name), locals())

	def delete(self):
		"""Delete this ospfv2-hello instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-hello instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddress(YangBase):
	"""Designated Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'designated-router-address'
		super(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressRandom(self))

	def delete(self):
		"""Delete this designated-router-address instance on the server.

		Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this designated-router-address instance on the server with any changed property values.

		Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloDesignatedRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddress(YangBase):
	"""Backup Designated Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'backup-designated-router-address'
		super(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom(self))

	def delete(self):
		"""Delete this backup-designated-router-address instance on the server.

		Backup Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this backup-designated-router-address instance on the server with any changed property values.

		Backup Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloHelloInterval(YangBase):
	"""Hello Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigPortTrafficFramesOspfv2HelloHelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Hello Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Hello Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloHelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadInterval(YangBase):
	"""Router Dead Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-dead-interval'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalRandom(self))

	def delete(self):
		"""Delete this router-dead-interval instance on the server.

		Router Dead Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-dead-interval instance on the server with any changed property values.

		Router Dead Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterDeadIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterPriority(YangBase):
	"""Router Priority.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-priority'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityRandom(self))

	def delete(self):
		"""Delete this router-priority instance on the server.

		Router Priority.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-priority instance on the server with any changed property values.

		Router Priority.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNetworkMask(YangBase):
	"""Network mask
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'network-mask'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskRandom(self))

	def delete(self):
		"""Delete this network-mask instance on the server.

		Network mask

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this network-mask instance on the server with any changed property values.

		Network mask

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNetworkMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAreaId(YangBase):
	"""Area Id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'area-id'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAreaId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAreaIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAreaIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAreaIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdRandom(self))

	def delete(self):
		"""Delete this area-id instance on the server.

		Area Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this area-id instance on the server with any changed property values.

		Area Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAreaIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAreaIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAreaIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAreaIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterId(YangBase):
	"""Router Id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-id'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdRandom(self))

	def delete(self):
		"""Delete this router-id instance on the server.

		Router Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-id instance on the server with any changed property values.

		Router Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigPortTrafficFramesOspfv2HelloChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloPacketLength(YangBase):
	"""Packet Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'packet-length'
		super(SessionsConfigPortTrafficFramesOspfv2HelloPacketLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthRandom(self))

	def delete(self):
		"""Delete this packet-length instance on the server.

		Packet Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this packet-length instance on the server with any changed property values.

		Packet Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloPacketLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthType(YangBase):
	"""Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-type'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthType, self).__init__(parent, None)

	def delete(self):
		"""Delete this auth-type instance on the server.

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-type instance on the server with any changed property values.

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1(YangBase):
	"""Authentication Value 1.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-value1'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Random(self))

	def delete(self):
		"""Delete this auth-value1 instance on the server.

		Authentication Value 1.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-value1 instance on the server with any changed property values.

		Authentication Value 1.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue1Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2(YangBase):
	"""Authentication Value 2.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-value2'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Random(self))

	def delete(self):
		"""Delete this auth-value2 instance on the server.

		Authentication Value 2.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-value2 instance on the server with any changed property values.

		Authentication Value 2.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloAuthValue2Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7(YangBase):
	"""Reserved Bit 7
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-7'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-7 instance on the server.

		Reserved Bit 7

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-7 instance on the server with any changed property values.

		Reserved Bit 7

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit7Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6(YangBase):
	"""Reserved Bit 6
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-6'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-6 instance on the server.

		Reserved Bit 6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-6 instance on the server with any changed property values.

		Reserved Bit 6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit6Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBit(YangBase):
	"""Demand Circuits Bit
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-dc-bit'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitRandom(self))

	def delete(self):
		"""Delete this options-dc-bit instance on the server.

		Demand Circuits Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-dc-bit instance on the server with any changed property values.

		Demand Circuits Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsDcBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBit(YangBase):
	"""EA Bit
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-ea-bit'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitRandom(self))

	def delete(self):
		"""Delete this options-ea-bit instance on the server.

		EA Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-ea-bit instance on the server with any changed property values.

		EA Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEaBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBit(YangBase):
	"""NSSA Support
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-np-bit'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitRandom(self))

	def delete(self):
		"""Delete this options-np-bit instance on the server.

		NSSA Support

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-np-bit instance on the server with any changed property values.

		NSSA Support

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsNpBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBit(YangBase):
	"""Multicast Capable
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-mc-bit'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitRandom(self))

	def delete(self):
		"""Delete this options-mc-bit instance on the server.

		Multicast Capable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-mc-bit instance on the server with any changed property values.

		Multicast Capable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsMcBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBit(YangBase):
	"""E Bit, External Routing Capability.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-e-bit'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitRandom(self))

	def delete(self):
		"""Delete this options-e-bit instance on the server.

		E Bit, External Routing Capability.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-e-bit instance on the server with any changed property values.

		E Bit, External Routing Capability.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsEBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0(YangBase):
	"""Reserved Bit 0
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-0'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-0 instance on the server.

		Reserved Bit 0

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-0 instance on the server with any changed property values.

		Reserved Bit 0

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloOptionsReservedBit0Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNeighbors(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbors'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNeighbors, self).__init__(parent, yang_key_value)

	def neighbors_id(self):
		"""Get the neighbors-id object from the server

		Neighbor ID

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsId`: An object encapsulating an instance of the neighbors-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsId(self))

	def delete(self):
		"""Delete this neighbors instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbors instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')


class SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsId(YangBase):
	"""Neighbor ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbors-id'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom(self))

	def delete(self):
		"""Delete this neighbors-id instance on the server.

		Neighbor ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbors-id instance on the server with any changed property values.

		Neighbor ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigPortTrafficFramesCustom(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom'
		super(SessionsConfigPortTrafficFramesCustom, self).__init__(parent, None)

	def payload(self):
		"""Get the payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFramesCustomPayload`: An object encapsulating an instance of the payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFramesCustomPayload(self))

	def delete(self):
		"""Delete this custom instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def payload_type(self):
		""":str:`enum`: TBD  
		Enums:  
			INCREMENT_BYTE: TBD  
			DECREMENT_BYTE: TBD  
			INCREMENT_WORD: TBD  
			DECREMENT_WORD: TBD  
			CRPAT: TBD  
			CJPAT: TBD  
			RANDOM: TBD  
			CUSTOM: TBD  
		"""
		return self._get_value('payload-type')

	@payload_type.setter
	def payload_type(self, value):
		return self._set_value('payload-type', value)


class SessionsConfigPortTrafficFramesCustomPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload'
		super(SessionsConfigPortTrafficFramesCustomPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def data(self):
		"""iyt:hex-string: TBD"""
		return self._get_value('data')

	@data.setter
	def data(self, value):
		return self._set_value('data', value)

	@property
	def repeat(self):
		"""bool: Repeat the payload data to fill the length specified"""
		return self._get_value('repeat')

	@repeat.setter
	def repeat(self, value):
		return self._set_value('repeat', value)

	@property
	def length(self):
		"""int32: TBD"""
		return self._get_value('length')

	@length.setter
	def length(self, value):
		return self._set_value('length', value)


class SessionsConfigPortTrafficFrameLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frame-length'
		super(SessionsConfigPortTrafficFrameLength, self).__init__(parent, None)

	def random(self):
		"""Get the random object from the server

		Random frame size options

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLengthRandom(self))

	def increment(self):
		"""Get the increment object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLengthDecrement(self))

	def imix(self):
		"""Get the imix object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLengthImix`: An object encapsulating an instance of the imix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLengthImix(self))

	def delete(self):
		"""Delete this frame-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frame-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def length_type(self):
		""":str:`enum`: TBD  
		Enums:  
			FIXED: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			AUTO: TBD  
			IMIX: TBD  
		"""
		return self._get_value('length-type')

	@length_type.setter
	def length_type(self, value):
		return self._set_value('length-type', value)

	@property
	def fixed(self):
		"""int32: Fixed value for frame length"""
		return self._get_value('fixed')

	@fixed.setter
	def fixed(self, value):
		return self._set_value('fixed', value)


class SessionsConfigPortTrafficFrameLengthRandom(YangBase):
	"""Random frame size options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigPortTrafficFrameLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		Random frame size options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		Random frame size options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def max(self):
		"""int32: TBD"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def min(self):
		"""int32: TBD"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)


class SessionsConfigPortTrafficFrameLengthIncrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigPortTrafficFrameLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""int32: Starting increment value for frame length"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def end(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('end')

	@end.setter
	def end(self, value):
		return self._set_value('end', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigPortTrafficFrameLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigPortTrafficFrameLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""int32: Starting decrement value for frame length"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def end(self):
		"""int32: Maximum decrement value for frame length"""
		return self._get_value('end')

	@end.setter
	def end(self, value):
		return self._set_value('end', value)

	@property
	def step(self):
		"""int32: Step decrement value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigPortTrafficFrameLengthImix(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'imix'
		super(SessionsConfigPortTrafficFrameLengthImix, self).__init__(parent, None)

	def custom_imix(self, name=None):
		"""Get the custom-imix object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-imix list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigPortTrafficFrameLengthImixCustomImix` | :obj:`SessionsConfigPortTrafficFrameLengthImixCustomImix`: If arg name is None a list of SessionsConfigPortTrafficFrameLengthImixCustomImix objects otherwise a single SessionsConfigPortTrafficFrameLengthImixCustomImix object.  

		Raises:  
			NotFoundError: name is not in the list of custom-imix objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameLengthImixCustomImix(self, name))

	def create_custom_imix(self, name):
		"""Create a child sibling custom-imix instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameLengthImixCustomImix`: An object encapsulating an instance of the custom-imix model.  

		Raises:  
			AlreadyExistsError: An instance of custom-imix with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigPortTrafficFrameLengthImixCustomImix(self, name), locals())

	def delete(self):
		"""Delete this imix instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this imix instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def imix_type(self):
		""":str:`enum`: TBD  
		Enums:  
			DEFAULT: TBD  
			CUSTOM_IMIX: TBD  
		"""
		return self._get_value('imix-type')

	@imix_type.setter
	def imix_type(self, value):
		return self._set_value('imix-type', value)


class SessionsConfigPortTrafficFrameLengthImixCustomImix(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-imix'
		self.YANG_KEY = 'name'
		super(SessionsConfigPortTrafficFrameLengthImixCustomImix, self).__init__(parent, yang_key_value)

	def delete(self):
		"""Delete this custom-imix instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-imix instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')

	@property
	def size(self):
		"""int32: TBD"""
		return self._get_value('size')

	@size.setter
	def size(self, value):
		return self._set_value('size', value)

	@property
	def weight(self):
		"""int32: TBD"""
		return self._get_value('weight')

	@weight.setter
	def weight(self, value):
		return self._set_value('weight', value)


class SessionsConfigPortTrafficFrameRate(YangBase):
	"""Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frame-rate'
		super(SessionsConfigPortTrafficFrameRate, self).__init__(parent, None)

	def fixed_rate(self):
		"""Get the fixed-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameRateFixedRate`: An object encapsulating an instance of the fixed-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameRateFixedRate(self))

	def decrement_rate(self):
		"""Get the decrement-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameRateDecrementRate`: An object encapsulating an instance of the decrement-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameRateDecrementRate(self))

	def increment_rate(self):
		"""Get the increment-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficFrameRateIncrementRate`: An object encapsulating an instance of the increment-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficFrameRateIncrementRate(self))

	def delete(self):
		"""Delete this frame-rate instance on the server.

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frame-rate instance on the server with any changed property values.

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def mode(self):
		""":str:`enum`: TBD  
		Enums:  
			FIXED: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			SHUFFLE: TBD  
			LIST: TBD  
		"""
		return self._get_value('mode')

	@mode.setter
	def mode(self, value):
		return self._set_value('mode', value)


class SessionsConfigPortTrafficFrameRateFixedRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fixed-rate'
		super(SessionsConfigPortTrafficFrameRateFixedRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this fixed-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fixed-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MPBS: TBD  
			FRAMES_PER_SECOND: TBD  
			INTER_PACKET_GAP: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def inter_packet_gap(self):
		"""int32: Load value set on the streamblock/traffic-item inter packet gap."""
		return self._get_value('inter-packet-gap')

	@inter_packet_gap.setter
	def inter_packet_gap(self, value):
		return self._set_value('inter-packet-gap', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)


class SessionsConfigPortTrafficFrameRateDecrementRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement-rate'
		super(SessionsConfigPortTrafficFrameRateDecrementRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MBPS: TBD  
			FRAMES_PER_SECOND: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)

	@property
	def count(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigPortTrafficFrameRateIncrementRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment-rate'
		super(SessionsConfigPortTrafficFrameRateIncrementRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MPBS: TBD  
			FRAMES_PER_SECOND: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)

	@property
	def count(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigPortTrafficPacketPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'packet-payload'
		super(SessionsConfigPortTrafficPacketPayload, self).__init__(parent, None)

	def payload(self):
		"""Get the payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigPortTrafficPacketPayloadPayload`: An object encapsulating an instance of the payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigPortTrafficPacketPayloadPayload(self))

	def delete(self):
		"""Delete this packet-payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this packet-payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def payload_type(self):
		""":str:`enum`: TBD  
		Enums:  
			INCREMENT_BYTE: TBD  
			DECREMENT_BYTE: TBD  
			INCREMENT_WORD: TBD  
			DECREMENT_WORD: TBD  
			CUSTOM: TBD  
		"""
		return self._get_value('payload-type')

	@payload_type.setter
	def payload_type(self, value):
		return self._set_value('payload-type', value)


class SessionsConfigPortTrafficPacketPayloadPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload'
		super(SessionsConfigPortTrafficPacketPayloadPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def data(self):
		"""iyt:hex-string: TBD"""
		return self._get_value('data')

	@data.setter
	def data(self, value):
		return self._set_value('data', value)

	@property
	def repeat(self):
		"""bool: Repeat the payload data to fill the length specified"""
		return self._get_value('repeat')

	@repeat.setter
	def repeat(self, value):
		return self._set_value('repeat', value)

	@property
	def length(self):
		"""int32: TBD"""
		return self._get_value('length')

	@length.setter
	def length(self, value):
		return self._set_value('length', value)


class SessionsConfigDeviceTraffic(YangBase):
	"""This is a collection of nodes that the server uses to generate 1..n traffic-streams. The source and destinations of the device traffic group can only come from the device-groups list or its children. 
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'device-traffic'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTraffic, self).__init__(parent, yang_key_value)

	def frames(self, name=None):
		"""Get the frames object(s) from the server.

		List of user defined frames. Frames that are generated as part of the traffic-streams will overwrite any user defined frame leafs that are part of the learned info due to the endpoint-type.

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the frames list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTrafficFrames` | :obj:`SessionsConfigDeviceTrafficFrames`: If arg name is None a list of SessionsConfigDeviceTrafficFrames objects otherwise a single SessionsConfigDeviceTrafficFrames object.  

		Raises:  
			NotFoundError: name is not in the list of frames objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrames(self, name))

	def create_frames(self, name):
		"""Create a child sibling frames instance on the server.

		List of user defined frames. Frames that are generated as part of the traffic-streams will overwrite any user defined frame leafs that are part of the learned info due to the endpoint-type.

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrames`: An object encapsulating an instance of the frames model.  

		Raises:  
			AlreadyExistsError: An instance of frames with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTrafficFrames(self, name), locals())

	def frame_length(self):
		"""Get the frame-length object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLength`: An object encapsulating an instance of the frame-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLength(self))

	def frame_rate(self):
		"""Get the frame-rate object from the server

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameRate`: An object encapsulating an instance of the frame-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameRate(self))

	def packet_payload(self):
		"""Get the packet-payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficPacketPayload`: An object encapsulating an instance of the packet-payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficPacketPayload(self))

	def delete(self):
		"""Delete this device-traffic instance on the server.

		This is a collection of nodes that the server uses to generate 1..n traffic-streams. The source and destinations of the device traffic group can only come from the device-groups list or its children. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this device-traffic instance on the server with any changed property values.

		This is a collection of nodes that the server uses to generate 1..n traffic-streams. The source and destinations of the device traffic group can only come from the device-groups list or its children. 

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: The unique name of a traffic group"""
		return self._get_value('name')

	@property
	def encapsulation(self):
		""":str:`enum`: The encapsulation determines the following: What frames will be generated. What the traffic-streams name will be.   
		Enums:  
			ETHERNET: TBD  
			VLAN: TBD  
			IPV4: TBD  
			IPV6: TBD  
		"""
		return self._get_value('encapsulation')

	@encapsulation.setter
	def encapsulation(self, value):
		return self._set_value('encapsulation', value)

	@property
	def sources(self):
		"""union: A list of device source endpoint references."""
		return self._get_value('sources')

	@sources.setter
	def sources(self, value):
		return self._set_value('sources', value)

	@property
	def destinations(self):
		"""union: A list of device destination endpoint references."""
		return self._get_value('destinations')

	@destinations.setter
	def destinations(self, value):
		return self._set_value('destinations', value)

	@property
	def bi_directional(self):
		"""bool: If true then traffic-streams objects will be generated from destination DEVICES to source DEVICES."""
		return self._get_value('bi-directional')

	@bi_directional.setter
	def bi_directional(self, value):
		return self._set_value('bi-directional', value)

	@property
	def mesh_type(self):
		""":str:`enum`: If true then generated-traffic-streams objects will be generated from every destination DEVICES to every source DEVICES.  
		Enums:  
			ONE_TO_ONE: TBD  
			FULL_MESHED: TBD  
		"""
		return self._get_value('mesh-type')

	@mesh_type.setter
	def mesh_type(self, value):
		return self._set_value('mesh-type', value)


class SessionsConfigDeviceTrafficFrames(YangBase):
	"""List of user defined frames. Frames that are generated as part of the traffic-streams will overwrite any user defined frame leafs that are part of the learned info due to the endpoint-type.
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frames'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTrafficFrames, self).__init__(parent, yang_key_value)

	def ethernet(self):
		"""Get the ethernet object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernet`: An object encapsulating an instance of the ethernet model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernet(self))

	def vlan(self):
		"""Get the vlan object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlan`: An object encapsulating an instance of the vlan model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlan(self))

	def ipv4(self):
		"""Get the ipv4 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4`: An object encapsulating an instance of the ipv4 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4(self))

	def ipv6(self):
		"""Get the ipv6 object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6`: An object encapsulating an instance of the ipv6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6(self))

	def tcp(self):
		"""Get the tcp object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcp`: An object encapsulating an instance of the tcp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcp(self))

	def udp(self):
		"""Get the udp object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdp`: An object encapsulating an instance of the udp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdp(self))

	def gre(self):
		"""Get the gre object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGre`: An object encapsulating an instance of the gre model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGre(self))

	def icmp_destination_unreachable(self):
		"""Get the icmp-destination-unreachable object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachable`: An object encapsulating an instance of the icmp-destination-unreachable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachable(self))

	def icmp_echo_reply(self):
		"""Get the icmp-echo-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReply`: An object encapsulating an instance of the icmp-echo-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReply(self))

	def icmp_echo_request(self):
		"""Get the icmp-echo-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequest`: An object encapsulating an instance of the icmp-echo-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequest(self))

	def icmp_information_reply(self):
		"""Get the icmp-information-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReply`: An object encapsulating an instance of the icmp-information-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReply(self))

	def icmp_information_request(self):
		"""Get the icmp-information-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequest`: An object encapsulating an instance of the icmp-information-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequest(self))

	def icmp_address_mask_reply(self):
		"""Get the icmp-address-mask-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReply`: An object encapsulating an instance of the icmp-address-mask-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReply(self))

	def icmp_address_mask_request(self):
		"""Get the icmp-address-mask-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequest`: An object encapsulating an instance of the icmp-address-mask-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequest(self))

	def icmp_parameter_problem(self):
		"""Get the icmp-parameter-problem object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblem`: An object encapsulating an instance of the icmp-parameter-problem model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblem(self))

	def icmp_redirect(self):
		"""Get the icmp-redirect object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirect`: An object encapsulating an instance of the icmp-redirect model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirect(self))

	def icmp_router_advertisement(self):
		"""Get the icmp-router-advertisement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisement`: An object encapsulating an instance of the icmp-router-advertisement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisement(self))

	def icmp_router_solicitation(self):
		"""Get the icmp-router-solicitation object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitation`: An object encapsulating an instance of the icmp-router-solicitation model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitation(self))

	def icmp_source_quench(self):
		"""Get the icmp-source-quench object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuench`: An object encapsulating an instance of the icmp-source-quench model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuench(self))

	def icmp_time_exceeded(self):
		"""Get the icmp-time-exceeded object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceeded`: An object encapsulating an instance of the icmp-time-exceeded model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceeded(self))

	def icmp_timestamp_reply(self):
		"""Get the icmp-timestamp-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReply`: An object encapsulating an instance of the icmp-timestamp-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReply(self))

	def icmp_timestamp_request(self):
		"""Get the icmp-timestamp-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequest`: An object encapsulating an instance of the icmp-timestamp-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequest(self))

	def icmpv6_destination_unreachable(self):
		"""Get the icmpv6-destination-unreachable object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachable`: An object encapsulating an instance of the icmpv6-destination-unreachable model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachable(self))

	def icmpv6_echo_reply(self):
		"""Get the icmpv6-echo-reply object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReply`: An object encapsulating an instance of the icmpv6-echo-reply model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReply(self))

	def icmpv6_echo_request(self):
		"""Get the icmpv6-echo-request object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequest`: An object encapsulating an instance of the icmpv6-echo-request model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequest(self))

	def icmpv6_parameter_problem(self):
		"""Get the icmpv6-parameter-problem object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblem`: An object encapsulating an instance of the icmpv6-parameter-problem model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblem(self))

	def icmpv6_time_exceeded(self):
		"""Get the icmpv6-time-exceeded object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceeded`: An object encapsulating an instance of the icmpv6-time-exceeded model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceeded(self))

	def icmpv6_packet_too_big(self):
		"""Get the icmpv6-packet-too-big object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBig`: An object encapsulating an instance of the icmpv6-packet-too-big model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBig(self))

	def igmpv1_query(self):
		"""Get the igmpv1-query object from the server

		IGMPv1 Query message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1Query`: An object encapsulating an instance of the igmpv1-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1Query(self))

	def igmpv1_report(self):
		"""Get the igmpv1-report object from the server

		IGMPv1 Report message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1Report`: An object encapsulating an instance of the igmpv1-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1Report(self))

	def igmpv2_query(self):
		"""Get the igmpv2-query object from the server

		IGMPv2 Query message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2Query`: An object encapsulating an instance of the igmpv2-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2Query(self))

	def igmpv2_report(self):
		"""Get the igmpv2-report object from the server

		IGMPv2 Report message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2Report`: An object encapsulating an instance of the igmpv2-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2Report(self))

	def igmpv3_query(self):
		"""Get the igmpv3-query object from the server

		IGMPv3 Query message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3Query`: An object encapsulating an instance of the igmpv3-query model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3Query(self))

	def igmpv3_report(self):
		"""Get the igmpv3-report object from the server

		IGMPv3 Report message

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3Report`: An object encapsulating an instance of the igmpv3-report model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3Report(self))

	def ospfv2_hello(self):
		"""Get the ospfv2-hello object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2Hello`: An object encapsulating an instance of the ospfv2-hello model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2Hello(self))

	def custom(self):
		"""Get the custom object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesCustom`: An object encapsulating an instance of the custom model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesCustom(self))

	def delete(self):
		"""Delete this frames instance on the server.

		List of user defined frames. Frames that are generated as part of the traffic-streams will overwrite any user defined frame leafs that are part of the learned info due to the endpoint-type.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frames instance on the server with any changed property values.

		List of user defined frames. Frames that are generated as part of the traffic-streams will overwrite any user defined frame leafs that are part of the learned info due to the endpoint-type.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')

	@property
	def frame_type(self):
		""":str:`enum`: TBD  
		Enums:  
			CUSTOM: TBD  
			ETHERNET: TBD  
			VLAN: TBD  
			IPV4: TBD  
			IPV6: TBD  
			TCP: TBD  
			UDP: TBD  
			GRE: TBD  
			ICMP_DESTINATION_UNREACHABLE: TBD  
			ICMP_ECHO_REPLY: TBD  
			ICMP_ECHO_REQUEST: TBD  
			ICMP_INFORMATION_REPLY: TBD  
			ICMP_INFORMATION_REQUEST: TBD  
			ICMP_ADDRESS_MASK_REPLY: TBD  
			ICMP_ADDRESS_MASK_REQUEST: TBD  
			ICMP_PARAMETER_PROBLEM: TBD  
			ICMP_REDIRECT: TBD  
			ICMP_ROUTER_ADVERTISEMENT: TBD  
			ICMP_ROUTER_SOLICITATION: TBD  
			ICMP_SOURCE_QUENCH: TBD  
			ICMP_TIME_EXCEEDED: TBD  
			ICMP_TIMESTAMP_REPLY: TBD  
			ICMP_TIMESTAMP_REQUEST: TBD  
			ICMPV6_DESTINATION_UNREACHABLE: TBD  
			ICMPV6_ECHO_REPLY: TBD  
			ICMPV6_ECHO_REQUEST: TBD  
			ICMPV6_PARAMETER_PROBLEM: TBD  
			ICMPV6_TIME_EXCEEDED: TBD  
			ICMPV6_PACKET_TOO_BIG: TBD  
			IGMPV1_QUERY: TBD  
			IGMPV1_REPORT: TBD  
			IGMPV2_QUERY: TBD  
			IGMPV2_REPORT: TBD  
			IGMPV3_QUERY: TBD  
			IGMPV3_REPORT: TBD  
			OSPFV2_HELLO: TBD  
		"""
		return self._get_value('frame-type')

	@frame_type.setter
	def frame_type(self, value):
		return self._set_value('frame-type', value)


class SessionsConfigDeviceTrafficFramesEthernet(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet'
		super(SessionsConfigDeviceTrafficFramesEthernet, self).__init__(parent, None)

	def destination(self):
		"""Get the destination object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetDestination`: An object encapsulating an instance of the destination model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetDestination(self))

	def source(self):
		"""Get the source object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetSource`: An object encapsulating an instance of the source model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetSource(self))

	def ethernet_type(self):
		"""Get the ethernet-type object from the server

		Ethernet type

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetEthernetType`: An object encapsulating an instance of the ethernet-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetEthernetType(self))

	def delete(self):
		"""Delete this ethernet instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesEthernetDestination(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination'
		super(SessionsConfigDeviceTrafficFramesEthernetDestination, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetDestinationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetDestinationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetDestinationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetDestinationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetDestinationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetDestinationRandom(self))

	def delete(self):
		"""Delete this destination instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)

	@property
	def count(self):
		"""int32: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetDestinationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesEthernetDestinationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetDestinationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesEthernetDestinationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetDestinationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesEthernetDestinationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesEthernetSource(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source'
		super(SessionsConfigDeviceTrafficFramesEthernetSource, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetSourceIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetSourceIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetSourceDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetSourceDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesEthernetSourceRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesEthernetSourceRandom(self))

	def delete(self):
		"""Delete this source instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)

	@property
	def count(self):
		"""int32: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetSourceIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesEthernetSourceIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetSourceDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesEthernetSourceDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesEthernetSourceRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesEthernetSourceRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesEthernetEthernetType(YangBase):
	"""Ethernet type
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ethernet-type'
		super(SessionsConfigDeviceTrafficFramesEthernetEthernetType, self).__init__(parent, None)

	def delete(self):
		"""Delete this ethernet-type instance on the server.

		Ethernet type

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ethernet-type instance on the server with any changed property values.

		Ethernet type

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesVlan(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'vlan'
		super(SessionsConfigDeviceTrafficFramesVlan, self).__init__(parent, None)

	def priority(self):
		"""Get the priority object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanPriority`: An object encapsulating an instance of the priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanPriority(self))

	def id(self):
		"""Get the id object from the server

		A single-tagged VLAN id

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanId`: An object encapsulating an instance of the id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanId(self))

	def delete(self):
		"""Delete this vlan instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this vlan instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesVlanPriority(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'priority'
		super(SessionsConfigDeviceTrafficFramesVlanPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanPriorityRandom(self))

	def delete(self):
		"""Delete this priority instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this priority instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesVlanPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesVlanPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesVlanPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesVlanPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesVlanPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesVlanPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesVlanId(YangBase):
	"""A single-tagged VLAN id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'id'
		super(SessionsConfigDeviceTrafficFramesVlanId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesVlanIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesVlanIdRandom(self))

	def delete(self):
		"""Delete this id instance on the server.

		A single-tagged VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this id instance on the server with any changed property values.

		A single-tagged VLAN id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesVlanIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesVlanIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesVlanIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesVlanIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesVlanIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesVlanIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4'
		super(SessionsConfigDeviceTrafficFramesIpv4, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4SourceAddress(self))

	def destination_address(self):
		"""Get the destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DestinationAddress`: An object encapsulating an instance of the destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DestinationAddress(self))

	def ttl(self):
		"""Get the ttl object from the server

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4Ttl`: An object encapsulating an instance of the ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4Ttl(self))

	def header_length(self):
		"""Get the header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4HeaderLength`: An object encapsulating an instance of the header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4HeaderLength(self))

	def identification(self):
		"""Get the identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4Identification`: An object encapsulating an instance of the identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4Identification(self))

	def fragment_offset(self):
		"""Get the fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4FragmentOffset`: An object encapsulating an instance of the fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4FragmentOffset(self))

	def protocol(self):
		"""Get the protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4Protocol`: An object encapsulating an instance of the protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4Protocol(self))

	def checksum(self):
		"""Get the checksum object from the server

		Verifies that packets are not corrupted.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4Checksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4Checksum(self))

	def reserved_bit(self):
		"""Get the reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ReservedBit`: An object encapsulating an instance of the reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ReservedBit(self))

	def mf_bit(self):
		"""Get the mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4MfBit`: An object encapsulating an instance of the mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4MfBit(self))

	def df_bit(self):
		"""Get the df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DfBit`: An object encapsulating an instance of the df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DfBit(self))

	def delete(self):
		"""Delete this ipv4 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigDeviceTrafficFramesIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-address'
		super(SessionsConfigDeviceTrafficFramesIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before            the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ttl'
		super(SessionsConfigDeviceTrafficFramesIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ttl instance on the server.

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before            the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'header-length'
		super(SessionsConfigDeviceTrafficFramesIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identification'
		super(SessionsConfigDeviceTrafficFramesIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'protocol'
		super(SessionsConfigDeviceTrafficFramesIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4Checksum(YangBase):
	"""Verifies that packets are not corrupted.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIpv4Checksum, self).__init__(parent, None)

	def delete(self):
		"""Delete this checksum instance on the server.

		Verifies that packets are not corrupted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Verifies that packets are not corrupted.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mf-bit'
		super(SessionsConfigDeviceTrafficFramesIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'df-bit'
		super(SessionsConfigDeviceTrafficFramesIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6'
		super(SessionsConfigDeviceTrafficFramesIpv6, self).__init__(parent, None)

	def source_address(self):
		"""Get the source-address object from the server

		Specifies the source IPv6 address of the first generated packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6SourceAddress`: An object encapsulating an instance of the source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6SourceAddress(self))

	def destination_address(self):
		"""Get the destination-address object from the server

		Specifies the destination IPv6 address of the first generated packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6DestinationAddress`: An object encapsulating an instance of the destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6DestinationAddress(self))

	def traffic_class(self):
		"""Get the traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6TrafficClass`: An object encapsulating an instance of the traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6TrafficClass(self))

	def flow_label(self):
		"""Get the flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6FlowLabel`: An object encapsulating an instance of the flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6FlowLabel(self))

	def payload_length(self):
		"""Get the payload-length object from the server

		The two-byte payload length field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6PayloadLength`: An object encapsulating an instance of the payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6PayloadLength(self))

	def next_header(self):
		"""Get the next-header object from the server

		The next header field in the IPv6 header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6NextHeader`: An object encapsulating an instance of the next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6NextHeader(self))

	def hop_limit(self):
		"""Get the hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6HopLimit`: An object encapsulating an instance of the hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6HopLimit(self))

	def delete(self):
		"""Delete this ipv6 instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6 instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIpv6SourceAddress(YangBase):
	"""Specifies the source IPv6 address of the first generated packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address'
		super(SessionsConfigDeviceTrafficFramesIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this source-address instance on the server.

		Specifies the source IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address instance on the server with any changed property values.

		Specifies the source IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6DestinationAddress(YangBase):
	"""Specifies the destination IPv6 address of the first generated packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-address'
		super(SessionsConfigDeviceTrafficFramesIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this destination-address instance on the server.

		Specifies the destination IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-address instance on the server with any changed property values.

		Specifies the destination IPv6 address of the first generated packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'traffic-class'
		super(SessionsConfigDeviceTrafficFramesIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'flow-label'
		super(SessionsConfigDeviceTrafficFramesIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6PayloadLength(YangBase):
	"""The two-byte payload length field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload-length'
		super(SessionsConfigDeviceTrafficFramesIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this payload-length instance on the server.

		The two-byte payload length field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload-length instance on the server with any changed property values.

		The two-byte payload length field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6NextHeader(YangBase):
	"""The next header field in the IPv6 header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'next-header'
		super(SessionsConfigDeviceTrafficFramesIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this next-header instance on the server.

		The next header field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this next-header instance on the server with any changed property values.

		The next header field in the IPv6 header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hop-limit'
		super(SessionsConfigDeviceTrafficFramesIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcp(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'tcp'
		super(SessionsConfigDeviceTrafficFramesTcp, self).__init__(parent, None)

	def source_port(self):
		"""Get the source-port object from the server

		Specifies the port on the sending TCP module.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSourcePort`: An object encapsulating an instance of the source-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSourcePort(self))

	def destination_port(self):
		"""Get the destination-port object from the server

		Specifies the port on the receiving TCP module.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpDestinationPort`: An object encapsulating an instance of the destination-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpDestinationPort(self))

	def acknowledgement_number(self):
		"""Get the acknowledgement-number object from the server

		Identifies the next expected TCP octet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumber`: An object encapsulating an instance of the acknowledgement-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumber(self))

	def header_length(self):
		"""Get the header-length object from the server

		Specifies the data offset field in the TCP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpHeaderLength`: An object encapsulating an instance of the header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpHeaderLength(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserves TCP bits.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpReserved(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Identifies the position of the data within the data stream.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSequenceNumber(self))

	def urgent_pointer(self):
		"""Get the urgent-pointer object from the server

		Specifies the position in the segment where urgent data ends.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpUrgentPointer`: An object encapsulating an instance of the urgent-pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpUrgentPointer(self))

	def window_size(self):
		"""Get the window-size object from the server

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpWindowSize`: An object encapsulating an instance of the window-size model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpWindowSize(self))

	def acknowledgement_flag(self):
		"""Get the acknowledgement-flag object from the server

		Indicates whether the data identified by the sequence number has been received.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpAcknowledgementFlag`: An object encapsulating an instance of the acknowledgement-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpAcknowledgementFlag(self))

	def fin_flag(self):
		"""Get the fin-flag object from the server

		Indicates whether a connection is terminated.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpFinFlag`: An object encapsulating an instance of the fin-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpFinFlag(self))

	def push_flag(self):
		"""Get the push-flag object from the server

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpPushFlag`: An object encapsulating an instance of the push-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpPushFlag(self))

	def reset_flag(self):
		"""Get the reset-flag object from the server

		Resets the connection when a segment arrives that is not intended for the current connection.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpResetFlag`: An object encapsulating an instance of the reset-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpResetFlag(self))

	def sync_flag(self):
		"""Get the sync-flag object from the server

		Indicates whether the port is open for connection.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSyncFlag`: An object encapsulating an instance of the sync-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSyncFlag(self))

	def urgent_flag(self):
		"""Get the urgent-flag object from the server

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpUrgentFlag`: An object encapsulating an instance of the urgent-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpUrgentFlag(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the TCP checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpChecksum(self))

	def delete(self):
		"""Delete this tcp instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this tcp instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesTcpSourcePort(YangBase):
	"""Specifies the port on the sending TCP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-port'
		super(SessionsConfigDeviceTrafficFramesTcpSourcePort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSourcePortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSourcePortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSourcePortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSourcePortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSourcePortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSourcePortRandom(self))

	def delete(self):
		"""Delete this source-port instance on the server.

		Specifies the port on the sending TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-port instance on the server with any changed property values.

		Specifies the port on the sending TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpSourcePortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpSourcePortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpSourcePortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpSourcePortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpSourcePortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpSourcePortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpDestinationPort(YangBase):
	"""Specifies the port on the receiving TCP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-port'
		super(SessionsConfigDeviceTrafficFramesTcpDestinationPort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpDestinationPortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpDestinationPortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpDestinationPortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpDestinationPortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpDestinationPortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpDestinationPortRandom(self))

	def delete(self):
		"""Delete this destination-port instance on the server.

		Specifies the port on the receiving TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-port instance on the server with any changed property values.

		Specifies the port on the receiving TCP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpDestinationPortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpDestinationPortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpDestinationPortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpDestinationPortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpDestinationPortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpDestinationPortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumber(YangBase):
	"""Identifies the next expected TCP octet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'acknowledgement-number'
		super(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberRandom(self))

	def delete(self):
		"""Delete this acknowledgement-number instance on the server.

		Identifies the next expected TCP octet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this acknowledgement-number instance on the server with any changed property values.

		Identifies the next expected TCP octet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpAcknowledgementNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpHeaderLength(YangBase):
	"""Specifies the data offset field in the TCP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'header-length'
		super(SessionsConfigDeviceTrafficFramesTcpHeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpHeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpHeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpHeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpHeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpHeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpHeaderLengthRandom(self))

	def delete(self):
		"""Delete this header-length instance on the server.

		Specifies the data offset field in the TCP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this header-length instance on the server with any changed property values.

		Specifies the data offset field in the TCP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpHeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpHeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpHeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpHeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpHeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpHeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpReserved(YangBase):
	"""Reserves TCP bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigDeviceTrafficFramesTcpReserved, self).__init__(parent, None)

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserves TCP bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserves TCP bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpSequenceNumber(YangBase):
	"""Identifies the position of the data within the data stream.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesTcpSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Identifies the position of the data within the data stream.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Identifies the position of the data within the data stream.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpUrgentPointer(YangBase):
	"""Specifies the position in the segment where urgent data ends.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'urgent-pointer'
		super(SessionsConfigDeviceTrafficFramesTcpUrgentPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpUrgentPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpUrgentPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpUrgentPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpUrgentPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpUrgentPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpUrgentPointerRandom(self))

	def delete(self):
		"""Delete this urgent-pointer instance on the server.

		Specifies the position in the segment where urgent data ends.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this urgent-pointer instance on the server with any changed property values.

		Specifies the position in the segment where urgent data ends.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpUrgentPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpUrgentPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpUrgentPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpUrgentPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpUrgentPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpUrgentPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpWindowSize(YangBase):
	"""This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'window-size'
		super(SessionsConfigDeviceTrafficFramesTcpWindowSize, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpWindowSizeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpWindowSizeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpWindowSizeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpWindowSizeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpWindowSizeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpWindowSizeRandom(self))

	def delete(self):
		"""Delete this window-size instance on the server.

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this window-size instance on the server with any changed property values.

		This field is used by the receiver to indicate to the sender the amount of data that it is able to accept.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpWindowSizeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpWindowSizeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpWindowSizeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpWindowSizeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpWindowSizeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpWindowSizeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesTcpAcknowledgementFlag(YangBase):
	"""Indicates whether the data identified by the sequence number has been received.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'acknowledgement-flag'
		super(SessionsConfigDeviceTrafficFramesTcpAcknowledgementFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this acknowledgement-flag instance on the server.

		Indicates whether the data identified by the sequence number has been received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this acknowledgement-flag instance on the server with any changed property values.

		Indicates whether the data identified by the sequence number has been received.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpFinFlag(YangBase):
	"""Indicates whether a connection is terminated.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fin-flag'
		super(SessionsConfigDeviceTrafficFramesTcpFinFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this fin-flag instance on the server.

		Indicates whether a connection is terminated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fin-flag instance on the server with any changed property values.

		Indicates whether a connection is terminated.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpPushFlag(YangBase):
	"""Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'push-flag'
		super(SessionsConfigDeviceTrafficFramesTcpPushFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this push-flag instance on the server.

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this push-flag instance on the server with any changed property values.

		Indicates whether to ensure that the data is given the appropriate priority and is processed at the sending or receiving end.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpResetFlag(YangBase):
	"""Resets the connection when a segment arrives that is not intended for the current connection.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reset-flag'
		super(SessionsConfigDeviceTrafficFramesTcpResetFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this reset-flag instance on the server.

		Resets the connection when a segment arrives that is not intended for the current connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reset-flag instance on the server with any changed property values.

		Resets the connection when a segment arrives that is not intended for the current connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpSyncFlag(YangBase):
	"""Indicates whether the port is open for connection.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sync-flag'
		super(SessionsConfigDeviceTrafficFramesTcpSyncFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this sync-flag instance on the server.

		Indicates whether the port is open for connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sync-flag instance on the server with any changed property values.

		Indicates whether the port is open for connection.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpUrgentFlag(YangBase):
	"""Identifies the incoming data as urgent, giving it priority over the other segments.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'urgent-flag'
		super(SessionsConfigDeviceTrafficFramesTcpUrgentFlag, self).__init__(parent, None)

	def delete(self):
		"""Delete this urgent-flag instance on the server.

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this urgent-flag instance on the server with any changed property values.

		Identifies the incoming data as urgent, giving it priority over the other segments.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpChecksum(YangBase):
	"""Specifies the TCP checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesTcpChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesTcpChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesTcpChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the TCP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the TCP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesTcpChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesTcpChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesTcpChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesTcpChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesTcpChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesUdp(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'udp'
		super(SessionsConfigDeviceTrafficFramesUdp, self).__init__(parent, None)

	def source_port(self):
		"""Get the source-port object from the server

		Specifies the port on the sending UDP module.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpSourcePort`: An object encapsulating an instance of the source-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpSourcePort(self))

	def destination_port(self):
		"""Get the destination-port object from the server

		Specifies the port on the receiving UDP module.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpDestinationPort`: An object encapsulating an instance of the destination-port model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpDestinationPort(self))

	def checksum(self):
		"""Get the checksum object from the server

		Specifies the UDP checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpChecksum(self))

	def delete(self):
		"""Delete this udp instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this udp instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesUdpSourcePort(YangBase):
	"""Specifies the port on the sending UDP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-port'
		super(SessionsConfigDeviceTrafficFramesUdpSourcePort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpSourcePortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpSourcePortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpSourcePortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpSourcePortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpSourcePortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpSourcePortRandom(self))

	def delete(self):
		"""Delete this source-port instance on the server.

		Specifies the port on the sending UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-port instance on the server with any changed property values.

		Specifies the port on the sending UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesUdpSourcePortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesUdpSourcePortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpSourcePortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesUdpSourcePortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpSourcePortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesUdpSourcePortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesUdpDestinationPort(YangBase):
	"""Specifies the port on the receiving UDP module.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'destination-port'
		super(SessionsConfigDeviceTrafficFramesUdpDestinationPort, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpDestinationPortIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpDestinationPortIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpDestinationPortDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpDestinationPortDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpDestinationPortRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpDestinationPortRandom(self))

	def delete(self):
		"""Delete this destination-port instance on the server.

		Specifies the port on the receiving UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this destination-port instance on the server with any changed property values.

		Specifies the port on the receiving UDP module.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesUdpDestinationPortIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesUdpDestinationPortIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpDestinationPortDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesUdpDestinationPortDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpDestinationPortRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesUdpDestinationPortRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesUdpChecksum(YangBase):
	"""Specifies the UDP checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesUdpChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesUdpChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesUdpChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Specifies the UDP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Specifies the UDP checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesUdpChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesUdpChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesUdpChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesUdpChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesUdpChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGre(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gre'
		super(SessionsConfigDeviceTrafficFramesGre, self).__init__(parent, None)

	def checksum_bit(self):
		"""Get the checksum-bit object from the server

		Checksum bit.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreChecksumBit`: An object encapsulating an instance of the checksum-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreChecksumBit(self))

	def routing_bit(self):
		"""Get the routing-bit object from the server

		Routing bit.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreRoutingBit`: An object encapsulating an instance of the routing-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreRoutingBit(self))

	def key_bit(self):
		"""Get the key-bit object from the server

		Key bit.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyBit`: An object encapsulating an instance of the key-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyBit(self))

	def sequence_number_bit(self):
		"""Get the sequence-number-bit object from the server

		Sequence Number bit.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberBit`: An object encapsulating an instance of the sequence-number-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberBit(self))

	def reserved_0(self):
		"""Get the reserved-0 object from the server

		Reserved-0 bits.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved0`: An object encapsulating an instance of the reserved-0 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved0(self))

	def version(self):
		"""Get the version object from the server

		GRE Version.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreVersion`: An object encapsulating an instance of the version model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreVersion(self))

	def protocol_type(self):
		"""Get the protocol-type object from the server

		Indicates the ether protocol type of the encapsulated payload.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreProtocolType`: An object encapsulating an instance of the protocol-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreProtocolType(self))

	def gre_checksum(self):
		"""Get the gre-checksum object from the server

		GRE Version.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreGreChecksum`: An object encapsulating an instance of the gre-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreGreChecksum(self))

	def reserved_1(self):
		"""Get the reserved-1 object from the server

		Reserved-1 bits.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved1`: An object encapsulating an instance of the reserved-1 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved1(self))

	def key(self):
		"""Get the key object from the server

		Key Value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKey`: An object encapsulating an instance of the key model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKey(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumber(self))

	def keep_alive_bit(self):
		"""Get the keep-alive-bit object from the server

		Keep Alive bit.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveBit`: An object encapsulating an instance of the keep-alive-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveBit(self))

	def keep_alive_period(self):
		"""Get the keep-alive-period object from the server

		Keep alive period.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAlivePeriod`: An object encapsulating an instance of the keep-alive-period model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriod(self))

	def keep_alive_retries(self):
		"""Get the keep-alive-retries object from the server

		Keep alive retries.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveRetries`: An object encapsulating an instance of the keep-alive-retries model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveRetries(self))

	def delete(self):
		"""Delete this gre instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gre instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesGreChecksumBit(YangBase):
	"""Checksum bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum-bit'
		super(SessionsConfigDeviceTrafficFramesGreChecksumBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreChecksumBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreChecksumBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreChecksumBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreChecksumBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreChecksumBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreChecksumBitRandom(self))

	def delete(self):
		"""Delete this checksum-bit instance on the server.

		Checksum bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum-bit instance on the server with any changed property values.

		Checksum bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreChecksumBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreChecksumBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreChecksumBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreChecksumBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreChecksumBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreChecksumBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreRoutingBit(YangBase):
	"""Routing bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'routing-bit'
		super(SessionsConfigDeviceTrafficFramesGreRoutingBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreRoutingBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreRoutingBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreRoutingBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreRoutingBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreRoutingBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreRoutingBitRandom(self))

	def delete(self):
		"""Delete this routing-bit instance on the server.

		Routing bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this routing-bit instance on the server with any changed property values.

		Routing bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreRoutingBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreRoutingBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreRoutingBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreRoutingBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreRoutingBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreRoutingBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreKeyBit(YangBase):
	"""Key bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key-bit'
		super(SessionsConfigDeviceTrafficFramesGreKeyBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyBitRandom(self))

	def delete(self):
		"""Delete this key-bit instance on the server.

		Key bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key-bit instance on the server with any changed property values.

		Key bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreKeyBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreKeyBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeyBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreKeyBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeyBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreKeyBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberBit(YangBase):
	"""Sequence Number bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number-bit'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitRandom(self))

	def delete(self):
		"""Delete this sequence-number-bit instance on the server.

		Sequence Number bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number-bit instance on the server with any changed property values.

		Sequence Number bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreReserved0(YangBase):
	"""Reserved-0 bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-0'
		super(SessionsConfigDeviceTrafficFramesGreReserved0, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved0Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved0Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved0Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved0Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved0Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved0Random(self))

	def delete(self):
		"""Delete this reserved-0 instance on the server.

		Reserved-0 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-0 instance on the server with any changed property values.

		Reserved-0 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreReserved0Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreReserved0Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreReserved0Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreReserved0Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreReserved0Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreReserved0Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreVersion(YangBase):
	"""GRE Version.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'version'
		super(SessionsConfigDeviceTrafficFramesGreVersion, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreVersionIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreVersionIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreVersionDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreVersionDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreVersionRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreVersionRandom(self))

	def delete(self):
		"""Delete this version instance on the server.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this version instance on the server with any changed property values.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreVersionIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreVersionIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreVersionDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreVersionDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreVersionRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreVersionRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreProtocolType(YangBase):
	"""Indicates the ether protocol type of the encapsulated payload.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'protocol-type'
		super(SessionsConfigDeviceTrafficFramesGreProtocolType, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreProtocolTypeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreProtocolTypeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreProtocolTypeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreProtocolTypeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreProtocolTypeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreProtocolTypeRandom(self))

	def delete(self):
		"""Delete this protocol-type instance on the server.

		Indicates the ether protocol type of the encapsulated payload.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this protocol-type instance on the server with any changed property values.

		Indicates the ether protocol type of the encapsulated payload.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreProtocolTypeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreProtocolTypeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreProtocolTypeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreProtocolTypeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreProtocolTypeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreProtocolTypeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreGreChecksum(YangBase):
	"""GRE Version.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gre-checksum'
		super(SessionsConfigDeviceTrafficFramesGreGreChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreGreChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreGreChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreGreChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreGreChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreGreChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreGreChecksumRandom(self))

	def delete(self):
		"""Delete this gre-checksum instance on the server.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gre-checksum instance on the server with any changed property values.

		GRE Version.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreGreChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreGreChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreGreChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreGreChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreGreChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreGreChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreReserved1(YangBase):
	"""Reserved-1 bits.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved-1'
		super(SessionsConfigDeviceTrafficFramesGreReserved1, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved1Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved1Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved1Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved1Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreReserved1Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreReserved1Random(self))

	def delete(self):
		"""Delete this reserved-1 instance on the server.

		Reserved-1 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved-1 instance on the server with any changed property values.

		Reserved-1 bits.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreReserved1Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreReserved1Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreReserved1Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreReserved1Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreReserved1Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreReserved1Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreKey(YangBase):
	"""Key Value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'key'
		super(SessionsConfigDeviceTrafficFramesGreKey, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeyRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeyRandom(self))

	def delete(self):
		"""Delete this key instance on the server.

		Key Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this key instance on the server with any changed property values.

		Key Value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreKeyIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreKeyIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeyDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreKeyDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeyRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreKeyRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveBit(YangBase):
	"""Keep Alive bit.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-bit'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveBitRandom(self))

	def delete(self):
		"""Delete this keep-alive-bit instance on the server.

		Keep Alive bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-bit instance on the server with any changed property values.

		Keep Alive bit.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreKeepAlivePeriod(YangBase):
	"""Keep alive period.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-period'
		super(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriod, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodRandom(self))

	def delete(self):
		"""Delete this keep-alive-period instance on the server.

		Keep alive period.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-period instance on the server with any changed property values.

		Keep alive period.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreKeepAlivePeriodRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveRetries(YangBase):
	"""Keep alive retries.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'keep-alive-retries'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveRetries, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesRandom(self))

	def delete(self):
		"""Delete this keep-alive-retries instance on the server.

		Keep alive retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this keep-alive-retries instance on the server with any changed property values.

		Keep alive retries.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesGreKeepAliveRetriesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachable(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-destination-unreachable'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachable, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Destination Unreachable code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableCode(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnused(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Data(self))

	def delete(self):
		"""Delete this icmp-destination-unreachable instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-destination-unreachable instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableCode(YangBase):
	"""ICMP Destination Unreachable code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Destination Unreachable code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Destination Unreachable code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpDestinationUnreachableIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-echo-reply'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Echo reply code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksum(self))

	def delete(self):
		"""Delete this icmp-echo-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-echo-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyCode(YangBase):
	"""ICMP Echo reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-echo-request'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Echo request code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksum(self))

	def delete(self):
		"""Delete this icmp-echo-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-echo-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestCode(YangBase):
	"""ICMP Echo request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpEchoRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-information-reply'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP information reply code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksum(self))

	def delete(self):
		"""Delete this icmp-information-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-information-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyCode(YangBase):
	"""ICMP information reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP information reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP information reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-information-request'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP information request code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksum(self))

	def delete(self):
		"""Delete this icmp-information-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-information-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestCode(YangBase):
	"""ICMP information request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP information request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP information request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpInformationRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-address-mask-reply'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP address mask reply code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksum(self))

	def address_mask(self):
		"""Get the address-mask object from the server

		Address Mask.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMask`: An object encapsulating an instance of the address-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMask(self))

	def delete(self):
		"""Delete this icmp-address-mask-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-address-mask-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCode(YangBase):
	"""ICMP address mask reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP address mask reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP address mask reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMask(YangBase):
	"""Address Mask.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-mask'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskRandom(self))

	def delete(self):
		"""Delete this address-mask instance on the server.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-mask instance on the server with any changed property values.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskReplyAddressMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-address-mask-request'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP address mask request code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifier(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksum(self))

	def address_mask(self):
		"""Get the address-mask object from the server

		Address Mask.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMask`: An object encapsulating an instance of the address-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMask(self))

	def delete(self):
		"""Delete this icmp-address-mask-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-address-mask-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCode(YangBase):
	"""ICMP address mask request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP address mask request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP address mask request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMask(YangBase):
	"""Address Mask.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-mask'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskRandom(self))

	def delete(self):
		"""Delete this address-mask instance on the server.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-mask instance on the server with any changed property values.

		Address Mask.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpAddressMaskRequestAddressMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblem(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-parameter-problem'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblem, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Destination Parameter problem code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemCode(self))

	def pointer(self):
		"""Get the pointer object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointer`: An object encapsulating an instance of the pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointer(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnused(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Data(self))

	def delete(self):
		"""Delete this icmp-parameter-problem instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-parameter-problem instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemCode(YangBase):
	"""ICMP Destination Parameter problem code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Destination Parameter problem code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Destination Parameter problem code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointer(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pointer'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerRandom(self))

	def delete(self):
		"""Delete this pointer instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pointer instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpParameterProblemIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirect(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-redirect'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirect, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Redirect code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectCode(self))

	def gateway_address(self):
		"""Get the gateway-address object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddress`: An object encapsulating an instance of the gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Data(self))

	def delete(self):
		"""Delete this icmp-redirect instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-redirect instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpRedirectCode(YangBase):
	"""ICMP Redirect code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Redirect code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Redirect code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddress(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressRandom(self))

	def delete(self):
		"""Delete this gateway-address instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this gateway-address instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectGatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRedirectIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-advertisement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisement, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Router Advertisement code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCode(self))

	def advertise_count(self):
		"""Get the advertise-count object from the server

		Advertise Count.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCount`: An object encapsulating an instance of the advertise-count model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCount(self))

	def life_time(self):
		"""Get the life-time object from the server

		Life Time.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTime`: An object encapsulating an instance of the life-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTime(self))

	def address_entry_size(self):
		"""Get the address-entry-size object from the server

		Address Entry Size.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySize`: An object encapsulating an instance of the address-entry-size model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySize(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksum(self))

	def icmp_router_address(self, name=None):
		"""Get the icmp-router-address object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the icmp-router-address list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress` | :obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress`: If arg name is None a list of SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress objects otherwise a single SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress object.  

		Raises:  
			NotFoundError: name is not in the list of icmp-router-address objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(self, name))

	def create_icmp_router_address(self, name):
		"""Create a child sibling icmp-router-address instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress`: An object encapsulating an instance of the icmp-router-address model.  

		Raises:  
			AlreadyExistsError: An instance of icmp-router-address with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(self, name), locals())

	def delete(self):
		"""Delete this icmp-router-advertisement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-advertisement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCode(YangBase):
	"""ICMP Router Advertisement code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Router Advertisement code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Router Advertisement code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCount(YangBase):
	"""Advertise Count.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'advertise-count'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCount, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom(self))

	def delete(self):
		"""Delete this advertise-count instance on the server.

		Advertise Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this advertise-count instance on the server with any changed property values.

		Advertise Count.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAdvertiseCountRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTime(YangBase):
	"""Life Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'life-time'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeRandom(self))

	def delete(self):
		"""Delete this life-time instance on the server.

		Life Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this life-time instance on the server with any changed property values.

		Life Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementLifeTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySize(YangBase):
	"""Address Entry Size.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address-entry-size'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySize, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom(self))

	def delete(self):
		"""Delete this address-entry-size instance on the server.

		Address Entry Size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address-entry-size instance on the server with any changed property values.

		Address Entry Size.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementAddressEntrySizeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-address'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddress, self).__init__(parent, yang_key_value)

	def router_address(self):
		"""Get the router-address object from the server

		Router Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress`: An object encapsulating an instance of the router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress(self))

	def preference_level(self):
		"""Get the preference-level object from the server

		Preference Level

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel`: An object encapsulating an instance of the preference-level model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel(self))

	def delete(self):
		"""Delete this icmp-router-address instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-address instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""str: Name"""
		return self._get_value('name')


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress(YangBase):
	"""Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-address'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom(self))

	def delete(self):
		"""Delete this router-address instance on the server.

		Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-address instance on the server with any changed property values.

		Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel(YangBase):
	"""Preference Level
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'preference-level'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom(self))

	def delete(self):
		"""Delete this preference-level instance on the server.

		Preference Level

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this preference-level instance on the server with any changed property values.

		Preference Level

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterAdvertisementIcmpRouterAddressPreferenceLevelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitation(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-router-solicitation'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitation, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Router Solicitation code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCode(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReserved(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksum(self))

	def delete(self):
		"""Delete this icmp-router-solicitation instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-router-solicitation instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCode(YangBase):
	"""ICMP Router Solicitation code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Router Solicitation code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Router Solicitation code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReserved(YangBase):
	"""Reserved.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpRouterSolicitationChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuench(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-source-quench'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuench, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Source Quench code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Data(self))

	def delete(self):
		"""Delete this icmp-source-quench instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-source-quench instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCode(YangBase):
	"""ICMP Source Quench code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Source Quench code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Source Quench code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpSourceQuenchIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceeded(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-time-exceeded'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceeded, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Time exceeded code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksum(self))

	def ipv4_source_address(self):
		"""Get the ipv4-source-address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddress`: An object encapsulating an instance of the ipv4-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddress(self))

	def ipv4_destination_address(self):
		"""Get the ipv4-destination-address object from the server

		Destination IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddress`: An object encapsulating an instance of the ipv4-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddress(self))

	def ipv4_gateway_address(self):
		"""Get the ipv4-gateway-address object from the server

		Gateway IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddress`: An object encapsulating an instance of the ipv4-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddress(self))

	def ipv4_ttl(self):
		"""Get the ipv4-ttl object from the server

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Ttl`: An object encapsulating an instance of the ipv4-ttl model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Ttl(self))

	def ipv4_protocol(self):
		"""Get the ipv4-protocol object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Protocol`: An object encapsulating an instance of the ipv4-protocol model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Protocol(self))

	def ipv4_fragment_offset(self):
		"""Get the ipv4-fragment-offset object from the server

		The byte count from the start of the original sent packet.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffset`: An object encapsulating an instance of the ipv4-fragment-offset model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffset(self))

	def ipv4_identification(self):
		"""Get the ipv4-identification object from the server

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Identification`: An object encapsulating an instance of the ipv4-identification model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Identification(self))

	def ipv4_reserved_bit(self):
		"""Get the ipv4-reserved-bit object from the server

		Specifies the reserved bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBit`: An object encapsulating an instance of the ipv4-reserved-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBit(self))

	def ipv4_mf_bit(self):
		"""Get the ipv4-mf-bit object from the server

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBit`: An object encapsulating an instance of the ipv4-mf-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBit(self))

	def ipv4_df_bit(self):
		"""Get the ipv4-df-bit object from the server

		Specifies whether the datagram is fragmented.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBit`: An object encapsulating an instance of the ipv4-df-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBit(self))

	def ipv4_header_length(self):
		"""Get the ipv4-header-length object from the server

		The length of the IP header field in number of bytes.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLength`: An object encapsulating an instance of the ipv4-header-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLength(self))

	def ipv4_total_length(self):
		"""Get the ipv4-total-length object from the server

		The total length of the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLength`: An object encapsulating an instance of the ipv4-total-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLength(self))

	def ipv4_checksum(self):
		"""Get the ipv4-checksum object from the server

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Checksum`: An object encapsulating an instance of the ipv4-checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Checksum(self))

	def ipv4_data(self):
		"""Get the ipv4-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Data`: An object encapsulating an instance of the ipv4-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Data(self))

	def delete(self):
		"""Delete this icmp-time-exceeded instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-time-exceeded instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededCode(YangBase):
	"""ICMP Time exceeded code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Time exceeded code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Time exceeded code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-source-address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-source-address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddress(YangBase):
	"""Destination IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-destination-address instance on the server.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-destination-address instance on the server with any changed property values.

		Destination IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddress(YangBase):
	"""Gateway IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv4-gateway-address instance on the server.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-gateway-address instance on the server with any changed property values.

		Gateway IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Ttl(YangBase):
	"""The limited number of iterations that a unit of data can experience before the data is discarded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-ttl'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Ttl, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlRandom(self))

	def delete(self):
		"""Delete this ipv4-ttl instance on the server.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-ttl instance on the server with any changed property values.

		The limited number of iterations that a unit of data can experience before the data is discarded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TtlRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Protocol(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-protocol'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Protocol, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolRandom(self))

	def delete(self):
		"""Delete this ipv4-protocol instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-protocol instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ProtocolRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffset(YangBase):
	"""The byte count from the start of the original sent packet.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-fragment-offset'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffset, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom(self))

	def delete(self):
		"""Delete this ipv4-fragment-offset instance on the server.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-fragment-offset instance on the server with any changed property values.

		The byte count from the start of the original sent packet.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4FragmentOffsetRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Identification(YangBase):
	"""Specifies the identifying value used to help assemble the fragments of a datagram.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-identification'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Identification, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationRandom(self))

	def delete(self):
		"""Delete this ipv4-identification instance on the server.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-identification instance on the server with any changed property values.

		Specifies the identifying value used to help assemble the fragments of a datagram.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4IdentificationRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBit(YangBase):
	"""Specifies the reserved bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-reserved-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom(self))

	def delete(self):
		"""Delete this ipv4-reserved-bit instance on the server.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-reserved-bit instance on the server with any changed property values.

		Specifies the reserved bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ReservedBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBit(YangBase):
	"""Specifies the More Fragment (MF) bit in the Flags field of the internet header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-mf-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-mf-bit instance on the server.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-mf-bit instance on the server with any changed property values.

		Specifies the More Fragment (MF) bit in the Flags field of the internet header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4MfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBit(YangBase):
	"""Specifies whether the datagram is fragmented.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-df-bit'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitRandom(self))

	def delete(self):
		"""Delete this ipv4-df-bit instance on the server.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-df-bit instance on the server with any changed property values.

		Specifies whether the datagram is fragmented.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DfBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLength(YangBase):
	"""The length of the IP header field in number of bytes.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-header-length'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-header-length instance on the server.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-header-length instance on the server with any changed property values.

		The length of the IP header field in number of bytes.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4HeaderLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLength(YangBase):
	"""The total length of the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-total-length'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom(self))

	def delete(self):
		"""Delete this ipv4-total-length instance on the server.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-total-length instance on the server with any changed property values.

		The total length of the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4TotalLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Checksum(YangBase):
	"""The header checksum is calculated over the IP header only. It does not cover any data that follows the header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Checksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumRandom(self))

	def delete(self):
		"""Delete this ipv4-checksum instance on the server.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-checksum instance on the server with any changed property values.

		The header checksum is calculated over the IP header only. It does not cover any data that follows the header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4ChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv4-data'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataRandom(self))

	def delete(self):
		"""Delete this ipv4-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv4-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimeExceededIpv4DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-timestamp-reply'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Timestamp reply code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifier(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksum(self))

	def sequence_number(self):
		"""Get the sequence_number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_number`: An object encapsulating an instance of the sequence_number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_number(self))

	def originate_timestamp(self):
		"""Get the originate-timestamp object from the server

		Originate timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestamp`: An object encapsulating an instance of the originate-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestamp(self))

	def receive_timestamp(self):
		"""Get the receive-timestamp object from the server

		Receive timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestamp`: An object encapsulating an instance of the receive-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestamp(self))

	def transmit_timestamp(self):
		"""Get the transmit-timestamp object from the server

		Transmit timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestamp`: An object encapsulating an instance of the transmit-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestamp(self))

	def delete(self):
		"""Delete this icmp-timestamp-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-timestamp-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCode(YangBase):
	"""ICMP Timestamp reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Timestamp reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Timestamp reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifier(YangBase):
	"""Identifier.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_number(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence_number'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_number, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberRandom(self))

	def delete(self):
		"""Delete this sequence_number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence_number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplySequence_numberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestamp(YangBase):
	"""Originate timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'originate-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampRandom(self))

	def delete(self):
		"""Delete this originate-timestamp instance on the server.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this originate-timestamp instance on the server with any changed property values.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyOriginateTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestamp(YangBase):
	"""Receive timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampRandom(self))

	def delete(self):
		"""Delete this receive-timestamp instance on the server.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-timestamp instance on the server with any changed property values.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyReceiveTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestamp(YangBase):
	"""Transmit timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampRandom(self))

	def delete(self):
		"""Delete this transmit-timestamp instance on the server.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-timestamp instance on the server with any changed property values.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampReplyTransmitTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmp-timestamp-request'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMP Timestamp request code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifier(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksum(self))

	def sequence_number(self):
		"""Get the sequence_number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_number`: An object encapsulating an instance of the sequence_number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_number(self))

	def originate_timestamp(self):
		"""Get the originate-timestamp object from the server

		Originate timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestamp`: An object encapsulating an instance of the originate-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestamp(self))

	def receive_timestamp(self):
		"""Get the receive-timestamp object from the server

		Receive timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestamp`: An object encapsulating an instance of the receive-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestamp(self))

	def transmit_timestamp(self):
		"""Get the transmit-timestamp object from the server

		Transmit timestamp.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestamp`: An object encapsulating an instance of the transmit-timestamp model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestamp(self))

	def delete(self):
		"""Delete this icmp-timestamp-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmp-timestamp-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCode(YangBase):
	"""ICMP Timestamp request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMP Timestamp request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMP Timestamp request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifier(YangBase):
	"""Identifier.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_number(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence_number'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_number, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberRandom(self))

	def delete(self):
		"""Delete this sequence_number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence_number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestSequence_numberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestamp(YangBase):
	"""Originate timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'originate-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampRandom(self))

	def delete(self):
		"""Delete this originate-timestamp instance on the server.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this originate-timestamp instance on the server with any changed property values.

		Originate timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestOriginateTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestamp(YangBase):
	"""Receive timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'receive-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampRandom(self))

	def delete(self):
		"""Delete this receive-timestamp instance on the server.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this receive-timestamp instance on the server with any changed property values.

		Receive timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestReceiveTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestamp(YangBase):
	"""Transmit timestamp.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'transmit-timestamp'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestamp, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampRandom(self))

	def delete(self):
		"""Delete this transmit-timestamp instance on the server.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this transmit-timestamp instance on the server with any changed property values.

		Transmit timestamp.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpTimestampRequestTransmitTimestampRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachable(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-destination-unreachable'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachable, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableCode(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused field.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnused(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-destination-unreachable instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-destination-unreachable instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableCode(YangBase):
	"""ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Destination Unreachable code.    NO_ROUTE            : No Route To Destination    PROHIBITED          : Communication with Destination Prohibited    ADDRESS_UNREACHABLE : Address Unreachable    PORT_UNREACHABLE    : Port Unreachable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnused(YangBase):
	"""Unused field.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused field.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6DestinationUnreachableIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReply(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-echo-reply'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReply, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Echo reply code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksum(self))

	def delete(self):
		"""Delete this icmpv6-echo-reply instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-echo-reply instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCode(YangBase):
	"""ICMPv6 Echo reply code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Echo reply code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplySequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoReplyChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequest(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-echo-request'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequest, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Echo request code.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCode(self))

	def identifier(self):
		"""Get the identifier object from the server

		Identifier value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifier`: An object encapsulating an instance of the identifier model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifier(self))

	def echo_data(self):
		"""Get the echo-data object from the server

		Data value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoData`: An object encapsulating an instance of the echo-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoData(self))

	def sequence_number(self):
		"""Get the sequence-number object from the server

		Sequence Number.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumber`: An object encapsulating an instance of the sequence-number model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumber(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksum(self))

	def delete(self):
		"""Delete this icmpv6-echo-request instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-echo-request instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCode(YangBase):
	"""ICMPv6 Echo request code.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Echo request code.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifier(YangBase):
	"""Identifier value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'identifier'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifier, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierRandom(self))

	def delete(self):
		"""Delete this identifier instance on the server.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this identifier instance on the server with any changed property values.

		Identifier value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestIdentifierRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoData(YangBase):
	"""Data value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'echo-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoData, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataRandom(self))

	def delete(self):
		"""Delete this echo-data instance on the server.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this echo-data instance on the server with any changed property values.

		Data value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestEchoDataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumber(YangBase):
	"""Sequence Number.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'sequence-number'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumber, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberRandom(self))

	def delete(self):
		"""Delete this sequence-number instance on the server.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this sequence-number instance on the server with any changed property values.

		Sequence Number.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestSequenceNumberRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6EchoRequestChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblem(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-parameter-problem'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblem, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemCode(self))

	def pointer(self):
		"""Get the pointer object from the server

		Pointer value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointer`: An object encapsulating an instance of the pointer model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointer(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-parameter-problem instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-parameter-problem instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemCode(YangBase):
	"""ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Destination Parameter problem code.    ERROR               : Erroneous Header Field    UNKNOWN_NEXT_HEADER : Unrecognized Next Header Type    UNKNOWN_IPV6_OPTION : Unrecognized IPv6 Option

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointer(YangBase):
	"""Pointer value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pointer'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointer, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerRandom(self))

	def delete(self):
		"""Delete this pointer instance on the server.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this pointer instance on the server with any changed property values.

		Pointer value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemPointerRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6ParameterProblemIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceeded(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-time-exceeded'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceeded, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededCode(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnused(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-time-exceeded instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-time-exceeded instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededCode(YangBase):
	"""ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededCode, self).__init__(parent, None)

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Time exceeded code.    HOP_LIMIT_EXCEED : Hop Limit exceeded.    FRAGMENT_ASSEMBLY_EXCEED : Fragment Reassembly Time Exceeded.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6TimeExceededIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBig(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'icmpv6-packet-too-big'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBig, self).__init__(parent, None)

	def code(self):
		"""Get the code object from the server

		ICMPv6 Packet too big.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCode`: An object encapsulating an instance of the code model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCode(self))

	def mtu(self):
		"""Get the mtu object from the server

		MTU.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtu`: An object encapsulating an instance of the mtu model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtu(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksum(self))

	def ipv6_source_address(self):
		"""Get the ipv6-source-address object from the server

		Source IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress`: An object encapsulating an instance of the ipv6-source-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress(self))

	def ipv6_destination_address(self):
		"""Get the ipv6-destination-address object from the server

		Destination IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress`: An object encapsulating an instance of the ipv6-destination-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress(self))

	def ipv6_gateway_address(self):
		"""Get the ipv6-gateway-address object from the server

		Gateway IPv6 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress`: An object encapsulating an instance of the ipv6-gateway-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress(self))

	def ipv6_hop_limit(self):
		"""Get the ipv6-hop-limit object from the server

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimit`: An object encapsulating an instance of the ipv6-hop-limit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimit(self))

	def ipv6_traffic_class(self):
		"""Get the ipv6-traffic-class object from the server

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass`: An object encapsulating an instance of the ipv6-traffic-class model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass(self))

	def ipv6_next_header(self):
		"""Get the ipv6-next-header object from the server

		Indicates the type of L4 protocol in the IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeader`: An object encapsulating an instance of the ipv6-next-header model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeader(self))

	def ipv6_payload_length(self):
		"""Get the ipv6-payload-length object from the server

		Payload Length.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength`: An object encapsulating an instance of the ipv6-payload-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength(self))

	def ipv6_flow_label(self):
		"""Get the ipv6-flow-label object from the server

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel`: An object encapsulating an instance of the ipv6-flow-label model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel(self))

	def ipv6_data(self):
		"""Get the ipv6-data object from the server

		Dataplane of IP header.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6Data`: An object encapsulating an instance of the ipv6-data model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6Data(self))

	def delete(self):
		"""Delete this icmpv6-packet-too-big instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this icmpv6-packet-too-big instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCode(YangBase):
	"""ICMPv6 Packet too big.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'code'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCode, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeRandom(self))

	def delete(self):
		"""Delete this code instance on the server.

		ICMPv6 Packet too big.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this code instance on the server with any changed property values.

		ICMPv6 Packet too big.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigCodeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtu(YangBase):
	"""MTU.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mtu'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtu, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuRandom(self))

	def delete(self):
		"""Delete this mtu instance on the server.

		MTU.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this mtu instance on the server with any changed property values.

		MTU.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigMtuRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksum(YangBase):
	"""Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value. Default: Automatically calculated for each packet. (If you set this to 0, the checksum will not be calculated and will be the same for each packet.)

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress(YangBase):
	"""Source IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-source-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-source-address instance on the server.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-source-address instance on the server with any changed property values.

		Source IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6SourceAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress(YangBase):
	"""Destination IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-destination-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-destination-address instance on the server.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-destination-address instance on the server with any changed property values.

		Destination IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DestinationAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress(YangBase):
	"""Gateway IPv6 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-gateway-address'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom(self))

	def delete(self):
		"""Delete this ipv6-gateway-address instance on the server.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-gateway-address instance on the server with any changed property values.

		Gateway IPv6 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6GatewayAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimit(YangBase):
	"""The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-hop-limit'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom(self))

	def delete(self):
		"""Delete this ipv6-hop-limit instance on the server.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-hop-limit instance on the server with any changed property values.

		The hop limit field in the IPv6 header, which is an eight- bit field similar to TTL in IPv4.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6HopLimitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass(YangBase):
	"""Specifies the IPv6 traffic class setting to use for application layer traffic.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-traffic-class'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClass, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom(self))

	def delete(self):
		"""Delete this ipv6-traffic-class instance on the server.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-traffic-class instance on the server with any changed property values.

		Specifies the IPv6 traffic class setting to use for application layer traffic.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6TrafficClassRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeader(YangBase):
	"""Indicates the type of L4 protocol in the IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-next-header'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeader, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom(self))

	def delete(self):
		"""Delete this ipv6-next-header instance on the server.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-next-header instance on the server with any changed property values.

		Indicates the type of L4 protocol in the IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6NextHeaderRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength(YangBase):
	"""Payload Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-payload-length'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom(self))

	def delete(self):
		"""Delete this ipv6-payload-length instance on the server.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-payload-length instance on the server with any changed property values.

		Payload Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6PayloadLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel(YangBase):
	"""The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-flow-label'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabel, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom(self))

	def delete(self):
		"""Delete this ipv6-flow-label instance on the server.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-flow-label instance on the server with any changed property values.

		The flow label value of the IPv6 stream, which is a twenty- bit field is used for QoS management.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6FlowLabelRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6Data(YangBase):
	"""Dataplane of IP header.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ipv6-data'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6Data, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataRandom(self))

	def delete(self):
		"""Delete this ipv6-data instance on the server.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ipv6-data instance on the server with any changed property values.

		Dataplane of IP header.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIcmpv6PacketTooBigIpv6DataRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1Query(YangBase):
	"""IGMPv1 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv1-query'
		super(SessionsConfigDeviceTrafficFramesIgmpv1Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnused(self))

	def delete(self):
		"""Delete this igmpv1-query instance on the server.

		IGMPv1 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv1-query instance on the server with any changed property values.

		IGMPv1 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1QueryUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1Report(YangBase):
	"""IGMPv1 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv1-report'
		super(SessionsConfigDeviceTrafficFramesIgmpv1Report, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksum(self))

	def unused(self):
		"""Get the unused object from the server

		Unused.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportUnused`: An object encapsulating an instance of the unused model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnused(self))

	def delete(self):
		"""Delete this igmpv1-report instance on the server.

		IGMPv1 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv1-report instance on the server with any changed property values.

		IGMPv1 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportUnused(YangBase):
	"""Unused.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'unused'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnused, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedRandom(self))

	def delete(self):
		"""Delete this unused instance on the server.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this unused instance on the server with any changed property values.

		Unused.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv1ReportUnusedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2Query(YangBase):
	"""IGMPv2 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv2-query'
		super(SessionsConfigDeviceTrafficFramesIgmpv2Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTime(self))

	def delete(self):
		"""Delete this igmpv2-query instance on the server.

		IGMPv2 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv2-query instance on the server with any changed property values.

		IGMPv2 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2QueryMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2Report(YangBase):
	"""IGMPv2 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv2-report'
		super(SessionsConfigDeviceTrafficFramesIgmpv2Report, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTime(self))

	def delete(self):
		"""Delete this igmpv2-report instance on the server.

		IGMPv2 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv2-report instance on the server with any changed property values.

		IGMPv2 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv2ReportMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3Query(YangBase):
	"""IGMPv3 Query message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv3-query'
		super(SessionsConfigDeviceTrafficFramesIgmpv3Query, self).__init__(parent, None)

	def group_address(self):
		"""Get the group-address object from the server

		Group IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddress`: An object encapsulating an instance of the group-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddress(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksum(self))

	def max_response_time(self):
		"""Get the max-response-time object from the server

		Maximum response Time.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTime`: An object encapsulating an instance of the max-response-time model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTime(self))

	def number_of_sources(self):
		"""Get the number-of-sources object from the server

		Number of Sources.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSources`: An object encapsulating an instance of the number-of-sources model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSources(self))

	def suppress_flag(self):
		"""Get the suppress-flag object from the server

		Suppress Flag.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlag`: An object encapsulating an instance of the suppress-flag model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlag(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryReserved(self))

	def qqic(self):
		"""Get the qqic object from the server

		QQIC(Querier's Query Interval Code).

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQqic`: An object encapsulating an instance of the qqic model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqic(self))

	def qrv(self):
		"""Get the qrv object from the server

		QRV bits(Querier's Robustness Variable).

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQrv`: An object encapsulating an instance of the qrv model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrv(self))

	def source_address_list(self, name=None):
		"""Get the source-address-list object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the source-address-list list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList` | :obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList`: If arg name is None a list of SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList objects otherwise a single SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList object.  

		Raises:  
			NotFoundError: name is not in the list of source-address-list objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList(self, name))

	def create_source_address_list(self, name):
		"""Create a child sibling source-address-list instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList`: An object encapsulating an instance of the source-address-list model.  

		Raises:  
			AlreadyExistsError: An instance of source-address-list with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList(self, name), locals())

	def delete(self):
		"""Delete this igmpv3-query instance on the server.

		IGMPv3 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv3-query instance on the server with any changed property values.

		IGMPv3 Query message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddress(YangBase):
	"""Group IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'group-address'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressRandom(self))

	def delete(self):
		"""Delete this group-address instance on the server.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this group-address instance on the server with any changed property values.

		Group IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryGroupAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTime(YangBase):
	"""Maximum response Time.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'max-response-time'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTime, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeRandom(self))

	def delete(self):
		"""Delete this max-response-time instance on the server.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this max-response-time instance on the server with any changed property values.

		Maximum response Time.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryMaxResponseTimeRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSources(YangBase):
	"""Number of Sources.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-sources'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSources, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesRandom(self))

	def delete(self):
		"""Delete this number-of-sources instance on the server.

		Number of Sources.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-sources instance on the server with any changed property values.

		Number of Sources.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryNumberOfSourcesRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlag(YangBase):
	"""Suppress Flag.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'suppress-flag'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlag, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagRandom(self))

	def delete(self):
		"""Delete this suppress-flag instance on the server.

		Suppress Flag.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this suppress-flag instance on the server with any changed property values.

		Suppress Flag.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySuppressFlagRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryReserved(YangBase):
	"""Reserved.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQqic(YangBase):
	"""QQIC(Querier's Query Interval Code).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'qqic'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqic, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicRandom(self))

	def delete(self):
		"""Delete this qqic instance on the server.

		QQIC(Querier's Query Interval Code).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this qqic instance on the server with any changed property values.

		QQIC(Querier's Query Interval Code).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQqicRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQrv(YangBase):
	"""QRV bits(Querier's Robustness Variable).
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'qrv'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrv, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvRandom(self))

	def delete(self):
		"""Delete this qrv instance on the server.

		QRV bits(Querier's Robustness Variable).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this qrv instance on the server with any changed property values.

		QRV bits(Querier's Robustness Variable).

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QueryQrvRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'source-address-list'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressList, self).__init__(parent, yang_key_value)

	def address(self):
		"""Get the address object from the server

		Source IPv4 Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddress`: An object encapsulating an instance of the address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddress(self))

	def delete(self):
		"""Delete this source-address-list instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this source-address-list instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddress(YangBase):
	"""Source IPv4 Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'address'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressRandom(self))

	def delete(self):
		"""Delete this address instance on the server.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this address instance on the server with any changed property values.

		Source IPv4 Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3QuerySourceAddressListAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3Report(YangBase):
	"""IGMPv3 Report message
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmpv3-report'
		super(SessionsConfigDeviceTrafficFramesIgmpv3Report, self).__init__(parent, None)

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksum(self))

	def number_of_group_records(self):
		"""Get the number-of-group-records object from the server

		Number of group records.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecords`: An object encapsulating an instance of the number-of-group-records model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecords(self))

	def reserved(self):
		"""Get the reserved object from the server

		Reserved

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved`: An object encapsulating an instance of the reserved model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved(self))

	def reserved2(self):
		"""Get the reserved2 object from the server

		Reserved2

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2`: An object encapsulating an instance of the reserved2 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2(self))

	def delete(self):
		"""Delete this igmpv3-report instance on the server.

		IGMPv3 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this igmpv3-report instance on the server with any changed property values.

		IGMPv3 Report message

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecords(YangBase):
	"""Number of group records.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'number-of-group-records'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecords, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom(self))

	def delete(self):
		"""Delete this number-of-group-records instance on the server.

		Number of group records.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this number-of-group-records instance on the server with any changed property values.

		Number of group records.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportNumberOfGroupRecordsRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved(YangBase):
	"""Reserved
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedRandom(self))

	def delete(self):
		"""Delete this reserved instance on the server.

		Reserved

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved instance on the server with any changed property values.

		Reserved

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReservedRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2(YangBase):
	"""Reserved2
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'reserved2'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Random(self))

	def delete(self):
		"""Delete this reserved2 instance on the server.

		Reserved2

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this reserved2 instance on the server with any changed property values.

		Reserved2

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesIgmpv3ReportReserved2Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2Hello(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-hello'
		super(SessionsConfigDeviceTrafficFramesOspfv2Hello, self).__init__(parent, None)

	def designated_router_address(self):
		"""Get the designated-router-address object from the server

		Designated Router Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddress`: An object encapsulating an instance of the designated-router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddress(self))

	def backup_designated_router_address(self):
		"""Get the backup-designated-router-address object from the server

		Backup Designated Router Address

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddress`: An object encapsulating an instance of the backup-designated-router-address model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddress(self))

	def hello_interval(self):
		"""Get the hello-interval object from the server

		Hello Interval.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloHelloInterval`: An object encapsulating an instance of the hello-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloInterval(self))

	def router_dead_interval(self):
		"""Get the router-dead-interval object from the server

		Router Dead Interval.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadInterval`: An object encapsulating an instance of the router-dead-interval model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadInterval(self))

	def router_priority(self):
		"""Get the router-priority object from the server

		Router Priority.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriority`: An object encapsulating an instance of the router-priority model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriority(self))

	def network_mask(self):
		"""Get the network-mask object from the server

		Network mask

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMask`: An object encapsulating an instance of the network-mask model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMask(self))

	def area_id(self):
		"""Get the area-id object from the server

		Area Id

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAreaId`: An object encapsulating an instance of the area-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaId(self))

	def router_id(self):
		"""Get the router-id object from the server

		Router Id

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterId`: An object encapsulating an instance of the router-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterId(self))

	def checksum(self):
		"""Get the checksum object from the server

		Checksum value.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloChecksum`: An object encapsulating an instance of the checksum model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksum(self))

	def packet_length(self):
		"""Get the packet-length object from the server

		Packet Length.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLength`: An object encapsulating an instance of the packet-length model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLength(self))

	def auth_type(self):
		"""Get the auth-type object from the server

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthType`: An object encapsulating an instance of the auth-type model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthType(self))

	def auth_value1(self):
		"""Get the auth-value1 object from the server

		Authentication Value 1.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1`: An object encapsulating an instance of the auth-value1 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1(self))

	def auth_value2(self):
		"""Get the auth-value2 object from the server

		Authentication Value 2.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2`: An object encapsulating an instance of the auth-value2 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2(self))

	def options_reserved_bit_7(self):
		"""Get the options-reserved-bit-7 object from the server

		Reserved Bit 7

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7`: An object encapsulating an instance of the options-reserved-bit-7 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7(self))

	def options_reserved_bit_6(self):
		"""Get the options-reserved-bit-6 object from the server

		Reserved Bit 6

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6`: An object encapsulating an instance of the options-reserved-bit-6 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6(self))

	def options_dc_bit(self):
		"""Get the options-dc-bit object from the server

		Demand Circuits Bit

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBit`: An object encapsulating an instance of the options-dc-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBit(self))

	def options_ea_bit(self):
		"""Get the options-ea-bit object from the server

		EA Bit

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBit`: An object encapsulating an instance of the options-ea-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBit(self))

	def options_np_bit(self):
		"""Get the options-np-bit object from the server

		NSSA Support

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBit`: An object encapsulating an instance of the options-np-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBit(self))

	def options_mc_bit(self):
		"""Get the options-mc-bit object from the server

		Multicast Capable

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBit`: An object encapsulating an instance of the options-mc-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBit(self))

	def options_e_bit(self):
		"""Get the options-e-bit object from the server

		E Bit, External Routing Capability.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBit`: An object encapsulating an instance of the options-e-bit model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBit(self))

	def options_reserved_bit_0(self):
		"""Get the options-reserved-bit-0 object from the server

		Reserved Bit 0

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0`: An object encapsulating an instance of the options-reserved-bit-0 model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0(self))

	def neighbors(self, name=None):
		"""Get the neighbors object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the neighbors list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors` | :obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors`: If arg name is None a list of SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors objects otherwise a single SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors object.  

		Raises:  
			NotFoundError: name is not in the list of neighbors objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors(self, name))

	def create_neighbors(self, name):
		"""Create a child sibling neighbors instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors`: An object encapsulating an instance of the neighbors model.  

		Raises:  
			AlreadyExistsError: An instance of neighbors with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors(self, name), locals())

	def delete(self):
		"""Delete this ospfv2-hello instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this ospfv2-hello instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()


class SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddress(YangBase):
	"""Designated Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'designated-router-address'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressRandom(self))

	def delete(self):
		"""Delete this designated-router-address instance on the server.

		Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this designated-router-address instance on the server with any changed property values.

		Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloDesignatedRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddress(YangBase):
	"""Backup Designated Router Address
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'backup-designated-router-address'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddress, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom(self))

	def delete(self):
		"""Delete this backup-designated-router-address instance on the server.

		Backup Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this backup-designated-router-address instance on the server with any changed property values.

		Backup Designated Router Address

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloBackupDesignatedRouterAddressRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloHelloInterval(YangBase):
	"""Hello Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'hello-interval'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalRandom(self))

	def delete(self):
		"""Delete this hello-interval instance on the server.

		Hello Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this hello-interval instance on the server with any changed property values.

		Hello Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloHelloIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadInterval(YangBase):
	"""Router Dead Interval.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-dead-interval'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadInterval, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalRandom(self))

	def delete(self):
		"""Delete this router-dead-interval instance on the server.

		Router Dead Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-dead-interval instance on the server with any changed property values.

		Router Dead Interval.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterDeadIntervalRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriority(YangBase):
	"""Router Priority.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-priority'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriority, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityRandom(self))

	def delete(self):
		"""Delete this router-priority instance on the server.

		Router Priority.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-priority instance on the server with any changed property values.

		Router Priority.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterPriorityRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMask(YangBase):
	"""Network mask
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'network-mask'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMask, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskRandom(self))

	def delete(self):
		"""Delete this network-mask instance on the server.

		Network mask

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this network-mask instance on the server with any changed property values.

		Network mask

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNetworkMaskRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAreaId(YangBase):
	"""Area Id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'area-id'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdRandom(self))

	def delete(self):
		"""Delete this area-id instance on the server.

		Area Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this area-id instance on the server with any changed property values.

		Area Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAreaIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterId(YangBase):
	"""Router Id
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'router-id'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdRandom(self))

	def delete(self):
		"""Delete this router-id instance on the server.

		Router Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this router-id instance on the server with any changed property values.

		Router Id

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloRouterIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloChecksum(YangBase):
	"""Checksum value.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'checksum'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksum, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumRandom(self))

	def delete(self):
		"""Delete this checksum instance on the server.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this checksum instance on the server with any changed property values.

		Checksum value.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloChecksumRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLength(YangBase):
	"""Packet Length.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'packet-length'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLength, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthRandom(self))

	def delete(self):
		"""Delete this packet-length instance on the server.

		Packet Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this packet-length instance on the server with any changed property values.

		Packet Length.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloPacketLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthType(YangBase):
	"""Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-type'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthType, self).__init__(parent, None)

	def delete(self):
		"""Delete this auth-type instance on the server.

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-type instance on the server with any changed property values.

		Authentication Type.    NO_AUTH : No Authentication.    SIMPLE_PASSWORD : Simple Password.    MD5 : MD5 Authentication.    USER_DEFINED : User Defined.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_format(self):
		"""str: Refine this leaf value with a regex of valid enum choices"""
		return self._get_value('pattern-format')

	@property
	def pattern_type(self):
		""":str:`enum`: TBD  
		Enums:  
			SINGLE: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def single(self):
		"""str: TBD"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: TBD"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1(YangBase):
	"""Authentication Value 1.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-value1'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Random(self))

	def delete(self):
		"""Delete this auth-value1 instance on the server.

		Authentication Value 1.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-value1 instance on the server with any changed property values.

		Authentication Value 1.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue1Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2(YangBase):
	"""Authentication Value 2.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'auth-value2'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Random(self))

	def delete(self):
		"""Delete this auth-value2 instance on the server.

		Authentication Value 2.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this auth-value2 instance on the server with any changed property values.

		Authentication Value 2.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloAuthValue2Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7(YangBase):
	"""Reserved Bit 7
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-7'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-7 instance on the server.

		Reserved Bit 7

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-7 instance on the server with any changed property values.

		Reserved Bit 7

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit7Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6(YangBase):
	"""Reserved Bit 6
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-6'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-6 instance on the server.

		Reserved Bit 6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-6 instance on the server with any changed property values.

		Reserved Bit 6

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit6Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBit(YangBase):
	"""Demand Circuits Bit
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-dc-bit'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitRandom(self))

	def delete(self):
		"""Delete this options-dc-bit instance on the server.

		Demand Circuits Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-dc-bit instance on the server with any changed property values.

		Demand Circuits Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsDcBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBit(YangBase):
	"""EA Bit
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-ea-bit'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitRandom(self))

	def delete(self):
		"""Delete this options-ea-bit instance on the server.

		EA Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-ea-bit instance on the server with any changed property values.

		EA Bit

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEaBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBit(YangBase):
	"""NSSA Support
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-np-bit'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitRandom(self))

	def delete(self):
		"""Delete this options-np-bit instance on the server.

		NSSA Support

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-np-bit instance on the server with any changed property values.

		NSSA Support

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsNpBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBit(YangBase):
	"""Multicast Capable
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-mc-bit'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitRandom(self))

	def delete(self):
		"""Delete this options-mc-bit instance on the server.

		Multicast Capable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-mc-bit instance on the server with any changed property values.

		Multicast Capable

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsMcBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBit(YangBase):
	"""E Bit, External Routing Capability.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-e-bit'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBit, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitRandom(self))

	def delete(self):
		"""Delete this options-e-bit instance on the server.

		E Bit, External Routing Capability.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-e-bit instance on the server with any changed property values.

		E Bit, External Routing Capability.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsEBitRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0(YangBase):
	"""Reserved Bit 0
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'options-reserved-bit-0'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Increment`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Increment(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Decrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Decrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Random`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Random(self))

	def delete(self):
		"""Delete this options-reserved-bit-0 instance on the server.

		Reserved Bit 0

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this options-reserved-bit-0 instance on the server with any changed property values.

		Reserved Bit 0

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Increment(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Increment, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Decrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Decrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Random(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloOptionsReservedBit0Random, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbors'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighbors, self).__init__(parent, yang_key_value)

	def neighbors_id(self):
		"""Get the neighbors-id object from the server

		Neighbor ID

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsId`: An object encapsulating an instance of the neighbors-id model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsId(self))

	def delete(self):
		"""Delete this neighbors instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbors instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')


class SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsId(YangBase):
	"""Neighbor ID
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'neighbors-id'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsId, self).__init__(parent, None)

	def increment(self):
		"""Get the increment object from the server

		The values that make up the increment pattern

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement(self))

	def random(self):
		"""Get the random object from the server

		The repeatable random pattern.

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom(self))

	def delete(self):
		"""Delete this neighbors-id instance on the server.

		Neighbor ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this neighbors-id instance on the server with any changed property values.

		Neighbor ID

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def pattern_type(self):
		""":str:`enum`: The selected pattern from the possible pattern types.  
		Enums:  
			SINGLE: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			VALUE_LIST: TBD  
		"""
		return self._get_value('pattern-type')

	@pattern_type.setter
	def pattern_type(self, value):
		return self._set_value('pattern-type', value)

	@property
	def pattern_format(self):
		"""str: The format of the pattern. This will almost always be a regular expression. It is used to determine the validity of the values being set in the child leaf nodes of the pattern."""
		return self._get_value('pattern-format')

	@property
	def single(self):
		"""str: The value of the single pattern"""
		return self._get_value('single')

	@single.setter
	def single(self, value):
		return self._set_value('single', value)

	@property
	def value_list(self):
		"""str: The value list pattern takes a list of values that will be repeated if they do not meet or exceed the count"""
		return self._get_value('value-list')

	@value_list.setter
	def value_list(self, value):
		return self._set_value('value-list', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement(YangBase):
	"""The values that make up the increment pattern
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		The values that make up the increment pattern

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the increment pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the increment pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""str: The start value of the decrement pattern"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def step(self):
		"""str: The step value of the decrement pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def count(self):
		"""str: TBD"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)


class SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom(YangBase):
	"""The repeatable random pattern.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFramesOspfv2HelloNeighborsNeighborsIdRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		The repeatable random pattern.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def min(self):
		"""str: The minimum random value of the random pattern"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)

	@property
	def max(self):
		"""str: The maximum random value of the random pattern"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def step(self):
		"""str: The step value of the random pattern"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)

	@property
	def seed(self):
		"""uint32: The seed value of the random pattern"""
		return self._get_value('seed')

	@seed.setter
	def seed(self, value):
		return self._set_value('seed', value)


class SessionsConfigDeviceTrafficFramesCustom(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom'
		super(SessionsConfigDeviceTrafficFramesCustom, self).__init__(parent, None)

	def payload(self):
		"""Get the payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFramesCustomPayload`: An object encapsulating an instance of the payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFramesCustomPayload(self))

	def delete(self):
		"""Delete this custom instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def payload_type(self):
		""":str:`enum`: TBD  
		Enums:  
			INCREMENT_BYTE: TBD  
			DECREMENT_BYTE: TBD  
			INCREMENT_WORD: TBD  
			DECREMENT_WORD: TBD  
			CRPAT: TBD  
			CJPAT: TBD  
			RANDOM: TBD  
			CUSTOM: TBD  
		"""
		return self._get_value('payload-type')

	@payload_type.setter
	def payload_type(self, value):
		return self._set_value('payload-type', value)


class SessionsConfigDeviceTrafficFramesCustomPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload'
		super(SessionsConfigDeviceTrafficFramesCustomPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def data(self):
		"""iyt:hex-string: TBD"""
		return self._get_value('data')

	@data.setter
	def data(self, value):
		return self._set_value('data', value)

	@property
	def repeat(self):
		"""bool: Repeat the payload data to fill the length specified"""
		return self._get_value('repeat')

	@repeat.setter
	def repeat(self, value):
		return self._set_value('repeat', value)

	@property
	def length(self):
		"""int32: TBD"""
		return self._get_value('length')

	@length.setter
	def length(self, value):
		return self._set_value('length', value)


class SessionsConfigDeviceTrafficFrameLength(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frame-length'
		super(SessionsConfigDeviceTrafficFrameLength, self).__init__(parent, None)

	def random(self):
		"""Get the random object from the server

		Random frame size options

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLengthRandom`: An object encapsulating an instance of the random model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLengthRandom(self))

	def increment(self):
		"""Get the increment object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLengthIncrement`: An object encapsulating an instance of the increment model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLengthIncrement(self))

	def decrement(self):
		"""Get the decrement object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLengthDecrement`: An object encapsulating an instance of the decrement model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLengthDecrement(self))

	def imix(self):
		"""Get the imix object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLengthImix`: An object encapsulating an instance of the imix model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLengthImix(self))

	def delete(self):
		"""Delete this frame-length instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frame-length instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def length_type(self):
		""":str:`enum`: TBD  
		Enums:  
			FIXED: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			RANDOM: TBD  
			AUTO: TBD  
			IMIX: TBD  
		"""
		return self._get_value('length-type')

	@length_type.setter
	def length_type(self, value):
		return self._set_value('length-type', value)

	@property
	def fixed(self):
		"""int32: Fixed value for frame length"""
		return self._get_value('fixed')

	@fixed.setter
	def fixed(self, value):
		return self._set_value('fixed', value)


class SessionsConfigDeviceTrafficFrameLengthRandom(YangBase):
	"""Random frame size options
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'random'
		super(SessionsConfigDeviceTrafficFrameLengthRandom, self).__init__(parent, None)

	def delete(self):
		"""Delete this random instance on the server.

		Random frame size options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this random instance on the server with any changed property values.

		Random frame size options

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def max(self):
		"""int32: TBD"""
		return self._get_value('max')

	@max.setter
	def max(self, value):
		return self._set_value('max', value)

	@property
	def min(self):
		"""int32: TBD"""
		return self._get_value('min')

	@min.setter
	def min(self, value):
		return self._set_value('min', value)


class SessionsConfigDeviceTrafficFrameLengthIncrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment'
		super(SessionsConfigDeviceTrafficFrameLengthIncrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""int32: Starting increment value for frame length"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def end(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('end')

	@end.setter
	def end(self, value):
		return self._set_value('end', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigDeviceTrafficFrameLengthDecrement(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement'
		super(SessionsConfigDeviceTrafficFrameLengthDecrement, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def start(self):
		"""int32: Starting decrement value for frame length"""
		return self._get_value('start')

	@start.setter
	def start(self, value):
		return self._set_value('start', value)

	@property
	def end(self):
		"""int32: Maximum decrement value for frame length"""
		return self._get_value('end')

	@end.setter
	def end(self, value):
		return self._set_value('end', value)

	@property
	def step(self):
		"""int32: Step decrement value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigDeviceTrafficFrameLengthImix(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'imix'
		super(SessionsConfigDeviceTrafficFrameLengthImix, self).__init__(parent, None)

	def custom_imix(self, name=None):
		"""Get the custom-imix object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the custom-imix list.  

		Returns:  
			:obj:`list` of :obj:`SessionsConfigDeviceTrafficFrameLengthImixCustomImix` | :obj:`SessionsConfigDeviceTrafficFrameLengthImixCustomImix`: If arg name is None a list of SessionsConfigDeviceTrafficFrameLengthImixCustomImix objects otherwise a single SessionsConfigDeviceTrafficFrameLengthImixCustomImix object.  

		Raises:  
			NotFoundError: name is not in the list of custom-imix objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameLengthImixCustomImix(self, name))

	def create_custom_imix(self, name):
		"""Create a child sibling custom-imix instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameLengthImixCustomImix`: An object encapsulating an instance of the custom-imix model.  

		Raises:  
			AlreadyExistsError: An instance of custom-imix with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsConfigDeviceTrafficFrameLengthImixCustomImix(self, name), locals())

	def delete(self):
		"""Delete this imix instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this imix instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def imix_type(self):
		""":str:`enum`: TBD  
		Enums:  
			DEFAULT: TBD  
			CUSTOM_IMIX: TBD  
		"""
		return self._get_value('imix-type')

	@imix_type.setter
	def imix_type(self, value):
		return self._set_value('imix-type', value)


class SessionsConfigDeviceTrafficFrameLengthImixCustomImix(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'custom-imix'
		self.YANG_KEY = 'name'
		super(SessionsConfigDeviceTrafficFrameLengthImixCustomImix, self).__init__(parent, yang_key_value)

	def delete(self):
		"""Delete this custom-imix instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this custom-imix instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: TBD"""
		return self._get_value('name')

	@property
	def size(self):
		"""int32: TBD"""
		return self._get_value('size')

	@size.setter
	def size(self, value):
		return self._set_value('size', value)

	@property
	def weight(self):
		"""int32: TBD"""
		return self._get_value('weight')

	@weight.setter
	def weight(self, value):
		return self._set_value('weight', value)


class SessionsConfigDeviceTrafficFrameRate(YangBase):
	"""Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'frame-rate'
		super(SessionsConfigDeviceTrafficFrameRate, self).__init__(parent, None)

	def fixed_rate(self):
		"""Get the fixed-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameRateFixedRate`: An object encapsulating an instance of the fixed-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameRateFixedRate(self))

	def decrement_rate(self):
		"""Get the decrement-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameRateDecrementRate`: An object encapsulating an instance of the decrement-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameRateDecrementRate(self))

	def increment_rate(self):
		"""Get the increment-rate object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficFrameRateIncrementRate`: An object encapsulating an instance of the increment-rate model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficFrameRateIncrementRate(self))

	def delete(self):
		"""Delete this frame-rate instance on the server.

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this frame-rate instance on the server with any changed property values.

		Configure the total stream rate. The vendor must ensure that the rate configured will be what goes out on the wire for the streams generated by this aggregate. i.e., 100% line rate / 10 vendor generated streams = 10% line rate per stream

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def mode(self):
		""":str:`enum`: TBD  
		Enums:  
			FIXED: TBD  
			INCREMENT: TBD  
			DECREMENT: TBD  
			SHUFFLE: TBD  
			LIST: TBD  
		"""
		return self._get_value('mode')

	@mode.setter
	def mode(self, value):
		return self._set_value('mode', value)


class SessionsConfigDeviceTrafficFrameRateFixedRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'fixed-rate'
		super(SessionsConfigDeviceTrafficFrameRateFixedRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this fixed-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this fixed-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MPBS: TBD  
			FRAMES_PER_SECOND: TBD  
			INTER_PACKET_GAP: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def inter_packet_gap(self):
		"""int32: Load value set on the streamblock/traffic-item inter packet gap."""
		return self._get_value('inter-packet-gap')

	@inter_packet_gap.setter
	def inter_packet_gap(self, value):
		return self._set_value('inter-packet-gap', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)


class SessionsConfigDeviceTrafficFrameRateDecrementRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'decrement-rate'
		super(SessionsConfigDeviceTrafficFrameRateDecrementRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this decrement-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this decrement-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MBPS: TBD  
			FRAMES_PER_SECOND: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)

	@property
	def count(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigDeviceTrafficFrameRateIncrementRate(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'increment-rate'
		super(SessionsConfigDeviceTrafficFrameRateIncrementRate, self).__init__(parent, None)

	def delete(self):
		"""Delete this increment-rate instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this increment-rate instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rate_type(self):
		""":str:`enum`: Load unit applied to the stream block.  
		Enums:  
			BPS: TBD  
			KBPS: TBD  
			MPBS: TBD  
			FRAMES_PER_SECOND: TBD  
			PERCENT_LINE_RATE: TBD  
		"""
		return self._get_value('rate-type')

	@rate_type.setter
	def rate_type(self, value):
		return self._set_value('rate-type', value)

	@property
	def bps(self):
		"""int32: Load value set on the streamblock/traffic-item in bits per second."""
		return self._get_value('bps')

	@bps.setter
	def bps(self, value):
		return self._set_value('bps', value)

	@property
	def kbps(self):
		"""int32: Load value set on the streamblock/traffic-item in kilo bits per second."""
		return self._get_value('kbps')

	@kbps.setter
	def kbps(self, value):
		return self._set_value('kbps', value)

	@property
	def mbps(self):
		"""int32: Load value set on the streamblock/traffic-item in mega bits per second."""
		return self._get_value('mbps')

	@mbps.setter
	def mbps(self, value):
		return self._set_value('mbps', value)

	@property
	def fps(self):
		"""int32: Load value set on the streamblock/traffic-item in frames per second."""
		return self._get_value('fps')

	@fps.setter
	def fps(self, value):
		return self._set_value('fps', value)

	@property
	def percent_value(self):
		"""int32: Load value set on the streamblock/traffic-item in percent."""
		return self._get_value('percent-value')

	@percent_value.setter
	def percent_value(self, value):
		return self._set_value('percent-value', value)

	@property
	def count(self):
		"""int32: Maximum increment value for frame length"""
		return self._get_value('count')

	@count.setter
	def count(self, value):
		return self._set_value('count', value)

	@property
	def step(self):
		"""int32: Step increment value for frame length"""
		return self._get_value('step')

	@step.setter
	def step(self, value):
		return self._set_value('step', value)


class SessionsConfigDeviceTrafficPacketPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'packet-payload'
		super(SessionsConfigDeviceTrafficPacketPayload, self).__init__(parent, None)

	def payload(self):
		"""Get the payload object from the server

		TBD

		Returns:  
			:obj:`SessionsConfigDeviceTrafficPacketPayloadPayload`: An object encapsulating an instance of the payload model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsConfigDeviceTrafficPacketPayloadPayload(self))

	def delete(self):
		"""Delete this packet-payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this packet-payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def payload_type(self):
		""":str:`enum`: TBD  
		Enums:  
			INCREMENT_BYTE: TBD  
			DECREMENT_BYTE: TBD  
			INCREMENT_WORD: TBD  
			DECREMENT_WORD: TBD  
			CUSTOM: TBD  
		"""
		return self._get_value('payload-type')

	@payload_type.setter
	def payload_type(self, value):
		return self._set_value('payload-type', value)


class SessionsConfigDeviceTrafficPacketPayloadPayload(YangBase):
	"""TBD
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'payload'
		super(SessionsConfigDeviceTrafficPacketPayloadPayload, self).__init__(parent, None)

	def delete(self):
		"""Delete this payload instance on the server.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._delete()

	def update(self):
		"""Update this payload instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def data(self):
		"""iyt:hex-string: TBD"""
		return self._get_value('data')

	@data.setter
	def data(self, value):
		return self._set_value('data', value)

	@property
	def repeat(self):
		"""bool: Repeat the payload data to fill the length specified"""
		return self._get_value('repeat')

	@repeat.setter
	def repeat(self, value):
		return self._set_value('repeat', value)

	@property
	def length(self):
		"""int32: TBD"""
		return self._get_value('length')

	@length.setter
	def length(self, value):
		return self._set_value('length', value)


class SessionsStatistics(YangBase):
	"""The statistics pull model
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'statistics'
		super(SessionsStatistics, self).__init__(parent, None)

	def physical_port(self, port_name=None):
		"""Get the physical-port object(s) from the server.

		TBD

		Args:  
			port_name (:obj:`str`, optional, default=None): A key value in the physical-port list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsPhysicalPort` | :obj:`SessionsStatisticsPhysicalPort`: If arg port_name is None a list of SessionsStatisticsPhysicalPort objects otherwise a single SessionsStatisticsPhysicalPort object.  

		Raises:  
			NotFoundError: port_name is not in the list of physical-port objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsPhysicalPort(self, port_name))

	def create_physical_port(self, port_name):
		"""Create a child sibling physical-port instance on the server.

		TBD

		Args:  
			port_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsPhysicalPort`: An object encapsulating an instance of the physical-port model.  

		Raises:  
			AlreadyExistsError: An instance of physical-port with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsPhysicalPort(self, port_name), locals())

	def port(self, name=None):
		"""Get the port object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the port list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsPort` | :obj:`SessionsStatisticsPort`: If arg name is None a list of SessionsStatisticsPort objects otherwise a single SessionsStatisticsPort object.  

		Raises:  
			NotFoundError: name is not in the list of port objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsPort(self, name))

	def create_port(self, name):
		"""Create a child sibling port instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsPort`: An object encapsulating an instance of the port model.  

		Raises:  
			AlreadyExistsError: An instance of port with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsPort(self, name), locals())

	def port_traffic(self, name=None):
		"""Get the port-traffic object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the port-traffic list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsPortTraffic` | :obj:`SessionsStatisticsPortTraffic`: If arg name is None a list of SessionsStatisticsPortTraffic objects otherwise a single SessionsStatisticsPortTraffic object.  

		Raises:  
			NotFoundError: name is not in the list of port-traffic objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsPortTraffic(self, name))

	def create_port_traffic(self, name):
		"""Create a child sibling port-traffic instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsPortTraffic`: An object encapsulating an instance of the port-traffic model.  

		Raises:  
			AlreadyExistsError: An instance of port-traffic with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsPortTraffic(self, name), locals())

	def device_traffic(self, name=None):
		"""Get the device-traffic object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the device-traffic list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDeviceTraffic` | :obj:`SessionsStatisticsDeviceTraffic`: If arg name is None a list of SessionsStatisticsDeviceTraffic objects otherwise a single SessionsStatisticsDeviceTraffic object.  

		Raises:  
			NotFoundError: name is not in the list of device-traffic objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDeviceTraffic(self, name))

	def create_device_traffic(self, name):
		"""Create a child sibling device-traffic instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDeviceTraffic`: An object encapsulating an instance of the device-traffic model.  

		Raises:  
			AlreadyExistsError: An instance of device-traffic with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDeviceTraffic(self, name), locals())

	def bgpv4_statistics(self, device_name=None):
		"""Get the bgpv4-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the bgpv4-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsBgpv4Statistics` | :obj:`SessionsStatisticsBgpv4Statistics`: If arg device_name is None a list of SessionsStatisticsBgpv4Statistics objects otherwise a single SessionsStatisticsBgpv4Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of bgpv4-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsBgpv4Statistics(self, device_name))

	def create_bgpv4_statistics(self, device_name):
		"""Create a child sibling bgpv4-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsBgpv4Statistics`: An object encapsulating an instance of the bgpv4-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of bgpv4-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsBgpv4Statistics(self, device_name), locals())

	def bgpv6_statistics(self, device_name=None):
		"""Get the bgpv6-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the bgpv6-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsBgpv6Statistics` | :obj:`SessionsStatisticsBgpv6Statistics`: If arg device_name is None a list of SessionsStatisticsBgpv6Statistics objects otherwise a single SessionsStatisticsBgpv6Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of bgpv6-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsBgpv6Statistics(self, device_name))

	def create_bgpv6_statistics(self, device_name):
		"""Create a child sibling bgpv6-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsBgpv6Statistics`: An object encapsulating an instance of the bgpv6-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of bgpv6-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsBgpv6Statistics(self, device_name), locals())

	def ospfv2_statistics(self, device_name=None):
		"""Get the ospfv2-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the ospfv2-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsOspfv2Statistics` | :obj:`SessionsStatisticsOspfv2Statistics`: If arg device_name is None a list of SessionsStatisticsOspfv2Statistics objects otherwise a single SessionsStatisticsOspfv2Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of ospfv2-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsOspfv2Statistics(self, device_name))

	def create_ospfv2_statistics(self, device_name):
		"""Create a child sibling ospfv2-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsOspfv2Statistics`: An object encapsulating an instance of the ospfv2-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of ospfv2-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsOspfv2Statistics(self, device_name), locals())

	def ospfv3_statistics(self, device_name=None):
		"""Get the ospfv3-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the ospfv3-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsOspfv3Statistics` | :obj:`SessionsStatisticsOspfv3Statistics`: If arg device_name is None a list of SessionsStatisticsOspfv3Statistics objects otherwise a single SessionsStatisticsOspfv3Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of ospfv3-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsOspfv3Statistics(self, device_name))

	def create_ospfv3_statistics(self, device_name):
		"""Create a child sibling ospfv3-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsOspfv3Statistics`: An object encapsulating an instance of the ospfv3-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of ospfv3-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsOspfv3Statistics(self, device_name), locals())

	def isis_statistics(self, device_name=None):
		"""Get the isis-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the isis-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIsisStatistics` | :obj:`SessionsStatisticsIsisStatistics`: If arg device_name is None a list of SessionsStatisticsIsisStatistics objects otherwise a single SessionsStatisticsIsisStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of isis-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIsisStatistics(self, device_name))

	def create_isis_statistics(self, device_name):
		"""Create a child sibling isis-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIsisStatistics`: An object encapsulating an instance of the isis-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of isis-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIsisStatistics(self, device_name), locals())

	def bfdv4_statistics(self, device_name=None):
		"""Get the bfdv4-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the bfdv4-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsBfdv4Statistics` | :obj:`SessionsStatisticsBfdv4Statistics`: If arg device_name is None a list of SessionsStatisticsBfdv4Statistics objects otherwise a single SessionsStatisticsBfdv4Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of bfdv4-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsBfdv4Statistics(self, device_name))

	def create_bfdv4_statistics(self, device_name):
		"""Create a child sibling bfdv4-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsBfdv4Statistics`: An object encapsulating an instance of the bfdv4-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of bfdv4-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsBfdv4Statistics(self, device_name), locals())

	def bfdv6_statistics(self, device_name=None):
		"""Get the bfdv6-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the bfdv6-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsBfdv6Statistics` | :obj:`SessionsStatisticsBfdv6Statistics`: If arg device_name is None a list of SessionsStatisticsBfdv6Statistics objects otherwise a single SessionsStatisticsBfdv6Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of bfdv6-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsBfdv6Statistics(self, device_name))

	def create_bfdv6_statistics(self, device_name):
		"""Create a child sibling bfdv6-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsBfdv6Statistics`: An object encapsulating an instance of the bfdv6-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of bfdv6-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsBfdv6Statistics(self, device_name), locals())

	def dhcpv4_statistics(self, device_name=None):
		"""Get the dhcpv4-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the dhcpv4-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv4Statistics` | :obj:`SessionsStatisticsDhcpv4Statistics`: If arg device_name is None a list of SessionsStatisticsDhcpv4Statistics objects otherwise a single SessionsStatisticsDhcpv4Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of dhcpv4-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv4Statistics(self, device_name))

	def create_dhcpv4_statistics(self, device_name):
		"""Create a child sibling dhcpv4-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv4Statistics`: An object encapsulating an instance of the dhcpv4-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv4-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv4Statistics(self, device_name), locals())

	def dhcpv4_port_statistics(self, port_name=None):
		"""Get the dhcpv4-port-statistics object(s) from the server.

		TBD

		Args:  
			port_name (:obj:`str`, optional, default=None): A key value in the dhcpv4-port-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv4PortStatistics` | :obj:`SessionsStatisticsDhcpv4PortStatistics`: If arg port_name is None a list of SessionsStatisticsDhcpv4PortStatistics objects otherwise a single SessionsStatisticsDhcpv4PortStatistics object.  

		Raises:  
			NotFoundError: port_name is not in the list of dhcpv4-port-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv4PortStatistics(self, port_name))

	def create_dhcpv4_port_statistics(self, port_name):
		"""Create a child sibling dhcpv4-port-statistics instance on the server.

		TBD

		Args:  
			port_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv4PortStatistics`: An object encapsulating an instance of the dhcpv4-port-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv4-port-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv4PortStatistics(self, port_name), locals())

	def dhcpv4_server_statistics(self, device_name=None):
		"""Get the dhcpv4-server-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the dhcpv4-server-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv4ServerStatistics` | :obj:`SessionsStatisticsDhcpv4ServerStatistics`: If arg device_name is None a list of SessionsStatisticsDhcpv4ServerStatistics objects otherwise a single SessionsStatisticsDhcpv4ServerStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of dhcpv4-server-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv4ServerStatistics(self, device_name))

	def create_dhcpv4_server_statistics(self, device_name):
		"""Create a child sibling dhcpv4-server-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv4ServerStatistics`: An object encapsulating an instance of the dhcpv4-server-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv4-server-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv4ServerStatistics(self, device_name), locals())

	def dhcpv6_statistics(self, device_name=None):
		"""Get the dhcpv6-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the dhcpv6-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv6Statistics` | :obj:`SessionsStatisticsDhcpv6Statistics`: If arg device_name is None a list of SessionsStatisticsDhcpv6Statistics objects otherwise a single SessionsStatisticsDhcpv6Statistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of dhcpv6-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv6Statistics(self, device_name))

	def create_dhcpv6_statistics(self, device_name):
		"""Create a child sibling dhcpv6-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv6Statistics`: An object encapsulating an instance of the dhcpv6-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv6-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv6Statistics(self, device_name), locals())

	def dhcpv6_port_statistics(self, port_name=None):
		"""Get the dhcpv6-port-statistics object(s) from the server.

		TBD

		Args:  
			port_name (:obj:`str`, optional, default=None): A key value in the dhcpv6-port-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv6PortStatistics` | :obj:`SessionsStatisticsDhcpv6PortStatistics`: If arg port_name is None a list of SessionsStatisticsDhcpv6PortStatistics objects otherwise a single SessionsStatisticsDhcpv6PortStatistics object.  

		Raises:  
			NotFoundError: port_name is not in the list of dhcpv6-port-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv6PortStatistics(self, port_name))

	def create_dhcpv6_port_statistics(self, port_name):
		"""Create a child sibling dhcpv6-port-statistics instance on the server.

		TBD

		Args:  
			port_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv6PortStatistics`: An object encapsulating an instance of the dhcpv6-port-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv6-port-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv6PortStatistics(self, port_name), locals())

	def dhcpv6_server_statistics(self, device_name=None):
		"""Get the dhcpv6-server-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the dhcpv6-server-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDhcpv6ServerStatistics` | :obj:`SessionsStatisticsDhcpv6ServerStatistics`: If arg device_name is None a list of SessionsStatisticsDhcpv6ServerStatistics objects otherwise a single SessionsStatisticsDhcpv6ServerStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of dhcpv6-server-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDhcpv6ServerStatistics(self, device_name))

	def create_dhcpv6_server_statistics(self, device_name):
		"""Create a child sibling dhcpv6-server-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDhcpv6ServerStatistics`: An object encapsulating an instance of the dhcpv6-server-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of dhcpv6-server-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDhcpv6ServerStatistics(self, device_name), locals())

	def igmp_statistics(self, device_name=None):
		"""Get the igmp-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the igmp-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIgmpStatistics` | :obj:`SessionsStatisticsIgmpStatistics`: If arg device_name is None a list of SessionsStatisticsIgmpStatistics objects otherwise a single SessionsStatisticsIgmpStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of igmp-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIgmpStatistics(self, device_name))

	def create_igmp_statistics(self, device_name):
		"""Create a child sibling igmp-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIgmpStatistics`: An object encapsulating an instance of the igmp-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of igmp-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIgmpStatistics(self, device_name), locals())

	def igmp_port_statistics(self, port_name=None):
		"""Get the igmp-port-statistics object(s) from the server.

		TBD

		Args:  
			port_name (:obj:`str`, optional, default=None): A key value in the igmp-port-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIgmpPortStatistics` | :obj:`SessionsStatisticsIgmpPortStatistics`: If arg port_name is None a list of SessionsStatisticsIgmpPortStatistics objects otherwise a single SessionsStatisticsIgmpPortStatistics object.  

		Raises:  
			NotFoundError: port_name is not in the list of igmp-port-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIgmpPortStatistics(self, port_name))

	def create_igmp_port_statistics(self, port_name):
		"""Create a child sibling igmp-port-statistics instance on the server.

		TBD

		Args:  
			port_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIgmpPortStatistics`: An object encapsulating an instance of the igmp-port-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of igmp-port-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIgmpPortStatistics(self, port_name), locals())

	def igmp_host_group_statistics(self, device_name=None):
		"""Get the igmp-host-group-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the igmp-host-group-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIgmpHostGroupStatistics` | :obj:`SessionsStatisticsIgmpHostGroupStatistics`: If arg device_name is None a list of SessionsStatisticsIgmpHostGroupStatistics objects otherwise a single SessionsStatisticsIgmpHostGroupStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of igmp-host-group-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIgmpHostGroupStatistics(self, device_name))

	def create_igmp_host_group_statistics(self, device_name):
		"""Create a child sibling igmp-host-group-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIgmpHostGroupStatistics`: An object encapsulating an instance of the igmp-host-group-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of igmp-host-group-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIgmpHostGroupStatistics(self, device_name), locals())

	def igmp_querier_statistics(self, device_name=None):
		"""Get the igmp-querier-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the igmp-querier-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIgmpQuerierStatistics` | :obj:`SessionsStatisticsIgmpQuerierStatistics`: If arg device_name is None a list of SessionsStatisticsIgmpQuerierStatistics objects otherwise a single SessionsStatisticsIgmpQuerierStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of igmp-querier-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIgmpQuerierStatistics(self, device_name))

	def create_igmp_querier_statistics(self, device_name):
		"""Create a child sibling igmp-querier-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIgmpQuerierStatistics`: An object encapsulating an instance of the igmp-querier-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of igmp-querier-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIgmpQuerierStatistics(self, device_name), locals())

	def mld_statistics(self, device_name=None):
		"""Get the mld-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the mld-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsMldStatistics` | :obj:`SessionsStatisticsMldStatistics`: If arg device_name is None a list of SessionsStatisticsMldStatistics objects otherwise a single SessionsStatisticsMldStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of mld-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsMldStatistics(self, device_name))

	def create_mld_statistics(self, device_name):
		"""Create a child sibling mld-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsMldStatistics`: An object encapsulating an instance of the mld-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of mld-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsMldStatistics(self, device_name), locals())

	def mld_port_statistics(self, port_name=None):
		"""Get the mld-port-statistics object(s) from the server.

		TBD

		Args:  
			port_name (:obj:`str`, optional, default=None): A key value in the mld-port-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsMldPortStatistics` | :obj:`SessionsStatisticsMldPortStatistics`: If arg port_name is None a list of SessionsStatisticsMldPortStatistics objects otherwise a single SessionsStatisticsMldPortStatistics object.  

		Raises:  
			NotFoundError: port_name is not in the list of mld-port-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsMldPortStatistics(self, port_name))

	def create_mld_port_statistics(self, port_name):
		"""Create a child sibling mld-port-statistics instance on the server.

		TBD

		Args:  
			port_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsMldPortStatistics`: An object encapsulating an instance of the mld-port-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of mld-port-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsMldPortStatistics(self, port_name), locals())

	def mld_host_group_statistics(self, device_name=None):
		"""Get the mld-host-group-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the mld-host-group-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsMldHostGroupStatistics` | :obj:`SessionsStatisticsMldHostGroupStatistics`: If arg device_name is None a list of SessionsStatisticsMldHostGroupStatistics objects otherwise a single SessionsStatisticsMldHostGroupStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of mld-host-group-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsMldHostGroupStatistics(self, device_name))

	def create_mld_host_group_statistics(self, device_name):
		"""Create a child sibling mld-host-group-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsMldHostGroupStatistics`: An object encapsulating an instance of the mld-host-group-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of mld-host-group-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsMldHostGroupStatistics(self, device_name), locals())

	def mld_querier_statistics(self, device_name=None):
		"""Get the mld-querier-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the mld-querier-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsMldQuerierStatistics` | :obj:`SessionsStatisticsMldQuerierStatistics`: If arg device_name is None a list of SessionsStatisticsMldQuerierStatistics objects otherwise a single SessionsStatisticsMldQuerierStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of mld-querier-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsMldQuerierStatistics(self, device_name))

	def create_mld_querier_statistics(self, device_name):
		"""Create a child sibling mld-querier-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsMldQuerierStatistics`: An object encapsulating an instance of the mld-querier-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of mld-querier-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsMldQuerierStatistics(self, device_name), locals())

	def pim_statistics(self, device_name=None):
		"""Get the pim-statistics object(s) from the server.

		TBD

		Args:  
			device_name (:obj:`str`, optional, default=None): A key value in the pim-statistics list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsPimStatistics` | :obj:`SessionsStatisticsPimStatistics`: If arg device_name is None a list of SessionsStatisticsPimStatistics objects otherwise a single SessionsStatisticsPimStatistics object.  

		Raises:  
			NotFoundError: device_name is not in the list of pim-statistics objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsPimStatistics(self, device_name))

	def create_pim_statistics(self, device_name):
		"""Create a child sibling pim-statistics instance on the server.

		TBD

		Args:  
			device_name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsPimStatistics`: An object encapsulating an instance of the pim-statistics model.  

		Raises:  
			AlreadyExistsError: An instance of pim-statistics with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsPimStatistics(self, device_name), locals())

	def update(self):
		"""Update this statistics instance on the server with any changed property values.

		The statistics pull model

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def first_activity_timestamp(self):
		"""str: Timestamp of the first request to this session."""
		return self._get_value('first-activity-timestamp')

	@property
	def last_activity_timestamp(self):
		"""str: Timestamp of the last request to this session"""
		return self._get_value('last-activity-timestamp')

	def clear(self):
		"""Execute the clear action on the server

		Clear all statistic counters.

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._execute(self.url + '/clear')


class SessionsStatisticsPhysicalPort(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'physical-port'
		self.YANG_KEY = 'port-name'
		super(SessionsStatisticsPhysicalPort, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this physical-port instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def connected_test_port_id(self):
		"""str: The id of the connected test port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('connected-test-port-id')

	@property
	def connected_test_port_description(self):
		"""str: Free form vendor specific description of the connected test port. Can include details such as make/model/productId of the underlying hardware. Empty if the abstract port is not connected to a test port."""
		return self._get_value('connected-test-port-description')

	@property
	def connection_state(self):
		""":str:`enum`: The state of the connection to the physical hardware test port or virtual machine test port  
		Enums:  
			CONNECTING: TBD  
			CONNECTED_LINK_UP: TBD  
			CONNECTED_LINK_DOWN: TBD  
			DISCONNECTING: TBD  
			DISCONNECTED: TBD  
		"""
		return self._get_value('connection-state')

	@property
	def connection_state_details(self):
		"""str: Free form vendor specific information about the state of the connection to the physical hardware test port or virtual machine test port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('connection-state-details')

	@property
	def speed(self):
		"""str: The actual speed of the test port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('speed')


class SessionsStatisticsPort(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'port'
		self.YANG_KEY = 'name'
		super(SessionsStatisticsPort, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this port instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""str: An abstract test port name"""
		return self._get_value('name')

	@property
	def tx_duration(self):
		"""decimal64: Generator on time in seconds."""
		return self._get_value('tx-duration')

	@property
	def tx_frames(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frames')

	@property
	def rx_frames(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frames')

	@property
	def tx_frame_rate(self):
		"""uint64: Total number of frames transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frame-rate')

	@property
	def rx_frame_rate(self):
		"""uint64: Total number of frames received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frame-rate')

	@property
	def dropped_frames(self):
		"""uint64: Total Number of dropped frames during transit. Empty if the abstract port is not connected to a test port."""
		return self._get_value('dropped-frames')

	@property
	def tx_bit_count(self):
		"""uint64: The total number of bits transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-count')

	@property
	def rx_bit_count(self):
		"""uint64: The total number of bits received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-count')

	@property
	def tx_bit_rate(self):
		"""uint64: Total number of bits transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-rate')

	@property
	def rx_bit_rate(self):
		"""uint64: Total number of bits received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-rate')

	@property
	def tx_byte_count(self):
		"""uint64: The total number of bytes transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-count')

	@property
	def rx_byte_count(self):
		"""uint64: The total number of bytes received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-count')

	@property
	def tx_byte_rate(self):
		"""uint64: Total number of bytes transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-rate')

	@property
	def rx_byte_rate(self):
		"""uint64: Total number of bytes received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-rate')

	@property
	def tx_counter_timestamp(self):
		"""uint64: Time when the counter was stored. This value is derived from the TestCenter chassis time sync source. The unit is 10 nanoseconds."""
		return self._get_value('tx-counter-timestamp')

	@property
	def abort_frame_count(self):
		"""uint64: Number of abort frames generated."""
		return self._get_value('abort-frame-count')

	@property
	def abort_frame_rate(self):
		"""uint64: Number of abort frames generated over the last 1-second interval."""
		return self._get_value('abort-frame-rate')

	@property
	def crc_error_frame_count(self):
		"""uint64: Number of CRC error frames generated."""
		return self._get_value('crc-error-frame-count')

	@property
	def crc_error_frame_rate(self):
		"""uint64: Number of CRC error frames generated over the last 1-second interval."""
		return self._get_value('crc-error-frame-rate')

	@property
	def ipv4_tx_frame_count(self):
		"""uint64: Number of IPv4 frames generated."""
		return self._get_value('ipv4-tx-frame-count')

	@property
	def ipv4_tx_frame_rate(self):
		"""uint64: Number of IPv4 frames generated over the last 1-second interval."""
		return self._get_value('ipv4-tx-frame-rate')

	@property
	def ipv4_rx_frame_count(self):
		"""uint64: Number of IPv4 frames received."""
		return self._get_value('ipv4-rx-frame-count')

	@property
	def ipv4_rx_frame_rate(self):
		"""uint64: Number of IPv4 frames received over the last 1-second interval."""
		return self._get_value('ipv4-rx-frame-rate')

	@property
	def ipv4_tx_total_frame_count(self):
		"""uint64: Total number of IPv4 frames transmitted."""
		return self._get_value('ipv4-tx-total-frame-count')

	@property
	def ipv4_tx_total_frame_rate(self):
		"""uint64: Total number of IPv4 frames transmitted over the last 1-second interval."""
		return self._get_value('ipv4-tx-total-frame-rate')

	@property
	def ipv6_tx_frame_count(self):
		"""uint64: Number of IPv6 frames generated."""
		return self._get_value('ipv6-tx-frame-count')

	@property
	def ipv6_tx_frame_rate(self):
		"""uint64: Number of IPv6 frames generated over the last 1-second interval."""
		return self._get_value('ipv6-tx-frame-rate')

	@property
	def ipv6_rx_frame_count(self):
		"""uint64: Number of IPv6 frames received."""
		return self._get_value('ipv6-rx-frame-count')

	@property
	def ipv6_rx_frame_rate(self):
		"""uint64: Number of IPv6 frames received over the last 1-second interval."""
		return self._get_value('ipv6-rx-frame-rate')

	@property
	def ipv6_tx_total_frame_count(self):
		"""uint64: Total number of IPv6 frames transmitted."""
		return self._get_value('ipv6-tx-total-frame-count')

	@property
	def ipv6_tx_total_frame_rate(self):
		"""uint64: Total number of IPv6 frames transmitted over the last 1-second interval."""
		return self._get_value('ipv6-tx-total-frame-rate')

	@property
	def jumbo_tx_frame_count(self):
		"""uint64: Number of jumbo frames generated."""
		return self._get_value('jumbo-tx-frame-count')

	@property
	def jumbo_tx_frame_rate(self):
		"""uint64: Number of jumbo frames generated over the last 1-second interval."""
		return self._get_value('jumbo-tx-frame-rate')

	@property
	def jumbo_rx_frame_count(self):
		"""uint64: Number of jumbo frames received."""
		return self._get_value('jumbo-rx-frame-count')

	@property
	def jumbo_rx_frame_rate(self):
		"""uint64: Number of jumbo frames received over the last 1-second interval."""
		return self._get_value('jumbo-rx-frame-rate')

	@property
	def l3_checksum_error_count(self):
		"""uint64: Number of frames generated with an L3 checksum error."""
		return self._get_value('l3-checksum-error-count')

	@property
	def l3_checksum_error_rate(self):
		"""uint64: Number of frames generated with an L3 header checksum, per second."""
		return self._get_value('l3-checksum-error-rate')

	@property
	def l4_checksum_error_count(self):
		"""uint64: Number of frames generated with an L4 checksum error."""
		return self._get_value('l4-checksum-error-count')

	@property
	def l4_checksum_error_rate(self):
		"""uint64: Number of frames generated with an L4 header checksum, per second."""
		return self._get_value('l4-checksum-error-rate')

	@property
	def mpls_tx_frame_count(self):
		"""uint64: Number of mpls frames generated."""
		return self._get_value('mpls-tx-frame-count')

	@property
	def mpls_tx_frame_rate(self):
		"""uint64: Number of mpls frames generated over the last 1-second interval."""
		return self._get_value('mpls-tx-frame-rate')

	@property
	def mpls_rx_frame_count(self):
		"""uint64: Number of mpls frames received."""
		return self._get_value('mpls-rx-frame-count')

	@property
	def mpls_rx_frame_rate(self):
		"""uint64: Number of mpls frames received over the last 1-second interval."""
		return self._get_value('mpls-rx-frame-rate')

	@property
	def mpls_tx_total_frame_count(self):
		"""uint64: Total number of MPLS frames transmitted."""
		return self._get_value('mpls-tx-total-frame-count')

	@property
	def mpls_tx_total_frame_rate(self):
		"""uint64: Total number of MPLS frames transmitted over the last 1-second interval."""
		return self._get_value('mpls-tx-total-frame-rate')

	@property
	def octet_tx_frame_count(self):
		"""uint64: Number of octet frames generated."""
		return self._get_value('octet-tx-frame-count')

	@property
	def octet_tx_frame_rate(self):
		"""uint64: Number of octet frames generated over the last 1-second interval."""
		return self._get_value('octet-tx-frame-rate')

	@property
	def octet_rx_frame_count(self):
		"""uint64: Number of octet frames received."""
		return self._get_value('octet-rx-frame-count')

	@property
	def octet_rx_frame_rate(self):
		"""uint64: Number of octet frames received over the last 1-second interval."""
		return self._get_value('octet-rx-frame-rate')

	@property
	def octet_tx_total_frame_count(self):
		"""uint64: Total number of octet frames transmitted."""
		return self._get_value('octet-tx-total-frame-count')

	@property
	def octet_tx_total_frame_rate(self):
		"""uint64: Total number of octet frames transmitted over the last 1-second interval."""
		return self._get_value('octet-tx-total-frame-rate')

	@property
	def cell_tx_total_frame_count(self):
		"""uint64: Count of total cells generated."""
		return self._get_value('cell-tx-total-frame-count')

	@property
	def cell_tx_total_frame_rate(self):
		"""uint64: Total number of bytes generated over last 1-second interval."""
		return self._get_value('cell-tx-total-frame-rate')

	@property
	def oversize_tx_frame_count(self):
		"""uint64: Number of oversize frames generated."""
		return self._get_value('oversize-tx-frame-count')

	@property
	def oversize_tx_frame_rate(self):
		"""uint64: Number of oversize frames generated over the last 1-second interval."""
		return self._get_value('oversize-tx-frame-rate')

	@property
	def oversize_rx_frame_count(self):
		"""uint64: Number of oversize frames received."""
		return self._get_value('oversize-rx-frame-count')

	@property
	def oversize_rx_frame_rate(self):
		"""uint64: Number of oversize frames received over the last 1-second interval."""
		return self._get_value('oversize-rx-frame-rate')

	@property
	def undersize_tx_frame_count(self):
		"""uint64: Number of undersize frames generated."""
		return self._get_value('undersize-tx-frame-count')

	@property
	def undersize_tx_frame_rate(self):
		"""uint64: Number of undersize frames generated over the last 1-second interval."""
		return self._get_value('undersize-tx-frame-rate')

	@property
	def undersize_rx_frame_count(self):
		"""uint64: Number of undersize frames received."""
		return self._get_value('undersize-rx-frame-count')

	@property
	def undersize_rx_frame_rate(self):
		"""uint64: Number of undersize frames received over the last 1-second interval."""
		return self._get_value('undersize-rx-frame-rate')

	@property
	def sig_tx_frame_count(self):
		"""uint64: Number of Spirent Signature frames generated."""
		return self._get_value('sig-tx-frame-count')

	@property
	def sig_tx_frame_rate(self):
		"""uint64: Number of Spirent Signature  frames generated over the last 1-second interval."""
		return self._get_value('sig-tx-frame-rate')

	@property
	def sig_rx_frame_count(self):
		"""uint64: Number of Spirent Signature  frames received."""
		return self._get_value('sig-rx-frame-count')

	@property
	def sig_rx_frame_rate(self):
		"""uint64: Number of Spirent Signature  frames received over the last 1-second interval."""
		return self._get_value('sig-rx-frame-rate')

	@property
	def vlan_tx_frame_count(self):
		"""uint64: Number of VLAN frames generated."""
		return self._get_value('vlan-tx-frame-count')

	@property
	def vlan_tx_frame_rate(self):
		"""uint64: Number of VLAN frames generated over the last 1-second interval."""
		return self._get_value('vlan-tx-frame-rate')

	@property
	def vlan_rx_frame_count(self):
		"""uint64: Number of VLAN frames received."""
		return self._get_value('vlan-rx-frame-count')

	@property
	def vlan_rx_frame_rate(self):
		"""uint64: Number of VLAN frames received over the last 1-second interval."""
		return self._get_value('vlan-rx-frame-rate')

	@property
	def hw_tx_frame_count(self):
		"""uint64: Number of Hardware frames generated."""
		return self._get_value('hw-tx-frame-count')

	@property
	def hw_rx_frame_count(self):
		"""uint64: Number of Hardware frames received."""
		return self._get_value('hw-rx-frame-count')

	@property
	def l1_tx_bit_count(self):
		"""uint64: Count of total layer1 bits generated."""
		return self._get_value('l1-tx-bit-count')

	@property
	def l1_tx_bit_rate(self):
		"""uint64: Total number of layer1 bits transmitted over last 1-second interval."""
		return self._get_value('l1-tx-bit-rate')

	@property
	def l1_tx_bit_rate_percent(self):
		"""decimal64: Total number of layer 1 bits transmitted in percentage."""
		return self._get_value('l1-tx-bit-rate-percent')

	@property
	def l1_rx_bit_count(self):
		"""uint64: Count of total layer1 bits received."""
		return self._get_value('l1-rx-bit-count')

	@property
	def l1_rx_bit_rate(self):
		"""uint64: Total number of layer1 bits received over last 1-second interval."""
		return self._get_value('l1-rx-bit-rate')

	@property
	def l1_rx_bit_rate_percent(self):
		"""decimal64: Total number of layer 1 bits received in percentage."""
		return self._get_value('l1-rx-bit-rate-percent')

	@property
	def pfc_tx_frame_count(self):
		"""uint64: Number of pause frames transmitted."""
		return self._get_value('pfc-tx-frame-count')

	@property
	def pfc_rx_frame_count(self):
		"""uint64: Number of pause frames received."""
		return self._get_value('pfc-rx-frame-count')

	@property
	def pfc_priority0_tx_frame_count(self):
		"""uint64: Number of priority-0 pause frames generated."""
		return self._get_value('pfc-priority0-tx-frame-count')

	@property
	def pfc_priority1_tx_frame_count(self):
		"""uint64: Number of priority-1 pause frames generated."""
		return self._get_value('pfc-priority1-tx-frame-count')

	@property
	def pfc_priority2_tx_frame_count(self):
		"""uint64: Number of priority-2 pause frames generated."""
		return self._get_value('pfc-priority2-tx-frame-count')

	@property
	def pfc_priority3_tx_frame_count(self):
		"""uint64: Number of priority-3 pause frames generated."""
		return self._get_value('pfc-priority3-tx-frame-count')

	@property
	def pfc_priority4_tx_frame_count(self):
		"""uint64: Number of priority-4 pause frames generated."""
		return self._get_value('pfc-priority4-tx-frame-count')

	@property
	def pfc_priority5_tx_frame_count(self):
		"""uint64: Number of priority-5 pause frames generated."""
		return self._get_value('pfc-priority5-tx-frame-count')

	@property
	def pfc_priority6_tx_frame_count(self):
		"""uint64: Number of priority-6 pause frames generated."""
		return self._get_value('pfc-priority6-tx-frame-count')

	@property
	def pfc_priority7_tx_frame_count(self):
		"""uint64: Number of priority-7 pause frames generated."""
		return self._get_value('pfc-priority7-tx-frame-count')

	@property
	def pfc_priority0_rx_frame_count(self):
		"""uint64: Number of priority-0 pause frames received."""
		return self._get_value('pfc-priority0-rx-frame-count')

	@property
	def pfc_priority1_rx_frame_count(self):
		"""uint64: Number of priority-1 pause frames received."""
		return self._get_value('pfc-priority1-rx-frame-count')

	@property
	def pfc_priority2_rx_frame_count(self):
		"""uint64: Number of priority-2 pause frames received."""
		return self._get_value('pfc-priority2-rx-frame-count')

	@property
	def pfc_priority3_rx_frame_count(self):
		"""uint64: Number of priority-3 pause frames received."""
		return self._get_value('pfc-priority3-rx-frame-count')

	@property
	def pfc_priority4_rx_frame_count(self):
		"""uint64: Number of priority-4 pause frames received."""
		return self._get_value('pfc-priority4-rx-frame-count')

	@property
	def pfc_priority5_rx_frame_count(self):
		"""uint64: Number of priority-5 pause frames received."""
		return self._get_value('pfc-priority5-rx-frame-count')

	@property
	def pfc_priority6_rx_frame_count(self):
		"""uint64: Number of priority-6 pause frames received."""
		return self._get_value('pfc-priority6-rx-frame-count')

	@property
	def pfc_priority7_rx_frame_count(self):
		"""uint64: Number of priority-7 pause frames received."""
		return self._get_value('pfc-priority7-rx-frame-count')

	@property
	def combo_trigger_count(self):
		"""uint64: Number of frames captured by all the triggers."""
		return self._get_value('combo-trigger-count')

	@property
	def combo_trigger_rate(self):
		"""uint64: Number of frames received by all the triggers over the last 1-second interval."""
		return self._get_value('combo-trigger-rate')

	@property
	def corrected_baser_fec_error_count(self):
		"""uint64: Number of Corrected BaseR FEC Errors received."""
		return self._get_value('corrected-baser-fec-error-count')

	@property
	def corrected_rs_fec_error_count(self):
		"""uint64: Number of Corrected RS FEC Errors received."""
		return self._get_value('corrected-rs-fec-error-count')

	@property
	def corrected_rs_fec_symbols(self):
		"""uint64: Number of Corrected RS FEC Symbols received."""
		return self._get_value('corrected-rs-fec-symbols')

	@property
	def rx_counter_timestamp(self):
		"""uint64: Time when the counter was stored."""
		return self._get_value('rx-counter-timestamp')

	@property
	def duplicate_frame_count(self):
		"""uint64: Number of duplicate frames received."""
		return self._get_value('duplicate-frame-count')

	@property
	def fcoe_rx_frame_count(self):
		"""uint64: Fiber channel over Ethernet frame count."""
		return self._get_value('fcoe-rx-frame-count')

	@property
	def fcoe_rx_frame_rate(self):
		"""uint64: Fiber channel over Ethernet frame rate."""
		return self._get_value('fcoe-rx-frame-rate')

	@property
	def fcs_rx_frame_count(self):
		"""uint64: Number of FCS error frames received."""
		return self._get_value('fcs-rx-frame-count')

	@property
	def fcs_rx_frame_rate(self):
		"""uint64: Number of FCS error frames received over the last 1-second interval."""
		return self._get_value('fcs-rx-frame-rate')

	@property
	def first_arrival_timestamp(self):
		"""decimal64: Backplane first arrival timestamp received."""
		return self._get_value('first-arrival-timestamp')

	@property
	def last_arrival_timestamp(self):
		"""decimal64: Backplane last arrival timestamp received."""
		return self._get_value('last-arrival-timestamp')

	@property
	def late_frame_count(self):
		"""uint64: Number of late frames received."""
		return self._get_value('late-frame-count')

	@property
	def max_frame_length(self):
		"""uint64: Maximum frame length received (in bytes)."""
		return self._get_value('max-frame-length')

	@property
	def min_frame_length(self):
		"""uint64: Minimum frame length received (in bytes)."""
		return self._get_value('min-frame-length')

	@property
	def icmp_rx_frame_count(self):
		"""uint64: Number of ICMP frames received."""
		return self._get_value('icmp-rx-frame-count')

	@property
	def icmp_rx_frame_rate(self):
		"""uint64: Number of ICMP frames received over the last 1-second interval."""
		return self._get_value('icmp-rx-frame-rate')

	@property
	def ipv4_checksum_error_rx_frame_count(self):
		"""uint64: Number of IPv4 checksum errors received."""
		return self._get_value('ipv4-checksum-error-rx-frame-count')

	@property
	def ipv4_checksum_error_rx_frame_rate(self):
		"""uint64: Number of IPv4 checksum errors received over the last 1-second interval."""
		return self._get_value('ipv4-checksum-error-rx-frame-rate')

	@property
	def in_sequence_rx_frame_count(self):
		"""uint64: Number of frames received in sequence order."""
		return self._get_value('in-sequence-rx-frame-count')

	@property
	def out_sequence_rx_frame_count(self):
		"""uint64: Number of frames received out of sequence order."""
		return self._get_value('out-sequence-rx-frame-count')

	@property
	def ipv6_over_ipv4_rx_frame_count(self):
		"""uint64: Number of IPv6 over IPv4 frames received."""
		return self._get_value('ipv6-over-ipv4-rx-frame-count')

	@property
	def ipv6_over_ipv4_rx_frame_rate(self):
		"""uint64: Number of IPv6 over IPv4 frames received over the last 1-second interval."""
		return self._get_value('ipv6-over-ipv4-rx-frame-rate')

	@property
	def pause_rx_frame_count(self):
		"""uint64: Number of pause frames received."""
		return self._get_value('pause-rx-frame-count')

	@property
	def pause_rx_frame_rate(self):
		"""uint64: Number of pause frames received over the last 1-second interval."""
		return self._get_value('pause-rx-frame-rate')


class SessionsStatisticsPortTraffic(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'port-traffic'
		self.YANG_KEY = 'name'
		super(SessionsStatisticsPortTraffic, self).__init__(parent, yang_key_value)

	def ports(self, name=None):
		"""Get the ports object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ports list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsPortTrafficPorts` | :obj:`SessionsStatisticsPortTrafficPorts`: If arg name is None a list of SessionsStatisticsPortTrafficPorts objects otherwise a single SessionsStatisticsPortTrafficPorts object.  

		Raises:  
			NotFoundError: name is not in the list of ports objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsPortTrafficPorts(self, name))

	def create_ports(self, name):
		"""Create a child sibling ports instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsPortTrafficPorts`: An object encapsulating an instance of the ports model.  

		Raises:  
			AlreadyExistsError: An instance of ports with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsPortTrafficPorts(self, name), locals())

	def update(self):
		"""Update this port-traffic instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""str: Port Traffic name"""
		return self._get_value('name')

	@property
	def tx_frames(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frames')

	@property
	def rx_frames(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frames')

	@property
	def tx_frame_rate(self):
		"""uint64: Total number of frames transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frame-rate')

	@property
	def rx_frame_rate(self):
		"""uint64: Total number of frames received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frame-rate')

	@property
	def dropped_frames(self):
		"""uint64: Total Number of dropped frames during transit. Empty if the abstract port is not connected to a test port."""
		return self._get_value('dropped-frames')

	@property
	def tx_bit_count(self):
		"""uint64: The total number of bits transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-count')

	@property
	def rx_bit_count(self):
		"""uint64: The total number of bits received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-count')

	@property
	def tx_bit_rate(self):
		"""uint64: Total number of bits transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-rate')

	@property
	def rx_bit_rate(self):
		"""uint64: Total number of bits received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-rate')

	@property
	def tx_byte_count(self):
		"""uint64: The total number of bytes transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-count')

	@property
	def rx_byte_count(self):
		"""uint64: The total number of bytes received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-count')

	@property
	def tx_byte_rate(self):
		"""uint64: Total number of bytes transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-rate')

	@property
	def rx_byte_rate(self):
		"""uint64: Total number of bytes received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-rate')


class SessionsStatisticsPortTrafficPorts(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ports'
		self.YANG_KEY = 'name'
		super(SessionsStatisticsPortTrafficPorts, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this ports instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""str: Port Streams name"""
		return self._get_value('name')

	@property
	def tx_frames(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frames')

	@property
	def rx_frames(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frames')

	@property
	def tx_frame_rate(self):
		"""uint64: Total number of frames transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frame-rate')

	@property
	def rx_frame_rate(self):
		"""uint64: Total number of frames received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frame-rate')

	@property
	def dropped_frames(self):
		"""uint64: Total Number of dropped frames during transit. Empty if the abstract port is not connected to a test port."""
		return self._get_value('dropped-frames')

	@property
	def tx_bit_count(self):
		"""uint64: The total number of bits transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-count')

	@property
	def rx_bit_count(self):
		"""uint64: The total number of bits received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-count')

	@property
	def tx_bit_rate(self):
		"""uint64: Total number of bits transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-rate')

	@property
	def rx_bit_rate(self):
		"""uint64: Total number of bits received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-rate')

	@property
	def tx_byte_count(self):
		"""uint64: The total number of bytes transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-count')

	@property
	def rx_byte_count(self):
		"""uint64: The total number of bytes received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-count')

	@property
	def tx_byte_rate(self):
		"""uint64: Total number of bytes transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-rate')

	@property
	def rx_byte_rate(self):
		"""uint64: Total number of bytes received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-rate')


class SessionsStatisticsDeviceTraffic(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'device-traffic'
		self.YANG_KEY = 'name'
		super(SessionsStatisticsDeviceTraffic, self).__init__(parent, yang_key_value)

	def ports(self, name=None):
		"""Get the ports object(s) from the server.

		TBD

		Args:  
			name (:obj:`str`, optional, default=None): A key value in the ports list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsDeviceTrafficPorts` | :obj:`SessionsStatisticsDeviceTrafficPorts`: If arg name is None a list of SessionsStatisticsDeviceTrafficPorts objects otherwise a single SessionsStatisticsDeviceTrafficPorts object.  

		Raises:  
			NotFoundError: name is not in the list of ports objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsDeviceTrafficPorts(self, name))

	def create_ports(self, name):
		"""Create a child sibling ports instance on the server.

		TBD

		Args:  
			name (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsDeviceTrafficPorts`: An object encapsulating an instance of the ports model.  

		Raises:  
			AlreadyExistsError: An instance of ports with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsDeviceTrafficPorts(self, name), locals())

	def update(self):
		"""Update this device-traffic instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: Device Traffic name"""
		return self._get_value('name')

	@property
	def tx_frames(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frames')

	@property
	def rx_frames(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frames')

	@property
	def tx_frame_rate(self):
		"""uint64: Total number of frames transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frame-rate')

	@property
	def rx_frame_rate(self):
		"""uint64: Total number of frames received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frame-rate')

	@property
	def dropped_frames(self):
		"""uint64: Total Number of dropped frames during transit. Empty if the abstract port is not connected to a test port."""
		return self._get_value('dropped-frames')

	@property
	def tx_bit_count(self):
		"""uint64: The total number of bits transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-count')

	@property
	def rx_bit_count(self):
		"""uint64: The total number of bits received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-count')

	@property
	def tx_bit_rate(self):
		"""uint64: Total number of bits transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-rate')

	@property
	def rx_bit_rate(self):
		"""uint64: Total number of bits received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-rate')

	@property
	def tx_byte_count(self):
		"""uint64: The total number of bytes transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-count')

	@property
	def rx_byte_count(self):
		"""uint64: The total number of bytes received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-count')

	@property
	def tx_byte_rate(self):
		"""uint64: Total number of bytes transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-rate')

	@property
	def rx_byte_rate(self):
		"""uint64: Total number of bytes received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-rate')


class SessionsStatisticsDeviceTrafficPorts(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ports'
		self.YANG_KEY = 'name'
		super(SessionsStatisticsDeviceTrafficPorts, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this ports instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def name(self):
		"""t:config-list-key: Port Streams name"""
		return self._get_value('name')

	@property
	def tx_frames(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frames')

	@property
	def rx_frames(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frames')

	@property
	def tx_frame_rate(self):
		"""uint64: Total number of frames transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-frame-rate')

	@property
	def rx_frame_rate(self):
		"""uint64: Total number of frames received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-frame-rate')

	@property
	def dropped_frames(self):
		"""uint64: Total Number of dropped frames during transit. Empty if the abstract port is not connected to a test port."""
		return self._get_value('dropped-frames')

	@property
	def tx_bit_count(self):
		"""uint64: The total number of bits transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-count')

	@property
	def rx_bit_count(self):
		"""uint64: The total number of bits received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-count')

	@property
	def tx_bit_rate(self):
		"""uint64: Total number of bits transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-bit-rate')

	@property
	def rx_bit_rate(self):
		"""uint64: Total number of bits received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-bit-rate')

	@property
	def tx_byte_count(self):
		"""uint64: The total number of bytes transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-count')

	@property
	def rx_byte_count(self):
		"""uint64: The total number of bytes received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-count')

	@property
	def tx_byte_rate(self):
		"""uint64: Total number of bytes transmitted over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-byte-rate')

	@property
	def rx_byte_rate(self):
		"""uint64: Total number of bytes received over the last 1-second interval. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-byte-rate')


class SessionsStatisticsBgpv4Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv4-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsBgpv4Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this bgpv4-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Abstract emulated bgpv4 device name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def router_state(self):
		""":str:`enum`: The state of the connection to the physical hardware test port or virtual machine test port  NONE	No state.  IDLE	Prior to invoking Open_BgpSession or Start_Router, an emulated router is in the Idle state.  CONNECT	Connecting.  ACTIVE	Active.  OPEN_SENT	Open is sent.  OPEN_CONFIRM	Open is confirmed.  ESTABLISHED	Session is confirmed by the peer. The router state is Established.  
		Enums:  
			IDLE: TBD  
			CONNECTING: TBD  
			ESTABLISHED: TBD  
			NONE: TBD  
			CONNECT: TBD  
			OPEN_SENT: TBD  
			OPEN_CONFIRM: TBD  
			ACTIVE: TBD  
		"""
		return self._get_value('router-state')

	@property
	def tx_advertise_route_count(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-advertise-route-count')

	@property
	def rx_advertise_route_count(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-advertise-route-count')

	@property
	def tx_withdraw_route_count(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-withdraw-route-count')

	@property
	def rx_withdraw_route_count(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-withdraw-route-count')

	@property
	def last_rx_update_route_count(self):
		"""uint64: Number of routes in the last-received UPDATE message."""
		return self._get_value('last-rx-update-route-count')

	@property
	def outstanding_route_count(self):
		"""uint64: Number of routes that should be in the DUT'scurrent route table."""
		return self._get_value('outstanding-route-count')

	@property
	def rx_advertise_update_count(self):
		"""uint64: Received update packet count. Number of Update packets received fromDUT."""
		return self._get_value('rx-advertise-update-count')

	@property
	def rx_keepalive_count(self):
		"""uint64: BGP KeepAlive count received. Total number of KeepAlive packets receivedfrom the DUT during the test. """
		return self._get_value('rx-keepalive-count')

	@property
	def rx_notification_count(self):
		"""uint64: BGP Notification count received. Number of Notification packets received by the emulated router during the test."""
		return self._get_value('rx-notification-count')

	@property
	def rx_notify_code(self):
		"""uint64: BGP Notify code received. The last NOTIFICATION code the emulated routerreceived from the DUT."""
		return self._get_value('rx-notify-code')

	@property
	def rx_notify_sub_code(self):
		"""uint64: BGP Notify subcode received. Each NOTIFICATION code has a sub-code. """
		return self._get_value('rx-notify-sub-code')

	@property
	def rx_open_count(self):
		"""uint64: BGP Open message count received. Opens received from DUT. In stable BGP scenarios, this should match the value in the previous field. """
		return self._get_value('rx-open-count')

	@property
	def rx_route_refresh_count(self):
		"""uint64: Number of advertised route refresh message received."""
		return self._get_value('rx-route-refresh-count')

	@property
	def rx_rt_constraint_count(self):
		"""uint64: Number of RT-Constraint routes received for this router."""
		return self._get_value('rx-rt-constraint-count')

	@property
	def session_up_count(self):
		"""uint64: Number of router sessions within the router block in the Establishedstate."""
		return self._get_value('session-up-count')

	@property
	def tx_advertise_update_count(self):
		"""uint64: Advertised update route count transmitted. Total number of UPDATE packetswith feasible routes sent to the DUT. """
		return self._get_value('tx-advertise-update-count')

	@property
	def tx_keepalive_count(self):
		"""uint64: BGP KeepAlive count transmitted. Total number of KEEPALIVE packets sent to the DUT during test. """
		return self._get_value('tx-keepalive-count')

	@property
	def tx_notification_count(self):
		"""uint64: BGP Notification count transmitted. Number of Notification packets sent from the emulated router. """
		return self._get_value('tx-notification-count')

	@property
	def tx_notify_code(self):
		"""uint64: BGP Notify code transmitted. Last Notification code the emulated router sent to the DUT."""
		return self._get_value('tx-notify-code')

	@property
	def tx_notify_sub_code(self):
		"""uint64: BGP Notify subcode received. Each NOTIFICATION code has a sub-code. """
		return self._get_value('tx-notify-sub-code')

	@property
	def tx_open_count(self):
		"""uint64: BGP Open message count transmitted. Total number of OPEN packets sent tothe DUT. Initial one, plus number of times the emulated router re-establishessessions. """
		return self._get_value('tx-open-count')

	@property
	def tx_route_refresh_count(self):
		"""uint64:  Number of advertised route refresh message transmitted."""
		return self._get_value('tx-route-refresh-count')

	@property
	def tx_rt_constraint_count(self):
		"""uint64: Number of RT-Constraint routes transmitted for this router."""
		return self._get_value('tx-rt-constraint-count')

	@property
	def tx_withdrawn_update_count(self):
		"""uint64: Withdrawn update route count transmitted. Total number of UPDATE packetswith unfeasible routes sent to the DUT (route flapping).  """
		return self._get_value('tx-withdrawn-update-count')


class SessionsStatisticsBgpv6Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bgpv6-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsBgpv6Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this bgpv6-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Abstract emulated bgpv6 device name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def router_state(self):
		""":str:`enum`: The state of the connection to the physical hardware test port or virtual machine test port  NONE	No state.  IDLE	Prior to invoking Open_BgpSession or Start_Router, an emulated router is in the Idle state.  CONNECT	Connecting.  ACTIVE	Active.  OPEN_SENT	Open is sent.  OPEN_CONFIRM	Open is confirmed.  ESTABLISHED	Session is confirmed by the peer. The router state is Established.  
		Enums:  
			IDLE: TBD  
			CONNECTING: TBD  
			ESTABLISHED: TBD  
			NONE: TBD  
			CONNECT: TBD  
			OPEN_SENT: TBD  
			OPEN_CONFIRM: TBD  
			ACTIVE: TBD  
		"""
		return self._get_value('router-state')

	@property
	def tx_advertise_route_count(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-advertise-route-count')

	@property
	def rx_advertise_route_count(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-advertise-route-count')

	@property
	def tx_withdraw_route_count(self):
		"""uint64: The total number of frames transmitted on the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('tx-withdraw-route-count')

	@property
	def rx_withdraw_route_count(self):
		"""uint64: The total number of frames received on the the port. Empty if the abstract port is not connected to a test port."""
		return self._get_value('rx-withdraw-route-count')

	@property
	def last_rx_update_route_count(self):
		"""uint64: Number of routes in the last-received UPDATE message."""
		return self._get_value('last-rx-update-route-count')

	@property
	def outstanding_route_count(self):
		"""uint64: Number of routes that should be in the DUT'scurrent route table."""
		return self._get_value('outstanding-route-count')

	@property
	def rx_advertise_update_count(self):
		"""uint64: Received update packet count. Number of Update packets received fromDUT."""
		return self._get_value('rx-advertise-update-count')

	@property
	def rx_keepalive_count(self):
		"""uint64: BGP KeepAlive count received. Total number of KeepAlive packets receivedfrom the DUT during the test. """
		return self._get_value('rx-keepalive-count')

	@property
	def rx_notification_count(self):
		"""uint64: BGP Notification count received. Number of Notification packets received by the emulated router during the test."""
		return self._get_value('rx-notification-count')

	@property
	def rx_notify_code(self):
		"""uint64: BGP Notify code received. The last NOTIFICATION code the emulated routerreceived from the DUT."""
		return self._get_value('rx-notify-code')

	@property
	def rx_notify_sub_code(self):
		"""uint64: BGP Notify subcode received. Each NOTIFICATION code has a sub-code. """
		return self._get_value('rx-notify-sub-code')

	@property
	def rx_open_count(self):
		"""uint64: BGP Open message count received. Opens received from DUT. In stable BGP scenarios, this should match the value in the previous field. """
		return self._get_value('rx-open-count')

	@property
	def rx_route_refresh_count(self):
		"""uint64: Number of advertised route refresh message received."""
		return self._get_value('rx-route-refresh-count')

	@property
	def rx_rt_constraint_count(self):
		"""uint64: Number of RT-Constraint routes received for this router."""
		return self._get_value('rx-rt-constraint-count')

	@property
	def session_up_count(self):
		"""uint64: Number of router sessions within the router block in the Establishedstate."""
		return self._get_value('session-up-count')

	@property
	def tx_advertise_update_count(self):
		"""uint64: Advertised update route count transmitted. Total number of UPDATE packetswith feasible routes sent to the DUT. """
		return self._get_value('tx-advertise-update-count')

	@property
	def tx_keepalive_count(self):
		"""uint64: BGP KeepAlive count transmitted. Total number of KEEPALIVE packets sent to the DUT during test. """
		return self._get_value('tx-keepalive-count')

	@property
	def tx_notification_count(self):
		"""uint64: BGP Notification count transmitted. Number of Notification packets sent from the emulated router. """
		return self._get_value('tx-notification-count')

	@property
	def tx_notify_code(self):
		"""uint64: BGP Notify code transmitted. Last Notification code the emulated router sent to the DUT."""
		return self._get_value('tx-notify-code')

	@property
	def tx_notify_sub_code(self):
		"""uint64: BGP Notify subcode received. Each NOTIFICATION code has a sub-code. """
		return self._get_value('tx-notify-sub-code')

	@property
	def tx_open_count(self):
		"""uint64: BGP Open message count transmitted. Total number of OPEN packets sent tothe DUT. Initial one, plus number of times the emulated router re-establishessessions. """
		return self._get_value('tx-open-count')

	@property
	def tx_route_refresh_count(self):
		"""uint64:  Number of advertised route refresh message transmitted."""
		return self._get_value('tx-route-refresh-count')

	@property
	def tx_rt_constraint_count(self):
		"""uint64: Number of RT-Constraint routes transmitted for this router."""
		return self._get_value('tx-rt-constraint-count')

	@property
	def tx_withdrawn_update_count(self):
		"""uint64: Withdrawn update route count transmitted. Total number of UPDATE packetswith unfeasible routes sent to the DUT (route flapping).  """
		return self._get_value('tx-withdrawn-update-count')


class SessionsStatisticsOspfv2Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv2-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsOspfv2Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this ospfv2-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: An abstract test port name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def router_state(self):
		""":str:`enum`: Reports the state of adjacency on the current port.  
		Enums:  
			DOWN: TBD  
			WAITING: TBD  
			DR: TBD  
			DR_OTHER: TBD  
			BACKUP: TBD  
		"""
		return self._get_value('router-state')

	@property
	def adjacency_status(self):
		""":str:`enum`: OSPFv2 Adjacency State.  DOWN    Initial state of a neighbor conversation. It indicates that therehas been no          recent information received from the neighbor.  ATTEMPT This state is only valid for neighbors attached to non-broadcastnetworks. It          indicates that no recent information has been received from theneighbor, but that          a more concerted effort should be made to contact the neighbor. This is done by          sending the neighbor Hello packets at intervals ofHelloInterval.   INIT    An Hello packet has recently been seen from the neighbor. However,bidirectional          communication has not yet been established with the neighbor(the router itself          did not appear in the neighbor's Hello packet). Allneighbors in this state (or higher)          are listed in the Hello packets sentfrom the associated interface.   TWOWAYS Communication between the two routers is bidirectional. This has been assured by the          operation of the Hello Protocol. This is the mostadvanced state short of beginning          adjacency establishment. The BackupDesignated Router (BDR) is selected from the set          of neighbors in the TWOWAYSstate or greater.   EXSTART This is the first step in creating an adjacency between the twoneighboring routers.          The goal of this step is to decide which router is the master, and to decide upon          the initial database description (DD) sequencenumber. Neighbor conversations in this          state or greater are calledadjacencies.   EXCHANGE In this state the router is describing its entire link statedatabase by sending          Database Description packets to the neighbor. EachDatabase Description Packet has          a DD sequence number, and is explicitlyacknowledged. Only one Database Description          Packet is allowed outstanding atany one time. In this state, Link State Request Packets          may also be sentasking for the neighbor's more recent advertisements. All adjacencies in          Exchange state or greater are used by the flooding procedure. In fact, these adjacencies          are fully capable of transmitting and receiving all types of OSPF routing protocol packets.          Loading Link State Request packets are sent to the neighbor asking for themore recent          advertisements that have been discovered (but not yet received)in the Exchange state.   FULL    Neighboring routers are fully adjacent. These adjacencies will nowappear in router          links and network links advertisements.   
		Enums:  
			DOWN: TBD  
			ATTEMPT: TBD  
			INIT: TBD  
			TWOWAYS: TBD  
			EXSTART: TBD  
			EXCHANGE: TBD  
			LOADING: TBD  
			FULL: TBD  
		"""
		return self._get_value('adjacency-status')

	@property
	def tx_hello_count(self):
		"""uint64: Number of Hello packets transmitted by the emulated router."""
		return self._get_value('tx-hello-count')

	@property
	def rx_hello_count(self):
		"""uint64: Number of Hello packets received by the emulated router."""
		return self._get_value('rx-hello-count')

	@property
	def rx_ack(self):
		"""uint64: Received Acks - Number of Link State Acknowledgment packets received by theemulated router."""
		return self._get_value('rx-ack')

	@property
	def rx_asbr_summary_lsa(self):
		"""uint64: Received ASBR-Summary-LSAs - Number of ASBR-Summary-LSAs received by theemulated router. """
		return self._get_value('rx-asbr-summary-lsa')

	@property
	def rx_as_external_lsa(self):
		"""uint64: Number of Extended Prefix LSAs received by the emulated router."""
		return self._get_value('rx-as-external-lsa')

	@property
	def rx_dd(self):
		"""uint64: Received DD - Number of Database Description packets (containing LSAheaders) received by the emulated router. """
		return self._get_value('rx-dd')

	@property
	def rx_extended_link_lsa(self):
		"""uint64: Number of Extended Link LSAs received by the emulated router."""
		return self._get_value('rx-extended-link-lsa')

	@property
	def rx_extended_prefix_lsa(self):
		"""uint64: Number of Extended Prefix LSAs received by the emulated router."""
		return self._get_value('rx-extended-prefix-lsa')

	@property
	def rx_network_lsa(self):
		"""uint64: Received Network-LSAs - Number of Network LSAs received by the emulatedrouter."""
		return self._get_value('rx-network-lsa')

	@property
	def rx_nssa_lsa(self):
		"""uint64: Received NSSA-LSAs - Number of NSSA LSAs received by the emulatedrouter."""
		return self._get_value('rx-nssa-lsa')

	@property
	def rx_request(self):
		"""uint64: Received Requests - Number of LS Request packets received by the emulatedrouter."""
		return self._get_value('rx-request')

	@property
	def rx_router_info_lsa(self):
		"""uint64: Number of Router Info LSAs received by the emulated router."""
		return self._get_value('rx-router-info-lsa')

	@property
	def rx_router_lsa(self):
		"""uint64: Received Router-LSAs - Number of Router LSAs received by the emulatedrouter."""
		return self._get_value('rx-router-lsa')

	@property
	def rx_summary_lsa(self):
		"""uint64: Received Summary-LSAs - Number of Summary LSAs received by the emulatedrouter."""
		return self._get_value('rx-summary-lsa')

	@property
	def rx_te_lsa(self):
		"""uint64: Received TE-LSAs - Number of TE-LSAs received by the emulatedrouter."""
		return self._get_value('rx-te-lsa')

	@property
	def tx_ack(self):
		"""uint64: Sent Acks - Number of Link State Acknowledgment packets sent by the emulated router."""
		return self._get_value('tx-ack')

	@property
	def tx_asbr_summary_lsa(self):
		"""uint64: Sent ASBR-Summary-LSAs - Number of ASBR-Summary LSAs sent by the emulatedrouter."""
		return self._get_value('tx-asbr-summary-lsa')

	@property
	def tx_as_external_lsa(self):
		"""uint64: Sent External-LSAs - Number of External LSAs sent by the emulatedrouter."""
		return self._get_value('tx-as-external-lsa')

	@property
	def tx_dd(self):
		"""uint64: Sent DD - Number of Database Description packets sent by the emulatedrouter."""
		return self._get_value('tx-dd')

	@property
	def tx_extended_link_lsa(self):
		"""uint64: Number of Extended Link LSAs sent by the emulated router."""
		return self._get_value('tx-extended-link-lsa')

	@property
	def tx_extended_prefix_lsa(self):
		"""uint64: Number of Extended Prefix LSAs sent by the emulated router."""
		return self._get_value('tx-extended-prefix-lsa')

	@property
	def tx_network_lsa(self):
		"""uint64: Sent Network-LSAs - Number of Network LSAs sent by the emulatedrouter."""
		return self._get_value('tx-network-lsa')

	@property
	def tx_nssa_lsa(self):
		"""uint64: Sent NSSA-LSAs - Number of NSSA LSAs sent by the emulatedrouter."""
		return self._get_value('tx-nssa-lsa')

	@property
	def tx_request(self):
		"""uint64: Sent Requests - Number of LS Request packets sent by the emulatedrouter."""
		return self._get_value('tx-request')

	@property
	def tx_router_info_lsa(self):
		"""uint64: Number of Router Info LSAs sent by the emulated router."""
		return self._get_value('tx-router-info-lsa')

	@property
	def tx_router_lsa(self):
		"""uint64: Sent Router-LSAs - Number of Router LSAs sent by the emulatedrouter."""
		return self._get_value('tx-router-lsa')

	@property
	def tx_summary_lsa(self):
		"""uint64: Sent Summary-LSAs - Number of Summary LSAs sent by the emulatedrouter."""
		return self._get_value('tx-summary-lsa')

	@property
	def tx_te_lsa(self):
		"""uint64: Sent TE-LSAs - Number of TE-LSAs sent by the emulated router."""
		return self._get_value('tx-te-lsa')


class SessionsStatisticsOspfv3Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'ospfv3-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsOspfv3Statistics, self).__init__(parent, yang_key_value)

	def extended_lsa_counters(self):
		"""Get the extended-lsa-counters object from the server

		Extended LSA counters.

		Returns:  
			:obj:`SessionsStatisticsOspfv3StatisticsExtendedLsaCounters`: An object encapsulating an instance of the extended-lsa-counters model.  

		Raises:  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsOspfv3StatisticsExtendedLsaCounters(self))

	def update(self):
		"""Update this ospfv3-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: An abstract test port name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def router_state(self):
		""":str:`enum`: Reports the state of adjacency on the current port.  
		Enums:  
			DOWN: TBD  
			WAITING: TBD  
			DR: TBD  
			DR_OTHER: TBD  
			BACKUP: TBD  
		"""
		return self._get_value('router-state')

	@property
	def adjacency_status(self):
		""":str:`enum`: OSPFv2 Adjacency State.  DOWN    Initial state of a neighbor conversation. It indicates that therehas been no          recent information received from the neighbor.  ATTEMPT This state is only valid for neighbors attached to non-broadcastnetworks. It          indicates that no recent information has been received from theneighbor, but that          a more concerted effort should be made to contact the neighbor. This is done by          sending the neighbor Hello packets at intervals ofHelloInterval.   INIT    An Hello packet has recently been seen from the neighbor. However,bidirectional          communication has not yet been established with the neighbor(the router itself          did not appear in the neighbor's Hello packet). Allneighbors in this state (or higher)          are listed in the Hello packets sentfrom the associated interface.   TWOWAYS Communication between the two routers is bidirectional. This has been assured by the          operation of the Hello Protocol. This is the mostadvanced state short of beginning          adjacency establishment. The BackupDesignated Router (BDR) is selected from the set          of neighbors in the TWOWAYSstate or greater.   EXSTART This is the first step in creating an adjacency between the twoneighboring routers.          The goal of this step is to decide which router is the master, and to decide upon          the initial database description (DD) sequencenumber. Neighbor conversations in this          state or greater are calledadjacencies.   EXCHANGE In this state the router is describing its entire link statedatabase by sending          Database Description packets to the neighbor. EachDatabase Description Packet has          a DD sequence number, and is explicitlyacknowledged. Only one Database Description          Packet is allowed outstanding atany one time. In this state, Link State Request Packets          may also be sentasking for the neighbor's more recent advertisements. All adjacencies in          Exchange state or greater are used by the flooding procedure. In fact, these adjacencies          are fully capable of transmitting and receiving all types of OSPF routing protocol packets.          Loading Link State Request packets are sent to the neighbor asking for themore recent          advertisements that have been discovered (but not yet received)in the Exchange state.   FULL    Neighboring routers are fully adjacent. These adjacencies will nowappear in router          links and network links advertisements.   
		Enums:  
			DOWN: TBD  
			ATTEMPT: TBD  
			INIT: TBD  
			TWOWAYS: TBD  
			EXSTART: TBD  
			EXCHANGE: TBD  
			LOADING: TBD  
			FULL: TBD  
		"""
		return self._get_value('adjacency-status')

	@property
	def tx_hello_count(self):
		"""uint64: Number of Hello packets transmitted by the emulated router."""
		return self._get_value('tx-hello-count')

	@property
	def rx_hello_count(self):
		"""uint64: Number of Hello packets received by the emulated router."""
		return self._get_value('rx-hello-count')

	@property
	def rx_ack(self):
		"""uint64: Received acks. The number of Link State Acknowledgment packets received by the emulated router. """
		return self._get_value('rx-ack')

	@property
	def rx_dd(self):
		"""uint64: Received DD - The number of Database Description packets (containing LSAheaders) received by the emulated router."""
		return self._get_value('rx-dd')

	@property
	def rx_request(self):
		"""uint64: Received requests. The number of LS requests received by the emulatedrouter. """
		return self._get_value('rx-request')

	@property
	def rx_update(self):
		"""uint64: Rx update."""
		return self._get_value('rx-update')

	@property
	def rx_router_info_lsa(self):
		"""uint64: The number of Router Information LSAs received by the emulated router."""
		return self._get_value('rx-router-info-lsa')

	@property
	def tx_ack(self):
		"""uint64: Sent acks. The number of Link State Acknowledgment packets sent by theemulated router."""
		return self._get_value('tx-ack')

	@property
	def tx_dd(self):
		"""uint64: Sent DD - Number of Database Description packets sent by the emulatedrouter."""
		return self._get_value('tx-dd')

	@property
	def tx_request(self):
		"""uint64: Sent requests. The number of LS request packets sent by the emulatedrouter."""
		return self._get_value('tx-request')

	@property
	def tx_update(self):
		"""uint64: Tx update."""
		return self._get_value('tx-update')

	@property
	def tx_router_info_lsa(self):
		"""uint64: The number of Router Information LSAs sent by the emulated router."""
		return self._get_value('tx-router-info-lsa')

	@property
	def rx_as_external_lsa(self):
		"""uint64: Received external-LSAs. The number of external LSAs received by theemulated router."""
		return self._get_value('rx-as-external-lsa')

	@property
	def rx_inter_area_prefix_lsa(self):
		"""uint64: Received inter-area-prefix LSAs. The number of inter-area-prefix LSAsreceived by the emulated router."""
		return self._get_value('rx-inter-area-prefix-lsa')

	@property
	def rx_inter_area_router_lsa(self):
		"""uint64: Received inter-area-router LSAs. The number of inter-area-router LSAsreceived by the emulated router."""
		return self._get_value('rx-inter-area-router-lsa')

	@property
	def rx_intra_area_prefix_lsa(self):
		"""uint64: Received Intra-Area-Prefix-LSAs - Number of Intra-Area-Prefix LSAs receivedby the emulated router."""
		return self._get_value('rx-intra-area-prefix-lsa')

	@property
	def rx_link_lsa(self):
		"""uint64: Received link-LSAs. The number of link LSAs received by the emulatedrouter."""
		return self._get_value('rx-link-lsa')

	@property
	def rx_network_lsa(self):
		"""uint64: Received Network-LSAs - Number of Network LSAs received by the emulated router."""
		return self._get_value('rx-network-lsa')

	@property
	def rx_nssa_lsa(self):
		"""uint64: Received Link-LSAs. The number of Link LSAs received by the emulatedrouter."""
		return self._get_value('rx-nssa-lsa')

	@property
	def rx_router_lsa(self):
		"""uint64: Received Router-LSAs - Number of Router LSAs received by the emulated router."""
		return self._get_value('rx-router-lsa')

	@property
	def tx_as_external_lsa(self):
		"""uint64: Sent external-LSAs. The number of external LSAs sent by the emulatedrouter."""
		return self._get_value('tx-as-external-lsa')

	@property
	def tx_inter_area_prefix_lsa(self):
		"""uint64: Sent inter-area-prefix LSAs. The number of inter-area-prefix LSAs sent bythe emulated router."""
		return self._get_value('tx-inter-area-prefix-lsa')

	@property
	def tx_inter_area_router_lsa(self):
		"""uint64: Sent inter-area-router LSAs. The number of inter-area-router LSAs sent bythe emulated router."""
		return self._get_value('tx-inter-area-router-lsa')

	@property
	def tx_intra_area_prefix_lsa(self):
		"""uint64: Sent Intra-Area-Prefix-LSAs - Number of Intra-Area-Prefix LSAs sent by theemulated router."""
		return self._get_value('tx-intra-area-prefix-lsa')

	@property
	def tx_link_lsa(self):
		"""uint64: Sent link-LSAs. The number of link LSAs sent by the emulated router."""
		return self._get_value('tx-link-lsa')

	@property
	def tx_network_lsa(self):
		"""uint64: Sent Network-LSAs - Number of Network LSAs sent by the emulated router."""
		return self._get_value('tx-network-lsa')

	@property
	def tx_nssa_lsa(self):
		"""uint64: Sent NSSA-LSAs. The number of NSSA LSAs sent by the emulated router."""
		return self._get_value('tx-nssa-lsa')

	@property
	def tx_router_lsa(self):
		"""uint64: Sent Router-LSAs - Number of Router LSAs sent by the emulated router."""
		return self._get_value('tx-router-lsa')


class SessionsStatisticsOspfv3StatisticsExtendedLsaCounters(YangBase):
	"""Extended LSA counters.
	"""
	def __init__(self, parent):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'extended-lsa-counters'
		super(SessionsStatisticsOspfv3StatisticsExtendedLsaCounters, self).__init__(parent, None)

	def update(self):
		"""Update this extended-lsa-counters instance on the server with any changed property values.

		Extended LSA counters.

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def rx_as_external_lsa(self):
		"""uint64: Received external-LSAs. The number of external LSAs received by theemulated router."""
		return self._get_value('rx-as-external-lsa')

	@property
	def rx_inter_area_prefix_lsa(self):
		"""uint64: Received inter-area-prefix LSAs. The number of inter-area-prefix LSAsreceived by the emulated router."""
		return self._get_value('rx-inter-area-prefix-lsa')

	@property
	def rx_inter_area_router_lsa(self):
		"""uint64: Received inter-area-router LSAs. The number of inter-area-router LSAsreceived by the emulated router."""
		return self._get_value('rx-inter-area-router-lsa')

	@property
	def rx_intra_area_prefix_lsa(self):
		"""uint64: Received Intra-Area-Prefix-LSAs - Number of Intra-Area-Prefix LSAs receivedby the emulated router."""
		return self._get_value('rx-intra-area-prefix-lsa')

	@property
	def rx_link_lsa(self):
		"""uint64: Received link-LSAs. The number of link LSAs received by the emulatedrouter."""
		return self._get_value('rx-link-lsa')

	@property
	def rx_network_lsa(self):
		"""uint64: Received Network-LSAs - Number of Network LSAs received by the emulated router."""
		return self._get_value('rx-network-lsa')

	@property
	def rx_nssa_lsa(self):
		"""uint64: Received Link-LSAs. The number of Link LSAs received by the emulatedrouter."""
		return self._get_value('rx-nssa-lsa')

	@property
	def rx_router_lsa(self):
		"""uint64: Received Router-LSAs - Number of Router LSAs received by the emulated router."""
		return self._get_value('rx-router-lsa')

	@property
	def tx_as_external_lsa(self):
		"""uint64: Sent external-LSAs. The number of external LSAs sent by the emulatedrouter."""
		return self._get_value('tx-as-external-lsa')

	@property
	def tx_inter_area_prefix_lsa(self):
		"""uint64: Sent inter-area-prefix LSAs. The number of inter-area-prefix LSAs sent bythe emulated router."""
		return self._get_value('tx-inter-area-prefix-lsa')

	@property
	def tx_inter_area_router_lsa(self):
		"""uint64: Sent inter-area-router LSAs. The number of inter-area-router LSAs sent bythe emulated router."""
		return self._get_value('tx-inter-area-router-lsa')

	@property
	def tx_intra_area_prefix_lsa(self):
		"""uint64: Sent Intra-Area-Prefix-LSAs - Number of Intra-Area-Prefix LSAs sent by theemulated router."""
		return self._get_value('tx-intra-area-prefix-lsa')

	@property
	def tx_link_lsa(self):
		"""uint64: Sent link-LSAs. The number of link LSAs sent by the emulated router."""
		return self._get_value('tx-link-lsa')

	@property
	def tx_network_lsa(self):
		"""uint64: Sent Network-LSAs - Number of Network LSAs sent by the emulated router."""
		return self._get_value('tx-network-lsa')

	@property
	def tx_nssa_lsa(self):
		"""uint64: Sent NSSA-LSAs. The number of NSSA LSAs sent by the emulated router."""
		return self._get_value('tx-nssa-lsa')

	@property
	def tx_router_lsa(self):
		"""uint64: Sent Router-LSAs - Number of Router LSAs sent by the emulated router."""
		return self._get_value('tx-router-lsa')


class SessionsStatisticsIsisStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'isis-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsIsisStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this isis-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def rx_l1_lan_hello_count(self):
		"""uint64: Number of LAN Hello packets received by the emulated router."""
		return self._get_value('rx-l1-lan-hello-count')

	@property
	def tx_l1_lan_hello_count(self):
		"""uint64: Number of LAN Hello packets transmitted by the emulated router."""
		return self._get_value('tx-l1-lan-hello-count')

	@property
	def rx_l2_lan_hello_count(self):
		"""uint64: Number of LAN Hello packets received by the emulated router."""
		return self._get_value('rx-l2-lan-hello-count')

	@property
	def tx_l2_lan_hello_count(self):
		"""uint64: Number of LAN Hello packets transmitted by the emulated router."""
		return self._get_value('tx-l2-lan-hello-count')

	@property
	def tx_l1_csnp_count(self):
		"""uint64: Number of Tx CSNPs sent to the SUT."""
		return self._get_value('tx-l1-csnp-count')

	@property
	def tx_l1_lsp_count(self):
		"""uint64: Number of Tx LSPs sent to the SUT."""
		return self._get_value('tx-l1-lsp-count')

	@property
	def tx_l1_psnp_count(self):
		"""uint64: Number of Tx PSNPs sent to the SUT."""
		return self._get_value('tx-l1-psnp-count')

	@property
	def rx_l1_csnp_count(self):
		"""uint64: Number of Rx CSNPs received from the SUT."""
		return self._get_value('rx-l1-csnp-count')

	@property
	def rx_l1_lsp_count(self):
		"""uint64: Number of Rx LSPs received from the SUT."""
		return self._get_value('rx-l1-lsp-count')

	@property
	def rx_l1_psnp_count(self):
		"""uint64: Number of Rx PSNPs received from the SUT."""
		return self._get_value('rx-l1-psnp-count')

	@property
	def tx_l2_csnp_count(self):
		"""uint64: Number of Tx CSNPs sent to the SUT."""
		return self._get_value('tx-l2-csnp-count')

	@property
	def tx_l2_lsp_count(self):
		"""uint64: Number of Tx LSPs sent to the SUT."""
		return self._get_value('tx-l2-lsp-count')

	@property
	def tx_l2_psnp_count(self):
		"""uint64: Number of Tx PSNPs sent to the SUT."""
		return self._get_value('tx-l2-psnp-count')

	@property
	def rx_l2_csnp_count(self):
		"""uint64: Number of Rx CSNPs received from the SUT."""
		return self._get_value('rx-l2-csnp-count')

	@property
	def rx_l2_lsp_count(self):
		"""uint64: Number of Rx LSPs received from the SUT."""
		return self._get_value('rx-l2-lsp-count')

	@property
	def rx_l2_psnp_count(self):
		"""uint64: Number of Rx PSNPs received from the SUT."""
		return self._get_value('rx-l2-psnp-count')

	@property
	def router_state(self):
		""":str:`enum`: State of adjacency with the SUT  
		Enums:  
			IDLE: TBD  
			INIT: TBD  
			UP: TBD  
			GR: TBD  
			GRHELPER: TBD  
		"""
		return self._get_value('router-state')

	@property
	def neighbor_extended_circuit_id(self):
		"""str: Learned the extended circuit ID of the adjacent neighbor after a three-way Hello exchange."""
		return self._get_value('neighbor-extended-circuit-id')

	@property
	def rx_ptp_hello_count(self):
		"""uint64: Number of Rx point-to-point hellos received from the SUT."""
		return self._get_value('rx-ptp-hello-count')

	@property
	def tx_ptp_hello_count(self):
		"""uint64: Number of Tx point-to-point hellos sent to the SUT."""
		return self._get_value('tx-ptp-hello-count')

	@property
	def neighbor_system_id(self):
		"""str: Learned System ID of the adjacent neighbor after three-way Helloexchange."""
		return self._get_value('neighbor-system-id')

	@property
	def three_way_p2p_adjacency_state(self):
		""":str:`enum`: Adjacency state of three-way Hello in point-to-pointnetwork.  
		Enums:  
			UP: TBD  
			INIT: TBD  
			DOWN: TBD  
			NOT_STARTED: TBD  
			NA: TBD  
		"""
		return self._get_value('three-way-p2p-adjacency-state')

	@property
	def l1_broadcast_adjacency_state(self):
		""":str:`enum`: Adjacency state of broadcast router.  
		Enums:  
			NOT_STARTED: TBD  
			IDLE: TBD  
			INIT: TBD  
			DIS_OTHER: TBD  
			DIS: TBD  
			GR: TBD  
			GRHELPER: TBD  
			NA: TBD  
		"""
		return self._get_value('l1-broadcast-adjacency-state')

	@property
	def l2_broadcast_adjacency_state(self):
		""":str:`enum`: Adjacency state of broadcast router.  
		Enums:  
			NOT_STARTED: TBD  
			IDLE: TBD  
			INIT: TBD  
			DIS_OTHER: TBD  
			DIS: TBD  
			GR: TBD  
			GRHELPER: TBD  
			NA: TBD  
		"""
		return self._get_value('l2-broadcast-adjacency-state')


class SessionsStatisticsBfdv4Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bfdv4-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsBfdv4Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this bfdv4-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def tx_count(self):
		"""uint64: Number of BFD packets sent on this router."""
		return self._get_value('tx-count')

	@property
	def rx_count(self):
		"""uint64: Number of BFD packets received on this router."""
		return self._get_value('rx-count')


class SessionsStatisticsBfdv6Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'bfdv6-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsBfdv6Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this bfdv6-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def tx_count(self):
		"""uint64: Number of BFD packets sent on this router."""
		return self._get_value('tx-count')

	@property
	def rx_count(self):
		"""uint64: Number of BFD packets received on this router."""
		return self._get_value('rx-count')


class SessionsStatisticsDhcpv4Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsDhcpv4Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv4-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def attempt_rate(self):
		"""decimal64: Attempt rate in sessions per second."""
		return self._get_value('attempt-rate')

	@property
	def bind_rate(self):
		"""decimal64: Binding  rate in sessions per second."""
		return self._get_value('bind-rate')

	@property
	def current_attempt_count(self):
		"""uint32: Number of hosts in the block that are in the process of attempting to bind and are sending or receiving DHCP messages."""
		return self._get_value('current-attempt-count')

	@property
	def current_bound_count(self):
		"""uint32: Number of DHCP sessions that are currently bound. This statistic should match the DUT's Leased-Count statistic."""
		return self._get_value('current-bound-count')

	@property
	def current_idle_count(self):
		"""uint32: Number of DHCP sessions currently in the Idle state (sessions that have been programmed but are not being attempted)."""
		return self._get_value('current-idle-count')

	@property
	def dna_v4_failed_count(self):
		"""uint32: Number of DNAv4 requests failed."""
		return self._get_value('dna-v4-failed-count')

	@property
	def dna_v4_retry_count(self):
		"""uint32: Number of DNAv4 requests retried."""
		return self._get_value('dna-v4-retry-count')

	@property
	def elapsed_time(self):
		"""decimal64: Elapsed time."""
		return self._get_value('elapsed-time')

	@property
	def rx_ack_count(self):
		"""uint32: Number of DHCPACK messages received."""
		return self._get_value('rx-ack-count')

	@property
	def rx_dna_v4_reply_count(self):
		"""uint32: Number of DNAv4 replies received."""
		return self._get_value('rx-dna-v4-reply-count')

	@property
	def rx_force_renew_count(self):
		"""uint32: Number of DHCP Force Renews (unicast DHCPREQUEST messages) received."""
		return self._get_value('rx-force-renew-count')

	@property
	def rx_nak_count(self):
		"""uint32: Number of DHCPNAK messages received."""
		return self._get_value('rx-nak-count')

	@property
	def rx_offer_count(self):
		"""uint32: Number of DHCPOFFER messages received."""
		return self._get_value('rx-offer-count')

	@property
	def total_attempt_count(self):
		"""uint32: Total number of sessions attempted (both past and current)."""
		return self._get_value('total-attempt-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of bound sessions (both past and current)."""
		return self._get_value('total-bound-count')

	@property
	def total_failed_count(self):
		"""uint32: Total number of DHCP hosts in the block that have failed to bind (both past and current). Failures may be due to DUT overload, a T1 timer expiration, or reaching the number of retries for the session."""
		return self._get_value('total-failed-count')

	@property
	def total_rebooted_count(self):
		"""uint32: Number of DHCPv4 reboot commands issued."""
		return self._get_value('total-rebooted-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of renewed sessions."""
		return self._get_value('total-renewed-count')

	@property
	def total_retried_count(self):
		"""uint32: Total number of times DHCPDISCOVER and DHCPREQUEST messages have been retried."""
		return self._get_value('total-retried-count')

	@property
	def tx_discover_count(self):
		"""uint32: Number of DHCPDISCOVER messages sent."""
		return self._get_value('tx-discover-count')

	@property
	def tx_dna_v4_request_count(self):
		"""uint32: Number of DNAv4 requests sent."""
		return self._get_value('tx-dna-v4-request-count')

	@property
	def tx_rebind_count(self):
		"""uint32: Number of DHCP Rebinds (broadcast DHCPREQUEST messages)."""
		return self._get_value('tx-rebind-count')

	@property
	def tx_reboot_count(self):
		"""uint32: Total number of rebooted sessions."""
		return self._get_value('tx-reboot-count')

	@property
	def tx_release_count(self):
		"""uint32: Number of DHCPRELEASE messages sent."""
		return self._get_value('tx-release-count')

	@property
	def tx_renew_count(self):
		"""uint32: Number of DHCP Renews (unicast DHCPREQUEST messages)."""
		return self._get_value('tx-renew-count')

	@property
	def tx_request_count(self):
		"""uint32: Number of DHCPREQUEST messages sent."""
		return self._get_value('tx-request-count')


class SessionsStatisticsDhcpv4PortStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4-port-statistics'
		self.YANG_KEY = 'port-name'
		super(SessionsStatisticsDhcpv4PortStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv4-port-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def attempt_rate(self):
		"""decimal64: Rate of attempts (in sessions/second) for the binding of DHCP client leases for all sessions on the port."""
		return self._get_value('attempt-rate')

	@property
	def avg_setup_time(self):
		"""decimal64: Average setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('avg-setup-time')

	@property
	def max_setup_time(self):
		"""decimal64: Maximum setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('max-setup-time')

	@property
	def min_setup_time(self):
		"""decimal64: Minimum setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('min-setup-time')

	@property
	def bind_rate(self):
		"""decimal64: Rate of binding (in sessions/second) of DHCP client leases or lease renewals for all sessions on the port."""
		return self._get_value('bind-rate')

	@property
	def current_bound_count(self):
		"""uint32: Number of DHCP hosts on the port that are bound (have been assigned an IP address). This statistic corresponds to the DUT's Leased Count statistic."""
		return self._get_value('current-bound-count')

	@property
	def current_idle_count(self):
		"""uint32: Number of DHCP hosts on the port that are in the Idle state."""
		return self._get_value('current-idle-count')

	@property
	def rx_ack_count(self):
		"""uint32: Number of DHCPACK messages received."""
		return self._get_value('rx-ack-count')

	@property
	def rx_force_renew_count(self):
		"""uint32: Number of DHCP Force Renews (unicast DHCPREQUEST messages) received."""
		return self._get_value('rx-force-renew-count')

	@property
	def rx_nak_count(self):
		"""uint32: Number of DHCPNAK messages received."""
		return self._get_value('rx-nak-count')

	@property
	def rx_offer_count(self):
		"""uint32: Number of DHCPOFFER messages received."""
		return self._get_value('rx-offer-count')

	@property
	def success_percent(self):
		"""decimal64: Percentage of all sessions on the port that have been successfully bound and have obtained DHCP client leases or lease renewals."""
		return self._get_value('success-percent')

	@property
	def total_attempt_count(self):
		"""uint32: Total number of DHCP hosts on the port that have attempted to bind (both past and current)."""
		return self._get_value('total-attempt-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of DHCP hosts on the port that are bound (both past and current)."""
		return self._get_value('total-bound-count')

	@property
	def total_failed_count(self):
		"""uint32: Total number of DHCP hosts on the port that have failed to bind (both past and current). Failures may be due to DUT overload, a T1 timer expiration, or reaching the number of retries for the session."""
		return self._get_value('total-failed-count')

	@property
	def total_rebooted_count(self):
		"""uint32: Number of DHCPv4 reboot commands issued."""
		return self._get_value('total-rebooted-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of DHCP hosts on the port that renewed a lease."""
		return self._get_value('total-renewed-count')

	@property
	def total_retried_count(self):
		"""uint32: Total number of DHCP hosts on the port that have retried to bind (both past and current)."""
		return self._get_value('total-retried-count')

	@property
	def tx_discover_count(self):
		"""uint32: Number of DHCPDISCOVER messages sent."""
		return self._get_value('tx-discover-count')

	@property
	def tx_rebind_count(self):
		"""uint32: Number of DHCP Rebinds (broadcast DHCPREQUEST messages)."""
		return self._get_value('tx-rebind-count')

	@property
	def tx_reboot_count(self):
		"""uint32: Total number of rebooted sessions."""
		return self._get_value('tx-reboot-count')

	@property
	def tx_release_count(self):
		"""uint32: Number of DHCPRELEASE messages sent."""
		return self._get_value('tx-release-count')

	@property
	def tx_renew_count(self):
		"""uint32: Number of DHCP Renews (unicast DHCPREQUEST messages)."""
		return self._get_value('tx-renew-count')

	@property
	def tx_request_count(self):
		"""uint32: Number of DHCPREQUEST messages sent."""
		return self._get_value('tx-request-count')


class SessionsStatisticsDhcpv4ServerStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv4-server-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsDhcpv4ServerStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv4-server-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def current_bound_count(self):
		"""uint32: Number of sessions that are currently bound."""
		return self._get_value('current-bound-count')

	@property
	def rx_decline_count(self):
		"""uint32: Number of DHCPDECLINE messages received."""
		return self._get_value('rx-decline-count')

	@property
	def rx_discover_count(self):
		"""uint32: Number of DHCPDISCOVER messages received."""
		return self._get_value('rx-discover-count')

	@property
	def rx_inform_count(self):
		"""uint32: Number of DHCPINFORM messages received."""
		return self._get_value('rx-inform-count')

	@property
	def rx_release_count(self):
		"""uint32: Number of DHCPRELEASE messages received."""
		return self._get_value('rx-release-count')

	@property
	def rx_request_count(self):
		"""uint32: Number of DHCPRELEASE messages received."""
		return self._get_value('rx-request-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of bound sessions."""
		return self._get_value('total-bound-count')

	@property
	def total_expired_count(self):
		"""uint32: Total number of expired offers or sessions."""
		return self._get_value('total-expired-count')

	@property
	def total_released_count(self):
		"""uint32: Total number of sessions released by clients."""
		return self._get_value('total-released-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of renewed sessions."""
		return self._get_value('total-renewed-count')

	@property
	def tx_ack_count(self):
		"""uint32: Number of ACK messages sent."""
		return self._get_value('tx-ack-count')

	@property
	def tx_force_renew_count(self):
		"""uint32: Number of DHCP Force Renews (unicast DHCPREQUEST messages) transmitted."""
		return self._get_value('tx-force-renew-count')

	@property
	def tx_nak_count(self):
		"""uint32: Number of NAK messages sent."""
		return self._get_value('tx-nak-count')

	@property
	def tx_offer_count(self):
		"""uint32: Number of OFFER messages sent."""
		return self._get_value('tx-offer-count')


class SessionsStatisticsDhcpv6Statistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsDhcpv6Statistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv6-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def attempt_rate(self):
		"""decimal64: Attempt rate in sessions per second."""
		return self._get_value('attempt-rate')

	@property
	def bind_rate(self):
		"""decimal64: Binding  rate in sessions per second."""
		return self._get_value('bind-rate')

	@property
	def current_attempt_count(self):
		"""uint32: Number of DHCPv6 sessions currently being attempted."""
		return self._get_value('current-attempt-count')

	@property
	def current_bound_count(self):
		"""uint32: Number of DHCPv6 sessions that are currently bound. This statistic should match the DUT's Leased Count statistic."""
		return self._get_value('current-bound-count')

	@property
	def current_idle_count(self):
		"""uint32: Number of DHCPv6 sessions currently in the Idle state (sessions that have been programmed but are not being attempted)."""
		return self._get_value('current-idle-count')

	@property
	def avg_rebind_reply_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a REBIND message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('avg-rebind-reply-time')

	@property
	def avg_release_reply_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a RELEASE message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('avg-release-reply-time')

	@property
	def avg_renew_reply_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a RENEW message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('avg-renew-reply-time')

	@property
	def avg_request_reply_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a REQUEST message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('avg-request-reply-time')

	@property
	def avg_solicit_advertise_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and an ADVERTISE message was received from the delegating server."""
		return self._get_value('avg-solicit-advertise-time')

	@property
	def avg_solicit_reply_time(self):
		"""decimal64: Average amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('avg-solicit-reply-time')

	@property
	def max_rebind_reply_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a REBIND message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('max-rebind-reply-time')

	@property
	def max_release_reply_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a RELEASE message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('max-release-reply-time')

	@property
	def max_renew_reply_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a RENEW message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('max-renew-reply-time')

	@property
	def max_request_reply_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a REQUEST message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('max-request-reply-time')

	@property
	def max_solicit_advertise_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and an ADVERTISE message was received from the delegating server."""
		return self._get_value('max-solicit-advertise-time')

	@property
	def max_solicit_reply_time(self):
		"""decimal64: Maximum amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('max-solicit-reply-time')

	@property
	def min_rebind_reply_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a REBIND message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('min-rebind-reply-time')

	@property
	def min_release_reply_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a RELEASE message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('min-release-reply-time')

	@property
	def min_renew_reply_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a RENEW message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('min-renew-reply-time')

	@property
	def min_request_reply_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a REQUEST message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('min-request-reply-time')

	@property
	def min_solicit_advertise_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and an ADVERTISE message was received from the delegating server."""
		return self._get_value('min-solicit-advertise-time')

	@property
	def min_solicit_reply_time(self):
		"""decimal64: Minimum amount of time (in milliseconds) that elapsed between when a SOLICIT message was sent by a host and a REPLY message was received from the delegating server."""
		return self._get_value('min-solicit-reply-time')

	@property
	def elapsed_time(self):
		"""decimal64: Elapsed time."""
		return self._get_value('elapsed-time')

	@property
	def prefix_count(self):
		"""uint32: Number of prefixes allocated to the block."""
		return self._get_value('prefix-count')

	@property
	def rebind_rate(self):
		"""uint32: Rebind rate in sessions per second."""
		return self._get_value('rebind-rate')

	@property
	def release_rate(self):
		"""uint32: Release rate in sessions per second."""
		return self._get_value('release-rate')

	@property
	def renew_rate(self):
		"""uint32: Renew rate in sessions per second."""
		return self._get_value('renew-rate')

	@property
	def rx_advertise_count(self):
		"""uint32: Number of ADVERTISE messages received."""
		return self._get_value('rx-advertise-count')

	@property
	def rx_reconfigure_count(self):
		"""uint32: Number of RECONFIGURE messages received."""
		return self._get_value('rx-reconfigure-count')

	@property
	def rx_relay_reply_count(self):
		"""uint32: Number of RELAY-REPLY messages received"""
		return self._get_value('rx-relay-reply-count')

	@property
	def rx_reply_count(self):
		"""uint32: Number of REPLY messages received."""
		return self._get_value('rx-reply-count')

	@property
	def total_attempt_count(self):
		"""uint32: Total number of sessions attempted."""
		return self._get_value('total-attempt-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of sessions bound."""
		return self._get_value('total-bound-count')

	@property
	def total_declined_count(self):
		"""uint32: Total number of session declined.."""
		return self._get_value('total-declined-count')

	@property
	def total_failed_count(self):
		"""uint32: Number of hosts in the block that failed to bind. Failures may be due to DUT overload, a T1 or T2 timer expiration, or reaching the number of retries for the session."""
		return self._get_value('total-failed-count')

	@property
	def total_rebound_count(self):
		"""uint32: Total number of sessions rebound."""
		return self._get_value('total-rebound-count')

	@property
	def total_released_count(self):
		"""uint32: Total number of sessions released."""
		return self._get_value('total-released-count')

	@property
	def total_release_retried_count(self):
		"""uint32: Number of RELEASE messages resent when no REPLY message was received by the associated timeout."""
		return self._get_value('total-release-retried-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of sessions renewed."""
		return self._get_value('total-renewed-count')

	@property
	def total_renew_retried_count(self):
		"""uint32: Total number of RENEW messages resent."""
		return self._get_value('total-renew-retried-count')

	@property
	def total_retried_count(self):
		"""uint32: Number of SOLICIT and REQUEST messages resent for all hosts in the block."""
		return self._get_value('total-retried-count')

	@property
	def tx_confirm_count(self):
		"""uint32: Number of CONFIRM messages sent.."""
		return self._get_value('tx-confirm-count')

	@property
	def tx_decline_count(self):
		"""uint32: Number of DECLINE messages sent."""
		return self._get_value('tx-decline-count')

	@property
	def tx_info_request_count(self):
		"""uint32: Number of INFO-REQUEST messages sent."""
		return self._get_value('tx-info-request-count')

	@property
	def tx_rebind_count(self):
		"""uint32: Number of REBIND messages sent."""
		return self._get_value('tx-rebind-count')

	@property
	def tx_relay_forward_count(self):
		"""uint32: Number of RELAY-FORWARD messages sent."""
		return self._get_value('tx-relay-forward-count')

	@property
	def tx_release_count(self):
		"""uint32: Number of RELEASE  messages sent."""
		return self._get_value('tx-release-count')

	@property
	def tx_renew_count(self):
		"""uint32: Number of RENEW messages sent."""
		return self._get_value('tx-renew-count')

	@property
	def tx_request_count(self):
		"""uint32: Number of REQUEST messages sent."""
		return self._get_value('tx-request-count')

	@property
	def tx_solicit_count(self):
		"""uint32: Number of SOLICIT messages sent."""
		return self._get_value('tx-solicit-count')


class SessionsStatisticsDhcpv6PortStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-port-statistics'
		self.YANG_KEY = 'port-name'
		super(SessionsStatisticsDhcpv6PortStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv6-port-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def attempt_rate(self):
		"""decimal64: Attempt rate in sessions per second."""
		return self._get_value('attempt-rate')

	@property
	def avg_setup_time(self):
		"""decimal64: Average setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('avg-setup-time')

	@property
	def max_setup_time(self):
		"""decimal64: Maximum setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('max-setup-time')

	@property
	def min_setup_time(self):
		"""decimal64: Minimum setup time that was required to bind a session (in milliseconds)."""
		return self._get_value('min-setup-time')

	@property
	def bind_rate(self):
		"""decimal64: Binding rate in sessions per second."""
		return self._get_value('bind-rate')

	@property
	def current_bound_count(self):
		"""uint32: Number of DHCP hosts on the port that are bound (have been assigned an IP address). This statistic corresponds to the DUT's Leased Count statistic."""
		return self._get_value('current-bound-count')

	@property
	def current_idle_count(self):
		"""uint32: Number of sessions currently in the idle state."""
		return self._get_value('current-idle-count')

	@property
	def current_attempt_count(self):
		"""uint32: Number of sessions currently being attempted."""
		return self._get_value('current-attempt-count')

	@property
	def release_rate(self):
		"""decimal64: Rate at which all sessions on the ports were released by initiating RELEASE messages."""
		return self._get_value('release-rate')

	@property
	def rx_advertise_count(self):
		"""uint32: Number of ADVERTISE messages received."""
		return self._get_value('rx-advertise-count')

	@property
	def rx_reconfigure_count(self):
		"""uint32: Number of RECONFIGURE messages received."""
		return self._get_value('rx-reconfigure-count')

	@property
	def rx_relay_reply_count(self):
		"""uint32: Number of RELAY-REPLY messages received."""
		return self._get_value('rx-relay-reply-count')

	@property
	def rx_reply_count(self):
		"""uint32: Number of REPLY messages received."""
		return self._get_value('rx-reply-count')

	@property
	def success_percent(self):
		"""decimal64: Percentage of sessions that have successfully bound."""
		return self._get_value('success-percent')

	@property
	def total_attempt_count(self):
		"""uint32: Total number of sessions that were attempted."""
		return self._get_value('total-attempt-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of bound sessions."""
		return self._get_value('total-bound-count')

	@property
	def total_failed_count(self):
		"""uint32: Number of hosts on the port that failed to bind."""
		return self._get_value('total-failed-count')

	@property
	def total_rebound_count(self):
		"""uint32: Total number of sessions that were rebound."""
		return self._get_value('total-rebound-count')

	@property
	def total_released_count(self):
		"""uint32: Total number of released sessions."""
		return self._get_value('total-released-count')

	@property
	def total_release_retried_count(self):
		"""uint32: Total number of RENEW messages resent."""
		return self._get_value('total-release-retried-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of renewed sessions."""
		return self._get_value('total-renewed-count')

	@property
	def total_renew_retried_count(self):
		"""uint32: Number of RELEASE messages resent when no REPLY message was received by the associated timeout."""
		return self._get_value('total-renew-retried-count')

	@property
	def total_retried_count(self):
		"""uint32: Number of SOLICIT and REQUEST messages resent for all hosts on the port."""
		return self._get_value('total-retried-count')

	@property
	def tx_confirm_count(self):
		"""uint32: Number of CONFIRM messages sent."""
		return self._get_value('tx-confirm-count')

	@property
	def tx_decline_count(self):
		"""uint32: Number of DECLINE messages sent."""
		return self._get_value('tx-decline-count')

	@property
	def tx_info_request_count(self):
		"""uint32: Number of INFO-REQUEST messages sent."""
		return self._get_value('tx-info-request-count')

	@property
	def tx_rebind_count(self):
		"""uint32: Number of REBIND messages sent."""
		return self._get_value('tx-rebind-count')

	@property
	def tx_relay_forward_count(self):
		"""uint32: Number of RELAY-FORWARD messages sent."""
		return self._get_value('tx-relay-forward-count')

	@property
	def tx_release_count(self):
		"""uint32: Number of RELEASE messages sent."""
		return self._get_value('tx-release-count')

	@property
	def tx_renew_count(self):
		"""uint32: Number of RENEW messages sent."""
		return self._get_value('tx-renew-count')

	@property
	def tx_request_count(self):
		"""uint32: Number of REQUEST messages sent."""
		return self._get_value('tx-request-count')

	@property
	def tx_solicit_count(self):
		"""uint32: Number of SOLICIT messages sent."""
		return self._get_value('tx-solicit-count')


class SessionsStatisticsDhcpv6ServerStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'dhcpv6-server-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsDhcpv6ServerStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this dhcpv6-server-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def current_bound_count(self):
		"""uint32: Number of sessions that are currently bound."""
		return self._get_value('current-bound-count')

	@property
	def rx_confirm_count(self):
		"""uint32: Number of Confirm messages received."""
		return self._get_value('rx-confirm-count')

	@property
	def rx_decline_count(self):
		"""uint32: Number of Decline messages received."""
		return self._get_value('rx-decline-count')

	@property
	def rx_info_request_count(self):
		"""uint32: Number of Information-Request messages received."""
		return self._get_value('rx-info-request-count')

	@property
	def rx_rebind_count(self):
		"""uint32: Number of Rebind messages received."""
		return self._get_value('rx-rebind-count')

	@property
	def rx_release_count(self):
		"""uint32: Number of Release messages received."""
		return self._get_value('rx-release-count')

	@property
	def rx_renew_count(self):
		"""uint32: Number of Renew messages received."""
		return self._get_value('rx-renew-count')

	@property
	def rx_request_count(self):
		"""uint32: Number of Request messages received."""
		return self._get_value('rx-request-count')

	@property
	def rx_solicit_count(self):
		"""uint32: Number of Solicit messages received."""
		return self._get_value('rx-solicit-count')

	@property
	def total_bound_count(self):
		"""uint32: Total number of times a bound event occurred including renew events."""
		return self._get_value('total-bound-count')

	@property
	def total_expired_count(self):
		"""uint32: Total number of times an expired event occurred."""
		return self._get_value('total-expired-count')

	@property
	def total_released_count(self):
		"""uint32: Total number of times a release event occurred."""
		return self._get_value('total-released-count')

	@property
	def total_renewed_count(self):
		"""uint32: Total number of times a renew event occurred."""
		return self._get_value('total-renewed-count')

	@property
	def tx_advertise_count(self):
		"""uint32: Number of Advertise messages transmitted."""
		return self._get_value('tx-advertise-count')

	@property
	def tx_reconfigure_count(self):
		"""uint32: Number of Reconfigure messages transmitted."""
		return self._get_value('tx-reconfigure-count')

	@property
	def tx_reconfigure_rebind_count(self):
		"""uint32: Number of Reconfigure Rebind messages transmitted."""
		return self._get_value('tx-reconfigure-rebind-count')

	@property
	def tx_reconfigure_renew_count(self):
		"""uint32: Number of Reconfigure Renew messages transmitted."""
		return self._get_value('tx-reconfigure-renew-count')

	@property
	def tx_reply_count(self):
		"""uint32: Number of Reply messages transmitted."""
		return self._get_value('tx-reply-count')


class SessionsStatisticsIgmpStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsIgmpStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this igmp-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def host_state(self):
		""":str:`enum`: State of the IGMP host block  
		Enums:  
			UNDEFINED: Undefined  
			NON_MEMBER: No device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			JOINING: At least one device in the block has yet to send its initial unsolicited non-Leave report.  
			MEMBER: At least one device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			LEAVING: One or more devices in the block have yet to send their initial unsolicited Leave reports,      and no device in the block is sending non-Leave reports.  
		"""
		return self._get_value('host-state')

	@property
	def igmp_version(self):
		""":str:`enum`: IGMP version the device block is currently operating as  
		Enums:  
			V1: IGMPv1  
			V2: IGMPv2  
			V3: IGMPv3  
		"""
		return self._get_value('igmp-version')

	@property
	def router_state(self):
		""":str:`enum`: State of the IGMP Host  
		Enums:  
			UNDEFINED: Undefined  
			NOT_STARTED: Not started  
			UP: Already started and up  
		"""
		return self._get_value('router-state')

	@property
	def avg_join_latency(self):
		"""uint64: Average Join Latency"""
		return self._get_value('avg-join-latency')

	@property
	def avg_leave_latency(self):
		"""uint64: Average Leave Latency"""
		return self._get_value('avg-leave-latency')

	@property
	def max_join_latency(self):
		"""uint64: Maximum Join Latency."""
		return self._get_value('max-join-latency')

	@property
	def max_leave_latency(self):
		"""uint64: Maximum Leave Latency."""
		return self._get_value('max-leave-latency')

	@property
	def min_join_latency(self):
		"""uint64: Minimum Join Latency."""
		return self._get_value('min-join-latency')

	@property
	def min_leave_latency(self):
		"""uint64: Minimum Leave Latency."""
		return self._get_value('min-leave-latency')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of IGMP frames received."""
		return self._get_value('rx-frame-count')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of IGMP frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def rx_igmp_checksum_error_count(self):
		"""uint32: Total number of IGMP messages received with checksum errors."""
		return self._get_value('rx-igmp-checksum-error-count')

	@property
	def rx_igmp_length_error_count(self):
		"""uint32: Total number of IGMP messages received with length errors."""
		return self._get_value('rx-igmp-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')


class SessionsStatisticsIgmpPortStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-port-statistics'
		self.YANG_KEY = 'port-name'
		super(SessionsStatisticsIgmpPortStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this igmp-port-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of IGMP frames received."""
		return self._get_value('rx-frame-count')

	@property
	def rx_general_query_count(self):
		"""uint32: Total number of IGMP General Queries received. The General Query is used to learn which groups have members on an attached network."""
		return self._get_value('rx-general-query-count')

	@property
	def rx_group_and_source_specific_query_count(self):
		"""uint32: Group- and source-specific queries are sent by a multicast router whenever a host leave a specific source of a group. This is to make sure that there are no other hosts of that source and group."""
		return self._get_value('rx-group-and-source-specific-query-count')

	@property
	def rx_group_specific_query_count(self):
		"""uint32: Total number of IGMP group-specific queries transmitted. The Group-Specific Query is used to learn if a particular group has any members on an attached network.."""
		return self._get_value('rx-group-specific-query-count')

	@property
	def rx_igmp_checksum_error_count(self):
		"""uint32: Total number of IGMP messages received with checksum errors."""
		return self._get_value('rx-igmp-checksum-error-count')

	@property
	def rx_igmp_length_error_count(self):
		"""uint32: Total number of IGMP messages received with length errors."""
		return self._get_value('rx-igmp-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def rx_v1_query_count(self):
		"""uint32: IGMPv1 Host membership query messages are sent to discover which group have members."""
		return self._get_value('rx-v1-query-count')

	@property
	def rx_v1_report_count(self):
		"""uint32: IGMPv1 reports are sent to multicast routers to indicate that hosts have listeners interested in joining multicast groups whose multicast address is listed in the router's list."""
		return self._get_value('rx-v1-report-count')

	@property
	def rx_v2_query_count(self):
		"""uint32: Total number of IGMPv2 queries received. Routers use Multicast Listener Query messages to query a subnet for multicast listeners.."""
		return self._get_value('rx-v2-query-count')

	@property
	def rx_v2_report_count(self):
		"""uint32: Similar to IGMPv1 reports, IGMPv2 reports are sent by IGMPv2 hosts if they detect an IGMPv2 router."""
		return self._get_value('rx-v2-report-count')

	@property
	def rx_v3_query_count(self):
		"""uint32: IGMPv3 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces."""
		return self._get_value('rx-v3-query-count')

	@property
	def rx_v3_report_count(self):
		"""uint32: While functionally similar to IGMPv2 reports, IGMPv3 reports add support for source filtering. This means a host may report interest in receiving packets only from specific addresses. Or, from all but specific addresses sent to a multicast address. This information may be used to avoid delivering multicast packets from specific sources to networks where there are no interested hosts."""
		return self._get_value('rx-v3-report-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of IGMP frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def tx_general_query_count(self):
		"""uint32: otal number of IGMP General Queries transmitted. General Query is used to learn which groups have members on an attached network."""
		return self._get_value('tx-general-query-count')

	@property
	def tx_group_and_source_specific_query_count(self):
		"""uint32: Group- and source-specific queries are sent by a multicast router whenever a host leave a specific source of a group. This is to make sure that there are no other hosts of that source and group"""
		return self._get_value('tx-group-and-source-specific-query-count')

	@property
	def tx_group_specific_query_count(self):
		"""uint32: Total number of IGMP group-specific queries transmitted. The Group-Specific Query is used to learn if a particular group has any members on an attached network."""
		return self._get_value('tx-group-specific-query-count')

	@property
	def tx_leave_group_count(self):
		"""uint32: When an IGMP host leaves a multicast group, it may send a Leave Group message to the all-routers multicast group."""
		return self._get_value('tx-leave-group-count')

	@property
	def tx_v1_query_count(self):
		"""uint32: IGMPv1 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces"""
		return self._get_value('tx-v1-query-count')

	@property
	def tx_v1_report_count(self):
		"""uint32: IGMPv1 reports are sent to multicast routers to indicate that hosts have listeners interested in joining multicast groups whose multicast address is listed in the router's list.."""
		return self._get_value('tx-v1-report-count')

	@property
	def tx_v2_query_count(self):
		"""uint32: IGMPv2 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces."""
		return self._get_value('tx-v2-query-count')

	@property
	def tx_v2_report_count(self):
		"""uint32: Similar to IGMPv1 reports, IGMPv2 reports are sent by IGMPv2 hosts if they detect an IGMPv2 router."""
		return self._get_value('tx-v2-report-count')

	@property
	def tx_v3_query_count(self):
		"""uint32: IGMPv3 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces."""
		return self._get_value('tx-v3-query-count')

	@property
	def tx_v3_report_count(self):
		"""uint32: While functionally similar to IGMPv2 reports, IGMPv3 reports add support for source filtering. This means a host may report interest in receiving packets only from specific addresses. Or, from all but specific addresses sent to a multicast address. This information may be used to avoid delivering multicast packets from specific sources to networks where there are no interested hosts."""
		return self._get_value('tx-v3-report-count')

	@property
	def tx_v3_allow_new_sources_count(self):
		"""uint32: A Source-List-Change Record (SLCR) indicating the group's associated sources have changed such that data from a new set of sources are to be received.."""
		return self._get_value('tx-v3-allow-new-sources-count')

	@property
	def tx_v3_block_old_sources_count(self):
		"""uint32: A Source-List-Change Record (SLCR) indicating the group's associated sources have changed such that data from an existing set of sources are not required."""
		return self._get_value('tx-v3-block-old-sources-count')

	@property
	def tx_v3_change_to_exclude_mode_count(self):
		"""uint32: A Filter-Mode-Change Record (FMCR) indicating the filter-mode of the reception state has changed to exclude mode."""
		return self._get_value('tx-v3-change-to-exclude-mode-count')

	@property
	def tx_v3_change_to_include_mode_count(self):
		"""uint32: A Filter-Mode-Change Record (FMCR) indicating the filter-mode of the reception state has changed to include mode."""
		return self._get_value('tx-v3-change-to-include-mode-count')

	@property
	def tx_v3_mode_is_exclude_count(self):
		"""uint32: A Current-State Record (CSR) indicating the current reception state with respect to 1 multicast group at a given interface. The state contains the exclude filter mode."""
		return self._get_value('tx-v3-mode-is-exclude-count')

	@property
	def tx_v3_mode_is_include_count(self):
		"""uint32: A Current-State Record (CSR) indicating the current reception state with respect to 1 multicast group at a given interface. The state contains the include filter mode."""
		return self._get_value('tx-v3-mode-is-include-count')


class SessionsStatisticsIgmpHostGroupStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-host-group-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsIgmpHostGroupStatistics, self).__init__(parent, yang_key_value)

	def igmp_host(self, host_address=None):
		"""Get the igmp-host object(s) from the server.

		TBD

		Args:  
			host_address (:obj:`str`, optional, default=None): A key value in the igmp-host list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsIgmpHostGroupStatisticsIgmpHost` | :obj:`SessionsStatisticsIgmpHostGroupStatisticsIgmpHost`: If arg host_address is None a list of SessionsStatisticsIgmpHostGroupStatisticsIgmpHost objects otherwise a single SessionsStatisticsIgmpHostGroupStatisticsIgmpHost object.  

		Raises:  
			NotFoundError: host_address is not in the list of igmp-host objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsIgmpHostGroupStatisticsIgmpHost(self, host_address))

	def create_igmp_host(self, host_address):
		"""Create a child sibling igmp-host instance on the server.

		TBD

		Args:  
			host_address (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsIgmpHostGroupStatisticsIgmpHost`: An object encapsulating an instance of the igmp-host model.  

		Raises:  
			AlreadyExistsError: An instance of igmp-host with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsIgmpHostGroupStatisticsIgmpHost(self, host_address), locals())

	def update(self):
		"""Update this igmp-host-group-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')


class SessionsStatisticsIgmpHostGroupStatisticsIgmpHost(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-host'
		self.YANG_KEY = 'host-address'
		super(SessionsStatisticsIgmpHostGroupStatisticsIgmpHost, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this igmp-host instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def host_address(self):
		"""inet:ipv4-address: Host IP address."""
		return self._get_value('host-address')

	@property
	def group_address(self):
		"""inet:ipv4-address: Group IP multicast address."""
		return self._get_value('group-address')

	@property
	def state(self):
		""":str:`enum`: Current state of the IGMP group  
		Enums:  
			UNDEFINED: Host is in an undefined state.  
			NON_MEMBER: Host does not belong to the group on the interface. This state applies only to IGMPv1 and IGMPv2 and is the initial state for all memberships on all network interfaces.  
			DELAYING_MEMBER: Host belongs to the group on the interface and has a report delay timer running for that membership. This state applies to IGMPv1 and IGMPv2only.  
			IDLE_MEMBER: Host belongs to the group on the interface and does not have a report delay timer running for that membership. This state applies to IGMPv1 and IGMPv2 only.  
			RETRYING_MEMBER: This state applies to IGMPv1/IGMPv2 hosts when force-robust-join is True. The host is retransmitting its initial join before transitioning from the NON_MEMBER state to the DELAYING_MEMBER or IDLE_MEMBER state  
			INCLUDE: Data from the specified sources are filtered and forwarded to the host by the multicast router. This state applies to IGMPv3 only.  
			EXCLUDE: Data from the specified sources are filtered and not forwarded to the host by the multicast router. This state applies to IGMPv3 only.  
		"""
		return self._get_value('state')

	@property
	def duplicate_join(self):
		"""bool: Indicates whether or not the DUT was already forwarding multicast prior to a join being sent. TRUE  : DUT was forwarding multicast traffic for this group prior to the join being sent. FALSE : No duplicate join report detected"""
		return self._get_value('duplicate-join')

	@property
	def join_fail(self):
		"""bool: Indicates whether or not multicast traffic was received. TRUE  : Multicast traffic was not received. FALSE : Multicast traffic was received"""
		return self._get_value('join-fail')

	@property
	def join_latency(self):
		"""uint64: Time between sending an IGMP join message, and receiving multicast data from the group specified in the join message."""
		return self._get_value('join-latency')

	@property
	def leave_latency(self):
		"""uint64: Time between sending an IGMP leave message to a multicast group, and when the multicast data stopped traffic."""
		return self._get_value('leave-latency')

	@property
	def join_timestamp(self):
		"""str: Transmit timestamp of the initial IGMP join message."""
		return self._get_value('join-timestamp')

	@property
	def leave_timestamp(self):
		"""str: Transmit timestamp of the IGMP leave message."""
		return self._get_value('leave-timestamp')

	@property
	def state_change_timestamp(self):
		"""str: Timestamp of the IGMP group membership state change."""
		return self._get_value('state-change-timestamp')


class SessionsStatisticsIgmpQuerierStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'igmp-querier-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsIgmpQuerierStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this igmp-querier-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def host_state(self):
		""":str:`enum`: State of the IGMP Querier block  
		Enums:  
			UNDEFINED: Undefined  
			NON_MEMBER: No device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			JOINING: At least one device in the block has yet to send its initial unsolicited non-Leave report.  
			MEMBER: At least one device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			LEAVING: One or more devices in the block have yet to send their initial unsolicited Leave reports,      and no device in the block is sending non-Leave reports.  
		"""
		return self._get_value('host-state')

	@property
	def igmp_querier_version(self):
		""":str:`enum`: IGMP version the device block is currently operating as  
		Enums:  
			V1: IGMPv1  
			V2: IGMPv2  
			V3: IGMPv3  
		"""
		return self._get_value('igmp-querier-version')

	@property
	def router_state(self):
		""":str:`enum`: State of the IGMP Host  
		Enums:  
			UNDEFINED: Undefined  
			NOT_STARTED: Not started  
			UP: Already started and up  
		"""
		return self._get_value('router-state')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of IGMP frames received."""
		return self._get_value('rx-frame-count')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of IGMP frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def rx_igmp_checksum_error_count(self):
		"""uint32: Total number of IGMP messages received with checksum errors."""
		return self._get_value('rx-igmp-checksum-error-count')

	@property
	def rx_igmp_length_error_count(self):
		"""uint32: Total number of IGMP messages received with length errors."""
		return self._get_value('rx-igmp-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')


class SessionsStatisticsMldStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsMldStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this mld-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def host_state(self):
		""":str:`enum`: State of the MLD host block  
		Enums:  
			UNDEFINED: Undefined  
			NON_MEMBER: No device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			JOINING: At least one device in the block has yet to send its initial unsolicited non-Leave report.  
			MEMBER: At least one device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			LEAVING: One or more devices in the block have yet to send their initial unsolicited Leave reports,      and no device in the block is sending non-Leave reports.  
		"""
		return self._get_value('host-state')

	@property
	def mld_version(self):
		""":str:`enum`: MLD version the device block is currently operating as  
		Enums:  
			V1: MLDv1  
			V2: MLDv2  
		"""
		return self._get_value('mld-version')

	@property
	def router_state(self):
		""":str:`enum`: State of the MLD Host  
		Enums:  
			UNDEFINED: Undefined  
			NOT_STARTED: Not started  
			UP: Already started and up  
		"""
		return self._get_value('router-state')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of MLD frames received."""
		return self._get_value('rx-frame-count')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of MLD frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def rx_mld_checksum_error_count(self):
		"""uint32: Total number of MLD messages received with checksum errors."""
		return self._get_value('rx-mld-checksum-error-count')

	@property
	def rx_mld_length_error_count(self):
		"""uint32: Total number of MLD messages received with length errors."""
		return self._get_value('rx-mld-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')


class SessionsStatisticsMldPortStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-port-statistics'
		self.YANG_KEY = 'port-name'
		super(SessionsStatisticsMldPortStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this mld-port-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of MLD frames received."""
		return self._get_value('rx-frame-count')

	@property
	def rx_general_query_count(self):
		"""uint32: Total number of multicast general queries received."""
		return self._get_value('rx-general-query-count')

	@property
	def rx_group_and_source_specific_query_count(self):
		"""uint32: Group- and source-specific queries are sent by a multicast router whenever a host leave a specific source of a group. This is to make sure that there are no other hosts of that source and group."""
		return self._get_value('rx-group-and-source-specific-query-count')

	@property
	def rx_group_specific_query_count(self):
		"""uint32: Group-Specific Query is used to learn if a particular group has any members on an attached network."""
		return self._get_value('rx-group-specific-query-count')

	@property
	def rx_mld_checksum_error_count(self):
		"""uint32: Total number of MLD messages received with checksum errors."""
		return self._get_value('rx-mld-checksum-error-count')

	@property
	def rx_mld_length_error_count(self):
		"""uint32: Total number of MLD messages received with length errors."""
		return self._get_value('rx-mld-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def rx_v1_query_count(self):
		"""uint32: Total number of MLDv1 queries received. Routers use Multicast Listener Query messages to query a subnet for multicast listeners."""
		return self._get_value('rx-v1-query-count')

	@property
	def rx_v1_report_count(self):
		"""uint32: MLDv1 reports are sent to multicast routers to indicate that hosts have listeners interested in joining multicast groups whose multicast address is listed in the router's list."""
		return self._get_value('rx-v1-report-count')

	@property
	def rx_v2_query_count(self):
		"""uint32: Total number of MLDv2 queries received. Routers use Multicast Listener Query messages to query a subnet for multicast listeners."""
		return self._get_value('rx-v2-query-count')

	@property
	def rx_v2_report_count(self):
		"""uint32: MLDv1 are used to report interest in receiving multicast traffic for a specific multicast address or to respond to a Multicast Listener Query message.."""
		return self._get_value('rx-v2-report-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of MLD frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def tx_general_query_count(self):
		"""uint32: General Queries are used to learn which multicast addresses have listeners on an attached link."""
		return self._get_value('tx-general-query-count')

	@property
	def tx_group_and_source_specific_query_count(self):
		"""uint32: Group- and source-specific queries are sent by a multicast router whenever a host leave a specific source of a group. This is to make sure that there are no other hosts of that source and group."""
		return self._get_value('tx-group-and-source-specific-query-count')

	@property
	def tx_group_specific_query_count(self):
		"""uint32: Total number of MLD group specific queries transmitted. The Group-Specific Query is used to learn if a particular group has any members on an attached network.."""
		return self._get_value('tx-group-specific-query-count')

	@property
	def tx_stop_listen_group_count(self):
		"""uint32: Stop listening events occur when the node stops listening to an address on the interface."""
		return self._get_value('tx-stop-listen-group-count')

	@property
	def tx_v1_query_count(self):
		"""uint32: MLDv1 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces."""
		return self._get_value('tx-v1-query-count')

	@property
	def tx_v1_report_count(self):
		"""uint32: MLDv1 reports are sent to multicast routers to indicate that hosts have listeners interested in joining multicast groups whose multicast address is listed in the router's list."""
		return self._get_value('tx-v1-report-count')

	@property
	def tx_v2_query_count(self):
		"""uint32: MLDv2 Membership Queries are sent by IP multicast routers to query the multicast reception state of neighboring interfaces."""
		return self._get_value('tx-v2-query-count')

	@property
	def tx_v2_report_count(self):
		"""uint32: MLDv2 reports are sent to multicast routers to indicate that hosts have listeners interested in joining multicast groups whose multicast address is listed in the router's list."""
		return self._get_value('tx-v2-report-count')

	@property
	def tx_v2_allow_new_sources_count(self):
		"""uint32: A Source-List-Change Record (SLCR) indicating the group's associated sources have changed such that data from a new set of sources are to be received"""
		return self._get_value('tx-v2-allow-new-sources-count')

	@property
	def tx_v2_block_old_sources_count(self):
		"""uint32: A Source-List-Change Record (SLCR) indicating the group's associated sources have changed such that data from an existing set of sources are not required.."""
		return self._get_value('tx-v2-block-old-sources-count')

	@property
	def tx_v2_change_to_exclude_mode_count(self):
		"""uint32: A Filter-Mode-Change Record (FMCR) indicating the filter-mode of the reception state has changed to exclude mode."""
		return self._get_value('tx-v2-change-to-exclude-mode-count')

	@property
	def tx_v2_change_to_include_mode_count(self):
		"""uint32: A Filter-Mode-Change Record (FMCR) indicating the filter-mode of the reception state has changed to include mode"""
		return self._get_value('tx-v2-change-to-include-mode-count')

	@property
	def tx_v2_mode_is_exclude_count(self):
		"""uint32: A Current-State Record (CSR) indicating the current reception state with respect to 1 multicast group at a given interface. The state contains the exclude filter mode."""
		return self._get_value('tx-v2-mode-is-exclude-count')

	@property
	def tx_v2_mode_is_include_count(self):
		"""uint32: A Current-State Record (CSR) indicating the current reception state with respect to 1 multicast group at a given interface. The state contains the include filter mode."""
		return self._get_value('tx-v2-mode-is-include-count')


class SessionsStatisticsMldHostGroupStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-host-group-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsMldHostGroupStatistics, self).__init__(parent, yang_key_value)

	def mld_host(self, host_address=None):
		"""Get the mld-host object(s) from the server.

		TBD

		Args:  
			host_address (:obj:`str`, optional, default=None): A key value in the mld-host list.  

		Returns:  
			:obj:`list` of :obj:`SessionsStatisticsMldHostGroupStatisticsMldHost` | :obj:`SessionsStatisticsMldHostGroupStatisticsMldHost`: If arg host_address is None a list of SessionsStatisticsMldHostGroupStatisticsMldHost objects otherwise a single SessionsStatisticsMldHostGroupStatisticsMldHost object.  

		Raises:  
			NotFoundError: host_address is not in the list of mld-host objects.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._read(SessionsStatisticsMldHostGroupStatisticsMldHost(self, host_address))

	def create_mld_host(self, host_address):
		"""Create a child sibling mld-host instance on the server.

		TBD

		Args:  
			host_address (str): A unique key value that does not exist in the list on the server.  

		Returns:  
			:obj:`SessionsStatisticsMldHostGroupStatisticsMldHost`: An object encapsulating an instance of the mld-host model.  

		Raises:  
			AlreadyExistsError: An instance of mld-host with the supplied key value already exists on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._create(SessionsStatisticsMldHostGroupStatisticsMldHost(self, host_address), locals())

	def update(self):
		"""Update this mld-host-group-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')


class SessionsStatisticsMldHostGroupStatisticsMldHost(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-host'
		self.YANG_KEY = 'host-address'
		super(SessionsStatisticsMldHostGroupStatisticsMldHost, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this mld-host instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def host_address(self):
		"""inet:ipv6-address: Host IP address."""
		return self._get_value('host-address')

	@property
	def group_address(self):
		"""inet:ipv6-address: Group IP multicast address."""
		return self._get_value('group-address')

	@property
	def state(self):
		""":str:`enum`: Current state of the MLD group  
		Enums:  
			UNDEFINED: Host is in an undefined state.  
			NON_MEMBER: Host does not belong to the group on the interface. This state applies only to MLDv1 and is the initial state for all memberships on all network interfaces.  
			DELAYING_MEMBER: Host belongs to the group on the interface and has a report delay timer running for that membership. This state applies to MLDv1 only.  
			IDLE_MEMBER: Host belongs to the group on the interface and does not have a report delay timer running for that membership. This state applies to MLDv1 only.  
			RETRYING_MEMBER: This state applies to MLDv1 hosts when force-robust-join is True. The host is retransmitting its initial join before transitioning from the NON_MEMBER state to the DELAYING_MEMBER or IDLE_MEMBER state. This state applies to MLDv1 only  
			INCLUDE: Data from the specified sources are filtered and forwarded to the host by the multicast router. This state applies to MLDv2 only.  
			EXCLUDE: Data from the specified sources are filtered and not forwarded to the host by the multicast router. This state applies to MLDv2 only.  
		"""
		return self._get_value('state')

	@property
	def duplicate_join(self):
		"""bool: Indicates whether or not the DUT was already forwarding multicast prior to a join being sent. TRUE  : DUT was forwarding multicast traffic for this group prior to the join being sent. FALSE : No duplicate join report detected"""
		return self._get_value('duplicate-join')

	@property
	def join_fail(self):
		"""bool: Indicates whether or not multicast traffic was received. TRUE  : Multicast traffic was not received. FALSE : Multicast traffic was received"""
		return self._get_value('join-fail')

	@property
	def join_latency(self):
		"""uint64: Time between sending an MLD join message, and receiving multicast data from the group specified in the join message."""
		return self._get_value('join-latency')

	@property
	def leave_latency(self):
		"""uint64: Time between sending an MLD leave message to a multicast group, and when the multicast data stopped traffic."""
		return self._get_value('leave-latency')

	@property
	def join_timestamp(self):
		"""str: Transmit timestamp of the initial MLD join message."""
		return self._get_value('join-timestamp')

	@property
	def leave_timestamp(self):
		"""str: Transmit timestamp of the MLD leave message."""
		return self._get_value('leave-timestamp')

	@property
	def state_change_timestamp(self):
		"""str: Timestamp of the MLD group membership state change."""
		return self._get_value('state-change-timestamp')


class SessionsStatisticsMldQuerierStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'mld-querier-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsMldQuerierStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this mld-querier-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def port_name(self):
		"""str: An abstract test port name"""
		return self._get_value('port-name')

	@property
	def host_state(self):
		""":str:`enum`: State of the MLD Querier block  
		Enums:  
			UNDEFINED: Undefined  
			NON_MEMBER: No device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			JOINING: At least one device in the block has yet to send its initial unsolicited non-Leave report.  
			MEMBER: At least one device in the block is joined to a group, and there are no initial unsolicited reports to be sent.  
			LEAVING: One or more devices in the block have yet to send their initial unsolicited Leave reports,      and no device in the block is sending non-Leave reports.  
		"""
		return self._get_value('host-state')

	@property
	def mld_querier_version(self):
		""":str:`enum`: MLD version the device block is currently operating as  
		Enums:  
			V1: MLDv1  
			V2: MLDv2  
		"""
		return self._get_value('mld-querier-version')

	@property
	def router_state(self):
		""":str:`enum`: State of the MLD Host  
		Enums:  
			UNDEFINED: Undefined  
			NOT_STARTED: Not started  
			UP: Already started and up  
		"""
		return self._get_value('router-state')

	@property
	def rx_frame_count(self):
		"""uint32: Total number of MLD frames received."""
		return self._get_value('rx-frame-count')

	@property
	def tx_frame_count(self):
		"""uint32: Total number of MLD frames transmitted."""
		return self._get_value('tx-frame-count')

	@property
	def rx_mld_checksum_error_count(self):
		"""uint32: Total number of MLD messages received with checksum errors."""
		return self._get_value('rx-mld-checksum-error-count')

	@property
	def rx_mld_length_error_count(self):
		"""uint32: Total number of MLD messages received with length errors."""
		return self._get_value('rx-mld-length-error-count')

	@property
	def rx_unknown_type_count(self):
		"""uint32: Total number of frames of unknown type received."""
		return self._get_value('rx-unknown-type-count')

	@property
	def time_stamp(self):
		"""uint32: Timestamp in seconds of last statistic update."""
		return self._get_value('time-stamp')


class SessionsStatisticsPimStatistics(YangBase):
	"""TBD
	"""
	def __init__(self, parent, yang_key_value=None):
		self.YANG_MODULE = 'openhltest'
		self.YANG_CLASS = 'pim-statistics'
		self.YANG_KEY = 'device-name'
		super(SessionsStatisticsPimStatistics, self).__init__(parent, yang_key_value)

	def update(self):
		"""Update this pim-statistics instance on the server with any changed property values.

		TBD

		Raises:  
			NotFoundError: This instance does not exist on the server.  
			ServerError: An abnormal server error has occurred.  
		"""
		return self._update()

	@property
	def device_name(self):
		"""str: Device Name"""
		return self._get_value('device-name')

	@property
	def mdt_join_count(self):
		"""uint64: Number of MDT join messages."""
		return self._get_value('mdt-join-count')

	@property
	def neighbor_count(self):
		"""uint64: Number of PIM neighbors for this router."""
		return self._get_value('neighbor-count')

	@property
	def rx_assert_count(self):
		"""uint64: Number of Assert messages received."""
		return self._get_value('rx-assert-count')

	@property
	def rx_bootstrap_count(self):
		"""uint64: Number of Bootstrap messages received."""
		return self._get_value('rx-bootstrap-count')

	@property
	def rx_c_and_rp_advert_count(self):
		"""uint64: Number of candidate Rendezvous Point (RP) Advertisements received."""
		return self._get_value('rx-c-and-rp-advert-count')

	@property
	def rx_group_rp_count(self):
		"""uint64: Number of (*,*,RP) groups received."""
		return self._get_value('rx-group-rp-count')

	@property
	def rx_group_s_g_count(self):
		"""uint64: Number of (S,G) groups received."""
		return self._get_value('rx-group-s-g-count')

	@property
	def rx_group_s_g_rpt_count(self):
		"""uint64: Number of (S,G,rpt) groups received."""
		return self._get_value('rx-group-s-g-rpt-count')

	@property
	def rx_group_star_g_count(self):
		"""uint64: Number of (*,G) groups received."""
		return self._get_value('rx-group-star-g-count')

	@property
	def rx_hello_count(self):
		"""uint64: Number of Hello messages received."""
		return self._get_value('rx-hello-count')

	@property
	def rx_join_prune_count(self):
		"""uint64: Number of Join/Prune messages received."""
		return self._get_value('rx-join-prune-count')

	@property
	def rx_register_count(self):
		"""uint64: Number of Register messages received."""
		return self._get_value('rx-register-count')

	@property
	def rx_register_stop_count(self):
		"""uint64: Number of Register Stop messages received."""
		return self._get_value('rx-register-stop-count')

	@property
	def tx_assert_count(self):
		"""uint64: Number of Assert messages sent."""
		return self._get_value('tx-assert-count')

	@property
	def tx_bootstrap_count(self):
		"""uint64: Number of Bootstrap messages sent."""
		return self._get_value('tx-bootstrap-count')

	@property
	def tx_c_and_rp_advert_count(self):
		"""uint64: Number of candidate Rendezvous Point (RP) Advertisements sent."""
		return self._get_value('tx-c-and-rp-advert-count')

	@property
	def tx_group_rp_count(self):
		"""uint64: Number of (*,*,RP) groups sent."""
		return self._get_value('tx-group-rp-count')

	@property
	def tx_group_s_g_count(self):
		"""uint64: Number of (S,G) groups sent."""
		return self._get_value('tx-group-s-g-count')

	@property
	def tx_group_s_g_rpt_count(self):
		"""uint64: Number of (S,G,rpt) groups sent."""
		return self._get_value('tx-group-s-g-rpt-count')

	@property
	def tx_group_star_g_count(self):
		"""uint64: Number of (*,G) groups sent."""
		return self._get_value('tx-group-star-g-count')

	@property
	def tx_hello_count(self):
		"""uint64: Number of Hello messages sent."""
		return self._get_value('tx-hello-count')

	@property
	def tx_join_prune_count(self):
		"""uint64: Number of Join/Prune messages sent."""
		return self._get_value('tx-join-prune-count')

	@property
	def tx_register_count(self):
		"""uint64: Number of Register messages sent."""
		return self._get_value('tx-register-count')

	@property
	def tx_register_stop_count(self):
		"""uint64: Number of Register Stop messages sent."""
		return self._get_value('tx-register-stop-count')


